@@ -1,6 +1,5 @@
-/*
-    Copyright (C) 2000, S.R.Haque <shaheedhaque@hotmail.com>.
-    This file is part of the KDE project
+/*  This file is part of the KDE project
+    Copyright (C) 2000, S.R.Haque <shaheedhaque@hotmail.com>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -14,8 +13,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 
 DESCRIPTION
 */
@@ -23,8 +22,7 @@ DESCRIPTION
 #ifndef WMFIMPORT_H
 #define WMFIMPORT_H
 
-#include <koFilter.h>
-#include <koStore.h>
+#include <KoFilter.h>
 
 
 class WMFImport : public KoFilter
@@ -32,10 +30,10 @@ class WMFImport : public KoFilter
     Q_OBJECT
 
 public:
-    WMFImport( KoFilter *parent, const char *name, const QStringList&);
+    WMFImport(QObject* parent, const QStringList&);
     virtual ~WMFImport();
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 
 };
 
