@@ -15,8 +15,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 
 DESCRIPTION
 */
@@ -26,21 +26,21 @@ DESCRIPTION
 
 #include <klibloader.h>
 
-class KInstance;
+class KComponentData;
 
 class XFIGImportFactory :
-    public KLibFactory
+            public KLibFactory
 {
     Q_OBJECT
 public:
-    XFIGImportFactory(QObject* parent = 0, const char* name = 0);
+    explicit XFIGImportFactory(QObject* parent = 0, const char* name = 0);
     virtual ~XFIGImportFactory();
 
     virtual QObject* createObject(QObject* parent = 0, const char* name = 0, const char* classname = "QObject", const QStringList &args = QStringList());
 
-    static KInstance* global();
+    static const KComponentData &global();
 
 private:
-    static KInstance* s_global;
+    static KComponentData* s_global;
 };
 #endif
