@@ -17,8 +17,11 @@
    Boston, MA  02110-1301  USA.
 */
 
-#include <qpicture.h>
-#include <qpainter.h>
+#include <q3picture.h>
+#include <QPainter>
+#include <QDomDocument>
+//Added by qt3to4:
+#include <QByteArray>
 
 #include <kmessagebox.h>
 
@@ -27,16 +30,16 @@
 //#include <KoStoreDevice.h>
 #include <kgenericfactory.h>
 
-#include "kchart_part.h"
+#include "KChartPart.h"
 
 #include "svgexport.h"
 
 
-typedef KGenericFactory<SvgExport, KoFilter> SvgExportFactory;
-K_EXPORT_COMPONENT_FACTORY( libkchartsvgexport, SvgExportFactory( "svgexport" ) )
+typedef KGenericFactory<SvgExport> SvgExportFactory;
+K_EXPORT_COMPONENT_FACTORY(libkchartsvgexport, SvgExportFactory("svgexport"))
 
-SvgExport::SvgExport(KoFilter *, const char *, const QStringList&) 
-    : KoFilter()
+SvgExport::SvgExport(QObject* parent, const QStringList&)
+        : KoFilter(parent)
 {
 }
 
@@ -46,44 +49,44 @@ SvgExport::~SvgExport()
 
 
 KoFilter::ConversionStatus
-SvgExport::convert(const QCString& from, const QCString& to)
+SvgExport::convert(const QByteArray& from, const QByteArray& to)
 {
     // Check for proper conversion.
-    if ( from != "application/x-kchart" || to != "image/svg+xml" )
+    if (from != "application/x-kchart" || to != "image/svg+xml")
         return KoFilter::NotImplemented;
 
     // Read the contents of the KChart file
-    KoStoreDevice* storeIn = m_chain->storageFile( "root", KoStore::Read );
-    if ( !storeIn ) {
-	KMessageBox::error( 0, i18n("Failed to read data." ), 
-			    i18n( "SVG Export Error" ) );
-	return KoFilter::FileNotFound;
+    KoStoreDevice* storeIn = m_chain->storageFile("root", KoStore::Read);
+    if (!storeIn) {
+        KMessageBox::error(0, i18n("Failed to read data."),
+                           i18n("SVG Export Error"));
+        return KoFilter::FileNotFound;
     }
 
     // Get the XML tree.
-    QDomDocument  domIn;
-    domIn.setContent( storeIn );
-    QDomElement   docNode = domIn.documentElement();
+    KoXmlDocument  domIn;
+    domIn.setContent(storeIn);
+    KoXmlElement   docNode = domIn.documentElement();
 
     // Read the document from the XML tree.
     KChart::KChartPart  kchartDoc;
-    if ( !kchartDoc.loadXML(0, domIn) ) {
-        KMessageBox::error( 0, i18n( "Malformed XML data." ), 
-			    i18n( "SVG Export Error" ) );
+    if (!kchartDoc.loadXML(domIn, 0)) {
+        KMessageBox::error(0, i18n("Malformed XML data."),
+                           i18n("SVG Export Error"));
         return KoFilter::WrongFormat;
     }
 
     // Draw the actual bitmap.
-    QPicture  picture;
+    Q3Picture  picture;
     QPainter  painter(&picture);
     QRect     rect(QPoint(0, 0), QPoint(500, 400));
-    kchartDoc.paintContent(painter, rect, false);
+    kchartDoc.paintContent(painter, rect);
     painter.end();
 
     // Save the image.
-    if ( !picture.save( m_chain->outputFile(), "SVG" ) ) {
-        KMessageBox::error( 0, i18n( "Failed to write file." ), 
-			    i18n( "SVG Export Error" ) );
+    if (!picture.save(m_chain->outputFile(), "SVG")) {
+        KMessageBox::error(0, i18n("Failed to write file."),
+                           i18n("SVG Export Error"));
     }
 
     return KoFilter::OK;
