@@ -13,78 +13,81 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
-#include <qstring.h>
-#include <qtextstream.h>
-#include <qfile.h>
+#include <QString>
+#include <QTextStream>
+#include <QFile>
+//Added by qt3to4:
+#include <QByteArray>
 
 #include <kapplication.h>
 #include <kdebug.h>
 #include <kgenericfactory.h>
 #include <kmessagebox.h>
 
-#include <koFilterChain.h>
-#include <koStoreDevice.h>
+#include <KoFilterChain.h>
+#include <KoStoreDevice.h>
 
 #include <kformuladocument.h>
 #include <kformulacontainer.h>
 #include <kformulamimesource.h>
+#include <kglobal.h>
 
 #include "latexexport.h"
 
 
-typedef KGenericFactory<LATEXExport, KoFilter> LATEXExportFactory;
-K_EXPORT_COMPONENT_FACTORY( libkfolatexexport, LATEXExportFactory( "kofficefilters" ) )
+typedef KGenericFactory<LATEXExport> LATEXExportFactory;
+K_EXPORT_COMPONENT_FACTORY(libkfolatexexport, LATEXExportFactory("kofficefilters"))
 
 
-LATEXExport::LATEXExport( KoFilter */*parent*/, const char */*name*/, const QStringList& )
-    : KoFilter()
+LATEXExport::LATEXExport(QObject* parent, const QStringList&)
+        : KoFilter(parent)
 {
 }
 
 
-KoFilter::ConversionStatus LATEXExport::convert( const QCString& from, const QCString& to )
+KoFilter::ConversionStatus LATEXExport::convert(const QByteArray& from, const QByteArray& to)
 {
-    if ( to != "text/x-tex" || from != "application/x-kformula" )
+    if (to != "text/x-tex" || from != "application/x-kformula")
         return KoFilter::NotImplemented;
 
-    KoStoreDevice* in = m_chain->storageFile( "root", KoStore::Read );
-    if(!in) {
+    KoStoreDevice* in = m_chain->storageFile("root", KoStore::Read);
+    if (!in) {
         QApplication::restoreOverrideCursor();
-        KMessageBox::error( 0, i18n( "Failed to read data." ), i18n( "LaTeX Export Error" ) );
+        KMessageBox::error(0, i18n("Failed to read data."), i18n("LaTeX Export Error"));
         return KoFilter::StorageCreationError;
     }
 
-    QDomDocument dom( "KFORMULA" );
-    if ( !dom.setContent( in, false ) ) {
+    QDomDocument dom("KFORMULA");
+    if (!dom.setContent(in, false)) {
         QApplication::restoreOverrideCursor();
-        KMessageBox::error( 0, i18n( "Malformed XML data." ), i18n( "LaTeX Export Error" ) );
+        KMessageBox::error(0, i18n("Malformed XML data."), i18n("LaTeX Export Error"));
         return KoFilter::WrongFormat;
     }
 
-    QFile f( m_chain->outputFile() );
-    if( !f.open( IO_Truncate | IO_ReadWrite ) ) {
+    QFile f(m_chain->outputFile());
+    if (!f.open(QIODevice::Truncate | QIODevice::ReadWrite)) {
         QApplication::restoreOverrideCursor();
-        KMessageBox::error( 0, i18n( "Failed to write file." ), i18n( "LaTeX Export Error" ) );
+        KMessageBox::error(0, i18n("Failed to write file."), i18n("LaTeX Export Error"));
         return KoFilter::FileNotFound;
     }
 
-    KFormula::DocumentWrapper* wrapper = new KFormula::DocumentWrapper( kapp->config(), 0 );
+    KFormula::DocumentWrapper* wrapper = new KFormula::DocumentWrapper(KGlobal::config(), 0);
     KFormula::Document* doc = new KFormula::Document;
-    wrapper->document( doc );
+    wrapper->document(doc);
     KFormula::Container* formula = doc->createFormula();
-    if ( !doc->loadXML( dom ) ) {
-        kdError(30522) << "Failed." << endl;
+    if (!doc->loadXML(dom)) {
+        kError(30522) << "Failed." << endl;
     }
 
     QTextStream stream(&f);
     //stream.setEncoding(QTextStream::UnicodeUTF8);
     stream << "\\documentclass{article}\n\\usepackage{amsmath}\n\\begin{document}\n\\[\n"
-           << formula->texString()
-           << "\n\\]\n\\end{document}";
+    << formula->texString()
+    << "\n\\]\n\\end{document}";
     f.close();
 
     delete formula;
