@@ -1,3 +1,5 @@
+//Added by qt3to4:
+#include <QByteArray>
 /* This file is part of the KDE project
    Copyright (C) 2002 Ulrich Kuettler <ulrich.kuettler@mailbox.tu-dresden.de>
 
@@ -20,7 +22,7 @@
 #ifndef LATEXEXPORT_H
 #define LATEXEXPORT_H
 
-class QCString;
+class QByteArray;
 class QStringList;
 
 #include <KoFilter.h>
@@ -29,10 +31,10 @@ class LATEXExport : public KoFilter
 {
     Q_OBJECT
 public:
-    LATEXExport( KoFilter *parent, const char *name, const QStringList& );
+    LATEXExport(QObject* parent, const QStringList&);
     virtual ~LATEXExport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
 
 #endif // LATEXEXPORT_H
