@@ -29,20 +29,20 @@ class ImageExportWidget;
 
 class ImageExportDialog : public KDialogBase
 {
-  Q_OBJECT
-  public:
-    ImageExportDialog(QWidget* parent = 0, const char* name = 0);
+    Q_OBJECT
+public:
+    explicit ImageExportDialog(QWidget* parent = 0, const char* name = 0);
 
     void setPageList(const QStringList& pages);
-    void setInitialDPI(const int dpi);
-    void setInitialmargin(const int margin);
+    void setInitialCustomSize(const QSize& size);
 
     QString selectedPage() const;
     bool usePageBorders() const;
-    int imageDPI() const;
+    bool useCustomSize() const;
+    QSize customSize() const;
     int margin() const;
 
-  private:
+private:
     ImageExportWidget* m_mainWidget;
 };
 
