@@ -32,20 +32,20 @@
 #ifndef ABIWORDEXPORT_H
 #define ABIWORDEXPORT_H
 
-#include <qstring.h>
-#include <qcstring.h>
+#include <QString>
 
 #include <KoFilter.h>
 
 
-class ABIWORDExport : public KoFilter {
+class ABIWORDExport : public KoFilter
+{
 
     Q_OBJECT
 
 public:
-    ABIWORDExport(KoFilter *parent, const char *name, const QStringList &);
+    ABIWORDExport(QObject* parent, const QStringList &);
     virtual ~ABIWORDExport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
 #endif // ABIWORDEXPORT_H
