@@ -33,22 +33,25 @@
 #ifndef ABIWORDIMPORT_H
 #define ABIWORDIMPORT_H
 
-#include <qstring.h>
-#include <qfile.h>
-#include <qobject.h>
+#include <QString>
+#include <QFile>
+#include <QObject>
+//Added by qt3to4:
+#include <QByteArray>
 
 #include <KoFilter.h>
 #include <KoStore.h>
 
-class ABIWORDImport : public KoFilter {
+class ABIWORDImport : public KoFilter
+{
 
     Q_OBJECT
 
 public:
-    ABIWORDImport(KoFilter *parent, const char *name, const QStringList &);
+    ABIWORDImport(QObject* parent, const QStringList &);
     ABIWORDImport();
     virtual ~ABIWORDImport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
 #endif // ABIWORDIMPORT_H
