@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2002 Ariya Hidayat <ariyahidayat@yahoo.de>
+   Copyright (C) 2002 Ariya Hidayat <ariya@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -22,19 +22,21 @@
 
 #include <KoFilter.h>
 #include <KoStore.h>
+//Added by qt3to4:
+#include <QByteArray>
 
 class AmiProExport : public KoFilter
 {
     Q_OBJECT
 
-  public:
+public:
 
-    AmiProExport(KoFilter *parent, const char *name, const QStringList&);
+    AmiProExport(QObject* parent, const QStringList&);
     AmiProExport();
 
     virtual ~AmiProExport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
 
 #endif // __AMIPROEXPORT_H
