@@ -27,7 +27,7 @@ class QStringList;
 
 class AmiPro
 {
-  public:
+public:
     static const float LS_Single, LS_OneAndHalf, LS_Double;
 };
 
@@ -35,7 +35,7 @@ class AmiProStyle;
 
 class AmiProFormat
 {
-  public:
+public:
     int pos, len;
     bool bold, italic, underline;
     bool word_underline, double_underline;
@@ -45,10 +45,10 @@ class AmiProFormat
     QColor fontColor;
     enum { Left, Right, Center, Justified } align;
     AmiProFormat();
-    AmiProFormat( const AmiProFormat& );
-    AmiProFormat& operator=( const AmiProFormat& );
-    void assign( const AmiProFormat& );
-    void applyStyle( const AmiProStyle& );
+    AmiProFormat(const AmiProFormat&);
+    AmiProFormat& operator=(const AmiProFormat&);
+    void assign(const AmiProFormat&);
+    void applyStyle(const AmiProStyle&);
 };
 
 typedef QList<AmiProFormat> AmiProFormatList;
@@ -56,7 +56,7 @@ typedef QList<AmiProFormat> AmiProFormatList;
 
 class AmiProLayout
 {
-  public:
+public:
     QString name;
     QString fontFamily;
     float fontSize;
@@ -64,19 +64,19 @@ class AmiProLayout
     bool bold, italic, underline;
     bool word_underline, double_underline;
     bool subscript, superscript, strikethrough;
-	Qt::AlignmentFlag align;
+    Qt::AlignmentFlag align;
     float linespace;
     float spaceBefore, spaceAfter;
     AmiProLayout();
-    AmiProLayout( const AmiProLayout& );
-    AmiProLayout& operator=( const AmiProLayout& );
-    void assign( const AmiProLayout& );
-    void applyStyle( const AmiProStyle& );
+    AmiProLayout(const AmiProLayout&);
+    AmiProLayout& operator=(const AmiProLayout&);
+    void assign(const AmiProLayout&);
+    void applyStyle(const AmiProStyle&);
 };
 
 class AmiProStyle
 {
-  public:
+public:
     QString name;
     QString fontFamily;
     float fontSize;
@@ -88,40 +88,42 @@ class AmiProStyle
     float linespace;
     float spaceBefore, spaceAfter;
     AmiProStyle();
-    AmiProStyle( const AmiProStyle& );
-    AmiProStyle& operator=( const AmiProStyle& );
-    void assign( const AmiProStyle& );
+    AmiProStyle(const AmiProStyle&);
+    AmiProStyle& operator=(const AmiProStyle&);
+    void assign(const AmiProStyle&);
 };
 
 typedef QList<AmiProStyle> AmiProStyleList;
 
 class AmiProListener
 {
-  public: 
+public:
     AmiProListener();
     virtual ~AmiProListener();
     virtual bool doOpenDocument();
-    virtual bool doCloseDocument();   
-    virtual bool doDefineStyle( const AmiProStyle& style );
-    virtual bool doParagraph( const QString& text, AmiProFormatList formatList, 
-      AmiProLayout& layout );
+    virtual bool doCloseDocument();
+    virtual bool doDefineStyle(const AmiProStyle& style);
+    virtual bool doParagraph(const QString& text, AmiProFormatList formatList,
+                             AmiProLayout& layout);
 };
 
 class AmiProParser
 {
 
-  public:
+public:
     AmiProParser();
     virtual ~AmiProParser();
 
     enum { OK, UnknownError, FileError, InvalidFormat } Error;
 
-    bool process( const QString& filename );
-    int result(){ return m_result; }
+    bool process(const QString& filename);
+    int result() {
+        return m_result;
+    }
 
-    void setListener( AmiProListener * );
+    void setListener(AmiProListener *);
 
-  private:
+private:
 
     int m_result;
     QString m_text;
@@ -135,16 +137,16 @@ class AmiProParser
 
     QString m_currentSection;
 
-    bool setResult( int );
-    bool parseParagraph( const QStringList& lines );
-    bool parseStyle( const QStringList& line );
-    AmiProStyle findStyle( const QString& name );
+    bool setResult(int);
+    bool parseParagraph(const QStringList& lines);
+    bool parseStyle(const QStringList& line);
+    AmiProStyle findStyle(const QString& name);
 
-    bool handleTag( const QString& tag );
+    bool handleTag(const QString& tag);
     bool processOpenDocument();
     bool processCloseDocument();
-    bool processParagraph( const QString& text, AmiProFormatList formatList,
-      AmiProLayout& layout );
+    bool processParagraph(const QString& text, AmiProFormatList formatList,
+                          AmiProLayout& layout);
 
 };
 
