@@ -21,24 +21,24 @@
 #ifndef EXPORT_DIALOG_H
 #define EXPORT_DIALOG_H
 
-#include <qwidget.h>
-#include <kdialogbase.h>
+#include <QWidget>
+#include <kdialog.h>
 
 class ExportDialogUI;
 
-class HtmlExportDialog : public KDialogBase
+class HtmlExportDialog : public KDialog
 {
     Q_OBJECT
 public:
 
     enum Mode {
-      Light =0,
-      Basic,
-      DefaultCSS,
-      CustomCSS
+        Light = 0,
+        Basic,
+        DefaultCSS,
+        CustomCSS
     };
 
-    HtmlExportDialog(QWidget* parent=NULL);
+    HtmlExportDialog(QWidget* parent = NULL);
     ~HtmlExportDialog(void);
     bool isXHtml(void) const;
     QTextCodec* getCodec(void) const;
@@ -46,7 +46,7 @@ public:
     QString cssURL(void) const;
 
 protected slots:
-    void setCSSEnabled( bool );
+    void setCSSEnabled(bool);
 
 private:
     ExportDialogUI* m_dialog;
