@@ -14,31 +14,34 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef HTMLEXPORT_H
 #define HTMLEXPORT_H
 
-#include <qstring.h>
-#include <qtextstream.h>
-#include <qfile.h>
-#include <qobject.h>
+#include <QString>
+#include <QTextStream>
+#include <QFile>
+#include <QObject>
+//Added by qt3to4:
+#include <QByteArray>
 
-#include <koFilter.h>
-#include <koStore.h>
+#include <KoFilter.h>
+#include <KoStore.h>
 
 
-class HTMLExport : public KoFilter {
+class HTMLExport : public KoFilter
+{
 
     Q_OBJECT
 
 public:
-    HTMLExport(KoFilter *parent, const char *name, const QStringList &);
+    HTMLExport(QObject* parent, const QStringList &);
     virtual ~HTMLExport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
 
 #endif // HTMLEXPORT_H
