@@ -30,31 +30,32 @@
 #include <khtmlreader.h>
 #include <kwdwriter.h>
 
-typedef KGenericFactory<HTMLImport, KoFilter> HTMLImportFactory;
-K_EXPORT_COMPONENT_FACTORY( libhtmlimport, HTMLImportFactory( "kofficefilters" ) )
+typedef KGenericFactory<HTMLImport> HTMLImportFactory;
+K_EXPORT_COMPONENT_FACTORY(libhtmlimport, HTMLImportFactory("kofficefilters"))
 
 
-HTMLImport::HTMLImport(KoFilter *, const char*, const QStringList &) :
-                     KoFilter() {
+HTMLImport::HTMLImport(QObject* parent, const QStringList &) :
+        KoFilter(parent)
+{
 // 123
 }
 
-KoFilter::ConversionStatus HTMLImport::convert( const QCString& from, const QCString& to )
+KoFilter::ConversionStatus HTMLImport::convert(const QByteArray& from, const QByteArray& to)
 {
-	if(to!="application/x-kword" || from!="text/html")
-		return KoFilter::NotImplemented;
+    if (to != "application/x-kword" || from != "text/html")
+        return KoFilter::NotImplemented;
 
-	KoStore *k= KoStore::createStore(m_chain->outputFile(), KoStore::Write, "application/x-kword");
-	KWDWriter *w= new KWDWriter(k);
-	KHTMLReader h(w);
-	KURL url;
-	url.setPath(m_chain->inputFile());
-	bool b= h.filter(url);
-	delete(w);
-	delete(k);
+    KoStore *k = KoStore::createStore(m_chain->outputFile(), KoStore::Write, "application/x-kword");
+    KWDWriter *w = new KWDWriter(k);
+    KHTMLReader h(w);
+    KUrl url;
+    url.setPath(m_chain->inputFile());
+    bool b = h.filter(url);
+    delete(w);
+    delete(k);
 
-        if ( b )
-            return KoFilter::OK;
-        else
-            return KoFilter::StupidError;
+    if (b)
+        return KoFilter::OK;
+    else
+        return KoFilter::StupidError;
 }
