@@ -1,5 +1,3 @@
-// $Header$
-
 /*
    This file is part of the KDE project
    Copyright (C) 2001, 2003 Nicolas GOUTTE <goutte@kde.org>
@@ -16,7 +14,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 /*
@@ -33,20 +32,21 @@
 #ifndef OOWRITEREXPORT_H
 #define OOWRITEREXPORT_H
 
-#include <qstring.h>
-#include <qcstring.h>
+#include <QString>
+#include <q3cstring.h>
 
-#include <koFilter.h>
+#include <KoFilter.h>
 
 
-class OOWRITERExport : public KoFilter {
+class OOWRITERExport : public KoFilter
+{
 
     Q_OBJECT
 
 public:
-    OOWRITERExport(KoFilter *parent, const char *name, const QStringList &);
+    OOWRITERExport(QObject* parent, const QStringList &);
     virtual ~OOWRITERExport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
 #endif // OOWRITEREXPORT_H
