@@ -22,19 +22,21 @@
 
 #include <KoFilter.h>
 #include <KoStore.h>
+//Added by qt3to4:
+#include <QByteArray>
 
 class WPExport : public KoFilter
 {
     Q_OBJECT
 
-  public:
+public:
 
-    WPExport(KoFilter *parent, const char *name, const QStringList&);
+    WPExport(QObject* parent, const QStringList&);
     WPExport();
 
     virtual ~WPExport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
 
 #endif // __WPEXPORT_H
