@@ -21,6 +21,8 @@
 #define __XSLTEXPORT_H__
 
 #include <KoFilter.h>
+//Added by qt3to4:
+#include <QByteArray>
 
 class XSLTExport : public KoFilter
 {
@@ -28,9 +30,9 @@ class XSLTExport : public KoFilter
     Q_OBJECT
 
 public:
-    XSLTExport(KoFilter *parent, const char *name, const QStringList&);
+    XSLTExport(QObject* parent, const QStringList&);
     virtual ~XSLTExport() {}
 
-    virtual KoFilter::ConversionStatus convert( const QCString& from, const QCString& to );
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
 #endif /* __XSLTEXPORT_H__ */
