@@ -66,7 +66,6 @@ using std::cerr;
 #include <KoOdfWriteStore.h>
 #include <KoShapeLoadingContext.h>
 #include <KoShapeRegistry.h>
-#include <KoDataCenter.h>
 
 // KDChart
 #include "KDChartChart"
@@ -98,7 +97,7 @@ KChartPart::KChartPart( QWidget *parentWidget,
     m_chartData( 0 )
 {
     // A KChart part contains a single chart Flake shape. 
-    m_chartShape = ( ChartShape* )( KoShapeRegistry::instance()->value("ChartShape") )->createDefaultShapeAndInit( dataCenterMap() );
+    m_chartShape = (ChartShape*)(KoShapeRegistry::instance()->value("ChartShape") )->createDefaultShape(resourceManager());
     setComponentData( KChartFactory::global(), false );
     setTemplateType( "kchart_template" );
 
@@ -159,6 +158,8 @@ KoView* KChartPart::createViewInstance( QWidget* parent )
 
 void KChartPart::paintContent( QPainter& painter, const QRect& rect)
 {
+    Q_UNUSED(painter);
+    Q_UNUSED(rect);
 #if 0
     int  numDatasets;
 
@@ -227,7 +228,7 @@ void KChartPart::paintContent( QPainter& painter, const QRect& rect)
 
 #else
     // Paint the shape that is the real chart.
-    kDebug() << "################## Painting!";
+    kDebug(35001) << "################## Painting!";
     //KoZoomHandler  zoomHandler;
     //m_chartShape->paint( painter, zoomHandler );
     return;
@@ -323,14 +324,6 @@ void KChartPart::removeShape( KoShape* )
 {
 }
 
-QMap<QString, KoDataCenter *> KChartPart::dataCenterMap() const
-{
-    //Q_ASSERT(false);
-    // if the kchart app ever supports more than a single shape this should be implemented
-    // along with several other methods - see any other app or ask boemann
-    return QMap<QString, KoDataCenter *>() ;
-}
-
 // ----------------------------------------------------------------
 
 
@@ -611,7 +604,7 @@ bool KChartPart::loadOdf( KoOdfReadStore & odfStore )
     KoXmlElement bodyElement = KoXml::namedItemNS( realBody,
                                                    KoXmlNS::office, "chart" );
     if ( bodyElement.isNull() ) {
-        kError(32001) << "No office:chart found!" << endl;
+        kError(32001) << "No office:chart found!";
 
         KoXmlElement  childElem;
         QString       localName;
@@ -641,7 +634,7 @@ bool KChartPart::loadOdf( KoOdfReadStore & odfStore )
 #endif
 
     KoOdfLoadingContext  context( odfStore.styles(), odfStore.store() );
-    KoShapeLoadingContext  shapeContext( context, dataCenterMap() );
+    KoShapeLoadingContext shapeContext(context, resourceManager());
 
     // ----------------------------------------------------------------
     // FIXME: Load chart here.
@@ -669,7 +662,7 @@ bool KChartPart::loadOdf( KoOdfReadStore & odfStore )
     KoXmlElement  bodyElem ( KoXml::namedItemNS( content,
 						KoXmlNS::office, "body" ) );
     if ( bodyElem.isNull() ) {
-        kError(32001) << "No office:body found!" << endl;
+        kError(32001) << "No office:body found!";
         setErrorMessage( i18n( "Invalid OASIS OpenDocument file. No office:body tag found." ) );
         return false;
     }
@@ -678,7 +671,7 @@ bool KChartPart::loadOdf( KoOdfReadStore & odfStore )
     KoXmlElement officeChartElem = KoXml::namedItemNS( bodyElem,
 						      KoXmlNS::office, "chart" );
     if ( officeChartElem.isNull() ) {
-        kError(32001) << "No office:chart found!" << endl;
+        kError(32001) << "No office:chart found!";
         KoXmlElement  childElem;
         QString      localName;
         forEachElement( childElem, bodyElem ) {
@@ -711,7 +704,7 @@ bool KChartPart::loadOdf( KoOdfReadStore & odfStore )
     loadingContext.fillStyleStack( chartElem, KoXmlNS::chart, "style-name" );
 
     const QString fillColor = styleStack.property( KoXmlNS::draw, "fill-color" );
-    kDebug() <<"fillColor=" << fillColor;
+    kDebug(35001) <<"fillColor=" << fillColor;
 
     styleStack.restore();
 #endif
@@ -775,10 +768,10 @@ bool KChartPart::loadOasisData( const KoXmlElement& tableElem )
     m_colLabels.clear();
     KoXmlElement tableHeaderRows = KoXml::namedItemNS( tableElem, KoXmlNS::table, "table-header-rows" );
     if ( tableHeaderRows.isNull() )
-        kWarning(35001) << "No table-header-rows element found!" << endl;
+        kWarning(35001) << "No table-header-rows element found!";
     KoXmlElement tableHeaderRow = KoXml::namedItemNS( tableHeaderRows, KoXmlNS::table, "table-row" );
     if ( tableHeaderRow.isNull() )
-        kWarning(35001) << "No table-row inside table-header-rows!" << endl;
+        kWarning(35001) << "No table-row inside table-header-rows!";
 
     int cellNum = 0;
     forEachElement( elem, tableHeaderRow ) {
@@ -816,11 +809,11 @@ bool KChartPart::loadOasisData( const KoXmlElement& tableElem )
 
     kDebug(35001) <<"numberHeaderColumns=" << numberHeaderColumns
 		   << " numberDataColumns=" << numberDataColumns
-                   << " numberDataRows=" << numberDataRows << endl;
+                   << " numberDataRows=" << numberDataRows;
 
     if ( (int)m_rowLabels.count() != numberDataRows)
         kWarning(35001) << "Got " << m_rowLabels.count()
-			 << " row labels, expected " << numberDataRows << endl;
+			 << " row labels, expected " << numberDataRows;
 
     m_currentData.expand( numberDataRows, numberDataColumns );
     m_currentData.setUsedCols( numberDataColumns );
@@ -840,10 +833,10 @@ bool KChartPart::loadOasisData( const KoXmlElement& tableElem )
                     if ( cellNum > numberHeaderColumns ) {
                         QString valueType = cellElem.attributeNS( KoXmlNS::office, "value-type", QString() );
                         if ( valueType != "float" )
-                            kWarning(35001) << "Don't know how to handle value-type " << valueType << endl;
+                            kWarning(35001) << "Don't know how to handle value-type " << valueType;
                         else {
                             QString  value = cellElem.attributeNS( KoXmlNS::office, "value", QString() );
-                            double   val = value.toDouble();
+                            qreal    val = value.toDouble();
 
                             m_currentData.setCell( row, col, val );
                         }
@@ -956,7 +949,7 @@ bool KChartPart::saveOdf( SavingContext & documentContext )
     settingsWriter->startElement("config:config-item-set");
     settingsWriter->addAttribute("config:name", "view-settings");
 
-    KoUnit::saveOasis(settingsWriter, unit());
+    saveUnitOdf(settingsWriter);
 
     //saveOasisSettings( *settingsWriter );
 
@@ -989,10 +982,10 @@ bool KChartPart::saveOdf( SavingContext & documentContext )
 void KChartPart::writeAutomaticStyles( KoXmlWriter& contentWriter, 
                                        KoGenStyles& mainStyles ) const
 {
-    QList<KoGenStyles::NamedStyle>  styles = mainStyles.styles( KoGenStyle::StyleAuto );
+    QList<KoGenStyles::NamedStyle>  styles = mainStyles.styles( KoGenStyle::ParagraphAutoStyle );
     QList<KoGenStyles::NamedStyle>::const_iterator  it;
     for ( it = styles.constBegin(); it != styles.constEnd(); ++it ) {
-        kDebug() << "Style: " << (*it).name;
+        kDebug(35001) << "Style: " << (*it).name;
         (*it).style->writeStyle( &contentWriter, mainStyles, "style:style",
                                  (*it).name, "style:chart-properties" );
     }
@@ -1067,13 +1060,13 @@ QDomDocument KChartPart::saveXML()
                 default: {
                     valType = "(unknown)";
                     kDebug(35001) <<"ERROR: cell" << i <<"," << j
-				   << " has unknown type." << endl;
+				   << " has unknown type.";
                 }
             }
 
             e.setAttribute( "valType", valType );
             //kDebug(35001) <<"      cell" << i <<"," << j
-	    //	   << " saved with type '" << valType << "'." << endl;
+	    //	   << " saved with type '" << valType << "'." ;
             switch ( value.type() ) {
                 case QVariant::String:  e.setAttribute( "value", value.toString() );
                               break;
@@ -1085,7 +1078,7 @@ QDomDocument KChartPart::saveXML()
                     e.setAttribute( "value", "" );
                     if( QVariant::Invalid != value.type() )
                         kDebug(35001) <<"ERROR: cell" << i <<"," << j
-				       << " has unknown type." << endl;
+				       << " has unknown type." ;
                 }
             }
         }
@@ -1301,7 +1294,7 @@ bool KChartPart::loadData( const KoXmlDocument& doc )
                     }
                     else if ( "double" == valueType ) {
                         bool bOk;
-                        double val = e.attribute("value").toDouble(&bOk);
+                        qreal val = e.attribute("value").toDouble(&bOk);
                         if ( !bOk )
                             val = 0.0;
                         t = val;
