@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
 
    Copyright 2007 Stefan Nikolaus     <stefan.nikolaus@kdemail.net>
-   Copyright 2007-2009 Inge Wallin    <inge@lysator.liu.se>
+   Copyright 2007-2010 Inge Wallin    <inge@lysator.liu.se>
    Copyright 2007-2008 Johannes Simon <johannes.simon@gmail.com>
 
    This library is free software; you can redistribute it and/or
@@ -89,6 +89,7 @@
 #include <KoShapeBorderModel.h>
 #include <KoColorBackground.h>
 #include <KoLineBorder.h>
+#include <KoOdfWorkaround.h>
 
 // KChart
 #include "Axis.h"
@@ -157,20 +158,13 @@ bool isCartesian( ChartType type )
     return !isPolar( type );
 }
 
-// FIXME: Rename Title to TitleLabelType and so on
-enum OdfLabelType {
-    Title,
-    SubTitle,
-    Footer
-};
-
 
 QString saveOdfFont( KoGenStyles& mainStyles,
-                                 const QFont& font,
-                                 const QColor& color )
+                     const QFont& font,
+                     const QColor& color )
 {
     KoGenStyle::PropertyType tt = KoGenStyle::TextType;
-    KoGenStyle autoStyle( KoGenStyle::StyleAuto, "chart", 0 );
+    KoGenStyle autoStyle( KoGenStyle::ParagraphAutoStyle, "chart", 0 );
     autoStyle.addProperty( "fo:font-family", font.family(), tt );
     autoStyle.addPropertyPt( "fo:font-size", font.pointSize(), tt );
     autoStyle.addProperty( "fo:color", color.isValid() ? color.name() : "#000000", tt );
@@ -178,16 +172,19 @@ QString saveOdfFont( KoGenStyles& mainStyles,
     autoStyle.addProperty( "fo:font-weight", w == 50 ? "normal" : w == 75 ? "bold" : QString::number( qRound(  w / 10 ) * 100 ), tt );
     autoStyle.addProperty( "fo:font-style", font.italic() ? "italic" : "normal", tt );
 
-    return mainStyles.lookup( autoStyle, "ch", KoGenStyles::ForceNumbering );
+    return mainStyles.insert( autoStyle, "ch" );
 }
 
 bool loadOdfLabel( KoShape *label, KoXmlElement &labelElement, KoShapeLoadingContext &context )
 {
+    Q_UNUSED(context);
     TextLabelData *labelData = qobject_cast<TextLabelData*>( label->userData() );
     if ( !labelData )
         return false;
 
-    label->loadOdf( labelElement, context );
+    // Following will always return false cause KoTextShapeData::loadOdf will try to load
+    // a frame while our text:p is not within a frame. So, let's just not call loadOdf then...
+    //label->loadOdf( labelElement, context );
 
     // TODO: Read optional attributes
     // 1. Table range
@@ -196,13 +193,11 @@ bool loadOdfLabel( KoShape *label, KoXmlElement &labelElement, KoShapeLoadingCon
 
     labelData->document()->setPlainText( pElement.text() );
 
-    label->setVisible( true );
-
     return true;
 }
 
 void saveOdfLabel( KoShape *label, KoXmlWriter &bodyWriter,
-                   KoGenStyles &mainStyles, OdfLabelType odfLabelType )
+                   KoGenStyles &mainStyles, LabelType labelType )
 {
     // Don't save hidden labels, as that's the way of removing them
     // from a chart.
@@ -213,15 +208,17 @@ void saveOdfLabel( KoShape *label, KoXmlWriter &bodyWriter,
     if ( !labelData )
         return;
 
-    if ( odfLabelType == Footer )
+    if ( labelType == FooterLabelType )
         bodyWriter.startElement( "chart:footer" );
-    else if ( odfLabelType == SubTitle )
+    else if ( labelType == SubTitleLabelType )
         bodyWriter.startElement( "chart:subtitle" );
-    else // if ( odfLabelType == Title )
+    else // if ( labelType == TitleLabelType )
         bodyWriter.startElement( "chart:title" );
 
     bodyWriter.addAttributePt( "svg:x", label->position().x() );
     bodyWriter.addAttributePt( "svg:y", label->position().y() );
+    bodyWriter.addAttributePt( "svg:width", label->size().width() );
+    bodyWriter.addAttributePt( "svg:height", label->size().height() );
     // TODO: Save text label color
     bodyWriter.addAttribute( "chart:style-name", saveOdfFont( mainStyles, labelData->document()->defaultFont(), QColor() ) );
 
@@ -267,7 +264,9 @@ public:
     Private( ChartShape *shape );
     ~Private();
 
-    void showLabel( KoShape *label );
+    void showLabel( KoShape *label, bool doShow );
+    QPointF relativePosition( KoShape *shape );
+    void setRelativePosition( KoShape *shape, const QPointF &relPos );
 
     // The components of a chart
     KoShape   *title;
@@ -284,11 +283,12 @@ public:
 
     ChartShape *shape;		// The chart that owns this ChartShape::Private
 
-    QMap<QString, KoDataCenter*> dataCenterMap;
+    KoResourceManager *resourceManager;
 };
 
 
 ChartShape::Private::Private( ChartShape *shape )
+    : resourceManager(0)
 {
     // Register the owner.
     this->shape = shape;
@@ -317,56 +317,50 @@ ChartShape::Private::~Private()
 //
 // If there is too little room, then make space by shrinking the Plotarea.
 //
-// FIXME: Make it take a bool parameter to show / hide the label in
-//        question and then also grow the Plotarea again if the label
-//        is hidden.
-void ChartShape::Private::showLabel( KoShape *label )
+void ChartShape::Private::showLabel( KoShape *label, bool doShow )
 {
     Q_ASSERT( label );
 
-    const QSizeF  plotAreaSize = plotArea->size();
-    if ( label->position().y() + label->size().height() / 2.0
-         < shape->size().height() / 2.0 )
-    {
-        const double verticalSpaceRemaining = plotArea->position().y();
-        const double spaceToExpand          = ( label->position().y() + label->size().height() ) - verticalSpaceRemaining;
+    if (label->isVisible() == doShow)
+        return;
 
-        if ( spaceToExpand > 0.0 ) {
-            plotArea->setSize( QSizeF( plotAreaSize.width(),
-                                       plotAreaSize.height() - spaceToExpand ) );
-            plotArea->setPosition( QPointF( plotArea->position().x(),
-                                            plotArea->position().y() + spaceToExpand ) );
-        }
+    label->setVisible( doShow );
+
+    // The bounds of the space that is left for the plotarea.
+    // Start with the full height of the shape.
+    qreal yTop = 0;
+    qreal yBottom = shape->size().height();
+
+    // Start calculate the available space for the plotarea.  First
+    // remove the space for the title/subtitle/footer
+    if (title->isVisible()) {
+        yTop = title->position().y() + title->size().height();
     }
-    else {
-        const double verticalSpaceRemaining = label->position().y() - plotArea->position().y() - plotArea->size().height();
-        double spaceToExpand = ( shape->size().height() - label->position().y() - label->size().height() ) - verticalSpaceRemaining;
-
-        if ( spaceToExpand < 0.0 )
-            spaceToExpand = 0.0;
-
-        foreach ( Axis *axis, plotArea->axes() ) {
-            if ( axis->position() != BottomAxisPosition )
-                continue;
-
-            double _spaceToExpand = ( label->size().height()
-                                      - ( shape->size().height()
-                                          - axis->title()->position().y()
-                                          - axis->title()->size().height() ) );
-            if ( _spaceToExpand > 0.0 ) {
-                axis->title()->setPosition( axis->title()->position()
-                                            - QPointF( 0.0, _spaceToExpand ) );
-                spaceToExpand += _spaceToExpand;
-            }
-        }
-
-        if ( spaceToExpand > 0.0 ) {
-            plotArea->setSize( QSizeF( plotAreaSize.width(),
-				       plotAreaSize.height() - spaceToExpand ) );
-        }
+    if (subTitle->isVisible()) {
+        // Here we assume that the subtitle is below the title.
+        yTop = subTitle->position().y() + subTitle->size().height();
+    }
+    if (footer->isVisible()) {
+        yBottom = footer->position().y();
     }
 
-    label->setVisible( true );
+    // Then remove the space that is taken by the axis titles.
+    foreach ( Axis *axis, plotArea->axes() ) {
+        if ( axis->position() != BottomAxisPosition )
+            continue;
+
+        // Make room for the Axis titles.
+        // FIXME: They should only be allocated space if they are visible.
+        yBottom -= axis->title()->size().height();
+        axis->title()->setPosition( QPointF(axis->title()->position().x(), yBottom) );
+    }
+
+    // Check if there is any room left for the plotarea.
+    // Maybe we should use a minimal size?
+    if (yTop < yBottom) {
+        plotArea->setPosition(QPointF(plotArea->position().x(), yTop));
+        plotArea->setSize(QSizeF(plotArea->size().width(), yBottom - yTop));
+    }
 }
 
 
@@ -375,10 +369,11 @@ void ChartShape::Private::showLabel( KoShape *label )
 // ================================================================
 
 
-ChartShape::ChartShape()
+ChartShape::ChartShape(KoResourceManager *resourceManager)
     : KoFrameShape( KoXmlNS::draw, "object" )
     , d ( new Private( this ) )
 {
+    d->resourceManager = resourceManager;
     setShapeId( ChartShapeId );
 
     // Instantiated all children first
@@ -392,7 +387,7 @@ ChartShape::ChartShape()
     // We need this as the very first step, because some methods
     // here rely on the d->plotArea pointer.
     addChild( d->plotArea );
-    d->plotArea->init();
+    d->plotArea->plotAreaInit();
     d->plotArea->setZIndex( 0 );
     setClipping( d->plotArea, true );
 
@@ -402,14 +397,11 @@ ChartShape::ChartShape()
     setClipping( d->legend, true );
 
     // A few simple defaults (chart type and subtype in this case)
-    d->plotArea->setChartType( BarChartType );
-    d->plotArea->setChartSubType( NormalChartSubtype );
+    setChartType( BarChartType );
+    setChartSubType( NormalChartSubtype );
 
     // Create the Title, which is a standard TextShape.
-    // We use a empty dataCenterMap here for the creation and then in
-    // init() we init the shapes with the correct data.
-    QMap<QString, KoDataCenter *> dataCenterMap;
-    d->title = KoShapeRegistry::instance()->value( TextShapeId )->createDefaultShapeAndInit( dataCenterMap );
+    d->title = KoShapeRegistry::instance()->value(TextShapeId)->createDefaultShape(resourceManager);
     if ( !d->title ) {
         d->title = new TextLabelDummy;
         KMessageBox::error( 0, i18n("The plugin needed for displaying text labels in a chart is not available."), i18n("Plugin Missing") );
@@ -421,12 +413,17 @@ ChartShape::ChartShape()
         d->title->setUserData( dataDummy );
     }
 
+    // Start with a reasonable default size that we can base all following relative
+    // positions of chart elements on.
+    KoShape::setSize( QSizeF( CM_TO_POINT( 8 ), CM_TO_POINT( 5 ) ) );
+
     // Add the title to the shape
     addChild( d->title );
     QFont font = titleData()->document()->defaultFont();
     font.setPointSizeF( 12.0 );
     titleData()->document()->setDefaultFont( font );
     titleData()->document()->setHtml( "<div align=\"center\">" + i18n( "Title" ) + "</font></div>" );
+    // Position the title center at the very top.
     d->title->setSize( QSizeF( CM_TO_POINT( 5 ), CM_TO_POINT( 0.7 ) ) );
     d->title->setPosition( QPointF( size().width() / 2.0 - d->title->size().width() / 2.0, 0.0 ) );
     d->title->setVisible( false );
@@ -434,7 +431,7 @@ ChartShape::ChartShape()
     setClipping( d->title, true );
 
     // Create the Subtitle and add it to the shape.
-    d->subTitle = KoShapeRegistry::instance()->value( TextShapeId )->createDefaultShapeAndInit( dataCenterMap );
+    d->subTitle = KoShapeRegistry::instance()->value(TextShapeId)->createDefaultShape(resourceManager);
     if ( !d->subTitle ) {
         d->subTitle = new TextLabelDummy;
     }
@@ -447,6 +444,8 @@ ChartShape::ChartShape()
     font.setPointSizeF( 10.0 );
     subTitleData()->document()->setDefaultFont( font );
     subTitleData()->document()->setHtml( "<div align=\"center\">" + i18n( "Subtitle" ) + "</div>" );
+
+    // Position it in the center, just below the title.
     d->subTitle->setSize( QSizeF( CM_TO_POINT( 5 ), CM_TO_POINT( 0.6 ) ) );
     d->subTitle->setPosition( QPointF( size().width() / 2.0 - d->title->size().width() / 2.0,
                                        d->title->size().height() ) );
@@ -455,7 +454,7 @@ ChartShape::ChartShape()
     setClipping( d->subTitle, true );
 
     // Create the Footer and add it to the shape.
-    d->footer = KoShapeRegistry::instance()->value( TextShapeId )->createDefaultShapeAndInit( dataCenterMap );
+    d->footer = KoShapeRegistry::instance()->value(TextShapeId)->createDefaultShape(resourceManager);
     if ( !d->footer ) {
         d->footer = new TextLabelDummy;
     }
@@ -468,8 +467,11 @@ ChartShape::ChartShape()
     font.setPointSizeF( 10.0 );
     footerData()->document()->setDefaultFont( font );
     footerData()->document()->setHtml( "<div align=\"center\">" + i18n( "Footer" ) + "</div>" );
+
+    // Position the footer in the center, at the bottom.
     d->footer->setSize( QSizeF( CM_TO_POINT( 5 ), CM_TO_POINT( 0.6 ) ) );
-    d->footer->setPosition( QPointF( size().width() / 2.0 - d->footer->size().width() / 2.0, size().height() - d->footer->size().height() ) );
+    d->footer->setPosition( QPointF( size().width() / 2.0 - d->footer->size().width() / 2.0,
+                                     size().height() - d->footer->size().height() ) );
     d->footer->setVisible( false );
     d->footer->setZIndex( 4 );
     setClipping( d->footer, true );
@@ -480,9 +482,6 @@ ChartShape::ChartShape()
     KoLineBorder *border = new KoLineBorder( 0, Qt::black );
     setBorder( border );
 
-    // Default size of the chart.
-    KoShape::setSize( QSizeF( CM_TO_POINT( 8 ), CM_TO_POINT( 5 ) ) );
-
     requestRepaint();
 }
 
@@ -564,19 +563,19 @@ PlotArea *ChartShape::plotArea() const
 }
 
 
-void ChartShape::showTitle()
+void ChartShape::showTitle(bool doShow)
 {
-    d->showLabel( d->title );
+    d->showLabel( d->title, doShow );
 }
 
-void ChartShape::showSubTitle()
+void ChartShape::showSubTitle(bool doShow)
 {
-    d->showLabel( d->subTitle );
+    d->showLabel( d->subTitle, doShow );
 }
 
-void ChartShape::showFooter()
+void ChartShape::showFooter(bool doShow)
 {
-    d->showLabel( d->footer );
+    d->showLabel( d->footer, doShow );
 }
 
 void ChartShape::setModel( QAbstractItemModel *model,
@@ -638,7 +637,7 @@ void ChartShape::setPosition( const QPointF &pos )
     KoShape::setPosition( pos );
 }
 
-static QPointF scalePointCenterLeft( QPointF center, double factorX, double factorY, const QSizeF &size )
+static QPointF scalePointCenterLeft( QPointF center, qreal factorX, qreal factorY, const QSizeF &size )
 {
     center.rx() -= size.width() / 2.0;
     center.rx() *= factorX;
@@ -648,7 +647,7 @@ static QPointF scalePointCenterLeft( QPointF center, double factorX, double fact
     return center;
 }
 
-static QPointF scalePointCenterRight( QPointF center, double factorX, double factorY, const QSizeF &size )
+static QPointF scalePointCenterRight( QPointF center, qreal factorX, qreal factorY, const QSizeF &size )
 {
     center.rx() += size.width() / 2.0;
     center.rx() *= factorX;
@@ -658,7 +657,7 @@ static QPointF scalePointCenterRight( QPointF center, double factorX, double fac
     return center;
 }
 
-static QPointF scalePointCenterTop( QPointF center, double factorX, double factorY, const QSizeF &size )
+static QPointF scalePointCenterTop( QPointF center, qreal factorX, qreal factorY, const QSizeF &size )
 {
     center.ry() -= size.height() / 2.0;
     center.rx() *= factorX;
@@ -668,7 +667,7 @@ static QPointF scalePointCenterTop( QPointF center, double factorX, double facto
     return center;
 }
 
-static QPointF scalePointCenterBottom( QPointF center, double factorX, double factorY, const QSizeF &size )
+static QPointF scalePointCenterBottom( QPointF center, qreal factorX, qreal factorY, const QSizeF &size )
 {
     center.ry() += size.height() / 2.0;
     center.rx() *= factorX;
@@ -678,28 +677,42 @@ static QPointF scalePointCenterBottom( QPointF center, double factorX, double fa
     return center;
 }
 
+QPointF ChartShape::Private::relativePosition( KoShape *shape )
+{
+    return shape->absolutePosition() - this->shape->absolutePosition( KoFlake::TopLeftCorner );
+}
+
+void ChartShape::Private::setRelativePosition( KoShape *shape, const QPointF &relPos )
+{
+    shape->setAbsolutePosition( relPos + this->shape->absolutePosition( KoFlake::TopLeftCorner ) );
+}
+
 void ChartShape::setSize( const QSizeF &newSize )
 {
     Q_ASSERT( d->plotArea );
 
-    const double factorX = newSize.width() / size().width();
-    const double factorY = newSize.height() / size().height();
+    const qreal factorX = newSize.width() / size().width();
+    const qreal factorY = newSize.height() / size().height();
 
     // Reposition the Axes within the shape.
     foreach( Axis *axis, d->plotArea->axes() ) {
         KoShape *title = axis->title();
         switch( axis->position() ) {
         case TopAxisPosition:
-            title->setAbsolutePosition( scalePointCenterTop( title->absolutePosition(), factorX, factorY, title->boundingRect().size() ) );
+            d->setRelativePosition( title,
+                                    scalePointCenterTop( d->relativePosition( title ), factorX, factorY, title->boundingRect().size() ) );
             break;
         case BottomAxisPosition:
-            title->setAbsolutePosition( scalePointCenterBottom( title->absolutePosition(), factorX, factorY, title->boundingRect().size() ) );
+            d->setRelativePosition( title,
+                                    scalePointCenterBottom( d->relativePosition( title ), factorX, factorY, title->boundingRect().size() ) );
             break;
         case LeftAxisPosition:
-            title->setAbsolutePosition( scalePointCenterLeft( title->absolutePosition(), factorX, factorY, title->boundingRect().size() ) );
+            d->setRelativePosition( title,
+                                    scalePointCenterLeft( d->relativePosition( title ), factorX, factorY, title->boundingRect().size() ) );
             break;
         case RightAxisPosition:
-            title->setAbsolutePosition( scalePointCenterRight( title->absolutePosition(), factorX, factorY, title->boundingRect().size() ) );
+            d->setRelativePosition( title,
+                                    scalePointCenterRight( d->relativePosition( title ), factorX, factorY, title->boundingRect().size() ) );
             break;
         }
     }
@@ -707,39 +720,92 @@ void ChartShape::setSize( const QSizeF &newSize )
     // Reposition the Legend within the shape.
     switch ( d->legend->legendPosition() ) {
     case TopLegendPosition:
-        d->legend->setAbsolutePosition( scalePointCenterTop( d->legend->absolutePosition(), factorX, factorY, d->legend->boundingRect().size() ) );
+        d->setRelativePosition( d->legend,
+                                scalePointCenterTop( d->relativePosition( d->legend ), factorX, factorY, d->legend->boundingRect().size() ) );
         break;
     case BottomLegendPosition:
-        d->legend->setAbsolutePosition( scalePointCenterBottom( d->legend->absolutePosition(), factorX, factorY, d->legend->boundingRect().size() ) );
+        d->setRelativePosition( d->legend,
+                                scalePointCenterBottom( d->relativePosition( d->legend ), factorX, factorY, d->legend->boundingRect().size() ) );
         break;
     case StartLegendPosition:
-        d->legend->setAbsolutePosition( scalePointCenterLeft( d->legend->absolutePosition(), factorX, factorY, d->legend->boundingRect().size() ) );
+        d->setRelativePosition( d->legend,
+                                scalePointCenterLeft( d->relativePosition( d->legend ), factorX, factorY, d->legend->boundingRect().size() ) );
         break;
     case EndLegendPosition:
-        d->legend->setAbsolutePosition( scalePointCenterRight( d->legend->absolutePosition(), factorX, factorY, d->legend->boundingRect().size() ) );
+        d->setRelativePosition( d->legend,
+                                scalePointCenterRight( d->relativePosition( d->legend ), factorX, factorY, d->legend->boundingRect().size() ) );
         break;
     case TopStartLegendPosition:
     case BottomStartLegendPosition:
     case TopEndLegendPosition:
     case BottomEndLegendPosition:
+    case FloatingLegendPosition:
         // FIXME: These are not handled.
         break;
     }
 
     // Reposition the Title, Subtitle and Footer within the shape.
-    d->title->setAbsolutePosition( scalePointCenterTop( d->title->absolutePosition(), factorX, factorY, d->title->boundingRect().size() ) );
-    d->subTitle->setAbsolutePosition( scalePointCenterTop( d->subTitle->absolutePosition(), factorX, factorY, d->subTitle->boundingRect().size() ) );
-    d->footer->setAbsolutePosition( scalePointCenterBottom( d->footer->absolutePosition(), factorX, factorY, d->footer->boundingRect().size() ) );
+    d->setRelativePosition( d->title,
+                            scalePointCenterTop( d->relativePosition( d->title ), factorX, factorY, d->title->boundingRect().size() ) );
+    d->setRelativePosition( d->subTitle,
+                            scalePointCenterTop( d->relativePosition( d->subTitle ), factorX, factorY, d->subTitle->boundingRect().size() ) );
+    d->setRelativePosition( d->footer,
+                            scalePointCenterBottom( d->relativePosition( d->footer ), factorX, factorY, d->footer->boundingRect().size() ) );
 
     // Finally, resize the plotarea.
     const QSizeF plotAreaSize = d->plotArea->size();
-    d->plotArea->setSize( QSizeF( plotAreaSize.width() + newSize.width() - size().width(), 
+    d->plotArea->setSize( QSizeF( plotAreaSize.width() + newSize.width() - size().width(),
                                   plotAreaSize.height() + newSize.height() - size().height() ) );
 
     // Oh yeah, the whole shape needs resizing too.
     KoShape::setSize( newSize );
 }
 
+void ChartShape::setLegendSize( const QSizeF &size )
+{
+    QPointF newPos = d->legend->position();
+    QSizeF oldSize = d->legend->size();
+    switch ( d->legend->legendPosition() ) {
+    case TopLegendPosition:
+        // Move it to the left as much as the width changed
+        newPos.rx() -= (size.width() - oldSize.width()) / 2.0;
+        // No adjustment of y position needed
+        break;
+    case BottomLegendPosition:
+        // Move it up as much as the height changed
+        newPos.ry() -= (size.height() - oldSize.height());
+        // Move it to the left as much as the width changed
+        newPos.rx() -= (size.width() - oldSize.width()) / 2.0;
+        break;
+    case StartLegendPosition:
+        // No adjustment of x position needed
+        newPos.ry() -= (size.height() - oldSize.height()) / 2.0;
+        break;
+    case EndLegendPosition:
+        newPos.ry() -= (size.height() - oldSize.height()) / 2.0;
+        // Move it to the left as much as the width changed
+        newPos.rx() -= (size.width() - oldSize.width());
+        break;
+    // FIXME: The following positions are not handled
+    // The new position is simply set as is
+    case TopStartLegendPosition:
+    case BottomStartLegendPosition:
+    case TopEndLegendPosition:
+    case BottomEndLegendPosition:
+
+    // If the legend is floating, the user has changed the position manually.
+    // There's no way to determine where the user wants it to be after a resize.
+    // (more to the left? more to the right?)
+    case FloatingLegendPosition:
+        break;
+    }
+
+    // TODO: Adjust plot area's size and position if necessary
+
+    d->legend->setPosition( newPos );
+    d->legend->setSize( size );
+}
+
 void ChartShape::updateChildrenPositions()
 {
     Q_ASSERT( d->plotArea );
@@ -752,8 +818,7 @@ void ChartShape::updateChildrenPositions()
         title->setPosition( titlePosition );
     }
 
-    const double legendXOffset = 10.0;
-    d->legend->setPosition( QPointF( size().width() + legendXOffset,
+    d->legend->setPosition( QPointF( size().width() - d->legend->size().width(),
                                      size().height() / 2.0 - d->legend->size().height() / 2.0 ) );
 }
 
@@ -804,6 +869,7 @@ void ChartShape::setChartType( ChartType type )
 {
     Q_ASSERT( d->plotArea );
     d->plotArea->setChartType( type );
+    emit chartTypeChanged( type );
 }
 
 void ChartShape::setChartSubType( ChartSubtype subType )
@@ -864,8 +930,8 @@ void ChartShape::paintDecorations( QPainter &painter,
 //                         Loading and Saving
 
 
-bool ChartShape::loadEmbeddedDocument( KoStore *store, 
-                                       const KoXmlElement &objectElement, 
+bool ChartShape::loadEmbeddedDocument( KoStore *store,
+                                       const KoXmlElement &objectElement,
                                        const KoXmlDocument &manifestDocument )
 {
     if ( !objectElement.hasAttributeNS( KoXmlNS::xlink, "href" ) ) {
@@ -1002,10 +1068,23 @@ bool ChartShape::loadEmbeddedDocument( KoStore *store,
 bool ChartShape::loadOdf( const KoXmlElement &element,
                           KoShapeLoadingContext &context )
 {
+    // Invalidate all data sets before we new load data sets from ODF
+    // and *before* we delete existing axes.
+    proxyModel()->invalidateDataSets();
+
+    // When loading from ODF, all data sets are added explicitly.
+    bool autoCreation = proxyModel()->automaticDataSetCreation();
+    proxyModel()->setAutomaticDataSetCreation( false );
+
     // Load common attributes of (frame) shapes.  If you change here,
     // don't forget to also change in saveOdf().
     loadOdfAttributes( element, context, OdfMandatories | OdfGeometry | OdfAdditionalAttributes );
-    return loadOdfFrame( element, context );
+    bool result = loadOdfFrame( element, context );
+
+    // Restore previous setting
+    proxyModel()->setAutomaticDataSetCreation( autoCreation );
+
+    return result;
 }
 
 // Used to load the actual contents from the ODF frame that surrounds
@@ -1034,7 +1113,15 @@ bool ChartShape::loadOdfEmbedded( const KoXmlElement &chartElement,
         styleStack.setTypeProperties( "graphic" );
     }
     loadOdfAttributes( chartElement, context,
-                       OdfAdditionalAttributes | OdfMandatories | OdfCommonChildElements );
+                       OdfAdditionalAttributes | OdfMandatories | OdfCommonChildElements | OdfStyle );
+
+#ifndef NWORKAROUND_ODF_BUGS
+    if ( !background() ) {
+        const QColor color = KoOdfWorkaround::fixMissingFillColor( chartElement, context );
+        if(color.isValid()) // invalid color means do not set KoColorBackground but be transparent instead
+            setBackground( new KoColorBackground( color ) );
+    }
+#endif
 
     // Check if we're loading an embedded document
     if ( !chartElement.hasAttributeNS( KoXmlNS::chart, "class" ) ) {
@@ -1054,7 +1141,7 @@ bool ChartShape::loadOdfEmbedded( const KoXmlElement &chartElement,
         if ( chartClass == ODF_CHARTTYPES[ (ChartType)type ] ) {
             //kDebug(35001) <<"found chart of type" << chartClass;
 
-            d->plotArea->setChartType( (ChartType)type );
+            setChartType( (ChartType)type );
             knownType = true;
             break;
         }
@@ -1068,40 +1155,7 @@ bool ChartShape::loadOdfEmbedded( const KoXmlElement &chartElement,
         return false;
     }
 
-    // 2. Load the title.
-    KoXmlElement titleElem = KoXml::namedItemNS( chartElement,
-                                                 KoXmlNS::chart, "title" );
-    if ( !titleElem.isNull() ) {
-        if ( !loadOdfLabel( d->title, titleElem, context) )
-            return false;
-    }
-
-    // 3. Load the subtitle.
-    KoXmlElement subTitleElem = KoXml::namedItemNS( chartElement,
-                                                    KoXmlNS::chart, "subtitle" );
-    if ( !subTitleElem.isNull() ) {
-        if ( !loadOdfLabel( d->subTitle, subTitleElem, context) )
-            return false;
-    }
-
-    // 4. Load the footer.
-    KoXmlElement footerElem = KoXml::namedItemNS( chartElement,
-                                                  KoXmlNS::chart, "footer" );
-    if ( !footerElem.isNull() ) {
-        if ( !loadOdfLabel( d->footer, footerElem, context ) )
-            return false;
-    }
-
-    // 5. Load the legend.
-    KoXmlElement legendElem = KoXml::namedItemNS( chartElement, KoXmlNS::chart,
-                          "legend" );
-    if ( !legendElem.isNull() ) {
-        if ( !d->legend->loadOdf( legendElem, context ) )
-            return false;
-    }
-    d->legend->update();
-
-    // 6. Load the data
+    // 2. Load the data
     KoXmlElement  dataElem = KoXml::namedItemNS( chartElement,
                                                  KoXmlNS::table, "table" );
     if ( !dataElem.isNull() ) {
@@ -1109,7 +1163,7 @@ bool ChartShape::loadOdfEmbedded( const KoXmlElement &chartElement,
             return false;
     }
 
-    // 7. Load the plot area (this is where the meat is!).
+    // 3. Load the plot area (this is where the meat is!).
     KoXmlElement  plotareaElem = KoXml::namedItemNS( chartElement,
                                                      KoXmlNS::chart, "plot-area" );
     if ( !plotareaElem.isNull() ) {
@@ -1117,6 +1171,42 @@ bool ChartShape::loadOdfEmbedded( const KoXmlElement &chartElement,
             return false;
     }
 
+    // 4. Load the title.
+    KoXmlElement titleElem = KoXml::namedItemNS( chartElement,
+                                                 KoXmlNS::chart, "title" );
+    if ( !titleElem.isNull() ) {
+        if ( !loadOdfLabel( d->title, titleElem, context) )
+            return false;
+        d->showLabel(d->title, true);
+    }
+
+    // 5. Load the subtitle.
+    KoXmlElement subTitleElem = KoXml::namedItemNS( chartElement,
+                                                    KoXmlNS::chart, "subtitle" );
+    if ( !subTitleElem.isNull() ) {
+        if ( !loadOdfLabel( d->subTitle, subTitleElem, context) )
+            return false;
+        d->showLabel(d->subTitle, true);
+    }
+
+    // 6. Load the footer.
+    KoXmlElement footerElem = KoXml::namedItemNS( chartElement,
+                                                  KoXmlNS::chart, "footer" );
+    if ( !footerElem.isNull() ) {
+        if ( !loadOdfLabel( d->footer, footerElem, context ) )
+            return false;
+        d->showLabel(d->footer, true);
+    }
+
+    // 7. Load the legend.
+    KoXmlElement legendElem = KoXml::namedItemNS( chartElement, KoXmlNS::chart,
+                          "legend" );
+    if ( !legendElem.isNull() ) {
+        if ( !d->legend->loadOdf( legendElem, context ) )
+            return false;
+    }
+    d->legend->update();
+
     requestRepaint();
 
     styleStack.restore();
@@ -1161,7 +1251,7 @@ void ChartShape::saveOdf( KoShapeSavingContext & context ) const
     //        if it works there too.
     //
     QList<const char*>  tagHierarchy = bodyWriter.tagHierarchy();
-    if ( tagHierarchy.isEmpty() 
+    if ( tagHierarchy.isEmpty()
          || QString( tagHierarchy.last() ) != "office:chart" )
     {
         bodyWriter.startElement( "draw:frame" );
@@ -1183,20 +1273,20 @@ void ChartShape::saveOdf( KoShapeSavingContext & context ) const
     saveOdfAttributes( context, OdfAllAttributes ^ OdfMandatories );
 
     KoGenStyle style;
-    style = KoGenStyle( KoGenStyle::StyleGraphicAuto, "chart" );
+    style = KoGenStyle( KoGenStyle::GraphicAutoStyle, "chart" );
     bodyWriter.addAttribute( "chart:style-name", saveStyle( style, context ) );
 
     // 1. Write the chart type.
     bodyWriter.addAttribute( "chart:class", ODF_CHARTTYPES[ d->plotArea->chartType() ] );
 
     // 2. Write the title.
-    saveOdfLabel( d->title, bodyWriter, mainStyles, Title );
+    saveOdfLabel( d->title, bodyWriter, mainStyles, TitleLabelType );
 
     // 3. Write the subtitle.
-    saveOdfLabel( d->subTitle, bodyWriter, mainStyles, SubTitle );
+    saveOdfLabel( d->subTitle, bodyWriter, mainStyles, SubTitleLabelType );
 
     // 4. Write the footer.
-    saveOdfLabel( d->footer, bodyWriter, mainStyles, Footer );
+    saveOdfLabel( d->footer, bodyWriter, mainStyles, FooterLabelType );
 
     // 5. Write the legend.
     d->legend->saveOdf( context );
@@ -1307,14 +1397,6 @@ void ChartShape::saveOdfData( KoXmlWriter &bodyWriter, KoGenStyles &mainStyles )
     bodyWriter.endElement(); // table:table
 }
 
-void ChartShape::init( const QMap<QString, KoDataCenter *> & dataCenterMap )
-{
-    d->dataCenterMap = dataCenterMap;
-    d->title->init( dataCenterMap );
-    d->subTitle->init( dataCenterMap );
-    d->footer->init( dataCenterMap );
-}
-
 void ChartShape::update() const
 {
     KoShape::update();
@@ -1333,9 +1415,9 @@ void ChartShape::requestRepaint() const
     d->plotArea->requestRepaint();
 }
 
-QMap<QString, KoDataCenter*> ChartShape::dataCenterMap() const
+KoResourceManager *ChartShape::resourceManager() const
 {
-    return d->dataCenterMap;
+    return d->resourceManager;
 }
 
 } // Namespace KChart
