@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
  *
- * Copyright (C) 2007      Inge Wallin <inge@lysator.liu.se>
+ * Copyright (C) 2007, 2010  Inge Wallin <inge@lysator.liu.se>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -88,7 +88,7 @@ ChartTool::Private::~Private()
 }
 
 ChartTool::ChartTool( KoCanvasBase *canvas )
-    : KoTool( canvas ),
+    : KoToolBase( canvas ),
       d( new Private() )
 {
     // Create QActions here.
@@ -120,7 +120,7 @@ void ChartTool::shapeSelectionChanged()
     
     // Get the chart shape that the tool is working on. 
     // Let d->shape point to it.
-    KoSelection  *selection = m_canvas->shapeManager()->selection();
+    KoSelection  *selection = canvas()->shapeManager()->selection();
     foreach ( KoShape *shape, selection->selectedShapes() ) {
         // Find out which type of shape that the user clicked on.
         // We support several here, since the chart shape is comprised
@@ -136,7 +136,7 @@ void ChartTool::shapeSelectionChanged()
                 Legend *legend = dynamic_cast<Legend*>( shape );
                 if ( legend ) {
                     selectedShape = legend;
-                    d->shape = dynamic_cast<ChartShape*>( legend->parent() );
+                    d->shape = dynamic_cast<ChartShape*>( shape->parent() );
                 }
             }
         // The selected shape is the chart
@@ -225,9 +225,9 @@ void ChartTool::mouseReleaseEvent( KoPointerEvent *event )
 }
 
 
-void ChartTool::activate( bool )
+void ChartTool::activate(ToolActivation, const QSet<KoShape*> &)
 {
-    useCursor( Qt::ArrowCursor, true );
+    useCursor(Qt::ArrowCursor);
 }
 
 void ChartTool::deactivate()
@@ -309,10 +309,10 @@ QWidget *ChartTool::createOptionWidget()
     		 this,   SLOT( setAxisShowGridLines( Axis*, bool ) ) );
     connect( widget, SIGNAL( axisUseLogarithmicScalingChanged( Axis*, bool ) ),
     		 this,   SLOT( setAxisUseLogarithmicScaling( Axis*, bool ) ) );
-    connect( widget, SIGNAL( axisStepWidthChanged( Axis*, double ) ),
-    		 this,   SLOT( setAxisStepWidth( Axis*, double ) ) );
-    connect( widget, SIGNAL( axisSubStepWidthChanged( Axis*, double ) ),
-    		 this,   SLOT( setAxisSubStepWidth( Axis*, double ) ) );
+    connect( widget, SIGNAL( axisStepWidthChanged( Axis*, qreal ) ),
+    		 this,   SLOT( setAxisStepWidth( Axis*, qreal ) ) );
+    connect( widget, SIGNAL( axisSubStepWidthChanged( Axis*, qreal ) ),
+    		 this,   SLOT( setAxisSubStepWidth( Axis*, qreal ) ) );
     connect( widget, SIGNAL( axisUseAutomaticStepWidthChanged( Axis*, bool ) ),
     		 this,   SLOT( setAxisUseAutomaticStepWidth( Axis*, bool ) ) );
     connect( widget, SIGNAL( axisUseAutomaticSubStepWidthChanged( Axis*, bool ) ),
@@ -485,14 +485,7 @@ void ChartTool::setShowTitle( bool show )
     if ( !d->shape )
         return;
 
-    // FIXME: Is this the reason why the plotarea doesn't grow again
-    //        if the title checkbox is unchecked?  I think it is.
-    //        Solve this by making showTitle take a bool.
-    //        Similar for the subtitle and footer below.
-    if ( show )
-        d->shape->showTitle();
-    else
-        d->shape->title()->setVisible( false );
+    d->shape->showTitle(show);
     d->shape->update();
 }
 
@@ -502,10 +495,7 @@ void ChartTool::setShowSubTitle( bool show )
     if ( !d->shape )
         return;
 
-    if ( show )
-        d->shape->showSubTitle();
-    else
-        d->shape->subTitle()->setVisible( false );
+    d->shape->showSubTitle(show);
     d->shape->update();
 }
 
@@ -515,10 +505,7 @@ void ChartTool::setShowFooter( bool show )
     if ( !d->shape )
         return;
 
-    if ( show )
-        d->shape->showFooter();
-    else
-        d->shape->footer()->setVisible( false );
+    d->shape->showFooter(show);
     d->shape->update();
 }
 
@@ -686,13 +673,13 @@ void ChartTool::setAxisUseLogarithmicScaling( Axis *axis, bool b )
     d->shape->update();
 }
 
-void ChartTool::setAxisStepWidth( Axis *axis, double width )
+void ChartTool::setAxisStepWidth( Axis *axis, qreal width )
 {
     axis->setMajorInterval( width );
     d->shape->update();
 }
 
-void ChartTool::setAxisSubStepWidth( Axis *axis, double width )
+void ChartTool::setAxisSubStepWidth( Axis *axis, qreal width )
 {
     axis->setMinorInterval( width );
     d->shape->update();
