@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2004 Jaroslaw Staniek <js@iidea.pl>
+   Copyright (C) 2004 Jarosław Staniek <staniek@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef KEXIACTIONPROXY_P_H
@@ -27,15 +27,15 @@ class KexiActionProxy;
 //! Helper class for KexiActionProxy::plugSharedActionToExternalGUI() method.
 class KAction_setEnabled_Helper : public QObject
 {
-	Q_OBJECT
-	public:
-		KAction_setEnabled_Helper(KexiActionProxy* proxy);
+    Q_OBJECT
+public:
+    KAction_setEnabled_Helper(KexiActionProxy* proxy);
 
-	public slots:
-		void slotSetEnabled(bool enabled);
+public slots:
+    void slotSetEnabled(bool enabled);
 
-	protected:
-		KexiActionProxy *m_proxy;
+protected:
+    KexiActionProxy *m_proxy;
 };
 
 #endif
