@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2004 Jaroslaw Staniek <js@iidea.pl>
+   Copyright (C) 2004 Jarosław Staniek <staniek@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -21,10 +21,10 @@
 #define KEXISHAREDACTIONHOST_P_H
 
 #include <qobject.h>
-#include <qptrdict.h>
+#include <q3ptrdict.h>
 #include <qsignalmapper.h>
 
-#include <kmainwindow.h>
+//#include <kxmlguiwindow.h>
 
 #include "kexiactionproxy.h"
 
@@ -32,32 +32,35 @@ class KexiSharedActionHost;
 
 class KexiVolatileActionData
 {
-	public:
-		KexiVolatileActionData() { plugged=false; }
-//		KAction *kaction;
-		bool plugged : 1;
+public:
+    KexiVolatileActionData() {
+        plugged = false;
+    }
+//  KAction *kaction;
+    bool plugged;
 };
 
 //! @internal
-class KEXICORE_EXPORT KexiSharedActionHostPrivate : public QObject
+class KexiSharedActionHostPrivate : public QObject
 {
-	Q_OBJECT
+    Q_OBJECT
 
-	public:
-		KexiSharedActionHostPrivate(KexiSharedActionHost *h);
+public:
+    KexiSharedActionHostPrivate(KexiSharedActionHost *h);
 
-	public slots:
-		void slotAction(const QString& act_id);
+    ~KexiSharedActionHostPrivate();
 
-	public:
-		QPtrDict<KexiActionProxy> actionProxies;
-		KMainWindow *mainWin;
-		KActionPtrList sharedActions;
-		QSignalMapper actionMapper;
-		QPtrDict<KexiVolatileActionData> volatileActions;
-		QAsciiDict<QWidget> enablers;
+public slots:
+    void slotAction(const QString& act_id);
 
-		KexiSharedActionHost *host;
+public:
+    QHash<QObject*, KexiActionProxy*> actionProxies;
+    KexiMainWindowIface *mainWin;
+    QList<KAction*> sharedActions;
+    QSignalMapper actionMapper;
+    QHash<KAction*, KexiVolatileActionData*> volatileActions;
+    QHash<QString, QWidget*> enablers;
+    KexiSharedActionHost *host;
 };
 
 #endif
