@@ -24,35 +24,36 @@ the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 
 #include <kexidb/driver.h>
 
-namespace KexiDB {
+namespace KexiDB
+{
 
 //! MySQL database driver.
 class MySqlDriver : public Driver
 {
-	Q_OBJECT
-	KEXIDB_DRIVER
+    Q_OBJECT
+    KEXIDB_DRIVER
 
-	public:
-		MySqlDriver(QObject *parent, const char *name, const QStringList &args=QStringList());
-		virtual ~MySqlDriver();
-		
-		virtual bool isSystemDatabaseName( const QString &n ) const;
+public:
+    MySqlDriver(QObject *parent, const QVariantList &args = QVariantList());
+    virtual ~MySqlDriver();
 
-		//! Escape a string for use as a value
-		virtual QString escapeString(const QString& str) const;
-		virtual QCString escapeString(const QCString& str) const;
+    virtual bool isSystemDatabaseName(const QString &n) const;
 
-		//! Escape BLOB value \a array
-		virtual QString escapeBLOB(const QByteArray& array) const;
+    //! Escape a string for use as a value
+    virtual QString escapeString(const QString& str) const;
+    virtual QByteArray escapeString(const QByteArray& str) const;
 
-	protected:
-		virtual QString drv_escapeIdentifier( const QString& str) const;
-		virtual QCString drv_escapeIdentifier( const QCString& str) const;
-		virtual Connection *drv_createConnection( ConnectionData &conn_data );
-		virtual bool drv_isSystemFieldName( const QString& n ) const;
-	
-	private:
-		static const char *keywords[];
+    //! Escape BLOB value \a array
+    virtual QString escapeBLOB(const QByteArray& array) const;
+
+protected:
+    virtual QString drv_escapeIdentifier(const QString& str) const;
+    virtual QByteArray drv_escapeIdentifier(const QByteArray& str) const;
+    virtual Connection *drv_createConnection(ConnectionData &conn_data);
+    virtual bool drv_isSystemFieldName(const QString& n) const;
+
+private:
+    static const char *keywords[];
 };
 }
 
