@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2006 Jaroslaw Staniek <js@iidea.pl>
+   Copyright (C) 2006 Jarosław Staniek <staniek@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -19,25 +19,32 @@
 
 #include "styleproxy.h"
 
+#include <QStyleFactory>
+#include <QApplication>
+
 using namespace KexiUtils;
 
-StyleProxy::StyleProxy(QStyle* parentStyle) 
- : QStyle()
+StyleProxy::StyleProxy(QStyle* parentStyle)
+        : QStyle()
 {
-	setParentStyle(parentStyle);
+    setParentStyle(parentStyle);
 }
 
 StyleProxy::~StyleProxy()
 {
-	delete m_style;
+    delete m_style;
 }
 
 void StyleProxy::setParentStyle(QStyle* style)
 {
-	m_style = QStyleFactory::create(style->name());
+    m_style = QStyleFactory::create(style->objectName());
+    if (!m_style)
+        m_style = style;
 }
 
 QStyle* StyleProxy::parentStyle() const
 {
-	return m_style;
+    if (m_style)
+        return m_style;
+    return QApplication::style();
 }
