@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2004 Jaroslaw Staniek <js@iidea.pl>
+   Copyright (C) 2004 Jarosław Staniek <staniek@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -13,27 +13,25 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef KEXIDBTITLEPAGE_H
 #define KEXIDBTITLEPAGE_H
 
-#include "KexiDBTitlePageBase.h"
+#include "ui_KexiDBTitlePage.h"
+#include <QWidget>
+#include <kexi_export.h>
 
-class KEXIMAIN_EXPORT KexiDBTitlePage : public KexiDBTitlePageBase
+//! @short A helper widget used to displaying a line edit with a label and layout
+class KEXIMAIN_EXPORT KexiDBTitlePage : public QWidget, public Ui_KexiDBTitlePage
 {
-    Q_OBJECT
-
 public:
-    KexiDBTitlePage( QWidget* parent = 0, const char* name = 0 );
+    //! Constructs title page. \a labelText can be provided to change default
+    //! "Project caption:" label.
+    KexiDBTitlePage(const QString& labelText, QWidget* parent = 0);
     ~KexiDBTitlePage();
-
-protected slots:
-    virtual void languageChange() { KexiDBTitlePageBase::languageChange(); }
-
 };
 
 #endif // KEXIDBTITLEPAGE_H
-
