@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2005-2006 Jaroslaw Staniek <js@iidea.pl>
+   Copyright (C) 2005-2009 Jarosław Staniek <staniek@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -23,46 +23,52 @@
 #include <kexiutils/utils.h>
 
 KexiFormDataItemInterface::KexiFormDataItemInterface()
- : KexiDataItemInterface()
- , m_columnInfo(0)
- , m_displayParametersForEnteredValue(0)
- , m_displayParametersForDefaultValue(0)
- , m_displayDefaultValue(false)
+        : KexiDataItemInterface()
+        , m_columnInfo(0)
+        , m_displayParametersForEnteredValue(0)
+        , m_displayParametersForDefaultValue(0)
+        , m_displayDefaultValue(false)
 {
 }
 
 KexiFormDataItemInterface::~KexiFormDataItemInterface()
 {
-	delete m_displayParametersForEnteredValue;
-	delete m_displayParametersForDefaultValue;
+    delete m_displayParametersForEnteredValue;
+    delete m_displayParametersForDefaultValue;
 }
 
 void KexiFormDataItemInterface::undoChanges()
 {
-//	m_disable_signalValueChanged = true;
-	setValueInternal(QString::null, false);
-//	m_disable_signalValueChanged = false;
+// m_disable_signalValueChanged = true;
+    setValueInternal(QString(), false);
+// m_disable_signalValueChanged = false;
 }
 
 KexiDB::Field* KexiFormDataItemInterface::field() const
 {
-	return m_columnInfo ? m_columnInfo->field : 0;
+    return m_columnInfo ? m_columnInfo->field : 0;
 }
 
 void KexiFormDataItemInterface::setDisplayDefaultValue(QWidget* widget, bool displayDefaultValue)
 {
-	m_displayDefaultValue = displayDefaultValue;
-	if (!m_displayParametersForDefaultValue) {
-		m_displayParametersForEnteredValue = new KexiDisplayUtils::DisplayParameters(widget);
-		m_displayParametersForDefaultValue = new KexiDisplayUtils::DisplayParameters();
-		KexiDisplayUtils::initDisplayForDefaultValue(*m_displayParametersForDefaultValue, widget);
-	}
+    m_displayDefaultValue = displayDefaultValue;
+    if (!m_displayParametersForDefaultValue) {
+        m_displayParametersForEnteredValue = new KexiDisplayUtils::DisplayParameters(widget);
+        m_displayParametersForDefaultValue = new KexiDisplayUtils::DisplayParameters();
+        KexiDisplayUtils::initDisplayForDefaultValue(*m_displayParametersForDefaultValue, widget);
+    }
 }
 
 void KexiFormDataItemInterface::cancelEditor()
 {
-	QWidget *parentWidget = dynamic_cast<QWidget*>(this)->parentWidget();
-	KexiFormScrollView* view = KexiUtils::findParent<KexiFormScrollView>(parentWidget, "KexiFormScrollView");
-	if (view)
-		view->cancelEditor();
+    QWidget *parentWidget = dynamic_cast<QWidget*>(this)->parentWidget();
+    KexiFormScrollView* view = KexiUtils::findParent<KexiFormScrollView*>(parentWidget);
+    if (view)
+        view->cancelEditor();
+}
+
+void KexiFormDataItemInterface::selectAllOnFocusIfNeeded()
+{
+    moveCursorToEnd();
+    selectAll();
 }
