@@ -3,9 +3,9 @@
 # This script prepares & uploads tarballs
 # Usage within KDE svn tree.
 # Based on cvs2pack
-# Copyright 2002 Nikolas Zimmermann <wildfox@kde.org>
-# Copyright 2004-2006 Jaroslaw Staniek <js@iidea.pl>
-# Copyright 2005 Martin Ellis <martin.ellis@kdemail.net>
+# Copyright (C) 2002 Nikolas Zimmermann <wildfox@kde.org>
+# Copyright (C) 2004-2006 Jarosław Staniek <staniek@kde.org>
+# Copyright (C) 2005 Martin Ellis <martin.ellis@kdemail.net>
 # License: GPL (http://www.gnu.org/)
 
 DIRNAME=`pwd`/`dirname $0`
@@ -61,7 +61,7 @@ fixAppSpecific()
 # Paths
 mkdir -p /tmp/kexi-dist
 DESTINATION=/tmp/kexi-dist/$DIST_VER		# CHANGE TO A NON-EXISTING DIR, WHICH WILL BE CREATED LATER!!!
-CONFIGURE_PREFIX=`kde-config --prefix`	# CHANGE
+CONFIGURE_PREFIX=`kde4-config --prefix`	# CHANGE
 EXPECT_PROGRAM=`which expect`				# CHANGE
 ED_LOCATION=/bin						# CHANGE
 
@@ -133,8 +133,6 @@ if ! [ -e $DESTINATION/source/$MODULE ]; then
 	(cd $MODULE/lib && $SVN_PROGRAM up koproperty ; cd ..) >> $DESTINATION/LOG 2>&1 || exit 1
 	echo "cd $MODULE/lib && $SVN_PROGRAM up kofficecore ; cd .." >> $DESTINATION/LOG
 	(cd $MODULE/lib && $SVN_PROGRAM up kofficecore ; cd ..) >> $DESTINATION/LOG 2>&1 || exit 1
-	echo "cd $MODULE/lib && $SVN_PROGRAM up kofficeui ; cd .." >> $DESTINATION/LOG
-	(cd $MODULE/lib && $SVN_PROGRAM up kofficeui ; cd ..) >> $DESTINATION/LOG 2>&1 || exit 1
 	echo "cd $MODULE/lib && $SVN_PROGRAM up kross ; cd .." >> $DESTINATION/LOG
 	(cd $MODULE/lib && $SVN_PROGRAM up kross ; cd ..) >> $DESTINATION/LOG 2>&1 || exit 1
 
