@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
    Copyright (C) 2003 Lucijan Busch <lucijan@gmx.at>
-   Copyright (C) 2004-2005 Jaroslaw Staniek <js@iidea.pl>
+   Copyright (C) 2004-2005 Jarosław Staniek <staniek@kde.org>
    Copyright (C) 2005 Cedric Pasteur <cedric.pasteur@free.fr>
 
    This program is free software; you can redistribute it and/or
@@ -29,11 +29,11 @@
  for entering query statements. */
 class KEXIEXTWIDGETS_EXPORT KexiQueryDesignerSQLEditor : public KexiEditor
 {
-	Q_OBJECT
+    Q_OBJECT
 
-	public:
-		KexiQueryDesignerSQLEditor(KexiMainWindow *mainWin, QWidget *parent, const char *name = 0);
-		virtual ~KexiQueryDesignerSQLEditor();
+public:
+    KexiQueryDesignerSQLEditor(QWidget *parent = 0);
+    virtual ~KexiQueryDesignerSQLEditor();
 };
 
 #endif
