@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
    Copyright (C) 2001 David Faure <faure@kde.org>
+   Copyright (C) 2007 Thorsten Zachmann <zachmann@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -20,43 +21,45 @@
 #include "kounavail.h"
 
 #include <qpainter.h>
-#include <qiconset.h>
+#include <qicon.h>
+#include <QPaintEvent>
 #include <kaction.h>
-#include <kinstance.h>
-#include <kstdaction.h>
+#include <kcomponentdata.h>
+#include <kstandardaction.h>
 #include <klocale.h>
 #include <kaboutdata.h>
 #include <kdebug.h>
 #include <kglobalsettings.h>
 #include <qapplication.h>
+#include <KoOdfReadStore.h>
 
-KoUnavailPart::KoUnavailPart( QWidget *parentWidget, const char *widgetName, QObject* parent, const char* name )
-    : KoDocument( parentWidget, widgetName, parent, name, false /*singleViewMode*/ )
+KoUnavailPart::KoUnavailPart( QWidget *parentWidget, QObject* parent )
+    : KoDocument( parentWidget, parent, false /*singleViewMode*/ )
 {
     setReadWrite( false );
 }
 
-KoView* KoUnavailPart::createViewInstance( QWidget* parent, const char* name )
+KoView* KoUnavailPart::createViewInstance( QWidget* parent)
 {
-    return new KoUnavailView( this, parent, name );
+    return new KoUnavailView( this, parent );
 }
 
-bool KoUnavailPart::loadOasis( const QDomDocument& doc, KoOasisStyles&, const QDomDocument&, KoStore* )
+bool KoUnavailPart::loadOdf( KoOdfReadStore & odfStore )
 {
-    m_doc = doc;
+    m_doc = KoXml::asQDomDocument( QDomDocument(), odfStore.contentDoc() );
     return true;
 }
 
-bool KoUnavailPart::loadXML( QIODevice *, const QDomDocument &doc )
+bool KoUnavailPart::loadXML( const KoXmlDocument &doc, KoStore *)
 {
     // Simply keep a copy of the whole document ;)
-    m_doc = doc;
+    m_doc = KoXml::asQDomDocument( QDomDocument(), doc );
     return true;
 }
 
 bool KoUnavailPart::saveFile()
 {
-    kdDebug() << "KoUnavailPart::saveFile m_url=" << m_url.prettyURL() << endl;
+    kDebug() <<"KoUnavailPart::saveFile m_url=" << url().prettyUrl();
     // This is called if the part points to an external file
     // In that case we have nothing to save, the file was unavailable !
     return true;
@@ -64,28 +67,28 @@ bool KoUnavailPart::saveFile()
 
 QDomDocument KoUnavailPart::saveXML()
 {
-    kdDebug() << "KoUnavailPart::saveXML" << endl;
+    kDebug() <<"KoUnavailPart::saveXML";
     return m_doc;
 }
 
 
-bool KoUnavailPart::saveOasis(KoStore*, KoXmlWriter*)
+bool KoUnavailPart::saveOdf( SavingContext & documentContext )
 {
+    Q_UNUSED( documentContext );
     // TODO
     return false;
 }
 
-void KoUnavailPart::setMimeType( const QCString& mime )
+void KoUnavailPart::setMimeType( const QByteArray& mime )
 {
-    kdDebug() << "KoUnavailPart::setMimeType " << mime << endl;
+    kDebug() <<"KoUnavailPart::setMimeType" << mime;
     m_mimetype = mime;
 }
 
-void KoUnavailPart::paintContent( QPainter& painter, const QRect& rect, bool /*transparent*/,
-                                double /*zoomX*/, double /*zoomY*/ )
+void KoUnavailPart::paintContent( QPainter& painter, const QRect& rect)
 {
     painter.save();
-    painter.setPen( QApplication::palette().color( QPalette::Active, QColorGroup::Text ) );
+    painter.setPen( QApplication::palette().color( QPalette::Active, QPalette::Text ) );
     // Need to draw only the document rectangle described in the parameter rect.
     int left = rect.left() / 20;
     int right = rect.right() / 20 + 1;
@@ -101,14 +104,14 @@ void KoUnavailPart::paintContent( QPainter& painter, const QRect& rect, bool /*t
     defaultFont.setPointSize( 16 ); // ###
     painter.setFont( defaultFont );
     //painter.drawText( 20, 20, m_reason );
-    painter.drawText( rect, Qt::AlignCenter | Qt::WordBreak, m_reason );
+    painter.drawText( rect, Qt::AlignCenter | Qt::TextWordWrap, m_reason );
     painter.restore();
 }
 
-KoUnavailView::KoUnavailView( KoUnavailPart* part, QWidget* parent, const char* name )
-    : KoView( part, parent, name )
+KoUnavailView::KoUnavailView( KoUnavailPart* part, QWidget* parent)
+    : KoView( part, parent )
 {
-    setInstance( KoUnavailFactory::global() );
+    setComponentData( KoUnavailFactory::global() );
     //setXMLFile( "kounavail.rc" );
 }
 
@@ -120,14 +123,14 @@ void KoUnavailView::paintEvent( QPaintEvent* ev )
     // ### TODO: Scaling
 
     // Let the document do the drawing
-    koDocument()->paintEverything( painter, ev->rect(), FALSE, this );
+    koDocument()->paintEverything( painter, ev->rect(), this );
 
     painter.end();
 }
 
 K_EXPORT_COMPONENT_FACTORY( libkounavailpart, KoUnavailFactory )
 
-KInstance* KoUnavailFactory::s_global = 0L;
+KComponentData* KoUnavailFactory::s_global = 0L;
 KAboutData* KoUnavailFactory::s_aboutData = 0L;
 
 KoUnavailFactory::KoUnavailFactory( QObject* parent, const char* name )
@@ -144,9 +147,9 @@ KoUnavailFactory::~KoUnavailFactory()
     s_global = 0L;
 }
 
-KParts::Part* KoUnavailFactory::createPartObject( QWidget *parentWidget, const char *widgetName, QObject* parent, const char* name, const char*, const QStringList & )
+KParts::Part* KoUnavailFactory::createPartObject( QWidget *parentWidget, QObject* parent, const char*, const QStringList & )
 {
-    return new KoUnavailPart( parentWidget, widgetName, parent, name );
+    return new KoUnavailPart( parentWidget, parent );
 }
 
 KAboutData* KoUnavailFactory::aboutData()
@@ -155,23 +158,23 @@ KAboutData* KoUnavailFactory::aboutData()
     {
         static const char* description=I18N_NOOP("KoUnavail KOffice Program");
         static const char* version="0.1";
-        s_aboutData=new KAboutData( "kounavail", I18N_NOOP("KoUnavail"),
-                                    version, description, KAboutData::License_LGPL,
-                                    "(c) 2001, David Faure");
-        s_aboutData->addAuthor("David Faure",0, "faure@kde.org");
+        s_aboutData=new KAboutData( "kounavail", 0, ki18n("KoUnavail"),
+                                    version, ki18n(description), KAboutData::License_LGPL,
+                                    ki18n("(c) 2001, David Faure"));
+        s_aboutData->addAuthor(ki18n("David Faure"),KLocalizedString(), "faure@kde.org");
     }
     return s_aboutData;
 }
 
-KInstance* KoUnavailFactory::global()
+const KComponentData &KoUnavailFactory::global()
 {
     if ( !s_global )
     {
-        s_global = new KInstance( aboutData() );
+        s_global = new KComponentData( aboutData() );
         // Tell the iconloader about share/apps/koffice/icons
         //s_global->iconLoader()->addAppDir("koffice");
     }
-    return s_global;
+    return *s_global;
 }
 
 #include "kounavail.moc"
