@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
   Copyright (C) 1998, 1999, 2000 Torben Weis <weis@kde.org>
-  Copyright (C) 2002 - 2007 Dag Andersen <danders@get2net.dk>
+  Copyright (C) 2002 - 2010 Dag Andersen <danders@get2net.dk>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -24,17 +24,18 @@
 #include "kplato_export.h"
 
 #include <KoView.h>
-#include <KoQueryTrader.h>
 
 #include "kptcontext.h"
 
 #include <QMenu>
 #include <QDockWidget>
-
+#include <QTimer>
+#include <QMap>
 
 class QProgressBar;
 class QStackedWidget;
 class QSplitter;
+class QUndoCommand;
 
 class KAction;
 class KToggleAction;
@@ -49,6 +50,7 @@ class View;
 class ViewBase;
 class ViewListItem;
 class ViewListWidget;
+struct ViewInfo;
 class AccountsView;
 class GanttView;
 class PertEditor;
@@ -58,14 +60,14 @@ class TaskEditor;
 class CalendarEditor;
 class ScheduleEditor;
 class ScheduleManager;
-
+class CalculateScheduleCmd;
 class ResourceAssignmentView;
 class TaskStatusView;
 class Calendar;
 class Part;
-class DocumentChild;
 class Node;
 class Project;
+class Task;
 class MainSchedule;
 class Schedule;
 class Resource;
@@ -73,7 +75,10 @@ class ResourceGroup;
 class Relation;
 class Context;
 class ViewAdaptor;
+class HtmlView;
+class ReportView;
 
+class ReportDesignDialog;
 
 //-------------
 class KPLATO_EXPORT View : public KoView
@@ -83,10 +88,6 @@ class KPLATO_EXPORT View : public KoView
 public:
     explicit View( Part* part, QWidget* parent = 0 );
     ~View();
-    /**
-     * Support zooming.
-     */
-    virtual void setZoom( double zoom );
 
     Part *getPart() const;
 
@@ -101,62 +102,71 @@ public:
 
     /// Load the workpackage from @p url into @p project. Return true if successful, else false.
     bool loadWorkPackage( Project &project, const KUrl &url );
-    
+
     QWidget *canvas() const;
 
-    //virtual QDockWidget *createToolBox();
-
-    KoDocument *hitTest( const QPoint &viewPos );
-
     ScheduleManager *currentScheduleManager() const;
     long activeScheduleId() const;
     void setActiveSchedule( long id ) const;
-    
-    ViewBase *createTaskEditor( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createResourcEditor( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createAccountsEditor( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createCalendarEditor( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createScheduleHandler( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
+
+    /// Returns the default view information like standard name and tooltip for view type @p type
+    ViewInfo defaultViewInfo( const QString type ) const;
+
+    ViewBase *createTaskEditor( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createResourceEditor( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createAccountsEditor( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createCalendarEditor( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createScheduleHandler( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
     ScheduleEditor *createScheduleEditor( QWidget *parent );
-    ViewBase *createScheduleEditor( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createDependencyEditor( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createPertEditor( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createProjectStatusView( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createPerformanceStatusView( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createTaskStatusView( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createTaskView( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createGanttView( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createMilestoneGanttView( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createResourceAppointmentsView( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createAccountsView( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createResourceAssignmentView( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
-    ViewBase *createChartView( ViewListItem *cat, const QString tag, const QString &name, const QString &tip, int index = -1 );
+    ViewBase *createScheduleEditor( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createDependencyEditor( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createPertEditor( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createProjectStatusView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createPerformanceStatusView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createTaskStatusView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createTaskView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createTaskWorkPackageView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createGanttView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createMilestoneGanttView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createResourceAppointmentsView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createResourceAppointmentsGanttView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createAccountsView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createResourceAssignmentView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createChartView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
+    ViewBase *createReportView( ViewListItem *cat, const QString tag, const QString &name = QString(), const QString &tip = QString(), int index = -1 );
 
     KoPrintJob * createPrintJob();
-    
+
+    virtual KoZoomController *zoomController() const {
+        return 0;
+    }
+
 signals:
     void currentScheduleManagerChanged( ScheduleManager *sm );
-    
+
 public slots:
     void slotUpdate();
     void slotCreateTemplate();
-    void slotAddResource( ResourceGroup *group );
     void slotEditResource();
     void slotEditCut();
     void slotEditCopy();
     void slotEditPaste();
+    void slotRefreshView();
     void slotViewSelector( bool show );
 
     void slotAddTask();
     void slotAddSubTask();
     void slotAddMilestone();
+    void slotAddSubMilestone();
     void slotProjectEdit();
     void slotDefineWBS();
+    void slotCurrencyConfig();
 
     void slotCreateView();
-    void slotCreateKofficeDocument( KoDocumentEntry& );
 
     void slotConfigure();
+    void slotIntroduction();
+
     void slotAddRelation( Node *par, Node *child );
     void slotModifyRelation( Relation *rel );
     void slotAddRelation( Node *par, Node *child, int linkType );
@@ -169,6 +179,9 @@ public slots:
     void slotPopupMenu( const QString& menuname, const QPoint &pos );
     void slotPopupMenu( const QString& menuname, const QPoint &pos, ViewListItem *item );
 
+    void addViewListItem( const ViewListItem *item, const ViewListItem *parent, int index );
+    void removeViewListItem( const ViewListItem *item );
+
 protected slots:
     void slotGuiActivated( ViewBase *view, bool );
     void slotViewActivated( ViewListItem*, ViewListItem* );
@@ -177,10 +190,13 @@ protected slots:
     void slotScheduleChanged( MainSchedule* );
     void slotScheduleAdded( const MainSchedule * );
     void slotScheduleRemoved( const MainSchedule * );
+    void slotSelectionChanged( ScheduleManager *sm );
 
     void slotAddScheduleManager( Project *project );
     void slotDeleteScheduleManager( Project *project, ScheduleManager *sm );
     void slotCalculateSchedule( Project*, ScheduleManager* );
+    void slotCalculationStarted( Project *project, ScheduleManager *sm );
+    void slotCalculationFinished( Project *project, ScheduleManager *sm );
     void slotBaselineSchedule( Project *project, ScheduleManager *sm );
     void slotProgressChanged( int value );
 
@@ -189,6 +205,7 @@ protected slots:
     void slotOpenNode();
     void slotOpenNode( Node *node );
     void slotTaskProgress();
+    void slotTaskDescription();
     void slotDeleteTask( QList<Node*> lst );
     void slotDeleteTask( Node *node );
     void slotDeleteTask();
@@ -196,8 +213,7 @@ protected slots:
     void slotUnindentTask();
     void slotMoveTaskUp();
     void slotMoveTaskDown();
-    void slotTaskWorkpackage();
-    
+
     void slotConnectNode();
 
     void slotDeleteResource( Resource *resource );
@@ -208,16 +224,35 @@ protected slots:
 
     void removeProgressBarItems();
 
+    void slotInsertFile();
+
+    void slotWorkPackageLoaded();
+    void slotMailWorkpackage( Node *node, Resource *resource = 0 );
+    void slotMailWorkpackages( QList<Node*> &nodes, Resource *resource = 0 );
+
+    void slotOpenUrlRequest( HtmlView *v, const KUrl &url );
+
+    void slotProjectCalculated( ScheduleManager *sm );
+
+    void slotUpdateViewInfo( ViewListItem *itm );
+
+    void slotEditReportDesign( ReportView *view );
+    void slotCreateReport();
+    void slotOpenReportFile();
+    void slotModifyReportDefinition( QUndoCommand *cmd );
+
 #ifndef NDEBUG
     void slotPrintDebug();
     void slotPrintSelectedDebug();
     void slotPrintCalendarDebug();
     void slotPrintTestDebug();
+    void slotToggleDebugInfo();
 #else
     static void slotPrintDebug() { };
     static void slotPrintSelectedDebug() { };
     static void slotPrintCalendarDebug() { };
     static void slotPrintTestDebug() { };
+    static void slotToggleDebugInfo() { };
 #endif
 
 protected:
@@ -227,23 +262,52 @@ protected:
     QList<QAction*> sortedActionList();
     QAction *addScheduleAction( Schedule *sch );
     void setLabel( ScheduleManager *sm = 0 );
-    Node *currentTask();
+    Task *currentTask() const;
+    Node *currentNode() const;
     Resource *currentResource();
     ResourceGroup *currentResourceGroup();
     Calendar *currentCalendar();
     void updateView( QWidget *widget );
 
+    ViewBase *currentView() const;
+    
+    ViewBase *createWelcomeView();
+
 private slots:
     void slotActionDestroyed( QObject *o );
     void slotViewListItemRemoved( ViewListItem *item );
-    void slotViewListItemInserted( ViewListItem *item );
+    void slotViewListItemInserted( ViewListItem *item, ViewListItem *parent, int index );
+
+    void slotCreateReportView( ReportDesignDialog *dlg );
+
+    void slotProjectEditFinished( int result );
+    void slotTaskEditFinished( int result );
+    void slotSummaryTaskEditFinished( int result );
+    void slotEditResourceFinished( int result );
+    void slotProjectWorktimeFinished( int result );
+    void slotDefineWBSFinished( int result );
+    void slotCurrencyConfigFinished( int result );
+    void slotInsertFileFinished( int result );
+    void slotAddSubTaskFinished( int result );
+    void slotAddTaskFinished( int result );
+    void slotAddSubMilestoneFinished( int result );
+    void slotAddMilestoneFinished( int result );
+    void slotTaskProgressFinished( int result );
+    void slotMilestoneProgressFinished( int result );
+    void slotTaskDescriptionFinished( int result );
+    void slotAddRelationFinished( int result );
+    void slotModifyRelationFinished( int result );
+    void slotReportDesignFinished( int result );
+    void slotOpenReportFileFinished( int result );
+    void slotCreateViewFinished( int result );
+
+    void slotRemoveCommands();
 
 private:
     void createViews();
-    
+    ViewBase *createView( ViewListItem *cat, const QString &type, const QString &tag, const QString &name, const QString &tip, int index = -1 );
 
-    void createChildDocumentViews();
-    ViewListItem *createChildDocumentView( DocumentChild *ch );
+    QString standardTaskStatusReport() const;
 
 private:
     QSplitter *m_sp;
@@ -265,17 +329,21 @@ private:
     QLabel *m_estlabel;
     QProgressBar *m_progress;
     QLabel *m_text;
+    QTimer m_progressBarTimer;
 
     ViewAdaptor* m_dbus;
 
     QActionGroup *m_scheduleActionGroup;
     QMap<QAction*, Schedule*> m_scheduleActions;
-    
+
+    QMultiMap<ScheduleManager*, CalculateScheduleCmd*> m_calculationcommands;
+    QList<QUndoCommand*> m_undocommands;
+
     bool m_readWrite;
-    
+
     // ------ File
     QAction *actionCreateTemplate;
-    
+
     // ------ Edit
     QAction *actionCut;
     QAction *actionCopy;
@@ -284,25 +352,30 @@ private:
     // ------ View
     KToggleAction *actionViewSelector;
 
-    KToggleAction *actionViewResourceAppointments;
-
     // ------ Insert
-
     // ------ Project
     KAction *actionEditMainProject;
-    KAction *actionEditStandardWorktime;
 
     // ------ Tools
+    KAction *actionEditStandardWorktime;
     KAction *actionDefineWBS;
+    KAction *actionInsertFile;
+    KAction *actionCurrencyConfig;
+
+    KAction *actionCreateReport;
+    KAction *actionOpenReportFile;
 
     // ------ Settings
     KAction *actionConfigure;
 
+    // ------ Help
+    KAction *actionIntroduction;
+
     // ------ Popup
     KAction *actionOpenNode;
     KAction *actionTaskProgress;
+    KAction *actionTaskDescription;
     KAction *actionDeleteTask;
-    KAction *actionTaskWorkpackage;
     KAction *actionIndentTask;
     KAction *actionUnindentTask;
     KAction *actionMoveTaskUp;
@@ -311,10 +384,11 @@ private:
     KAction *actionEditResource;
     KAction *actionEditRelation;
     KAction *actionDeleteRelation;
-    
+
     //Test
     KAction *actNoInformation;
 
+    QMap<ViewListItem*, QAction*> m_reportActionMap;
 };
 
 } //Kplato namespace
