@@ -44,6 +44,7 @@ class QTime;
 namespace KPlato
 {
 
+class Account;
 class Risk;
 class Effort;
 class Appointment;
@@ -56,10 +57,11 @@ class ResourceGroupRequest;
 class Calendar;
 class ResourceRequestCollection;
 class Schedule;
-class Schedule;
+class NodeSchedule;
 class ResourceSchedule;
 class Schedule;
 class XMLLoaderObject;
+class DateTimeInterval;
 
 /**
   * This class represents a group of similar resources to be assigned to a task
@@ -92,7 +94,7 @@ public:
     void setName( const QString& n );
     const QString &name() const { return m_name;}
     void setType( Type type );
-    //void setType(const QString &type);
+    void setType(const QString &type);
     Type type() const { return m_type; }
     QString typeToString( bool trans = false ) const;
     static QStringList typeToStringList( bool trans = false );
@@ -108,7 +110,7 @@ public:
          */
     void addResource( int index, Resource*, Risk* );
     Resource *takeResource( Resource *resource );
-    QList<Resource*> &resources() { return m_resources; }
+    QList<Resource*> resources() const { return m_resources; }
     int indexOf( const Resource *resource ) const;
     Resource *resourceAt( int pos ) const { return m_resources.value( pos ); }
     int numResources() const { return m_resources.count(); }
@@ -154,7 +156,7 @@ public:
 
     Calendar *defaultCalendar() { return m_defaultCalendar; }
 
-    int units();
+    int units() const;
 
     void registerRequest( ResourceGroupRequest *request )
     { m_requests.append( request ); }
@@ -181,6 +183,9 @@ public:
 
     void copy( const ResourceGroup *group );
     
+    DateTime startTime( long id ) const;
+    DateTime endTime( long id ) const;
+
 #ifndef NDEBUG
 
     void printDebug( const QString& ident );
@@ -278,7 +283,6 @@ public:
     Appointment *appointmentAt( int index, long id = -1 ) const { return appointments( id ).value( index ); }
     int indexOf( Appointment *a, long id = -1 ) const { return appointments( id ).indexOf( a ); }
     
-    Appointment *findAppointment( Node *node );
     /// Adds appointment to current schedule
     virtual bool addAppointment( Appointment *appointment );
     /// Adds appointment to schedule sch
@@ -288,7 +292,7 @@ public:
 
     void initiateCalculation( Schedule &sch );
     bool isAvailable( Task *task );
-    void makeAppointment( Schedule *schedule );
+    void makeAppointment( Schedule *schedule, int load, const QList<Resource*> &required = QList<Resource*>() );
 
     bool isOverbooked() const;
     /// check if overbooked on date.
@@ -342,13 +346,13 @@ public:
     /// The current schedule is used to check for appointments.
     /// If @p  backward is true, checks backward in time.
     /// Status is returned in @p ok
-    Duration effort( const DateTime &start, const Duration &duration, bool backward = false, bool *ok = 0 ) const;
+    Duration effort( const DateTime &start, const Duration &duration, bool backward = false, const QList<Resource*> &required = QList<Resource*>(), bool *ok = 0 ) const;
 
     /// Returns the effort that can be done starting at @p start within @p duration.
     /// The schedule @p sch is used to check for appointments.
     /// If @p  backward is true, checks backward in time.
     /// Status is returned in @p ok
-    Duration effort( Schedule *sch, const DateTime &start, const Duration &duration, bool backward = false, bool *ok = 0 ) const;
+    Duration effort( Schedule *sch, const DateTime &start, const Duration &duration, bool backward = false, const QList<Resource*> &required = QList<Resource*>(), bool *ok = 0 ) const;
 
 
     /**
@@ -387,6 +391,8 @@ public:
 
     Appointment appointmentIntervals( long id ) const;
     Appointment appointmentIntervals() const;
+    
+    EffortCostMap plannedEffortCostPrDay(const QDate &start, const QDate &end, long id);
     Duration plannedEffort( const QDate &date ) const;
 
     void setCurrentSchedulePtr( Schedule *schedule ) { m_currentSchedule = schedule; }
@@ -394,7 +400,7 @@ public:
     Schedule *currentSchedule() const { return m_currentSchedule; }
 
     bool isScheduled() const;
-    QHash<long, Schedule*> &schedules() { return m_schedules; }
+    QHash<long, Schedule*> schedules() const { return m_schedules; }
     /**
      * Return schedule with @id
      * If @id == -1, return m_currentSchedule
@@ -428,6 +434,24 @@ public:
     int numExternalAppointments() const { return m_externalAppointments.count(); }
     QList<Appointment*> externalAppointmentList() const { return m_externalAppointments.values(); }
     
+    /// Return a measure of how suitable the resource is for allocation
+    long allocationSuitability( const DateTime &time, const Duration &duration, bool backward );
+
+    DateTime startTime( long id ) const;
+    DateTime endTime( long id ) const;
+
+    QList<Resource*> requiredResources() const { return m_required; }
+    void setRequiredResources( const QList<Resource*> &lst ) { m_required = lst; }
+
+    /// Used by Project::load() after all resources have been loaded
+    /// to translate resource ids to resources
+    void resolveRequiredResources( Project &project );
+
+    /// Return the account
+    Account *account() const { return cost.account; }
+    /// Set the @p account
+    void setAccount( Account *account );
+
 signals:
     void externalAppointmentToBeAdded( Resource *r, int row );
     void externalAppointmentAdded( Resource*, Appointment* );
@@ -436,7 +460,8 @@ signals:
     void externalAppointmentChanged( Resource *r, Appointment *a );
 
 protected:
-    void makeAppointment( Schedule *node, const DateTime &from, const DateTime &end );
+    DateTimeInterval requiredAvailable(Schedule *node, const DateTime &start, const DateTime &end ) const;
+    void makeAppointment( Schedule *node, const DateTime &from, const DateTime &end, int load, const QList<Resource*> &required = QList<Resource*>() );
     virtual void changed();
 
 private:
@@ -460,12 +485,14 @@ private:
         double normalRate;
         double overtimeRate;
         double fixed ;
+        Account *account;
     }
     cost;
     
     Calendar *m_calendar;
-    QList<ResourceRequest*>
-    m_requests;
+    QList<ResourceRequest*> m_requests;
+    QList<Resource*> m_required;
+    QStringList m_requiredIds;
     
     Schedule *m_currentSchedule;
 
@@ -509,6 +536,7 @@ class KPLATOKERNEL_EXPORT ResourceRequest
 {
 public:
     explicit ResourceRequest( Resource *resource = 0, int units = 1 );
+    explicit ResourceRequest( const ResourceRequest &r );
 
     ~ResourceRequest();
 
@@ -525,6 +553,7 @@ public:
     * Get amount of requested resource units in percent
     */
     int units() const;
+    void setUnits( int value );
 
     /**
     * Get amount of requested work units in percent
@@ -542,20 +571,39 @@ public:
             m_resource->unregisterRequest( this );
     }
 
+    void makeAppointment( Schedule *schedule, int amount );
     void makeAppointment( Schedule *schedule );
     Task *task() const;
 
-    Schedule *resourceSchedule( Schedule *ns );
+    Schedule *resourceSchedule( Schedule *ns, Resource *resource = 0 );
     DateTime availableAfter(const DateTime &time, Schedule *ns);
     DateTime availableBefore(const DateTime &time, Schedule *ns);
     Duration effort( const DateTime &time, const Duration &duration, Schedule *ns, bool backward, bool *ok = 0 );
-    DateTime workTimeAfter(const DateTime &dt);
-    DateTime workTimeBefore(const DateTime &dt);
+    DateTime workTimeAfter(const DateTime &dt, Schedule *ns = 0);
+    DateTime workTimeBefore(const DateTime &dt, Schedule *ns = 0);
 
+    /// Resource is allocated dynamically by the group request
+    bool isDynamicallyAllocated() const { return m_dynamic; }
+    /// Set resource is allocated dynamically
+    void setAllocatedDynaically( bool dyn ) { m_dynamic = dyn; }
+
+    /// Return a measure of how suitable the resource is for allocation
+    long allocationSuitability( const DateTime &time, const Duration &duration, Schedule *ns, bool backward );
+
+    const QList<Resource*> requiredResources() const { return m_required; }
+    void setRequiredResources( const QList<Resource*> &lst ) { m_required = lst; }
+
+protected:
+    void changed();
+
+    void setCurrentSchedulePtr( Schedule *ns );
+    
 private:
     Resource *m_resource;
     int m_units;
     ResourceGroupRequest *m_parent;
+    bool m_dynamic;
+    QList<Resource*> m_required;
 
 #ifndef NDEBUG
 public:
@@ -567,45 +615,43 @@ class KPLATOKERNEL_EXPORT ResourceGroupRequest
 {
 public:
     explicit ResourceGroupRequest( ResourceGroup *group = 0, int units = 0 );
+    explicit ResourceGroupRequest( const ResourceGroupRequest &group );
     ~ResourceGroupRequest();
 
     void setParent( ResourceRequestCollection *parent ) { m_parent = parent;}
     ResourceRequestCollection *parent() const { return m_parent; }
 
     ResourceGroup *group() const { return m_group; }
+    void setGroup( ResourceGroup *group ) { m_group = group; }
     void unregister( const ResourceGroup *group ) { if ( group == m_group ) m_group = 0; }
-    QList<ResourceRequest*> &resourceRequests() { return m_resourceRequests; }
+    QList<ResourceRequest*> resourceRequests() { return m_resourceRequests; }
     void addResourceRequest( ResourceRequest *request );
-    void deleteResourceRequest( ResourceRequest *request )
-    {
-        int i = m_resourceRequests.indexOf( request );
-        if ( i != -1 )
-            m_resourceRequests.removeAt( i );
-        delete request;
-        changed();
-    }
-    
+    void deleteResourceRequest( ResourceRequest *request );
+    int count() const { return m_resourceRequests.count(); }
+    ResourceRequest *requestAt( int idx ) const { return m_resourceRequests.value( idx ); }
+
     ResourceRequest *takeResourceRequest( ResourceRequest *request );
-    ResourceRequest *find( Resource *resource );
+    ResourceRequest *find( const Resource *resource ) const;
     ResourceRequest *resourceRequest( const QString &name );
-    QStringList requestNameList() const;
+    /// Return a list of allocated resources, allocation to group is not included by default.
+    QStringList requestNameList( bool includeGroup = false ) const;
     
     bool load( KoXmlElement &element, Project &project );
     void save( QDomElement &element ) const;
 
-    /**
-    * Get total amount of resource units in percent
-    */
+    /// The number of requested resources
     int units() const;
+    void setUnits( int value ) { m_units = value; changed(); }
 
     /**
-    * Get amount of work units in percent
+    * Get amount of allocated work units in percent
     */
     int workUnits() const;
 
-    Duration effort( const DateTime &time, const Duration &duration, Schedule *ns, bool backward, bool *ok = 0 ) const;
-
-    int numDays( const DateTime &time, bool backward ) const;
+    /**
+    * Get the number of resources allocated
+    */
+    int workAllocation() const;
 
     /**
      * Returns the duration needed to do the effort  effort
@@ -615,8 +661,8 @@ public:
 
     DateTime availableAfter( const DateTime &time, Schedule *ns );
     DateTime availableBefore( const DateTime &time, Schedule *ns );
-    DateTime workTimeAfter(const DateTime &dt);
-    DateTime workTimeBefore(const DateTime &dt);
+    DateTime workTimeAfter(const DateTime &dt, Schedule *ns = 0);
+    DateTime workTimeBefore(const DateTime &dt, Schedule *ns = 0);
 
     /**
      * Makes appointments for task @param task to the 
@@ -635,6 +681,11 @@ public:
 
     void changed();
     
+    /// Reset dynamic resource allocations
+    void resetDynamicAllocations();
+    /// Allocate dynamic requests. Do nothing if already allocated.
+    void allocateDynamicRequests( const DateTime &time, const Duration &effort, Schedule *ns, bool backward );
+
 private:
     ResourceGroup *m_group;
     int m_units;
@@ -653,10 +704,10 @@ public:
 class KPLATOKERNEL_EXPORT ResourceRequestCollection
 {
 public:
-    ResourceRequestCollection( Task &task );
+    explicit ResourceRequestCollection( Task *task = 0 );
     ~ResourceRequestCollection();
 
-    const QList<ResourceGroupRequest*> &requests() const { return m_requests; }
+    QList<ResourceGroupRequest*> requests() const { return m_requests; }
     void addRequest( ResourceGroupRequest *request );
     void deleteRequest( ResourceGroupRequest *request )
     {
@@ -667,24 +718,33 @@ public:
         changed();
     }
 
-    void takeRequest( ResourceGroupRequest *request )
+    int takeRequest( ResourceGroupRequest *request )
     {
         int i = m_requests.indexOf( request );
         if ( i != -1 ) {
             m_requests.removeAt( i );
             changed();
         }
+        return i;
     }
 
-    ResourceGroupRequest *find( ResourceGroup *resource ) const;
-    ResourceRequest *find( Resource *resource ) const;
+    ResourceGroupRequest *find( const ResourceGroup *resource ) const;
+    ResourceRequest *find( const Resource *resource ) const;
     ResourceRequest *resourceRequest( const QString &name ) const;
+    /// The ResourceRequestCollection has no requests
     bool isEmpty() const;
+    /// Empty the ResourceRequestCollection of all requets
     void clear() { m_requests.clear(); }
+    /// Reset dynamic resource allocations
+    void resetDynamicAllocations();
+
     bool contains( const QString &identity ) const;
     ResourceGroupRequest *findGroupRequestById( const QString &id ) const;
-    QStringList requestNameList() const;
+    /// Return a list of allocated resources, allocations to groups are not included by default.
+    QStringList requestNameList( bool includeGroup = false ) const;
     QList<Resource*> requestedResources() const;
+    /// Return a list of all resource requests
+    QList<ResourceRequest*> resourceRequests() const;
     
     //bool load(KoXmlElement &element, Project &project);
     void save( QDomElement &element ) const;
@@ -692,13 +752,18 @@ public:
     /**
     * Returns the total amount of resource units in percent
     */
-    int units() const;
+//    int units() const;
 
     /**
-    * Returns the amount of work units in percent
+    * Returns the amount of allocated work units in percent
     */
     int workUnits() const;
 
+    /**
+    * Returns the number of allocated working resources
+    */
+    int workAllocation() const;
+
     /**
     * Returns the duration needed to do the effort @param effort
     * starting at @param time.
@@ -707,9 +772,11 @@ public:
 
     DateTime availableAfter( const DateTime &time, Schedule *ns );
     DateTime availableBefore( const DateTime &time, Schedule *ns );
-    DateTime workTimeAfter(const DateTime &dt);
-    DateTime workTimeBefore(const DateTime &dt);
-    
+    DateTime workTimeAfter(const DateTime &dt, Schedule *ns = 0) const;
+    DateTime workTimeBefore(const DateTime &dt, Schedule *ns = 0) const;
+    DateTime workStartAfter(const DateTime &time, Schedule *ns);
+    DateTime workFinishBefore(const DateTime &time, Schedule *ns);
+
     /**
     * Makes appointments for the task @param task to the requested resources.
     * Assumes that @ref duration() has been run.
@@ -720,10 +787,15 @@ public:
      */
     void reserve( const DateTime &start, const Duration &duration );
 
-    Task &task() const { return m_task; }
+    Task *task() const { return m_task; }
+    void setTask( Task *t ) { m_task = t; }
 
     void changed();
     
+    Duration effort(const QList<ResourceRequest*> &lst, const DateTime &time, const Duration &duration, Schedule *ns, bool backward, bool *ok) const;
+    int numDays(const QList<ResourceRequest*> &lst, const DateTime &time, bool backward) const;
+    Duration duration(const QList<ResourceRequest*> &lst, const DateTime &time, const Duration &_effort, Schedule *ns, bool backward);
+
 protected:
     struct Interval
     {
@@ -734,7 +806,7 @@ protected:
 
 
 private:
-    Task &m_task;
+    Task *m_task;
     QList<ResourceGroupRequest*> m_requests;
 
 #ifndef NDEBUG
