@@ -168,6 +168,12 @@ bool Schedule::usePert() const
     return false;
 }
 
+void Schedule::setAllowOverbooking( bool state )
+{
+    if ( m_parent )
+        m_parent->setAllowOverbooking( state );
+}
+
 bool Schedule::allowOverbooking() const
 {
     if ( m_parent ) {
@@ -221,6 +227,26 @@ void Schedule::calcResourceOverbooked()
     }
 }
 
+DateTimeInterval Schedule::firstBookedInterval( const DateTimeInterval &interval, const Schedule *node  ) const
+{
+    QList<Appointment*> lst = m_appointments;
+    switch ( m_calculationMode ) {
+        case CalculateForward: lst = m_forward; break;
+        case CalculateBackward: lst = m_backward; break;
+        default: break;
+    }
+    foreach ( Appointment *a, lst ) {
+        if ( a->node() == node ) {
+            AppointmentIntervalList i = a->intervals( interval.first, interval.second );
+            if ( i.isEmpty() ) {
+                break;
+            }
+            return DateTimeInterval( i.values().first().startTime(), i.values().first().endTime() );
+        }
+    }
+    return DateTimeInterval();
+}
+
 QStringList Schedule::overbookedResources() const
 {
     QStringList rl;
@@ -338,6 +364,7 @@ bool Schedule::add( Appointment *appointment )
 
 void Schedule::takeAppointment( Appointment *appointment, int mode )
 {
+    Q_UNUSED(mode);
     //kDebug()<<"("<<this<<")"<<mode<<":"<<appointment<<","<<appointment->calculationMode();
     int i = m_forward.indexOf( appointment );
     if ( i != -1 ) {
@@ -385,7 +412,28 @@ Appointment *Schedule::findAppointment( Schedule *resource, Schedule *node, int
     return 0;
 }
 
-QList<Appointment*> &Schedule::appointments( int which)
+DateTime Schedule::appointmentStartTime() const
+{
+    DateTime dt;
+    foreach ( const Appointment *a, m_appointments ) {
+        if ( ! dt.isValid() || dt > a->startTime() ) {
+            dt = a->startTime();
+        }
+    }
+    return dt;
+}
+DateTime Schedule::appointmentEndTime() const
+{
+    DateTime dt;
+    foreach ( const Appointment *a, m_appointments ) {
+        if ( ! dt.isValid() || dt > a->endTime() ) {
+            dt = a->endTime();
+        }
+    }
+    return dt;
+}
+
+QList<Appointment*> Schedule::appointments( int which ) const
 {
     if ( which == CalculateForward ) {
         return m_forward;
@@ -441,6 +489,19 @@ EffortCostMap Schedule::plannedEffortCostPrDay( const QDate &start, const QDate
     return ec;
 }
 
+EffortCostMap Schedule::plannedEffortCostPrDay( const Resource *resource, const QDate &start, const QDate &end ) const
+{
+    //kDebug()<<m_name<<m_appointments;
+    EffortCostMap ec;
+    foreach ( Appointment *a, m_appointments ) {
+        if ( a->resource() && a->resource()->resource() == resource ) {
+            ec += a->plannedPrDay( start, end );
+            break;
+        }
+    }
+    return ec;
+}
+
 Duration Schedule::plannedEffort() const
 {
     //kDebug();
@@ -1038,6 +1099,12 @@ bool MainSchedule::usePert() const
     return m_manager == 0 ? false : m_manager->usePert();
 }
 
+void MainSchedule::setAllowOverbooking( bool state )
+{
+    if ( m_manager )
+        m_manager->setAllowOverbooking( state );
+}
+
 bool MainSchedule::allowOverbooking() const
 {
     return m_manager == 0 ? false : m_manager->allowOverbooking();
@@ -1243,12 +1310,7 @@ void MainSchedule::addCriticalPathNode( Node *node )
     m_currentCriticalPath->append( node );
 }
 
-QList<Schedule::Log> MainSchedule::logs()
-{
-    return m_log;
-}
-
-const QList<Schedule::Log> &MainSchedule::logs() const
+QList<Schedule::Log> MainSchedule::logs() const
 {
     return m_log;
 }
@@ -1295,6 +1357,9 @@ ScheduleManager::ScheduleManager( Project &project, const QString name )
     m_usePert( false ),
     m_recalculate( false ),
     m_schedulingDirection( false ),
+    m_scheduling( false ),
+    m_progress( 0 ),
+    m_maxprogress( 0 ),
     m_expected( 0 ),
     m_optimistic( 0 ),
     m_pessimistic( 0 )
@@ -1479,6 +1544,15 @@ void ScheduleManager::setSchedulingDirection( bool on )
     m_project.changed( this );
 }
 
+void ScheduleManager::setScheduling( bool on )
+{
+    m_scheduling = on;
+    if ( ! on ) {
+        m_project.setProgress( 0, this );
+    }
+    m_project.changed( this );
+}
+
 const QList<SchedulerPlugin*> ScheduleManager::schedulerPlugins() const
 {
     return m_project.schedulerPlugins().values();
@@ -1525,6 +1599,8 @@ int ScheduleManager::schedulerPluginIndex() const
 
 void ScheduleManager::setSchedulerPlugin( int index )
 {
+    schedulerPlugin()->stopCalculation( this ); // in case...
+
     m_schedulerPluginId = m_project.schedulerPlugins().keys().at( index );
     kDebug()<<index<<m_schedulerPluginId;
     m_project.changed( this );
@@ -1532,9 +1608,21 @@ void ScheduleManager::setSchedulerPlugin( int index )
 
 void ScheduleManager::calculateSchedule()
 {
+    m_calculationresult = CalculationRunning;
     schedulerPlugin()->calculate( m_project, this );
 }
 
+void ScheduleManager::stopCalculation()
+{
+    schedulerPlugin()->stopCalculation( this );
+}
+
+void ScheduleManager::setProgress( int progress )
+{
+    m_progress = progress;
+    m_project.changed( this );
+}
+
 void ScheduleManager::setDeleted( bool on )
 {
     if ( m_expected ) {
@@ -1621,6 +1709,9 @@ QStringList ScheduleManager::state() const
     if ( isBaselined() ) {
         return lst << i18n( "Baselined" );
     }
+    if ( m_scheduling ) {
+        return lst << i18n( "Scheduling" );
+    }
     if ( m_expected == 0 && m_optimistic == 0 && m_pessimistic == 0 ) {
         return lst << i18n( "Not scheduled" );
     }
@@ -1907,3 +1998,4 @@ void MainSchedule::printDebug( const QString& _indent )
 
 } //namespace KPlato
 
+#include "kptschedule.moc"
