@@ -19,6 +19,7 @@
 
 #include "kptresourcemodel.h"
 
+#include "kptcommonstrings.h"
 #include "kptcommand.h"
 #include "kptitemmodelbase.h"
 #include "kptcalendar.h"
@@ -112,9 +113,9 @@ QVariant ResourceModel::type( const Resource *res, int role ) const
         case Qt::EditRole:
         case Qt::ToolTipRole:
             return res->typeToString( true );
-        case Role::EnumList: 
+        case Role::EnumList:
             return res->typeToStringList( true );
-        case Role::EnumListValue: 
+        case Role::EnumListValue:
             return (int)res->type();
         case Qt::TextAlignmentRole:
             return Qt::AlignCenter;
@@ -132,9 +133,9 @@ QVariant ResourceModel::type( const ResourceGroup *res, int role ) const
         case Qt::EditRole:
         case Qt::ToolTipRole:
             return res->typeToString( true );
-        case Role::EnumList: 
+        case Role::EnumList:
             return res->typeToStringList( true );
-        case Role::EnumListValue: 
+        case Role::EnumListValue:
             return (int)res->type();
         case Qt::TextAlignmentRole:
             return Qt::AlignCenter;
@@ -306,8 +307,40 @@ QVariant ResourceModel::overtimeRate( const Resource *res, int role ) const
     return QVariant();
 }
 
+QVariant ResourceModel::account( const Resource *resource, int role ) const
+{
+    switch ( role ) {
+        case Qt::DisplayRole: {
+            Account *a = resource->account();
+            return a == 0 ? i18n( "None" ) : a->name();
+        }
+        case Qt::ToolTipRole: {
+            Account *a = resource->account();
+            return i18n( "Account: %1", (a == 0 ? i18n( "None" ) : a->name() ) );
+        }
+        case Role::EnumListValue:
+        case Qt::EditRole: {
+            Account *a = resource->account();
+            return a == 0 ? 0 : ( m_project->accounts().costElements().indexOf( a->name() ) + 1 );
+        }
+        case Role::EnumList: {
+            QStringList lst;
+            lst << i18n("None");
+            lst += m_project->accounts().costElements();
+            return lst;
+        }
+        case Qt::StatusTipRole:
+        case Qt::WhatsThisRole:
+            return QVariant();
+    }
+    return QVariant();
+}
+
 QVariant ResourceModel::data( const Resource *resource, int property, int role ) const
 {
+    if ( role == Role::ObjectType ) {
+        return OT_Resource;
+    }
     QVariant result;
     if ( resource == 0 ) {
         return result;
@@ -323,6 +356,7 @@ QVariant ResourceModel::data( const Resource *resource, int property, int role )
         case ResourceAvailableUntil: result = availableUntil( resource, role ); break;
         case ResourceNormalRate: result = normalRate( resource, role ); break;
         case ResourceOvertimeRate: result = overtimeRate( resource, role ); break;
+        case ResourceAccount: result = account( resource, role ); break;
         default:
             kDebug()<<"data: invalid display value: property="<<property;
             break;
@@ -332,6 +366,9 @@ QVariant ResourceModel::data( const Resource *resource, int property, int role )
 
 QVariant ResourceModel::data( const ResourceGroup *group, int property, int role ) const
 {
+    if ( role == Role::ObjectType ) {
+        return OT_ResourceGroup;
+    }
     QVariant result;
     if ( group == 0 ) {
         return result;
@@ -368,6 +405,7 @@ QVariant ResourceModel::headerData( int section, int role )
             case ResourceAvailableUntil: return i18n( "Available Until" );
             case ResourceNormalRate: return i18n( "Normal Rate" );
             case ResourceOvertimeRate: return i18n( "Overtime Rate" );
+            case ResourceAccount: return i18n( "Account" );
             default: return QVariant();
         }
     } else if ( role == Qt::TextAlignmentRole ) {
@@ -485,23 +523,23 @@ void ResourceItemModel::setProject( Project *project )
         disconnect( m_project, SIGNAL( localeChanged() ), this, SLOT( slotLayoutChanged() ) );
         disconnect( m_project, SIGNAL( resourceChanged( Resource* ) ), this, SLOT( slotResourceChanged( Resource* ) ) );
         disconnect( m_project, SIGNAL( resourceGroupChanged( ResourceGroup* ) ), this, SLOT( slotResourceGroupChanged( ResourceGroup* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceGroupToBeAdded( const ResourceGroup*, int ) ), this, SLOT( slotResourceGroupToBeInserted( const ResourceGroup*, int ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceGroupToBeRemoved( const ResourceGroup* ) ), this, SLOT( slotResourceGroupToBeRemoved( const ResourceGroup* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceToBeAdded( const ResourceGroup*, int ) ), this, SLOT( slotResourceToBeInserted( const ResourceGroup*, int ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceToBeRemoved( const Resource* ) ), this, SLOT( slotResourceToBeRemoved( const Resource* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceGroupAdded( const ResourceGroup* ) ), this, SLOT( slotResourceGroupInserted( const ResourceGroup* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceGroupRemoved( const ResourceGroup* ) ), this, SLOT( slotResourceGroupRemoved( const ResourceGroup* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceAdded( const Resource* ) ), this, SLOT( slotResourceInserted( const Resource* ) ) );
-        
+
         disconnect( m_project, SIGNAL( resourceRemoved( const Resource* ) ), this, SLOT( slotResourceRemoved( const Resource* ) ) );
-        
+
         disconnect( m_project, SIGNAL( defaultCalendarChanged( Calendar* ) ), this, SLOT( slotCalendarChanged( Calendar* ) ) );
     }
     m_project = project;
@@ -509,23 +547,23 @@ void ResourceItemModel::setProject( Project *project )
         connect( m_project, SIGNAL( localeChanged() ), this, SLOT( slotLayoutChanged() ) );
         connect( m_project, SIGNAL( resourceChanged( Resource* ) ), this, SLOT( slotResourceChanged( Resource* ) ) );
         connect( m_project, SIGNAL( resourceGroupChanged( ResourceGroup* ) ), this, SLOT( slotResourceGroupChanged( ResourceGroup* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceGroupToBeAdded( const ResourceGroup*, int ) ), this, SLOT( slotResourceGroupToBeInserted( const ResourceGroup*, int ) ) );
-        
+
         connect( m_project, SIGNAL( resourceGroupToBeRemoved( const ResourceGroup* ) ), this, SLOT( slotResourceGroupToBeRemoved( const ResourceGroup* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceToBeAdded( const ResourceGroup*, int ) ), this, SLOT( slotResourceToBeInserted( const ResourceGroup*, int ) ) );
-        
+
         connect( m_project, SIGNAL( resourceToBeRemoved( const Resource* ) ), this, SLOT( slotResourceToBeRemoved( const Resource* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceGroupAdded( const ResourceGroup* ) ), this, SLOT( slotResourceGroupInserted( const ResourceGroup* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceGroupRemoved( const ResourceGroup* ) ), this, SLOT( slotResourceGroupRemoved( const ResourceGroup* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceAdded( const Resource* ) ), this, SLOT( slotResourceInserted( const Resource* ) ) );
-        
+
         connect( m_project, SIGNAL( resourceRemoved( const Resource* ) ), this, SLOT( slotResourceRemoved( const Resource* ) ) );
-    
+
         connect( m_project, SIGNAL( defaultCalendarChanged( Calendar* ) ), this, SLOT( slotCalendarChanged( Calendar* ) ) );
     }
     m_model.setProject( m_project );
@@ -574,7 +612,7 @@ QModelIndex ResourceItemModel::parent( const QModelIndex &index ) const
         int row = m_project->indexOf(  r->parentGroup() );
         return createIndex( row, 0, r->parentGroup() );
     }
-    
+
     return QModelIndex();
 }
 
@@ -696,9 +734,9 @@ QVariant ResourceItemModel::type( const ResourceGroup *res, int role ) const
         case Qt::EditRole:
         case Qt::ToolTipRole:
             return res->typeToString( true );
-        case Role::EnumList: 
+        case Role::EnumList:
             return res->typeToStringList( true );
-        case Role::EnumListValue: 
+        case Role::EnumListValue:
             return (int)res->type();
         case Qt::TextAlignmentRole:
             return Qt::AlignCenter;
@@ -811,7 +849,7 @@ bool ResourceItemModel::setAvailableFrom( Resource *res, const QVariant &value,
     }
     return false;
 }
-    
+
 bool ResourceItemModel::setAvailableUntil( Resource *res, const QVariant &value, int role )
 {
     switch ( role ) {
@@ -851,6 +889,23 @@ bool ResourceItemModel::setOvertimeRate( Resource *res, const QVariant &value, i
     return false;
 }
 
+bool ResourceItemModel::setAccount( Resource *res, const QVariant &value, int role )
+{
+    switch ( role ) {
+        case Qt::EditRole:
+            QStringList lst = m_model.account( res, Role::EnumList ).toStringList();
+            if ( value.toInt() >= lst.count() ) {
+                return false;
+            }
+            Account *a = m_project->accounts().findAccount( lst.at( value.toInt() ) );
+            Account *old = res->account();
+            if ( old != a ) {
+                emit executeCommand( new ResourceModifyAccountCmd( *res, old, a, i18n( "Modify resource account" ) ) );
+                return true;
+            }
+    }
+    return false;
+}
 
 QVariant ResourceItemModel::notUsed( const ResourceGroup *, int role ) const
 {
@@ -916,11 +971,12 @@ bool ResourceItemModel::setData( const QModelIndex &index, const QVariant &value
             case ResourceModel::ResourceAvailableUntil: return setAvailableUntil( r, value, role );
             case ResourceModel::ResourceNormalRate: return setNormalRate( r, value, role );
             case ResourceModel::ResourceOvertimeRate: return setOvertimeRate( r, value, role );
+            case ResourceModel::ResourceAccount: return setAccount( r, value, role );
             default:
                 qWarning("data: invalid display value column %d", index.column());
                 return false;
         }
-    } else { 
+    } else {
         ResourceGroup *g = qobject_cast<ResourceGroup*>( obj );
         if ( g ) {
             switch (index.column()) {
@@ -960,6 +1016,7 @@ QAbstractItemDelegate *ResourceItemModel::createDelegate( int col, QWidget *pare
     switch ( col ) {
         case ResourceModel::ResourceType: return new EnumDelegate( parent );
         case ResourceModel::ResourceCalendar: return new EnumDelegate( parent );
+        case ResourceModel::ResourceAccount: return new EnumDelegate( parent );
         default: break;
     }
     return 0;
@@ -975,6 +1032,11 @@ QObject *ResourceItemModel::object( const QModelIndex &index ) const
     return o;
 }
 
+Resource *ResourceItemModel::resource( const QModelIndex &index ) const
+{
+    return qobject_cast<Resource*>( object( index ) );
+}
+
 void ResourceItemModel::slotCalendarChanged( Calendar* )
 {
     foreach ( Resource *r, m_project->resourceList() ) {
@@ -1010,6 +1072,7 @@ Qt::DropActions ResourceItemModel::supportedDropActions() const
 
 bool ResourceItemModel::dropAllowed( const QModelIndex &index, int dropIndicatorPosition, const QMimeData *data )
 {
+    Q_UNUSED(data);
     //kDebug()<<index;
     // TODO: if internal, don't allow dropping on my own parent
     // TODO: if internal, only MoveAction is allowed, but there is no indication...
@@ -1024,7 +1087,7 @@ bool ResourceItemModel::dropAllowed( const QModelIndex &index, int dropIndicator
 
 QStringList ResourceItemModel::mimeTypes() const
 {
-    return QStringList() 
+    return QStringList()
             << "text/x-vcard"
             << "text/directory"
             << "application/x-vnd.kde.kplato.resourceitemmodel.internal";
@@ -1085,7 +1148,7 @@ bool ResourceItemModel::dropMimeData( const QMimeData *data, Qt::DropAction acti
         KABC::VCardConverter vc;
         KABC::Addressee::List lst = vc.parseVCards( vcard );
         foreach( const KABC::Addressee &a, lst ) {
-            if ( m == 0 ) m = new MacroCommand( i18np( "Add resource from addressbook", "Add %1 resources from addressbook", lst.count() ) );
+            if ( m == 0 ) m = new MacroCommand( i18np( "Add resource from address book", "Add %1 resources from address book", lst.count() ) );
             Resource *r = new Resource();
             QString uid = a.uid();
             if ( ! m_project->findResource( uid ) ) {
@@ -1167,6 +1230,56 @@ QModelIndex ResourceItemModel::insertResource( ResourceGroup *g, Resource *r, Re
     return QModelIndex();
 }
 
+//-------------------
+ResourceItemSFModel::ResourceItemSFModel( QObject *parent )
+    : QSortFilterProxyModel( parent )
+{
+    setDynamicSortFilter( true );
+    setSourceModel( new ResourceItemModel( this ) );
+}
+
+void ResourceItemSFModel::setProject( Project *project )
+{
+    static_cast<ResourceItemModel*>( sourceModel() )->setProject( project );
+}
+
+Resource *ResourceItemSFModel::resource( const QModelIndex &idx ) const
+{
+    return static_cast<ResourceItemModel*>( sourceModel() )->resource( mapToSource( idx ) );
+}
+
+QModelIndex ResourceItemSFModel::index( Resource *r ) const
+{
+    return mapFromSource( static_cast<ResourceItemModel*>( sourceModel() )->index( r ) );
+}
+
+Qt::ItemFlags ResourceItemSFModel::flags( const QModelIndex & index ) const
+{
+    Qt::ItemFlags f = QSortFilterProxyModel::flags( index );
+    if ( index.isValid() && ! parent( index ).isValid() ) {
+        // group, not selectable
+        f &= ~Qt::ItemIsSelectable;
+    }
+    return f;
+}
+
+void ResourceItemSFModel::addFilteredResource( const Resource *r )
+{
+    if ( ! m_filteredResources.contains( r ) ) {
+        m_filteredResources << r;
+    }
+}
+
+bool ResourceItemSFModel::filterAcceptsRow( int source_row, const QModelIndex & source_parent ) const
+{
+    //TODO make this general filter
+    ResourceItemModel *m = static_cast<ResourceItemModel*>( sourceModel() );
+    if ( m->index( source_row, ResourceModel::ResourceType, source_parent ).data( Role::EnumListValue ).toInt() == ResourceGroup::Type_Work ) {
+        return false;
+    }
+    QModelIndex idx = m->index( source_row, 0, source_parent );
+    return ! m_filteredResources.contains( m->resource( idx ) );
+}
 
 } // namespace KPlato
 
