@@ -20,6 +20,7 @@
 #include "kptschedulemodel.h"
 
 #include "kptglobal.h"
+#include "kptcommonstrings.h"
 #include "kptcommand.h"
 #include "kptitemmodelbase.h"
 #include "kptduration.h"
@@ -28,6 +29,7 @@
 #include "kpttask.h"
 #include "kptschedule.h"
 #include "kptdatetime.h"
+#include "kptschedulerplugin.h"
 
 #include <QObject>
 #include <QStringList>
@@ -35,7 +37,7 @@
 
 #include <kglobal.h>
 #include <klocale.h>
-
+#include <KIcon>
 #include <kdebug.h>
 
 namespace KPlato
@@ -66,7 +68,8 @@ int ScheduleModel::propertyCount() const
 
 ScheduleItemModel::ScheduleItemModel( QObject *parent )
     : ItemModelBase( parent ),
-    m_manager( 0 )
+    m_manager( 0 ),
+    m_flat( false )
 {
 }
 
@@ -77,6 +80,9 @@ ScheduleItemModel::~ScheduleItemModel()
 void ScheduleItemModel::slotScheduleManagerToBeInserted( const ScheduleManager *parent, int row )
 {
     kDebug()<<parent;
+    if ( m_flat ) {
+        return; // handle in *Inserted();
+    }
     Q_ASSERT( m_manager == 0 );
     m_manager = const_cast<ScheduleManager*>(parent);
     beginInsertRows( index( parent ), row, row );
@@ -84,15 +90,31 @@ void ScheduleItemModel::slotScheduleManagerToBeInserted( const ScheduleManager *
 
 void ScheduleItemModel::slotScheduleManagerInserted( const ScheduleManager *manager )
 {
-    kDebug()<<manager->name();
+    if ( m_flat ) {
+        int row = m_project->allScheduleManagers().indexOf( const_cast<ScheduleManager*>( manager ) );
+        Q_ASSERT( row >= 0 );
+        beginInsertRows( QModelIndex(), row, row );
+        m_managerlist.insert( row, const_cast<ScheduleManager*>( manager ) );
+        endInsertRows();
+        emit scheduleManagerAdded( const_cast<ScheduleManager*>( manager ) );
+        return;
+    }
     Q_ASSERT( manager->parentManager() == m_manager );
     endInsertRows();
     m_manager = 0;
+    emit scheduleManagerAdded( const_cast<ScheduleManager*>( manager ) );
 }
 
 void ScheduleItemModel::slotScheduleManagerToBeRemoved( const ScheduleManager *manager )
 {
     kDebug()<<manager->name();
+    if ( m_flat ) {
+        int row = m_managerlist.indexOf( const_cast<ScheduleManager*>( manager ) );
+        beginRemoveRows( QModelIndex(), row, row );
+        m_managerlist.removeAt( row );
+        return;
+    }
+
     Q_ASSERT( m_manager == 0 );
     m_manager = const_cast<ScheduleManager*>(manager);
     QModelIndex i = index( manager );
@@ -103,6 +125,10 @@ void ScheduleItemModel::slotScheduleManagerToBeRemoved( const ScheduleManager *m
 void ScheduleItemModel::slotScheduleManagerRemoved( const ScheduleManager *manager )
 {
     kDebug()<<manager->name();
+    if ( m_flat ) {
+        endRemoveRows();
+        return;
+    }
     Q_ASSERT( manager == m_manager );
     endRemoveRows();
     m_manager = 0;
@@ -128,53 +154,60 @@ void ScheduleItemModel::setProject( Project *project )
 {
     if ( m_project ) {
         disconnect( m_project, SIGNAL( scheduleManagerChanged( ScheduleManager* ) ), this, SLOT( slotManagerChanged( ScheduleManager* ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleManagerToBeAdded( const ScheduleManager*, int ) ), this, SLOT( slotScheduleManagerToBeInserted( const ScheduleManager*, int) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleManagerToBeRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerToBeRemoved( const ScheduleManager* ) ) );
-    
+
         disconnect( m_project, SIGNAL( scheduleManagerAdded( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerInserted( const ScheduleManager* ) ) );
-    
+
         disconnect( m_project, SIGNAL( scheduleManagerRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerRemoved( const ScheduleManager* ) ) );
-    
+
         disconnect( m_project, SIGNAL( scheduleChanged( MainSchedule* ) ), this, SLOT( slotScheduleChanged( MainSchedule* ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleToBeAdded( const ScheduleManager*, int ) ), this, SLOT( slotScheduleToBeInserted( const ScheduleManager*, int ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleToBeRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleToBeRemoved( const MainSchedule* ) ) );
-    
+
         disconnect( m_project, SIGNAL( scheduleAdded( const MainSchedule* ) ), this, SLOT( slotScheduleInserted( const MainSchedule* ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleRemoved( const MainSchedule* ) ) );
     }
     m_project = project;
     if ( m_project ) {
         connect( m_project, SIGNAL( scheduleManagerChanged( ScheduleManager* ) ), this, SLOT( slotManagerChanged( ScheduleManager* ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleManagerToBeAdded( const ScheduleManager*, int ) ), this, SLOT( slotScheduleManagerToBeInserted( const ScheduleManager*, int) ) );
-        
+
         connect( m_project, SIGNAL( scheduleManagerToBeRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerToBeRemoved( const ScheduleManager* ) ) );
-    
+
         connect( m_project, SIGNAL( scheduleManagerAdded( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerInserted( const ScheduleManager* ) ) );
-    
+
         connect( m_project, SIGNAL( scheduleManagerRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerRemoved( const ScheduleManager* ) ) );
-    
+
         connect( m_project, SIGNAL( scheduleChanged( MainSchedule* ) ), this, SLOT( slotScheduleChanged( MainSchedule* ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleToBeAdded( const ScheduleManager*, int ) ), this, SLOT( slotScheduleToBeInserted( const ScheduleManager*, int ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleToBeRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleToBeRemoved( const MainSchedule* ) ) );
-    
+
         connect( m_project, SIGNAL( scheduleAdded( const MainSchedule* ) ), this, SLOT( slotScheduleInserted( const MainSchedule* ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleRemoved( const MainSchedule* ) ) );
     }
+    setFlat( m_flat ); // update m_managerlist
     reset();
 }
 
 void ScheduleItemModel::slotManagerChanged( ScheduleManager *sch )
 {
-    int r = m_project->indexOf( sch );
+    if ( m_flat ) {
+        int row = m_managerlist.indexOf( sch );
+        emit dataChanged( createIndex( row, 0, sch ), createIndex( row, columnCount(), sch ) );
+        return;
+    }
+
+    int r = sch->parentManager() ? sch->parentManager()->indexOf( sch ) : m_project->indexOf( sch );
     //kDebug()<<sch<<":"<<r;
     emit dataChanged( createIndex( r, 0, sch ), createIndex( r, columnCount(), sch ) );
 }
@@ -215,7 +248,7 @@ Qt::ItemFlags ScheduleItemModel::flags( const QModelIndex &index ) const
 
 QModelIndex ScheduleItemModel::parent( const QModelIndex &inx ) const
 {
-    if ( !inx.isValid() || m_project == 0 ) {
+    if ( !inx.isValid() || m_project == 0 || m_flat ) {
         return QModelIndex();
     }
     //kDebug()<<inx.internalPointer()<<":"<<inx.row()<<","<<inx.column();
@@ -233,6 +266,10 @@ QModelIndex ScheduleItemModel::index( int row, int column, const QModelIndex &pa
         //kDebug()<<row<<","<<column<<" out of bounce";
         return QModelIndex();
     }
+    if ( m_flat ) {
+        return createIndex( row, column, m_managerlist[ row ] );
+    }
+
     if ( parent.isValid() ) {
         return createIndex( row, column, manager( parent )->children().value( row ) );
     }
@@ -244,6 +281,10 @@ QModelIndex ScheduleItemModel::index( const ScheduleManager *manager ) const
     if ( m_project == 0 || manager == 0 ) {
         return QModelIndex();
     }
+    if ( m_flat ) {
+        return createIndex( m_managerlist.indexOf( const_cast<ScheduleManager*>( manager ) ), 0, const_cast<ScheduleManager*>(manager) );
+    }
+
     if ( manager->parentManager() == 0 ) {
         return createIndex( m_project->indexOf( manager ), 0, const_cast<ScheduleManager*>(manager) );
     }
@@ -260,6 +301,9 @@ int ScheduleItemModel::rowCount( const QModelIndex &parent ) const
     if ( m_project == 0 ) {
         return 0;
     }
+    if ( m_flat ) {
+        return m_managerlist.count();
+    }
     if ( !parent.isValid() ) {
         return m_project->numScheduleManagers();
     }
@@ -285,6 +329,13 @@ QVariant ScheduleItemModel::name( const QModelIndex &index, int role ) const
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
             return QVariant();
+        case Qt::DecorationRole:
+            if ( sm->isBaselined() ) {
+                return KIcon( "view-time-schedule-baselined" );
+            }
+            return QVariant();
+        default:
+            break;
     }
     return QVariant();
 }
@@ -297,7 +348,7 @@ bool ScheduleItemModel::setName( const QModelIndex &index, const QVariant &value
     }
     switch ( role ) {
         case Qt::EditRole:
-            emit executeCommand(new ModifyScheduleManagerNameCmd( *sm, value.toString(), "Modify Schedule Name" ) );
+            emit executeCommand( new ModifyScheduleManagerNameCmd( *sm, value.toString(), i18n( "Modify schedule name" ) ) );
             return true;
     }
     return false;
@@ -311,7 +362,17 @@ QVariant ScheduleItemModel::state( const QModelIndex &index, int role ) const
     }
     switch ( role ) {
         case Qt::DisplayRole:
-        case Qt::EditRole: 
+        {
+            if ( sm->progress() > 0 ) {
+                return sm->progress();
+            }
+            QStringList l = sm->state();
+            if ( l.isEmpty() ) {
+                return "";
+            }
+            return l.first();
+        }
+        case Qt::EditRole:
         {
             QStringList l = sm->state();
             if ( l.isEmpty() ) {
@@ -326,6 +387,10 @@ QVariant ScheduleItemModel::state( const QModelIndex &index, int role ) const
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
             return QVariant();
+        case Role::Maximum:
+            return sm->maxProgress();
+        case Role::Minimum:
+            return 0;
     }
     return QVariant();
 }
@@ -346,7 +411,7 @@ QVariant ScheduleItemModel::allowOverbooking( const QModelIndex &index, int role
         return QVariant();
     }
     switch ( role ) {
-        case Qt::EditRole: 
+        case Qt::EditRole:
             return sm->allowOverbooking();
         case Qt::DisplayRole:
         case Qt::ToolTipRole:
@@ -372,7 +437,7 @@ bool ScheduleItemModel::setAllowOverbooking( const QModelIndex &index, const QVa
     }
     switch ( role ) {
         case Qt::EditRole:
-            emit executeCommand(new ModifyScheduleManagerAllowOverbookingCmd( *sm, value.toBool(), "Modify Schedule Allow Overbooking" ) );
+            emit executeCommand( new ModifyScheduleManagerAllowOverbookingCmd( *sm, value.toBool(), i18n( "Modify allow overbooking" ) ) );
             return true;
     }
     return false;
@@ -386,7 +451,7 @@ QVariant ScheduleItemModel::usePert( const QModelIndex &index, int role ) const
         return QVariant();
     }
     switch ( role ) {
-        case Qt::EditRole: 
+        case Qt::EditRole:
             return sm->usePert();
         case Qt::DisplayRole:
         case Qt::ToolTipRole:
@@ -412,7 +477,7 @@ bool ScheduleItemModel::setUsePert( const QModelIndex &index, const QVariant &va
     }
     switch ( role ) {
         case Qt::EditRole:
-            emit executeCommand(new ModifyScheduleManagerDistributionCmd( *sm, value.toBool(), "Modify Schedule Distribution" ) );
+            emit executeCommand( new ModifyScheduleManagerDistributionCmd( *sm, value.toBool(), i18n( "Modify scheduling distribution" ) ) );
             emit slotManagerChanged( static_cast<ScheduleManager*>( sm ) );
             return true;
     }
@@ -426,7 +491,7 @@ QVariant ScheduleItemModel::calculateAll( const QModelIndex &index, int role ) c
         return QVariant();
     }
     switch ( role ) {
-        case Qt::EditRole: 
+        case Qt::EditRole:
             return sm->calculateAll();
         case Qt::DisplayRole:
         case Qt::ToolTipRole:
@@ -452,7 +517,8 @@ bool ScheduleItemModel::setCalculateAll( const QModelIndex &index, const QVarian
     }
     switch ( role ) {
         case Qt::EditRole:
-            emit executeCommand(new ModifyScheduleManagerCalculateAllCmd( *sm, value.toBool(), "Modify Schedule Calculate" ) );
+            //FIXME remove?
+            emit executeCommand( new ModifyScheduleManagerCalculateAllCmd( *sm, value.toBool(), "Modify schedule calculate" ) );
             return true;
     }
     return false;
@@ -465,7 +531,7 @@ QVariant ScheduleItemModel::projectStart( const QModelIndex &index, int role ) c
         return QVariant();
     }
     switch ( role ) {
-        case Qt::EditRole: 
+        case Qt::EditRole:
         case Qt::DisplayRole:
         case Qt::ToolTipRole:
             if ( sm->expected() ) {
@@ -488,7 +554,7 @@ QVariant ScheduleItemModel::projectEnd( const QModelIndex &index, int role ) con
         return QVariant();
     }
     switch ( role ) {
-        case Qt::EditRole: 
+        case Qt::EditRole:
         case Qt::DisplayRole:
         case Qt::ToolTipRole:
             if ( sm->expected() ) {
@@ -511,7 +577,7 @@ QVariant ScheduleItemModel::schedulingDirection( const QModelIndex &index, int r
         return QVariant();
     }
     switch ( role ) {
-        case Qt::EditRole: 
+        case Qt::EditRole:
             return sm->schedulingDirection();
         case Qt::DisplayRole:
         case Qt::ToolTipRole:
@@ -537,13 +603,75 @@ bool ScheduleItemModel::setSchedulingDirection( const QModelIndex &index, const
     }
     switch ( role ) {
         case Qt::EditRole:
-            emit executeCommand(new ModifyScheduleManagerSchedulingDirectionCmd( *sm, value.toBool(), "Modify Scheduling Direction" ) );
+            emit executeCommand(new ModifyScheduleManagerSchedulingDirectionCmd( *sm, value.toBool(), i18n( "Modify scheduling direction" ) ) );
             emit slotManagerChanged( static_cast<ScheduleManager*>( sm ) );
             return true;
     }
     return false;
 }
 
+QVariant ScheduleItemModel::scheduler( const QModelIndex &index, int role ) const
+{
+    ScheduleManager *sm = manager( index );
+    if ( sm == 0 ) {
+        return QVariant();
+    }
+    switch ( role ) {
+        case Qt::EditRole:
+            return sm->schedulerPluginId();
+        case Qt::DisplayRole:
+            return sm->schedulerPlugin()->name();
+        case Qt::ToolTipRole:
+            return sm->schedulerPlugin()->comment();
+        case Role::EnumList:
+            return sm->schedulerPluginNames();
+        case Role::EnumListValue:
+            return sm->schedulerPluginIndex();
+        case Qt::TextAlignmentRole:
+            return Qt::AlignCenter;
+        case Qt::StatusTipRole:
+        case Qt::WhatsThisRole:
+            return QVariant();
+    }
+    return QVariant();
+}
+
+bool ScheduleItemModel::setScheduler( const QModelIndex &index, const QVariant &value, int role )
+{
+    ScheduleManager *sm = manager( index );
+    if ( sm != 0 ) {
+        switch ( role ) {
+            case Qt::EditRole: {
+                emit executeCommand( new ModifyScheduleManagerSchedulerCmd( *sm, value.toInt(), i18n( "Modify scheduler" ) ) );
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
+QVariant ScheduleItemModel::isScheduled( const QModelIndex &index, int role ) const
+{
+    ScheduleManager *sm = manager( index );
+    if ( sm == 0 ) {
+        return QVariant();
+    }
+    switch ( role ) {
+        case Qt::EditRole:
+            return sm->isScheduled();
+        case Qt::DisplayRole:
+            return sm->isScheduled() ? i18n( "Scheduled" ) : i18n( "Not scheduled" );
+        case Qt::ToolTipRole:
+            break;
+        case Qt::TextAlignmentRole:
+            return Qt::AlignCenter;
+        case Qt::StatusTipRole:
+        case Qt::WhatsThisRole:
+            return QVariant();
+    }
+    return QVariant();
+}
+
 QVariant ScheduleItemModel::data( const QModelIndex &index, int role ) const
 {
     //kDebug()<<index.row()<<","<<index.column();
@@ -554,11 +682,13 @@ QVariant ScheduleItemModel::data( const QModelIndex &index, int role ) const
         case ScheduleModel::ScheduleDirection: result = schedulingDirection( index, role ); break;
         case ScheduleModel::ScheduleOverbooking: result = allowOverbooking( index, role ); break;
         case ScheduleModel::ScheduleDistribution: result = usePert( index, role ); break;
-        case ScheduleModel::ScheduleCalculate: result = calculateAll( index, role ); break;
+//        case ScheduleModel::ScheduleCalculate: result = calculateAll( index, role ); break;
         case ScheduleModel::SchedulePlannedStart: result = projectStart(  index, role ); break;
         case ScheduleModel::SchedulePlannedFinish: result = projectEnd( index, role ); break;
+        case ScheduleModel::ScheduleScheduler: result = scheduler( index, role ); break;
+        case ScheduleModel::ScheduleScheduled: result = isScheduled( index, role ); break;
         default:
-            kDebug()<<"data: invalid display value column"<<index.column();;
+            kDebug()<<"data: invalid display value column"<<index.column();
             return QVariant();
     }
     if ( result.isValid() ) {
@@ -585,9 +715,11 @@ bool ScheduleItemModel::setData( const QModelIndex &index, const QVariant &value
         case ScheduleModel::ScheduleDirection: return setSchedulingDirection( index, value, role );
         case ScheduleModel::ScheduleOverbooking: return setAllowOverbooking( index, value, role );
         case ScheduleModel::ScheduleDistribution: return setUsePert( index, value, role );
-        case ScheduleModel::ScheduleCalculate: return setCalculateAll( index, value, role );
+//        case ScheduleModel::ScheduleCalculate: return setCalculateAll( index, value, role );
         case ScheduleModel::SchedulePlannedStart: return false;
         case ScheduleModel::SchedulePlannedFinish: return false;
+        case ScheduleModel::ScheduleScheduler: return setScheduler( index, value, role ); break;
+        case ScheduleModel::ScheduleScheduled: break;
         default:
             qWarning("data: invalid display value column %d", index.column());
             break;
@@ -605,9 +737,11 @@ QVariant ScheduleItemModel::headerData( int section, Qt::Orientation orientation
                 case ScheduleModel::ScheduleDirection: return i18n( "Direction" );
                 case ScheduleModel::ScheduleOverbooking: return i18n( "Overbooking" );
                 case ScheduleModel::ScheduleDistribution: return i18n( "Distribution" );
-                case ScheduleModel::ScheduleCalculate: return i18n( "Calculate" );
+//                case ScheduleModel::ScheduleCalculate: return i18n( "Calculate" );
                 case ScheduleModel::SchedulePlannedStart: return i18n( "Planned Start" );
                 case ScheduleModel::SchedulePlannedFinish: return i18n( "Planned Finish" );
+                case ScheduleModel::ScheduleScheduler: return i18n( "Scheduler" );
+                case ScheduleModel::ScheduleScheduled: return i18n( "Scheduled" );
                 default: return QVariant();
             }
         } else if ( role == Qt::TextAlignmentRole ) {
@@ -623,28 +757,43 @@ QVariant ScheduleItemModel::headerData( int section, Qt::Orientation orientation
             case ScheduleModel::ScheduleDirection: return ToolTip::schedulingDirection();
             case ScheduleModel::ScheduleOverbooking: return ToolTip::scheduleOverbooking();
             case ScheduleModel::ScheduleDistribution: return ToolTip::scheduleDistribution();
-            case ScheduleModel::ScheduleCalculate: return ToolTip::scheduleCalculate();
+//            case ScheduleModel::ScheduleCalculate: return ToolTip::scheduleCalculate();
             case ScheduleModel::SchedulePlannedStart: return ToolTip::scheduleStart();
             case ScheduleModel::SchedulePlannedFinish: return ToolTip::scheduleFinish();
+            case ScheduleModel::ScheduleScheduler: return ToolTip::scheduleScheduler();
+            case ScheduleModel::ScheduleScheduled: return QVariant();
+            default: return QVariant();
+        }
+    } else if ( role == Qt::WhatsThisRole ) {
+        switch ( section ) {
+            case ScheduleModel::ScheduleDirection: return WhatsThis::schedulingDirection();
+            case ScheduleModel::ScheduleOverbooking: return WhatsThis::scheduleOverbooking();
+            case ScheduleModel::ScheduleDistribution: return WhatsThis::scheduleDistribution();
+            case ScheduleModel::ScheduleScheduler: return WhatsThis::scheduleScheduler();
             default: return QVariant();
         }
     }
+
     return ItemModelBase::headerData(section, orientation, role);
 }
 
-QItemDelegate *ScheduleItemModel::createDelegate( int column, QWidget *parent ) const
+QAbstractItemDelegate *ScheduleItemModel::createDelegate( int column, QWidget *parent ) const
 {
     switch ( column ) {
+        case ScheduleModel::ScheduleState: return new ProgressBarDelegate( parent );
         case ScheduleModel::ScheduleDirection: return new EnumDelegate( parent );
         case ScheduleModel::ScheduleOverbooking: return new EnumDelegate( parent );
         case ScheduleModel::ScheduleDistribution: return new EnumDelegate( parent );
-        case ScheduleModel::ScheduleCalculate: return new EnumDelegate( parent );
+//        case ScheduleModel::ScheduleCalculate: return new EnumDelegate( parent );
+        case ScheduleModel::ScheduleScheduler: return new EnumDelegate( parent );
     }
     return 0;
 }
 
 void ScheduleItemModel::sort( int column, Qt::SortOrder order )
 {
+    Q_UNUSED(column);
+    Q_UNUSED(order);
 }
 
 QMimeData * ScheduleItemModel::mimeData( const QModelIndexList & ) const
@@ -667,8 +816,34 @@ ScheduleManager *ScheduleItemModel::manager( const QModelIndex &index ) const
     return o;
 }
 
-//--------------------------------------
+void ScheduleItemModel::setFlat( bool flat )
+{
+    m_flat = flat;
+    m_managerlist.clear();
+    if ( ! flat || m_project == 0 ) {
+        return;
+    }
+    m_managerlist = m_project->allScheduleManagers();
+}
 
+//--------------------------------------
+ScheduleSortFilterModel::ScheduleSortFilterModel( QObject *parent )
+    : QSortFilterProxyModel( parent )
+{
+}
+
+ScheduleSortFilterModel::~ScheduleSortFilterModel()
+{
+}
+
+ScheduleManager *ScheduleSortFilterModel::manager( const QModelIndex &index ) const
+{
+    QModelIndex i = mapToSource( index );
+    const ScheduleItemModel *m = qobject_cast<const ScheduleItemModel*>( i.model() );
+    return m == 0 ? 0 : m->manager( i );
+}
+
+//--------------------------------------
 ScheduleLogItemModel::ScheduleLogItemModel( QObject *parent )
     : QStandardItemModel( parent ),
     m_project( 0 ),
@@ -695,6 +870,8 @@ void ScheduleLogItemModel::slotScheduleManagerRemoved( const ScheduleManager *ma
 
 void ScheduleLogItemModel::slotScheduleToBeInserted( const ScheduleManager *manager, int row )
 {
+    Q_UNUSED(manager);
+    Q_UNUSED(row);
     if ( m_manager && m_manager->expected() /*== ??*/ ) {
         //TODO
     }
@@ -728,37 +905,37 @@ void ScheduleLogItemModel::setProject( Project *project )
     kDebug()<<m_project<<"->"<<project;
     if ( m_project ) {
         disconnect( m_project, SIGNAL( scheduleManagerChanged( ScheduleManager* ) ), this, SLOT( slotManagerChanged( ScheduleManager* ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleManagerToBeRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerToBeRemoved( const ScheduleManager* ) ) );
-    
+
         disconnect( m_project, SIGNAL( scheduleManagerRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerRemoved( const ScheduleManager* ) ) );
-    
+
         disconnect( m_project, SIGNAL( scheduleChanged( MainSchedule* ) ), this, SLOT( slotScheduleChanged( MainSchedule* ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleToBeAdded( const ScheduleManager*, int ) ), this, SLOT( slotScheduleToBeInserted( const ScheduleManager*, int ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleToBeRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleToBeRemoved( const MainSchedule* ) ) );
-    
+
         disconnect( m_project, SIGNAL( scheduleAdded( const MainSchedule* ) ), this, SLOT( slotScheduleInserted( const MainSchedule* ) ) );
-        
+
         disconnect( m_project, SIGNAL( scheduleRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleRemoved( const MainSchedule* ) ) );
     }
     m_project = project;
     if ( m_project ) {
         connect( m_project, SIGNAL( scheduleManagerChanged( ScheduleManager* ) ), this, SLOT( slotManagerChanged( ScheduleManager* ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleManagerToBeRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerToBeRemoved( const ScheduleManager* ) ) );
-    
+
         connect( m_project, SIGNAL( scheduleManagerRemoved( const ScheduleManager* ) ), this, SLOT( slotScheduleManagerRemoved( const ScheduleManager* ) ) );
-    
+
         connect( m_project, SIGNAL( scheduleChanged( MainSchedule* ) ), this, SLOT( slotScheduleChanged( MainSchedule* ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleToBeAdded( const ScheduleManager*, int ) ), this, SLOT( slotScheduleToBeInserted( const ScheduleManager*, int ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleToBeRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleToBeRemoved( const MainSchedule* ) ) );
-    
+
         connect( m_project, SIGNAL( scheduleAdded( const MainSchedule* ) ), this, SLOT( slotScheduleInserted( const MainSchedule* ) ) );
-        
+
         connect( m_project, SIGNAL( scheduleRemoved( const MainSchedule* ) ), this, SLOT( slotScheduleRemoved( const MainSchedule* ) ) );
     }
 }
@@ -783,26 +960,29 @@ void ScheduleLogItemModel::refresh()
     QStringList lst;
     lst << i18n( "Name" ) << i18n( "Phase" ) << i18n( "Severity" ) << i18n( "Message" );
     setHorizontalHeaderLabels( lst );
-    
+
     if ( m_schedule == 0 ) {
         kDebug()<<"No main schedule";
         return;
     }
     kDebug()<<m_schedule<<m_schedule->logs().count();
     QStandardItem *parentItem = invisibleRootItem();
-    QList<Schedule::Log>::ConstIterator it;
-    for ( it = m_schedule->logs().constBegin(); it != m_schedule->logs().constEnd(); ++it ) {
+    QListIterator<Schedule::Log> it( m_schedule->logs() );
+    while ( it.hasNext() ) {
+        Schedule::Log log = it.next();
         QList<QStandardItem*> lst;
-        if ( (*it).resource ) {
-            lst.append( new QStandardItem( (*it).resource->name() ) );
+        if ( log.resource ) {
+            lst.append( new QStandardItem( log.resource->name() ) );
+        } else if ( log.node ) {
+            lst.append( new QStandardItem( log.node->name() ) );
         } else {
-            lst.append( new QStandardItem( (*it).node->name() ) );
+            lst.append(  new QStandardItem( "" ) );
         }
-        lst.append( new QStandardItem( m_schedule->logPhase( (*it).phase ) ) );
-        QStandardItem *item = new QStandardItem( m_schedule->logSeverity( (*it).severity ) );
-        item->setData( (*it).severity );
+        lst.append( new QStandardItem( m_schedule->logPhase( log.phase ) ) );
+        QStandardItem *item = new QStandardItem( m_schedule->logSeverity( log.severity ) );
+        item->setData( log.severity );
         lst.append( item );
-        lst.append( new QStandardItem( (*it).message ) );
+        lst.append( new QStandardItem( log.message ) );
         parentItem->appendRow( lst );
         //kDebug()<<rowCount()<<columnCount()<<parentItem<<lst;
     }
@@ -829,6 +1009,7 @@ void ScheduleLogItemModel::slotScheduleChanged( MainSchedule *sch )
 
 Qt::ItemFlags ScheduleLogItemModel::flags( const QModelIndex &index ) const
 {
+    Q_UNUSED(index);
     return Qt::ItemIsSelectable | Qt::ItemIsEnabled;
 }
 
