@@ -20,6 +20,7 @@
 #include "kptdependencyeditor.h"
 
 #include "kptglobal.h"
+#include "kptcommonstrings.h"
 #include "kptitemmodelbase.h"
 #include "kptcommand.h"
 #include "kptproject.h"
@@ -65,6 +66,14 @@ int DependecyViewPrintingDialog::documentLastPage() const
     return documentFirstPage();
 }
 
+QList<QWidget*> DependecyViewPrintingDialog::createOptionWidgets() const
+{
+    QList<QWidget*> lst;
+    lst << createPageLayoutWidget();
+    lst += PrintingDialog::createOptionWidgets();
+    return  lst;
+}
+
 void DependecyViewPrintingDialog::printPage( int page, QPainter &painter )
 {
     painter.save();
@@ -72,7 +81,7 @@ void DependecyViewPrintingDialog::printPage( int page, QPainter &painter )
     QRect hRect = headerRect();
     QRect fRect = footerRect();
     QRect pageRect = printer().pageRect();
-
+    pageRect.moveTo( 0, 0 );
     kDebug()<<pageRect<<hRect<<fRect;
 
     painter.translate( pageRect.topLeft() );
@@ -449,19 +458,16 @@ QPointF DependencyConnectorItem::connectorPoint() const
 
 void DependencyConnectorItem::hoverEnterEvent( QGraphicsSceneHoverEvent * /*event*/ )
 {
-    //qDebug()<<"DependencyConnectorItem::hoverEnterEvent:"<<node()->name();
     itemScene()->connectorEntered( this, true );
 }
 
 void DependencyConnectorItem::hoverLeaveEvent( QGraphicsSceneHoverEvent * /*event*/ )
 {
-    //qDebug()<<"DependencyConnectorItem::hoverLeaveEvent:"<<node()->name();
     itemScene()->connectorEntered( this, false );
 }
 
 void DependencyConnectorItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
 {
-    //qDebug()<<"DependencyConnectorItem::mousePressEvent:"<<node()->name()<<isEditable();
     if ( ! isEditable() ) {
         event->ignore();
         return;
@@ -475,7 +481,6 @@ void DependencyConnectorItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
 
 void DependencyConnectorItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 {
-    //qDebug()<<"DependencyConnectorItem::mouseReleaseEvent:"<<node()->name()<<m_mousePressPos;
     m_mousePressPos = QPointF();
     if (event->button() != Qt::LeftButton ) { 
         event->ignore();
@@ -483,7 +488,6 @@ void DependencyConnectorItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
     }
     if ( rect().contains( event->scenePos() ) ) {
         // user clicked on this item
-        //qDebug()<<"DependencyConnectorItem::mouseReleaseEvent:"<<node()->name()<<"click";
         bool multiSelect = (event->modifiers() & Qt::ControlModifier) != 0;
         if (multiSelect) {
             itemScene()->multiConnectorClicked( this );
@@ -509,7 +513,6 @@ void DependencyConnectorItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 void DependencyConnectorItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 {
     if (event->buttons() == Qt::LeftButton ) {
-        //qDebug()<<"DependencyConnectorItem::mouseMoveEvent:"<<node()->name()<<event->pos();
         if ( ! m_mousePressPos.isNull() ) {
             itemScene()->setFromItem( this );
             m_mousePressPos = QPointF();
@@ -1172,9 +1175,9 @@ DependencyNodeItem *DependencyScene::createItem( Node *node )
     item->startConnector()->setEditable( m_readwrite );
     item->finishConnector()->setEditable( m_readwrite );
     //kDebug()<<item->text()<<item;
-    int col = item->nodeLevel();
+    int col = 0;
     if ( parent ) {
-        col += parent->column();
+        col += parent->column() + 1;
     }
     item->setRectangle( QRectF( itemX( col ), itemY(), itemWidth(), itemHeight() ) );
     m_allItems.insert( i+1, item );
@@ -1324,7 +1327,7 @@ void DependencyScene::keyPressEvent( QKeyEvent *keyEvent )
                 return;
             }
             if ( fitem->type() == DependencyConnectorItem::Type ) {
-                DependencyConnectorItem *citem = static_cast<DependencyConnectorItem*>( fitem );;
+                DependencyConnectorItem *citem = static_cast<DependencyConnectorItem*>( fitem );
                 if ( citem->ctype() == DependencyNodeItem::Start ) {
                     //Goto prev nodes finishConnector
                     DependencyNodeItem *nitem = static_cast<DependencyNodeItem*>( citem->parentItem() );
@@ -1359,7 +1362,7 @@ void DependencyScene::keyPressEvent( QKeyEvent *keyEvent )
                 return;
             }
             if ( fitem->type() == DependencyConnectorItem::Type ) {
-                DependencyConnectorItem *citem = static_cast<DependencyConnectorItem*>( fitem );;
+                DependencyConnectorItem *citem = static_cast<DependencyConnectorItem*>( fitem );
                 if ( citem->ctype() == DependencyNodeItem::Finish ) {
                     //Goto prev nodes startConnector
                     DependencyNodeItem *nitem = static_cast<DependencyNodeItem*>( citem->parentItem() );
@@ -1821,7 +1824,6 @@ void DependencyView::keyPressEvent(QKeyEvent *event)
 
 void DependencyView::mouseMoveEvent( QMouseEvent *mouseEvent )
 {
-    //qDebug()<<"DependencyView::mouseMoveEvent:"<<viewport()->cursor().shape();
     m_cursorPos = mouseEvent->pos();
     if ( itemScene()->connectionMode() && itemScene()->mouseGrabberItem() ) {
         QPointF spos = mapToScene( m_cursorPos );
@@ -1877,7 +1879,6 @@ DependencyEditor::DependencyEditor( KoDocument *part, QWidget *parent )
 
 void DependencyEditor::updateReadWrite( bool on )
 {
-    qDebug()<<"DependencyEditor::updateReadWrite:"<<on;
     m_view->itemScene()->setReadWrite( on );
     ViewBase::updateReadWrite( on );
 }
@@ -2087,6 +2088,7 @@ void DependencyEditor::updateActionsEnabled( bool on )
     Project *p = m_view->project();
     
     bool o = ( on && p && selectedNodeCount() <= 1 );
+    menuAddTask->setEnabled( o );
     actionAddTask->setEnabled( o );
     actionAddMilestone->setEnabled( o );
     
@@ -2094,6 +2096,7 @@ void DependencyEditor::updateActionsEnabled( bool on )
     
     o = ( on && p && selectedNodeCount() == 1 );
     
+    menuAddSubTask->setEnabled( o );
     actionAddSubtask->setEnabled( o );
 }
 
@@ -2102,24 +2105,38 @@ void DependencyEditor::setupGui()
     KActionCollection *coll = actionCollection();
     
     QString name = "taskeditor_add_list";
-    actionAddTask  = new KAction(KIcon( "view-task-add" ), i18n("Add Task..."), this);
+
+    menuAddTask = new KActionMenu( KIcon( "view-task-add" ), "Add Task...", this );
+    coll->addAction("add_task", menuAddTask );
+    connect( menuAddTask, SIGNAL( triggered( bool ) ), SLOT( slotAddTask() ) );
+    addAction( name, menuAddTask );
+
+    actionAddTask  = new KAction( i18n("Add Task..."), this);
     actionAddTask->setShortcut( KShortcut( Qt::CTRL + Qt::Key_I ) );
-    coll->addAction("add_task", actionAddTask );
     connect( actionAddTask, SIGNAL( triggered( bool ) ), SLOT( slotAddTask() ) );
-    addAction( name, actionAddTask );
-    
-    actionAddSubtask  = new KAction(KIcon( "view-task-child-add" ), i18n("Add Sub-Task..."), this);
-    actionAddSubtask->setShortcut( KShortcut( Qt::CTRL + Qt::SHIFT + Qt::Key_I ) );
-    coll->addAction("add_sub_task", actionAddSubtask );
-    connect( actionAddSubtask, SIGNAL( triggered( bool ) ), SLOT( slotAddSubtask() ) );
-    addAction( name, actionAddSubtask );
-    
-    actionAddMilestone  = new KAction(KIcon( "view-milestone-add" ), i18n("Add Milestone..."), this);
+    menuAddTask->addAction( actionAddTask );
+
+    actionAddMilestone  = new KAction( i18n("Add Milestone..."), this );
     actionAddMilestone->setShortcut( KShortcut( Qt::CTRL + Qt::ALT + Qt::Key_I ) );
-    coll->addAction("add_milestone", actionAddMilestone );
     connect( actionAddMilestone, SIGNAL( triggered( bool ) ), SLOT( slotAddMilestone() ) );
-    addAction( name, actionAddMilestone );
+    menuAddTask->addAction( actionAddMilestone );
     
+
+    menuAddSubTask = new KActionMenu( KIcon( "view-task-child-add" ), "Add Sub-Task...", this );
+    coll->addAction("add_subtask", menuAddTask );
+    connect( menuAddSubTask, SIGNAL( triggered( bool ) ), SLOT( slotAddSubtask() ) );
+    addAction( name, menuAddSubTask );
+
+    actionAddSubtask  = new KAction( i18n("Add Sub-Task..."), this );
+    actionAddSubtask->setShortcut( KShortcut( Qt::CTRL + Qt::SHIFT + Qt::Key_I ) );
+    connect( actionAddSubtask, SIGNAL( triggered( bool ) ), SLOT( slotAddSubtask() ) );
+    menuAddSubTask->addAction( actionAddSubtask );
+
+    actionAddSubMilestone = new KAction( i18n("Add Sub-Milestone..."), this );
+    actionAddSubMilestone->setShortcut( KShortcut( Qt::CTRL + Qt::SHIFT + Qt::ALT + Qt::Key_I ) );
+    connect( actionAddSubMilestone, SIGNAL( triggered( bool ) ), SLOT( slotAddSubMilestone() ) );
+    menuAddSubTask->addAction( actionAddSubMilestone );
+
     actionDeleteTask  = new KAction(KIcon( "edit-delete" ), i18n("Delete Task"), this);
     actionDeleteTask->setShortcut( KShortcut( Qt::Key_Delete ) );
     coll->addAction("delete_task", actionDeleteTask );
@@ -2154,6 +2171,17 @@ void DependencyEditor::slotAddSubtask()
     m_currentnode = 0;
 }
 
+void DependencyEditor::slotAddSubMilestone()
+{
+    kDebug();
+    m_currentnode = selectedNode();
+    if ( m_currentnode == 0 ) {
+        return;
+    }
+    emit addSubMilestone();
+    m_currentnode = 0;
+}
+
 void DependencyEditor::edit( QModelIndex i )
 {
     if ( i.isValid() ) {
