@@ -26,13 +26,13 @@
 #include "kpttask.h"
 #include "kptresource.h"
 #include "kptdatetime.h"
-#include "kpttaskappointmentsview.h"
 #include "kptrelation.h"
 #include "kptschedule.h"
 #include "kptviewbase.h"
 #include "kptitemviewsettup.h"
 #include "kptduration.h"
 #include "kptdatetime.h"
+#include "kptresourceappointmentsmodel.h"
 
 #include <kdganttproxymodel.h>
 #include <kdganttconstraintmodel.h>
@@ -42,6 +42,7 @@
 #include <kdgantttreeviewrowcontroller.h>
 
 #include <KoDocument.h>
+#include <KoPageLayoutWidget.h>
 
 #include <kdebug.h>
 
@@ -57,6 +58,7 @@
 #include <kglobal.h>
 #include <kmessagebox.h>
 #include <kaction.h>
+#include <KToggleAction>
 
 #include "kdganttglobal.h"
 #include "kdganttstyleoptionganttitem.h"
@@ -140,22 +142,21 @@ GanttPrintingDialog::GanttPrintingDialog( ViewBase *view, KDGantt::View *gantt )
     m_singlePage( true ),
     m_printRowLabels( true )
 {
-    m_pageRect = printer().pageRect();
     m_headerHeight = gantt->graphicsView()->headerHeight();
     m_sceneRect = m_gantt->graphicsView()->printRect();
     m_horPages = 1;
-    qreal c = m_sceneRect.width() - m_pageRect.width();
+    qreal c = m_sceneRect.width() - printer().pageRect().width();
     while ( c > 0 ) {
         ++m_horPages;
-        c -= m_pageRect.width();
+        c -= printer().pageRect().width();
     }
     m_vertPages = 1;
-    c = m_sceneRect.height() - m_pageRect.height() - m_headerHeight;
+    c = m_sceneRect.height() - printer().pageRect().height() - m_headerHeight;
     while ( c > 0 ) {
         ++m_vertPages;
-        c -= m_pageRect.height();
+        c -= printer().pageRect().height();
     }
-    kDebug()<<m_sceneRect<<m_pageRect<<m_horPages<<m_vertPages;
+    kDebug()<<m_sceneRect<<printer().pageRect()<<m_horPages<<m_vertPages;
     printer().setFromTo( documentFirstPage(), documentLastPage() );
 }
 
@@ -165,7 +166,7 @@ void GanttPrintingDialog::startPrinting(RemovePolicy removePolicy )
         m_singlePage = m_options->singlePage();
         //m_printRowLabels = m_options->printRowLabels();
     }
-    KoPrintingDialog::startPrinting( removePolicy );
+    PrintingDialog::startPrinting( removePolicy );
 }
 
 QList<QWidget*> GanttPrintingDialog::createOptionWidgets() const
@@ -177,7 +178,7 @@ QList<QWidget*> GanttPrintingDialog::createOptionWidgets() const
     
     const_cast<GanttPrintingDialog*>( this )->m_options = w;
     
-    return QList<QWidget*>() << m_options;
+    return QList<QWidget*>() << createPageLayoutWidget() << m_options;
 }
 
 int GanttPrintingDialog::documentLastPage() const
@@ -191,7 +192,8 @@ void GanttPrintingDialog::printPage( int page, QPainter &painter )
 {
     QRectF sourceRect = m_sceneRect;
     int p = page - documentFirstPage();
-    QRectF pageRect = m_pageRect;
+    QRectF pageRect = printer().pageRect();
+    pageRect.moveTo( 0, 0 );
     int vert = m_singlePage ? 0 : p / m_horPages;
     int hor = m_singlePage ? 0 : p % m_horPages;
     if ( ! m_singlePage && documentLastPage() > documentFirstPage() ) {
@@ -204,7 +206,6 @@ void GanttPrintingDialog::printPage( int page, QPainter &painter )
     m_gantt->print( &painter, pageRect, sourceRect, m_printRowLabels, vert == 0 );
 }
 
-
 //---------------------
 class HeaderView : public QHeaderView
 {
@@ -238,6 +239,8 @@ GanttViewBase::GanttViewBase( QWidget *parent )
     graphicsView()->setItemDelegate( m_ganttdelegate );
     GanttTreeView *tv = new GanttTreeView( this );
     tv->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+    tv->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    tv->setVerticalScrollMode( QAbstractItemView::ScrollPerPixel ); // needed since qt 4.2
     setLeftView( tv );
     setRowController( new KDGantt::TreeViewRowController( tv, ganttProxyModel() ) );
     tv->header()->setStretchLastSection( true );
@@ -276,6 +279,7 @@ void GanttViewBase::setProject( Project *project )
 bool GanttViewBase::loadContext( const KoXmlElement &settings )
 {
     treeView()->loadContext( model()->columnMap(), settings );
+
     KoXmlElement e = settings.namedItem( "ganttchart" ).toElement();
     if ( ! e.isNull() ) {
         m_ganttdelegate->showTaskLinks = (bool)( e.attribute( "show-dependencies", "0" ).toInt() );
@@ -326,7 +330,7 @@ MyKDGanttView::MyKDGanttView( QWidget *parent )
         }
     }
 
-    setConstraintModel( new KDGantt::ConstraintModel() );
+    setConstraintModel( new KDGantt::ConstraintModel( this ) );
     KDGantt::ProxyModel *m = static_cast<KDGantt::ProxyModel*>( ganttProxyModel() );
 
     m->setRole( KDGantt::ItemTypeRole, KDGantt::ItemTypeRole ); // To provide correct format
@@ -339,7 +343,7 @@ MyKDGanttView::MyKDGanttView( QWidget *parent )
     m->setColumn( KDGantt::TaskCompletionRole, NodeModel::NodeCompleted );
     
     static_cast<KDGantt::DateTimeGrid*>( grid() )->setDayWidth( 30 );
-    static_cast<KDGantt::DateTimeGrid*>( grid() )->setRowSeparators( treeView()->alternatingRowColors() );
+    //static_cast<KDGantt::DateTimeGrid*>( grid() )->setRowSeparators( treeView()->alternatingRowColors() );
     
     connect( model(), SIGNAL( nodeInserted( Node* ) ), this, SLOT( slotNodeInserted( Node* ) ) );
 }
@@ -378,7 +382,7 @@ void MyKDGanttView::setScheduleManager( ScheduleManager *sm )
 {
     //kDebug()<<id<<endl;
     clearDependencies();
-    model()->setManager( sm );
+    model()->setScheduleManager( sm );
     m_manager = sm;
     if ( sm && project() ) {
         QDateTime start = project()->startTime( sm->id() ).dateTime().addDays( -1 );
@@ -419,7 +423,6 @@ void MyKDGanttView::removeDependency( Relation *rel )
 {
     QModelIndex par = sfModel()->mapFromSource( model()->index( rel->parent() ) );
     QModelIndex ch = sfModel()->mapFromSource( model()->index( rel->child() ) );
-    qDebug()<<"removeDependency() "<<model()<<par.model();
     KDGantt::Constraint con( par, ch, KDGantt::Constraint::TypeSoft, 
                              static_cast<KDGantt::Constraint::RelationType>( rel->type() )/*NOTE!!*/
                            );
@@ -448,7 +451,6 @@ void MyKDGanttView::createDependencies()
 GanttView::GanttView( KoDocument *part, QWidget *parent, bool readWrite )
     : ViewBase( part, parent ),
     m_readWrite( readWrite ),
-    m_taskView( 0 ),
     m_project( 0 )
 {
     kDebug() <<" ---------------- KPlato: Creating GanttView ----------------";
@@ -463,9 +465,6 @@ GanttView::GanttView( KoDocument *part, QWidget *parent, bool readWrite )
 
     setupGui();
     
-    m_taskView = new TaskAppointmentsView( m_splitter );
-    m_taskView->hide();
-
     updateReadWrite( readWrite );
     //connect( m_gantt->constraintModel(), SIGNAL( constraintAdded( const Constraint& )), this, SLOT( update() ) );
     kDebug() <<m_gantt->constraintModel();
@@ -484,27 +483,40 @@ void GanttView::setZoom( double )
 {
     //kDebug() <<"setting gantt zoom:" << zoom;
     //m_gantt->setZoomFactor(zoom,true); NO!!! setZoomFactor() is something else
-    //m_taskView->setZoom( zoom );
 }
 
 void GanttView::setupGui()
 {
+    // create context menu actions
+    actionShowProject = new KToggleAction( i18n( "Show Project" ), this );
+    connect(actionShowProject, SIGNAL(triggered(bool) ), m_gantt->model(), SLOT(setShowProject(bool)));
+    addContextAction( actionShowProject );
+
     createOptionAction();
 }
 
 void GanttView::slotOptions()
 {
     kDebug();
-    GanttViewSettingsDialog dlg( m_gantt->treeView(), m_gantt->delegate() );
-    if ( dlg.exec() == QDialog::Accepted ) {
+    GanttViewSettingsDialog *dlg = new GanttViewSettingsDialog( m_gantt->treeView(), m_gantt->delegate(), this );
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotOptionsFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
+}
+
+void GanttView::slotOptionsFinished( int result )
+{
+    GanttViewSettingsDialog *dlg = qobject_cast<GanttViewSettingsDialog*>( sender() );
+    if ( dlg && result == QDialog::Accepted ) {
         m_gantt->graphicsView()->updateScene();
     }
+    ViewBase::slotOptionsFinished( result );
 }
 
 void GanttView::clear()
 {
 //    m_gantt->clear();
-    m_taskView->clear();
 }
 
 void GanttView::setShowResources( bool on )
@@ -595,6 +607,7 @@ void GanttView::slotContextMenuRequested( QModelIndex idx, const QPoint &pos )
                 name = "taskview_milestone_popup";
                 break;
             case Node::Type_Summarytask:
+                name = "taskview_summary_popup";
                 break;
             default:
                 break;
@@ -611,13 +624,20 @@ void GanttView::slotContextMenuRequested( QModelIndex idx, const QPoint &pos )
 bool GanttView::loadContext( const KoXmlElement &settings )
 {
     kDebug();
+    bool show = (bool)(settings.attribute( "show-project", "0" ).toInt() );
+    actionShowProject->setChecked( show );
+    m_gantt->model()->setShowProject( show ); // why is this not called by the action?
+
     return m_gantt->loadContext( settings );
 }
 
 void GanttView::saveContext( QDomElement &settings ) const
 {
     kDebug();
+    settings.setAttribute( "show-project", actionShowProject->isChecked() );
+
     m_gantt->saveContext( settings );
+
 }
 
 void GanttView::updateReadWrite( bool on )
@@ -663,7 +683,7 @@ MilestoneKDGanttView::MilestoneKDGanttView( QWidget *parent )
     m->setColumn( KDGantt::TaskCompletionRole, NodeModel::NodeCompleted );
 
     static_cast<KDGantt::DateTimeGrid*>( grid() )->setDayWidth( 30 );
-    static_cast<KDGantt::DateTimeGrid*>( grid() )->setRowSeparators( treeView()->alternatingRowColors() );
+    //static_cast<KDGantt::DateTimeGrid*>( grid() )->setRowSeparators( treeView()->alternatingRowColors() );
 }
 
 MilestoneItemModel *MilestoneKDGanttView::model() const
@@ -692,7 +712,7 @@ void MilestoneKDGanttView::slotProjectCalculated( ScheduleManager *sm )
 void MilestoneKDGanttView::setScheduleManager( ScheduleManager *sm )
 {
     //kDebug()<<id<<endl;
-    model()->setManager( sm );
+    model()->setScheduleManager( sm );
     m_manager = sm;
     if ( sm && m_project ) {
         QDateTime start;
@@ -812,6 +832,7 @@ void MilestoneGanttView::slotContextMenuRequested( QModelIndex idx, const QPoint
                 name = "taskview_milestone_popup";
                 break;
             case Node::Type_Summarytask:
+                name = "taskview_summary_popup";
                 break;
             default:
                 break;
@@ -828,8 +849,11 @@ void MilestoneGanttView::slotContextMenuRequested( QModelIndex idx, const QPoint
 void MilestoneGanttView::slotOptions()
 {
     kDebug();
-    ItemViewSettupDialog dlg( m_gantt->treeView(), true );
-    dlg.exec();
+    ItemViewSettupDialog *dlg =  new ItemViewSettupDialog( m_gantt->treeView(), true, this );
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotOptionsFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
 }
 
 bool MilestoneGanttView::loadContext( const KoXmlElement &settings )
@@ -854,6 +878,128 @@ KoPrintJob *MilestoneGanttView::createPrintJob()
     return new GanttPrintingDialog( this, m_gantt );
 }
 
+//------------------------------------------
+
+ResourceAppointmentsGanttView::ResourceAppointmentsGanttView( KoDocument *part, QWidget *parent, bool readWrite )
+    : ViewBase( part, parent ),
+    m_project( 0 ),
+    m_model( new ResourceAppointmentsGanttModel( this ) )
+{
+    kDebug() <<" ---------------- KPlato: Creating ResourceAppointmentsGanttView ----------------";
+
+    m_gantt = new KDGantt::View( this );
+    m_gantt->graphicsView()->setItemDelegate( new ResourceGanttItemDelegate( m_gantt ) );
+
+    GanttTreeView *tv = new GanttTreeView( m_gantt );
+    tv->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+    tv->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    tv->setVerticalScrollMode( QAbstractItemView::ScrollPerPixel ); // needed since qt 4.2
+    m_gantt->setLeftView( tv );
+    m_gantt->setRowController( new KDGantt::TreeViewRowController( tv, m_gantt->ganttProxyModel() ) );
+    tv->header()->setStretchLastSection( true );
+    
+
+    KDGantt::ProxyModel *m = static_cast<KDGantt::ProxyModel*>( m_gantt->ganttProxyModel() );
+    m->setRole( KDGantt::ItemTypeRole, KDGantt::ItemTypeRole );
+    m->setRole( KDGantt::StartTimeRole, KDGantt::StartTimeRole );
+    m->setRole( KDGantt::EndTimeRole, KDGantt::EndTimeRole );
+    m->setRole( KDGantt::TaskCompletionRole, KDGantt::TaskCompletionRole );
+
+    m_gantt->setModel( m_model );
+
+    QVBoxLayout *l = new QVBoxLayout( this );
+    l->setMargin( 0 );
+    l->addWidget( m_gantt );
+
+    setupGui();
+
+    updateReadWrite( readWrite );
+
+    connect( m_gantt->leftView(), SIGNAL( contextMenuRequested( QModelIndex, const QPoint& ) ), SLOT( slotContextMenuRequested( QModelIndex, const QPoint& ) ) );
+
+    connect( m_gantt->leftView(), SIGNAL( headerContextMenuRequested( const QPoint& ) ), SLOT( slotHeaderContextMenuRequested( const QPoint& ) ) );
+}
+
+void ResourceAppointmentsGanttView::setZoom( double )
+{
+    //kDebug() <<"setting gantt zoom:" << zoom;
+    //m_gantt->setZoomFactor(zoom,true); NO!!! setZoomFactor() is something else
+}
+
+Project *ResourceAppointmentsGanttView::project() const
+{
+    return m_model->project();
+}
+
+void ResourceAppointmentsGanttView::setProject( Project *project )
+{
+    static_cast<KDGantt::DateTimeGrid*>( m_gantt->grid() )->setStartDateTime( project->startTime().dateTime() );
+    m_model->setProject( project );
+}
+
+void ResourceAppointmentsGanttView::setScheduleManager( ScheduleManager *sm )
+{
+    //kDebug()<<id<<endl;
+    m_model->setScheduleManager( sm );
+}
+
+void ResourceAppointmentsGanttView::setupGui()
+{
+    createOptionAction();
+}
+
+Node *ResourceAppointmentsGanttView::currentNode() const
+{
+    QModelIndex idx = treeView()->selectionModel()->currentIndex();
+    return m_model->node( idx );
+}
+
+void ResourceAppointmentsGanttView::slotContextMenuRequested( QModelIndex idx, const QPoint &pos )
+{
+    kDebug();
+    QString name;
+    if ( idx.isValid() ) {
+        Node *n = m_model->node( idx );
+        if ( n ) {
+            name = "taskview_popup";
+        }
+    }
+    if ( name.isEmpty() ) {
+        slotHeaderContextMenuRequested( pos );
+        return;
+    }
+    emit requestPopupMenu( name, pos );
+}
+
+void ResourceAppointmentsGanttView::slotOptions()
+{
+    kDebug();
+    ItemViewSettupDialog *dlg = new ItemViewSettupDialog( treeView(), true, this );
+    dlg->exec();
+    delete dlg;
+}
+
+bool ResourceAppointmentsGanttView::loadContext( const KoXmlElement &settings )
+{
+    kDebug();
+    return treeView()->loadContext( m_model->columnMap(), settings );
+}
+
+void ResourceAppointmentsGanttView::saveContext( QDomElement &settings ) const
+{
+    kDebug();
+    treeView()->saveContext( m_model->columnMap(), settings );
+}
+
+void ResourceAppointmentsGanttView::updateReadWrite( bool on )
+{
+    m_readWrite = on;
+}
+
+KoPrintJob *ResourceAppointmentsGanttView::createPrintJob()
+{
+    return 0;
+}
 
 }  //KPlato namespace
 
