@@ -167,6 +167,7 @@ void PertResult::slotSplitView()
 {
     kDebug();
     widget.treeWidgetTaskResult->setViewSplitMode( ! widget.treeWidgetTaskResult->isViewSplit() );
+    emit optionsModified();
 }
 
 Node *PertResult::currentNode() const
@@ -221,8 +222,10 @@ void PertResult::slotOptions()
 {
     kDebug();
     SplitItemViewSettupDialog *dlg = new SplitItemViewSettupDialog( widget.treeWidgetTaskResult, this );
-    dlg->exec();
-    delete dlg;
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotOptionsFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
 }
 
 void PertResult::slotUpdate(){
@@ -362,6 +365,7 @@ void PertCpmView::slotSplitView()
 {
     kDebug();
     widget.cpmTable->setViewSplitMode( ! widget.cpmTable->isViewSplit() );
+    emit optionsModified();
 }
 
 Node *PertCpmView::currentNode() const
@@ -416,8 +420,10 @@ void PertCpmView::slotOptions()
 {
     kDebug();
     SplitItemViewSettupDialog *dlg = new SplitItemViewSettupDialog( widget.cpmTable, this );
-    dlg->exec();
-    delete dlg;
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotOptionsFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
 }
 
 void PertCpmView::slotScheduleSelectionChanged( ScheduleManager *sm )
