@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-  Copyright (C) 2007 Dag Andersen <danders@get2net.dk>
+  Copyright (C) 2007 - 2010 Dag Andersen <danders@get2net.dk>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -20,6 +20,7 @@
 #include "kptrelationeditor.h"
 
 #include "kptglobal.h"
+#include "kptcommonstrings.h"
 #include "kptcommand.h"
 #include "kptproject.h"
 #include "kptitemviewsettup.h"
@@ -70,11 +71,6 @@ void RelationTreeView::slotCurrentChanged(const QModelIndex &curr, const QModelI
     selectionModel()->select( curr, QItemSelectionModel::Rows | QItemSelectionModel::ClearAndSelect );
 }
 
-void RelationTreeView::slotActivated( const QModelIndex index )
-{
-    kDebug()<<index.column();
-}
-
 //-----------------------------------
 RelationEditor::RelationEditor( KoDocument *part, QWidget *parent )
     : ViewBase( part, parent )
@@ -191,8 +187,10 @@ void RelationEditor::slotOptions()
         col0 = true;
     }
     ItemViewSettupDialog *dlg = new ItemViewSettupDialog( v, col0, this );
-    dlg->exec();
-    delete dlg;
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotOptionsFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
 }
 
 void RelationEditor::slotAddRelation()
