@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-  Copyright (C) 2007 Dag Andersen <danders@get2net.dk>
+  Copyright (C) 2007 - 2010 Dag Andersen <danders@get2net.dk>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -21,6 +21,7 @@
 #include "kpttaskstatusmodel.h"
 
 #include "kptglobal.h"
+#include "kptcommonstrings.h"
 #include "kptcommand.h"
 #include "kptproject.h"
 #include "kptschedule.h"
@@ -225,7 +226,7 @@ void TaskStatusView::updateReadWrite( bool rw )
 void TaskStatusView::setScheduleManager( ScheduleManager *sm )
 {
     //kDebug();
-    static_cast<TaskStatusItemModel*>( m_view->model() )->setManager( sm );
+    static_cast<TaskStatusItemModel*>( m_view->model() )->setScheduleManager( sm );
 }
 
 Node *TaskStatusView::currentNode() const 
@@ -304,15 +305,22 @@ void TaskStatusView::slotSplitView()
 {
     kDebug();
     m_view->setViewSplitMode( ! m_view->isViewSplit() );
+    emit optionsModified();
 }
 
+void TaskStatusView::slotRefreshView()
+{
+    model()->refresh();
+}
 
 void TaskStatusView::slotOptions()
 {
     kDebug();
     TaskStatusViewSettingsDialog *dlg = new TaskStatusViewSettingsDialog( m_view, this );
-    dlg->exec();
-    delete dlg;
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotOptionsFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
 }
 
 bool TaskStatusView::loadContext( const KoXmlElement &context )
@@ -461,8 +469,10 @@ void ProjectStatusView::slotOptions()
 {
     kDebug();
     ProjectStatusViewSettingsDialog *dlg = new ProjectStatusViewSettingsDialog( m_view, this );
-    dlg->exec();
-    delete dlg;
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotOptionsFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
 }
 
 bool ProjectStatusView::loadContext( const KoXmlElement &context )
@@ -487,9 +497,9 @@ PerformanceStatusBase::PerformanceStatusBase( QWidget *parent )
     labelBCWS->setToolTip( ToolTip::nodeBCWS() );
     labelBCWP->setToolTip( ToolTip::nodeBCWP() );
     labelACWP->setToolTip( ToolTip::nodeACWP() );
-    labelPI->setToolTip( ToolTip::nodePerformanceIndex() );
-    labelCPI->setToolTip( i18nc( "@info:tooltip", "Costbased performance index" ) );
-    labelSPI->setToolTip( i18nc( "@info:tooltip", "Effortbased performance index" ) );
+    labelPI->setToolTip(  i18nc( "@info:tooltip", "Performance indexes" ) );
+    labelCPI->setToolTip( i18nc( "@info:tooltip", "Cost performance index (BCWP/ACWP)" ) );
+    labelSPI->setToolTip( ToolTip::nodePerformanceIndex() );
 
     plotwidget->setAntialiasing(false);
     plotwidget->setShowGrid(false);
@@ -783,7 +793,7 @@ NodeItemModel *PerformanceStatusTreeView::nodeModel() const
 
 void PerformanceStatusTreeView::setScheduleManager( ScheduleManager *sm )
 {
-    nodeModel()->setManager( sm );
+    nodeModel()->setScheduleManager( sm );
     m_chart->setScheduleManager( sm );
 }
 
@@ -916,8 +926,10 @@ void PerformanceStatusView::slotOptions()
 {
     kDebug();
     PerformanceStatusViewSettingsDialog *dlg = new PerformanceStatusViewSettingsDialog( m_view, this );
-    dlg->exec();
-    delete dlg;
+    connect(dlg, SIGNAL(finished(int)), SLOT(slotOptionsFinished(int)));
+    dlg->show();
+    dlg->raise();
+    dlg->activateWindow();
 }
 
 bool PerformanceStatusView::loadContext( const KoXmlElement &context )
