@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-  Copyright (C) 2006 Dag Andersen <danders@get2net.dk>
+  Copyright (C) 2006 -2010 Dag Andersen <danders@get2net.dk>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -28,6 +28,7 @@
 
 #include <KoView.h>
 #include <KoPrintingDialog.h>
+#include <KoPageLayout.h>
 
 #include <QMap>
 #include <QTreeView>
@@ -43,6 +44,7 @@ class QAbstractItemModel;
 
 class KoDocument;
 class KoPrintJob;
+class KoPageLayoutWidget;
 
 /// The main namespace
 namespace KPlato
@@ -124,6 +126,9 @@ public:
     PrintingOptions printingOptions() const;
     void setPrintingOptions( const PrintingOptions &opt);
     
+    QWidget *createPageLayoutWidget() const;
+    QAbstractPrintDialog::PrintDialogOptions printDialogOptions() const;
+
 signals:
     void changed ( const PrintingOptions &opt );
     
@@ -135,23 +140,58 @@ protected:
     int headerFooterHeight( const PrintingOptions::Data &options ) const;
     void drawBottomRect( QPainter &p, const QRect &r );
 
+    void setPrinterPageLayout();
+
 protected:
     ViewBase *m_view;
     QPointer<PrintingHeaderFooter> m_widget;
 };
 
+class KPLATOUI_EXPORT ViewActionLists
+{
+public:
+    ViewActionLists() : actionOptions( 0 ) {}
+    virtual ~ViewActionLists() {}
+
+    /// Returns the list of action lists that shall be plugged/unplugged
+    virtual QStringList actionListNames() const { return m_actionListMap.keys(); }
+    /// Returns the list of actions associated with the action list name
+    virtual QList<QAction*> actionList( const QString name ) const { return m_actionListMap[name]; }
+    /// Add an action to the specified action list
+    void addAction( const QString list, QAction *action ) { m_actionListMap[list].append( action ); }
+
+    virtual QList<QAction*> viewlistActionList() const { return m_viewlistActionList; }
+    void addViewlistAction( QAction *action ) { m_viewlistActionList.append( action ); }
+    
+    QList<QAction*> contextActionList() const { return m_contextActionList; }
+    void addContextAction( QAction *action ) { m_contextActionList.append( action ); }
+
+protected:
+    /// List of all menu/toolbar actions (used for plug/unplug)
+    QMap<QString, QList<QAction*> > m_actionListMap;
+
+    /// List of actions that will be shown in the viewlist context menu
+    QList<QAction*> m_viewlistActionList;
+    
+    /// List of actions that will be shown in the views header context menu
+    QList<QAction*> m_contextActionList;
+    
+    // View options context menu
+    KAction *actionOptions;
+};
+
 /** 
  ViewBase is the baseclass of all sub-views to View.
 
 */
-class KPLATOUI_EXPORT ViewBase : public KoView
+class KPLATOUI_EXPORT ViewBase : public KoView, public ViewActionLists
 {
     Q_OBJECT
 public:
     /// Contructor
     ViewBase(KoDocument *doc, QWidget *parent);
     /// Destructor
-    virtual ~ViewBase() {}
+    virtual ~ViewBase();
     /// Return the part (document) this view handles
     KoDocument *part() const;
     
@@ -161,11 +201,10 @@ public:
     /// Returns true if this view or any child widget has focus
     bool isActive() const;
     
-    virtual void setZoom(double /*zoom*/) {}
     /// Set the project this view shall handle.
-    virtual void setProject( Project * /*project*/ ) {}
+    virtual void setProject( Project *project ) { m_proj = project; }
     /// Return the project
-    virtual Project *project() const { return 0; }
+    virtual Project *project() const { return m_proj; }
     /// Draw data from current part / project
     virtual void draw() {}
     /// Draw data from project.
@@ -186,67 +225,59 @@ public:
     virtual Calendar* currentCalendar() const { return 0; }
     /// Reimplement if your view handles relations
     virtual Relation *currentRelation() const { return 0; }
-    
-    /// Returns the list of action lists that shall be plugged/unplugged
-    virtual QStringList actionListNames() const { return m_actionListMap.keys(); }
-    /// Returns the list of actions associated with the action list name
-    virtual QList<QAction*> actionList( const QString name ) const { return m_actionListMap[name]; }
-    /// Add an action to the specified action list
-    void addAction( const QString list, QAction *action ) { m_actionListMap[list].append( action ); }
-    
+    /// Reimplement if your view handles zoom
+    virtual KoZoomController *zoomController() const { return 0; }
+
     /// Loads context info into this view. Reimplement.
     virtual bool loadContext( const KoXmlElement &/*context*/ ) { return false; }
     /// Save context info from this view. Reimplement.
     virtual void saveContext( QDomElement &/*context*/ ) const {}
-    
-    virtual QList<QAction*> viewlistActionList() const { return m_viewlistActionList; }
-    void addViewlistAction( QAction *action ) { m_viewlistActionList.append( action ); }
-    
-    virtual QList<QAction*> contextActionList() const { return m_contextActionList; }
-    void addContextAction( QAction *action ) { m_contextActionList.append( action ); }
-    
-    virtual ViewBase *hitView( const QPoint &pos );
 
     virtual KoPrintJob *createPrintJob();
     PrintingOptions printingOptions() const { return m_printingOptions; }
     void setPrintingOptions( const PrintingOptions &opt ) { m_printingOptions = opt; }
     
+    void addAction( const QString list, QAction *action ) { ViewActionLists::addAction( list, action );  }
+
+    KoPageLayout pageLayout() const { return m_pagelayout; }
+
 public slots:
     /// Activate/deactivate the gui
     virtual void setGuiActive( bool activate );
-
+    virtual void setScheduleManager( ScheduleManager *sm ) { m_schedulemanager = sm; }
     void slotUpdateReadWrite( bool );
     virtual void slotHeaderContextMenuRequested( const QPoint &pos );
 
+    virtual void slotEditCopy() {}
+    virtual void slotEditCut() {}
+    virtual void slotEditPaste() {}
+    virtual void slotRefreshView() {}
     
+    void setPageLayout( const KoPageLayout &layout ) { m_pagelayout = layout; }
+
 signals:
     /// Emitted when the gui has been activated or deactivated
     void guiActivated( ViewBase*, bool );
     /// Request for a context menu popup
     void requestPopupMenu( const QString&, const QPoint & );
     
+    /// Emitted when options are modified
+    void optionsModified();
+
 protected slots:
     virtual void slotOptions() {}
+    virtual void slotOptionsFinished( int result );
 
 protected:
     void createOptionAction();
     
-    /// List of all menu/toolbar actions (used for plug/unplug)
-    QMap<QString, QList<QAction*> > m_actionListMap;
-
-    /// List of actions that will be shown in the viewlist context menu
-    QList<QAction*> m_viewlistActionList;
-    
-    /// List of actions that will be shown in the views header context menu
-    QList<QAction*> m_contextActionList;
-    
     bool m_readWrite;
-    
     PrintingOptions m_printingOptions;
     
-    // View options context menu
-    KAction *actionOptions;
-
+    Project *m_proj;
+    ScheduleManager *m_schedulemanager;
+    
+    KoPageLayout m_pagelayout;
 };
 
 //------------------
@@ -260,6 +291,8 @@ public:
     virtual int documentFirstPage() const { return 1; }
     virtual int documentLastPage() const;
 
+    QList<QWidget*> createOptionWidgets() const;
+
 protected:
     virtual void printPage( int pageNumber, QPainter &painter );
 
@@ -404,6 +437,8 @@ public:
     virtual int documentFirstPage() const { return 1; }
     virtual int documentLastPage() const;
 
+    QList<QWidget*> createOptionWidgets() const;
+
 protected:
     virtual void printPage( int pageNumber, QPainter &painter );
     
@@ -486,6 +521,20 @@ public:
 
     void setParentsExpanded( const QModelIndex &idx, bool expanded );
 
+    void setSortingEnabled( bool on ) {
+        m_leftview->setSortingEnabled( on );
+        m_rightview->setSortingEnabled( on );
+    }
+    void sortByColumn( int col, Qt::SortOrder order = Qt::AscendingOrder ) {
+        if ( ! m_leftview->isColumnHidden( col ) || 
+             ! m_rightview->isVisible() ||
+             m_rightview->isColumnHidden( col ) )
+        {
+            m_leftview->sortByColumn( col, order );
+        } else {
+            m_rightview->sortByColumn( col, order );
+        }
+    }
 signals:
     /// Context menu requested from the viewport, pointer over @p index at global position @p pos
     void contextMenuRequested( QModelIndex index, const QPoint& pos );
@@ -503,7 +552,8 @@ signals:
 
 public slots:
     void edit( const QModelIndex &index );
-
+    void expandAll();
+    
 protected slots:
     void slotSelectionChanged( const QItemSelection &sel, const QItemSelection & );
     void slotToRightView( const QModelIndex &index );
@@ -514,6 +564,9 @@ protected slots:
     void slotRightHeaderContextMenuRequested( const QPoint &pos );
     void slotLeftHeaderContextMenuRequested( const QPoint &pos );
 
+    void slotLeftSortIndicatorChanged( int logicalIndex, Qt::SortOrder order );
+    void slotRightSortIndicatorChanged( int logicalIndex, Qt::SortOrder order );
+
 protected:
     void init();
     QList<int> expandColumnList( const QList<int> lst ) const;
