@@ -32,6 +32,8 @@
 #include <KoPASavingContext.h>
 
 #include "KPrDocument.h"
+#include "KPrDeclarations.h"
+#include "KPresenter.h"
 #include "KPrPageApplicationData.h"
 #include "KPrMasterPage.h"
 #include "KPrNotes.h"
@@ -51,14 +53,17 @@ class KPrPage::Private
 public:
     Private( KPrPage * page, KPrDocument * document )
     : pageNotes( new KPrNotes( page, document ) )
+    , declarations( document->declarations() )
     {}
 
     ~Private()
     {
         delete pageNotes;
     }
-
     KPrNotes * pageNotes;
+    QHash<KPrDeclarations::Type, QString> usedDeclaration;
+    KPrDeclarations *declarations;
+
 };
 
 KPrPage::KPrPage( KoPAMasterPage * masterPage, KPrDocument * document )
@@ -115,16 +120,31 @@ KPrPageLayout * KPrPage::layout() const
 
 void KPrPage::saveOdfPageContent( KoPASavingContext & paContext ) const
 {
+    KoXmlWriter &writer(paContext.xmlWriter());
     if ( layout() ) {
         KPrPageLayoutSharedSavingData * layouts = dynamic_cast<KPrPageLayoutSharedSavingData *>( paContext.sharedData( KPR_PAGE_LAYOUT_SHARED_SAVING_ID ) );
         Q_ASSERT( layouts );
         if ( layouts ) {
             QString layoutStyle = layouts->pageLayoutStyle( layout() );
             if ( ! layoutStyle.isEmpty() ) {
-                paContext.xmlWriter().addAttribute( "presentation:presentation-page-layout-name", layoutStyle );
+                writer.addAttribute( "presentation:presentation-page-layout-name", layoutStyle );
             }
         }
     }
+    QHash<KPrDeclarations::Type, QString>::const_iterator it(d->usedDeclaration.constBegin());
+    for (; it != d->usedDeclaration.constEnd(); ++it) {
+        switch (it.key()) {
+        case KPrDeclarations::Footer:
+            writer.addAttribute("presentation:use-footer-name", it.value());
+            break;
+        case KPrDeclarations::Header:
+            writer.addAttribute("presentation:use-header-name", it.value());
+            break;
+        case KPrDeclarations::DateTime:
+            writer.addAttribute("presentation:use-date-time-name", it.value());
+            break;
+        }
+    }
     KoPAPageBase::saveOdfPageContent( paContext );
 }
 
@@ -151,10 +171,22 @@ void KPrPage::loadOdfPageTag( const KoXmlElement &element, KoPALoadingContext &l
 
     int pageProperties = m_pageProperties & UseMasterBackground;
     if ( styleStack.property( KoXmlNS::presentation, "background-objects-visible" ) == "true" ) {
-        pageProperties = pageProperties | DisplayMasterShapes;
+        pageProperties |= DisplayMasterShapes;
     }
     if ( styleStack.property( KoXmlNS::presentation, "background-visible" ) == "true" ) {
-        pageProperties = pageProperties | DisplayMasterBackground;
+        pageProperties |= DisplayMasterBackground;
+    }
+    if ( styleStack.property( KoXmlNS::presentation, "display-header" ) == "true" ) {
+        pageProperties |= DisplayHeader;
+    }
+    if ( styleStack.property( KoXmlNS::presentation, "display-footer" ) == "true" ) {
+        pageProperties |= DisplayFooter;
+    }
+    if ( styleStack.property( KoXmlNS::presentation, "display-page-number" ) == "true" ) {
+        pageProperties |= DisplayPageNumber;
+    }
+    if ( styleStack.property( KoXmlNS::presentation, "display-date-time" ) == "true" ) {
+        pageProperties |= DisplayDateTime;
     }
     m_pageProperties = pageProperties;
 
@@ -188,7 +220,8 @@ void KPrPage::loadOdfPageExtra( const KoXmlElement &element, KoPALoadingContext
     // the layout needs to be loaded after the shapes are already loaded so the initialization of the data works
     KPrPageLayout * layout = 0;
     if ( element.hasAttributeNS( KoXmlNS::presentation, "presentation-page-layout-name" ) ) {
-        KPrPageLayouts * layouts = dynamic_cast<KPrPageLayouts *>( loadingContext.dataCenter( PageLayouts ) );
+        KPrPageLayouts *layouts = loadingContext.documentResourceManager()->resource(KPresenter::PageLayouts).value<KPrPageLayouts*>();
+
         Q_ASSERT( layouts );
         if ( layouts ) {
             QString layoutName = element.attributeNS( KoXmlNS::presentation, "presentation-page-layout-name" );
@@ -198,6 +231,19 @@ void KPrPage::loadOdfPageExtra( const KoXmlElement &element, KoPALoadingContext
         }
     }
     placeholders().init( layout, childShapes() );
+
+    if (element.hasAttributeNS(KoXmlNS::presentation, "use-footer-name")) {
+        QString name = element.attributeNS (KoXmlNS::presentation, "use-footer-name");
+        d->usedDeclaration.insert(KPrDeclarations::Footer, name);
+    }
+    if (element.hasAttributeNS( KoXmlNS::presentation, "use-header-name")) {
+        QString name = element.attributeNS (KoXmlNS::presentation, "use-header-name");
+        d->usedDeclaration.insert(KPrDeclarations::Header, name);
+    }
+    if (element.hasAttributeNS( KoXmlNS::presentation, "use-date-time-name")) {
+        QString name = element.attributeNS (KoXmlNS::presentation, "use-date-time-name");
+        d->usedDeclaration.insert(KPrDeclarations::DateTime, name);
+    }
 }
 
 bool KPrPage::saveOdfPresentationNotes(KoPASavingContext &paContext) const
@@ -211,6 +257,11 @@ KoPageApp::PageType KPrPage::pageType() const
     return KoPageApp::Slide;
 }
 
+QString KPrPage::declaration(KPrDeclarations::Type type) const
+{
+    return d->declarations->declaration(type, d->usedDeclaration.value(type));
+}
+
 KoShapeManagerPaintingStrategy * KPrPage::getPaintingStrategy() const
 {
     return new KPrShapeManagerDisplayMasterStrategy(0);
