@@ -17,13 +17,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
-
 #include "filter/kis_filter_registry.h"
 
 #include <math.h>
 
 #include <QString>
 
+#include <kglobal.h>
 #include <kaction.h>
 #include <klocale.h>
 #include <kparts/plugin.h>
@@ -38,28 +38,24 @@
 
 #include "kis_paint_device.h"
 #include "filter/kis_filter.h"
-
-KisFilterRegistry *KisFilterRegistry::m_singleton = 0;
+#include "kis_filter_configuration.h"
 
 KisFilterRegistry::KisFilterRegistry()
 {
-    Q_ASSERT(KisFilterRegistry::m_singleton == 0);
-    KisFilterRegistry::m_singleton = this;
 }
 
 KisFilterRegistry::~KisFilterRegistry()
 {
+    dbgRegistry << "deleting KisFilterRegistry";
 }
 
 KisFilterRegistry* KisFilterRegistry::instance()
 {
-    if (KisFilterRegistry::m_singleton == 0) {
-        KisFilterRegistry::m_singleton = new KisFilterRegistry();
-        Q_CHECK_PTR( KisFilterRegistry::m_singleton );
-        KoPluginLoader::instance()->load( "Krita/Filter", "Type == 'Service' and ([X-Krita-Version] == 3)" );
-
+    K_GLOBAL_STATIC(KisFilterRegistry, s_instance);
+    if (!s_instance.exists()) {
+        KoPluginLoader::instance()->load("Krita/Filter", "Type == 'Service' and ([X-Krita-Version] == 3)");
     }
-    return KisFilterRegistry::m_singleton;
+    return s_instance;
 }
 
 void KisFilterRegistry::add(KisFilterSP item)
@@ -73,4 +69,13 @@ void KisFilterRegistry::add(const QString &id, KisFilterSP item)
     emit(filterAdded(id));
 }
 
+KisFilterConfiguration* KisFilterRegistry::cloneConfiguration(KisFilterConfiguration* kfc)
+{
+    Q_ASSERT(kfc);
+    KisFilterSP filter = value(kfc->name());
+    KisFilterConfiguration* newkfc = filter->defaultConfiguration(0);
+    newkfc->fromXML(kfc->toXML());
+    return newkfc;
+}
+
 #include "kis_filter_registry.moc"
