@@ -24,13 +24,9 @@
 #include <stdlib.h>
 #include <math.h>
 
-#include <lcms.h>
-
 #include <QImage>
 #include <QPainter>
 #include <QSize>
-#include <QApplication>
-#include <QThread>
 #include <QDateTime>
 #include <QRect>
 #include <QRegion>
@@ -41,11 +37,13 @@
 #include "KoColorSpaceRegistry.h"
 #include "KoColor.h"
 #include "KoColorConversionTransformation.h"
-#include "colorprofiles/KoIccColorProfile.h"
+#include "KoColorProfile.h"
 
 #include "recorder/kis_action_recorder.h"
 #include "kis_adjustment_layer.h"
 #include "kis_annotation.h"
+#include "kis_projection.h"
+#include "kis_background.h"
 #include "kis_change_profile_visitor.h"
 #include "kis_colorspace_convert_visitor.h"
 #include "kis_count_visitor.h"
@@ -54,10 +52,9 @@
 #include "commands/kis_image_commands.h"
 #include "kis_iterators_pixel.h"
 #include "kis_layer.h"
-
+#include "kis_meta_data_merge_strategy_registry.h"
 #include "kis_name_server.h"
 #include "kis_paint_device.h"
-#include "kis_paint_device_action.h"
 #include "kis_paint_layer.h"
 #include "kis_painter.h"
 #include "kis_perspective_grid.h"
@@ -67,19 +64,16 @@
 #include "kis_transform_visitor.h"
 #include "kis_types.h"
 #include "kis_crop_visitor.h"
-#include "kis_projection_update_strategy.h"
 #include "kis_meta_data_merge_strategy.h"
 
 class KisImage::KisImagePrivate
 {
 public:
-    KoColor backgroundColor;
+    KisBackgroundSP  backgroundPattern;
     quint32 lockCount;
     bool sizeChangedWhileLocked;
     KisPerspectiveGrid* perspectiveGrid;
 
-    KUrl uri;
-
     qint32 width;
     qint32 height;
 
@@ -102,13 +96,19 @@ public:
     KisSelectionSP globalSelection;
     KisSelectionSP deselectedGlobalSelection;
 
+    KisProjection* projection;
+
+    bool startProjection;
+
 };
-KisImage::KisImage(KisUndoAdapter *adapter, qint32 width, qint32 height, const KoColorSpace * colorSpace, const QString& name)
+KisImage::KisImage(KisUndoAdapter *adapter, qint32 width, qint32 height, const KoColorSpace * colorSpace, const QString& name, bool startProjection)
         : QObject(0)
         , KisShared()
         , m_d(new KisImagePrivate())
 {
     setObjectName(name);
+    dbgImage << "creating" << name;
+    m_d->startProjection = startProjection;
     init(adapter, width, height, colorSpace);
 }
 
@@ -116,17 +116,19 @@ KisImage::KisImage(const KisImage& rhs)
         : QObject()
         , KisNodeFacade(rhs)
         , KisNodeGraphListener(rhs)
-        , KisShared(rhs)
+        , KisShared()
         , m_d(new KisImagePrivate())
 {
+
     if (this != &rhs) {
 
+        dbgImage << "copying" << objectName() << "from" << rhs.objectName();
+
         if (rhs.m_d->perspectiveGrid)
             m_d->perspectiveGrid = new KisPerspectiveGrid(*rhs.m_d->perspectiveGrid);
         else
             m_d->perspectiveGrid = 0;
 
-        m_d->uri = rhs.m_d->uri;
         m_d->width = rhs.m_d->width;
         m_d->height = rhs.m_d->height;
         m_d->xres = rhs.m_d->xres;
@@ -138,13 +140,25 @@ KisImage::KisImage(const KisImage& rhs)
         m_d->deselectedGlobalSelection = 0;
         setRootLayer(static_cast<KisGroupLayer*>(rhs.m_d->rootLayer->clone().data()));
         m_d->annotations = rhs.m_d->annotations; // XXX the annotations would probably need to be deep-copied
-        m_d->nserver = new KisNameServer( *rhs.m_d->nserver );
+        m_d->nserver = new KisNameServer(*rhs.m_d->nserver);
+        m_d->startProjection = rhs.m_d->startProjection;
         Q_CHECK_PTR(m_d->nserver);
 
+        m_d->projection = 0;
+        if (m_d->startProjection) {
+            m_d->projection = new KisProjection(this);
+            m_d->projection->start();
+        }
     }
 }
+
 KisImage::~KisImage()
 {
+    dbgImage << "deleting kisimage" << objectName();
+    if (m_d->projection) {
+        m_d->projection->stop();
+        m_d->projection->deleteLater();
+    }
     delete m_d->perspectiveGrid;
     delete m_d->nserver;
     delete m_d;
@@ -158,30 +172,6 @@ void KisImage::aboutToAddANode(KisNode *parent, int index)
 
 void KisImage::nodeHasBeenAdded(KisNode *parent, int index)
 {
-    // XXX: Temporarily for compatibility
-
-    KisLayer * layer = dynamic_cast<KisLayer*>(parent->at(index).data());
-    if (layer) {
-
-        KisPaintLayerSP player = KisPaintLayerSP(dynamic_cast<KisPaintLayer*>(layer));
-        if (!player.isNull()) {
-            // XXX: This should also be done whenever a layer grows!
-            QList<KisPaintDeviceAction *> actions =
-                KoColorSpaceRegistry::instance()->paintDeviceActionsFor(player->paintDevice()->colorSpace());
-            for (int i = 0; i < actions.count(); i++) {
-                actions.at(i)->act(player.data()->paintDevice(), width(), height());
-            }
-
-        }
-
-        // The addition of temporary layers is not interesting
-        KoProperties props;
-        props.setProperty("temporary", false);
-
-        if (!layer->check(props)) {
-            emit sigLayerMoved(layer);
-        }
-    }
     emit sigNodeHasBeenAdded(parent, index);
 }
 
@@ -194,43 +184,16 @@ void KisImage::nodeHasBeenRemoved(KisNode *parent, int index)
 {
     // XXX: Temporarily for compatibility
     emit sigNodeHasBeenRemoved(parent, index);
-
-    KisLayer * l = dynamic_cast<KisLayer*>(parent->at(index).data());
-    if (l) {
-        // The removal of temporary layers is not interesting
-        KoProperties props;
-        props.setProperty("temporary", false);
-
-        if (!l->check(props)) {
-            emit sigLayerRemoved(l);
-        }
-    }
 }
 
-void KisImage::aboutToMoveNode(KisNode * parent, int oldIndex, int newIndex)
+void KisImage::aboutToMoveNode(KisNode *parent, int oldIndex, int newIndex)
 {
-    Q_UNUSED(parent);
-    Q_UNUSED(oldIndex);
-    Q_UNUSED(newIndex);
+    emit sigAboutToMoveNode(parent, oldIndex, newIndex);
 }
 
-void KisImage::nodeHasBeenMoved(KisNode * parent, int oldIndex, int newIndex)
+void KisImage::nodeHasBeenMoved(KisNode *parent, int oldIndex, int newIndex)
 {
-    Q_UNUSED(oldIndex);
-
-    KisLayer * l = dynamic_cast<KisLayer*>(parent->at(newIndex).data());
-    if (l) {
-
-        // The moving of temporary layers is not interesting
-
-        KoProperties props;
-        props.setProperty("temporary", false);
-
-        if (!l->check(props)) {
-
-            emit sigLayerMoved(l);
-        }
-    }
+    emit sigNodeHasBeenMoved(parent, oldIndex, newIndex);
 }
 
 
@@ -242,7 +205,7 @@ KisSelectionSP KisImage::globalSelection() const
 void KisImage::setGlobalSelection(KisSelectionSP globalSelection)
 {
     if (globalSelection == 0)
-        m_d->globalSelection = new KisSelection(m_d->rootLayer->projection());
+        m_d->globalSelection = new KisSelection(m_d->rootLayer->projection(), KisDefaultBounds(this));
     else
         m_d->globalSelection = globalSelection;
 }
@@ -252,7 +215,7 @@ void KisImage::removeGlobalSelection()
     m_d->globalSelection = 0;
 }
 
-KisSelectionSP KisImage::deleselectedGlobalSelection()
+KisSelectionSP KisImage::deselectedGlobalSelection()
 {
     return m_d->deselectedGlobalSelection;
 }
@@ -262,18 +225,19 @@ void KisImage::setDeleselectedGlobalSelection(KisSelectionSP selection)
     m_d->deselectedGlobalSelection = selection;
 }
 
-
-KoColor KisImage::backgroundColor() const
+KisBackgroundSP KisImage::backgroundPattern() const
 {
-    return m_d->backgroundColor;
+    return m_d->backgroundPattern;
 }
 
-void KisImage::setBackgroundColor(const KoColor & color)
+void KisImage::setBackgroundPattern(KisBackgroundSP background)
 {
-    m_d->backgroundColor = color;
+    if (background != m_d->backgroundPattern) {
+        m_d->backgroundPattern = background;
+        emit sigImageUpdated(bounds());
+    }
 }
 
-
 QString KisImage::nextLayerName() const
 {
     if (m_d->nserver->currentSeed() == 0) {
@@ -289,13 +253,12 @@ void KisImage::rollBackLayerName()
     m_d->nserver->rollback();
 }
 
-void KisImage::init(KisUndoAdapter *adapter, qint32 width, qint32 height, const KoColorSpace * colorSpace)
+void KisImage::init(KisUndoAdapter *adapter, qint32 width, qint32 height, const KoColorSpace *colorSpace)
 {
     if (colorSpace == 0) {
         colorSpace = KoColorSpaceRegistry::instance()->rgb8();
     }
 
-    m_d->backgroundColor = KoColor(Qt::white, colorSpace);
     m_d->lockCount = 0;
     m_d->sizeChangedWhileLocked = false;
     m_d->perspectiveGrid = 0;
@@ -306,7 +269,7 @@ void KisImage::init(KisUndoAdapter *adapter, qint32 width, qint32 height, const
 
     m_d->colorSpace = colorSpace;
 
-    setRootLayer(new KisGroupLayer(this, "root", OPACITY_OPAQUE));
+    setRootLayer(new KisGroupLayer(this, "root", OPACITY_OPAQUE_U8));
 
     m_d->xres = 1.0;
     m_d->yres = 1.0;
@@ -315,6 +278,12 @@ void KisImage::init(KisUndoAdapter *adapter, qint32 width, qint32 height, const
     m_d->height = height;
 
     m_d->recorder = new KisActionRecorder(this);
+
+    m_d->projection = 0;
+    if (m_d->startProjection) {
+        m_d->projection = new KisProjection(this);
+        m_d->projection->start();
+    }
 }
 
 bool KisImage::locked() const
@@ -324,11 +293,12 @@ bool KisImage::locked() const
 
 void KisImage::lock()
 {
+//  blockSignals(true);
     if (!locked()) {
-        if (m_d->rootLayer)
-            m_d->rootLayer->updateStrategy()->lock();
+        if (m_d->projection) {
+            m_d->projection->lock();
+        }
         m_d->sizeChangedWhileLocked = false;
-//         blockSignals(true);
     }
     m_d->lockCount++;
 }
@@ -341,16 +311,15 @@ void KisImage::unlock()
         m_d->lockCount--;
 
         if (m_d->lockCount == 0) {
-//             blockSignals(false);
-
             if (m_d->sizeChangedWhileLocked) {
                 emit sigSizeChanged(m_d->width, m_d->height);
             }
 
-            if (m_d->rootLayer)
-                m_d->rootLayer->updateStrategy()->unlock();
-
+            if (m_d->projection) {
+                m_d->projection->unlock();
+            }
         }
+//      blockSignals(false);
     }
 }
 
@@ -376,16 +345,14 @@ void KisImage::resize(qint32 w, qint32 h, qint32 x, qint32 y, bool cropLayers)
 {
     if (w != width() || h != height()) {
 
-        lock();
-
         if (undo()) {
             if (cropLayers)
                 m_d->adapter->beginMacro(i18n("Crop Image"));
             else
                 m_d->adapter->beginMacro(i18n("Resize Image"));
 
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), true));
-            m_d->adapter->addCommand(new KisImageResizeCommand(KisImageSP(this), w, h, width(), height()));
+            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
+            m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
         }
 
         m_d->width = w;
@@ -398,15 +365,38 @@ void KisImage::resize(qint32 w, qint32 h, qint32 x, qint32 y, bool cropLayers)
 
         emitSizeChanged();
 
-        unlock();
-
         if (undo()) {
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), false));
+            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
             m_d->adapter->endMacro();
         }
     }
 }
 
+void KisImage::resizeWithOffset(qint32 w, qint32 h, qint32 xOffset, qint32 yOffset)
+{
+    if (w == width() && h == height() && xOffset == 0 && yOffset == 0)
+        return;
+
+    lock();
+    if (undo()) {
+        m_d->adapter->beginMacro(i18n("Size Canvas"));
+        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
+        m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
+    }
+
+    KisCropVisitor v(QRect(-xOffset, -yOffset, w, h), m_d->adapter);
+    m_d->rootLayer->accept(v);
+
+    emitSizeChanged();
+
+    unlock();
+
+    if (undo()) {
+        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
+        m_d->adapter->endMacro();
+    }
+
+}
 
 void KisImage::emitSizeChanged()
 {
@@ -439,16 +429,16 @@ void KisImage::scale(double sx, double sy, KoUpdater *progress, KisFilterStrateg
 
         if (undo()) {
             m_d->adapter->beginMacro(i18n("Scale Image"));
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), true));
+            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
         }
 
         {
-            KisTransformVisitor visitor(KisImageSP(this), sx, sy, 0.0, 0.0, 0.0, 0, 0, progress, filterStrategy);
+            KisTransformVisitor visitor(KisImageWSP(this), sx, sy, 0.0, 0.0, 0.0, 0, 0, progress, filterStrategy);
             m_d->rootLayer->accept(visitor);
         }
 
         if (undo()) {
-            m_d->adapter->addCommand(new KisImageResizeCommand(KisImageSP(this), w, h, width(), height()));
+            m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
         }
 
         m_d->width = w;
@@ -459,7 +449,7 @@ void KisImage::scale(double sx, double sy, KoUpdater *progress, KisFilterStrateg
         unlock();
 
         if (undo()) {
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), false));
+            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
             m_d->adapter->endMacro();
         }
     }
@@ -472,22 +462,22 @@ void KisImage::rotate(double radians, KoUpdater *progress)
     qint32 h = height();
     qint32 tx = qint32((w * cos(radians) - h * sin(radians) - w) / 2 + 0.5);
     qint32 ty = qint32((h * cos(radians) + w * sin(radians) - h) / 2 + 0.5);
-    w = (qint32)(width() * QABS(cos(radians)) + height() * QABS(sin(radians)) + 0.5);
-    h = (qint32)(height() * QABS(cos(radians)) + width() * QABS(sin(radians)) + 0.5);
+    w = (qint32)(width() * qAbs(cos(radians)) + height() * qAbs(sin(radians)) + 0.5);
+    h = (qint32)(height() * qAbs(cos(radians)) + width() * qAbs(sin(radians)) + 0.5);
 
     tx -= (w - width()) / 2;
     ty -= (h - height()) / 2;
 
     if (undo()) {
         m_d->adapter->beginMacro(i18n("Rotate Image"));
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), true));
+        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
     }
 
     KisFilterStrategy *filter = KisFilterStrategyRegistry::instance()->value("Triangle");
-    KisTransformVisitor visitor(KisImageSP(this), 1.0, 1.0, 0, 0, radians, -tx, -ty, progress, filter);
+    KisTransformVisitor visitor(KisImageWSP(this), 1.0, 1.0, 0, 0, radians, -tx, -ty, progress, filter);
     m_d->rootLayer->accept(visitor);
 
-    if (undo()) m_d->adapter->addCommand(new KisImageResizeCommand(KisImageSP(this), w, h, width(), height()));
+    if (undo()) m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
 
     m_d->width = w;
     m_d->height = h;
@@ -497,7 +487,7 @@ void KisImage::rotate(double radians, KoUpdater *progress)
     unlock();
 
     if (undo()) {
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), false));
+        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
         m_d->adapter->endMacro();
     }
 }
@@ -512,18 +502,18 @@ void KisImage::shear(double angleX, double angleY, KoUpdater *progress)
 
 
     if (angleX != 0 || angleY != 0) {
-        double deltaY = height() * QABS(tan(angleX * pi / 180) * tan(angleY * pi / 180));
-        w = (qint32)(width() + QABS(height() * tan(angleX * pi / 180)));
+        double deltaY = height() * qAbs(tan(angleX * pi / 180) * tan(angleY * pi / 180));
+        w = (qint32)(width() + qAbs(height() * tan(angleX * pi / 180)));
         //ugly fix for the problem of having two extra pixels if only a shear along one
-        //axis is done. This has to be fixed in the cropping code in KisRotateVisitor!
+        //axis is done.
         if (angleX == 0 || angleY == 0)
-            h = (qint32)(height() + QABS(w * tan(angleY * pi / 180)));
+            h = (qint32)(height() + qAbs(w * tan(angleY * pi / 180)));
         else if (angleX > 0 && angleY > 0)
-            h = (qint32)(height() + QABS(w * tan(angleY * pi / 180)) - 2 * deltaY + 2);
+            h = (qint32)(height() + qAbs(w * tan(angleY * pi / 180)) - 2 * deltaY + 2);
         else if (angleX < 0 && angleY < 0)
-            h = (qint32)(height() + QABS(w * tan(angleY * pi / 180)) - 2 * deltaY + 2);
+            h = (qint32)(height() + qAbs(w * tan(angleY * pi / 180)) - 2 * deltaY + 2);
         else
-            h = (qint32)(height() + QABS(w * tan(angleY * pi / 180)));
+            h = (qint32)(height() + qAbs(w * tan(angleY * pi / 180)));
     }
 
     if (w != width() || h != height()) {
@@ -532,13 +522,14 @@ void KisImage::shear(double angleX, double angleY, KoUpdater *progress)
 
         if (undo()) {
             m_d->adapter->beginMacro(i18n("Shear Image"));
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), true));
+            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
         }
 
         KisShearVisitor v(angleX, angleY, progress);
+        v.setUndoAdapter(m_d->adapter);
         rootLayer()->accept(v);
 
-        if (undo()) m_d->adapter->addCommand(new KisImageResizeCommand(KisImageSP(this), w, h, width(), height()));
+        if (undo()) m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
 
         m_d->width = w;
         m_d->height = h;
@@ -548,13 +539,13 @@ void KisImage::shear(double angleX, double angleY, KoUpdater *progress)
         unlock();
 
         if (undo()) {
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), false));
+            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
             m_d->adapter->endMacro();
         }
     }
 }
 
-void KisImage::convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTransformation::Intent renderingIntent)
+void KisImage::convertTo(const KoColorSpace *dstColorSpace, KoColorConversionTransformation::Intent renderingIntent)
 {
     if (*m_d->colorSpace == *dstColorSpace) {
         return;
@@ -566,22 +557,23 @@ void KisImage::convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTr
 
     if (undo()) {
         m_d->adapter->beginMacro(i18n("Convert Image Type"));
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), true));
+        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
     }
 
-    setColorSpace(dstColorSpace);
-
-    KisColorSpaceConvertVisitor visitor(dstColorSpace, renderingIntent);
+    KisColorSpaceConvertVisitor visitor(this, dstColorSpace, renderingIntent);
     m_d->rootLayer->accept(visitor);
 
-    unlock();
-
     if (undo()) {
 
-        m_d->adapter->addCommand(new KisImageConvertTypeCommand(KisImageSP(this), oldCs, dstColorSpace));
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), false));
+        m_d->adapter->addCommand(new KisImageConvertTypeCommand(KisImageWSP(this), oldCs, dstColorSpace));
+        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
         m_d->adapter->endMacro();
+    } else {
+        setColorSpace(dstColorSpace);
     }
+
+    unlock();
+    refreshGraph();
 }
 
 const KoColorProfile * KisImage::profile() const
@@ -589,19 +581,20 @@ const KoColorProfile * KisImage::profile() const
     return colorSpace()->profile();
 }
 
-void KisImage::setProfile(const KoColorProfile * profile)
+void KisImage::setProfile(const KoColorProfile *profile)
 {
     if (profile == 0) return;
 
-    const KoColorSpace * dstCs = KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->id(),
-                                 profile);
+    kDebug() << profile;
+
+    const KoColorSpace *dstCs = KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->colorModelId().id(), colorSpace()->colorDepthId().id(), profile);
     if (dstCs) {
 
         lock();
 
-        const KoColorSpace * oldCs = colorSpace();
+        const KoColorSpace *oldCs = colorSpace();
         setColorSpace(dstCs);
-        emit(sigProfileChanged(const_cast<KoColorProfile *>(profile)));
+        emit(sigProfileChanged(const_cast<KoColorProfile*>(profile)));
 
         KisChangeProfileVisitor visitor(oldCs, dstCs);
         m_d->rootLayer->accept(visitor);
@@ -658,6 +651,11 @@ QPointF KisImage::pixelToDocument(const QPoint &pixelCoord) const
     return QPointF((pixelCoord.x() + 0.5) / xRes(), (pixelCoord.y() + 0.5) / yRes());
 }
 
+QRectF KisImage::pixelToDocument(const QRectF &pixelCoord) const
+{
+    return QRectF(pixelToDocument(pixelCoord.topLeft()), pixelToDocument(pixelCoord.bottomRight()));
+}
+
 qint32 KisImage::width() const
 {
     return m_d->width;
@@ -676,9 +674,6 @@ KisGroupLayerSP KisImage::rootLayer() const
 
 KisPaintDeviceSP KisImage::projection()
 {
-    // XXX: Projection moved to the updateStrategy -- is synching really needed?
-    //m_d->projection->sync();
-
     Q_ASSERT(m_d->rootLayer);
     KisPaintDeviceSP projection = m_d->rootLayer->projection();
     Q_ASSERT(projection);
@@ -710,89 +705,121 @@ qint32 KisImage::nHiddenLayers() const
 void KisImage::flatten()
 {
     KisGroupLayerSP oldRootLayer = m_d->rootLayer;
+    KisGroupLayerSP newRootLayer =
+        new KisGroupLayer(this, "root", OPACITY_OPAQUE_U8);
 
-    KisPaintLayer *dst = new KisPaintLayer(this, nextLayerName(), OPACITY_OPAQUE, colorSpace());
-    Q_CHECK_PTR(dst);
-
-    QRect rc = mergedImage()->extent();
-
-    KisPainter gc(dst->paintDevice());
-    gc.bitBlt(rc.x(), rc.y(), COMPOSITE_COPY, mergedImage(), OPACITY_OPAQUE, rc.left(), rc.top(), rc.width(), rc.height());
-
-    setRootLayer(new KisGroupLayer(this, "root", OPACITY_OPAQUE));
-
-    if (undo()) {
-        m_d->adapter->beginMacro(i18n("Flatten Image"));
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), true));
-        m_d->adapter->addCommand(new KisImageChangeLayersCommand(KisImageSP(this), oldRootLayer, m_d->rootLayer, ""));
-    }
+    // - synchronous?
+    // - no =(
+    // - KisProjection::lock() should work like a barier
+    // - fixme?
+    //refreshGraph();
 
     lock();
-
-    addNode(dst, m_d->rootLayer.data(), 0);
-
+    KisPaintDeviceSP projectionCopy =
+        new KisPaintDevice(*oldRootLayer->projection());
     unlock();
 
+    KisPaintLayerSP flattenLayer =
+        new KisPaintLayer(this, nextLayerName(), OPACITY_OPAQUE_U8, projectionCopy);
+    Q_CHECK_PTR(flattenLayer);
+
+    m_d->adapter->beginMacro(i18n("Flatten Image"));
+    m_d->adapter->addCommand(new KisImageChangeLayersCommand(KisImageWSP(this), oldRootLayer, newRootLayer, ""));
+
+    addNode(flattenLayer, newRootLayer, 0);
+
+    // notify once again to be sure tools have set
+    // their current paint devices
     notifyLayersChanged();
 
-    if (undo()) {
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), false));
-        m_d->adapter->endMacro();
-    }
+    m_d->adapter->endMacro();
 }
 
-
+// FIXME: Rename to Merge Down?
 KisLayerSP KisImage::mergeLayer(KisLayerSP layer, const KisMetaData::MergeStrategy* strategy)
 {
-    if (!layer->prevSibling()) return 0;
+    if(!layer->prevSibling()) return 0;
+
     // XXX: this breaks if we allow free mixing of masks and layers
-    KisLayerSP layer2 = dynamic_cast<KisLayer*>(layer->prevSibling().data());
-    dbgImage << "Merge " << layer << " with " << layer2;
-    if (!layer2) return 0;
-
-    KisPaintLayerSP newLayer = new KisPaintLayer(this, layer->name(), OPACITY_OPAQUE, colorSpace());
-    Q_CHECK_PTR(newLayer);
+    KisLayerSP prevLayer = dynamic_cast<KisLayer*>(layer->prevSibling().data());
+    if (!prevLayer) return 0;
 
+    dbgImage << "Merge " << layer << " with " << prevLayer;
 
     QRect layerExtent = layer->extent();
-    QRect layerPrevSiblingExtent = layer->prevSibling()->extent();
-
-    QRect rc = layerExtent | layerPrevSiblingExtent;
-
-    undoAdapter()->beginMacro(i18n("Merge with Layer Below"));
-
-    KisPainter gc(newLayer->paintDevice());
-    gc.setCompositeOp(newLayer->colorSpace()->compositeOp(COMPOSITE_COPY));
-    gc.bitBlt(rc.topLeft(), layer2->projection(), rc);
+    QRect prevLayerExtent = prevLayer->extent();
+    KisPaintDeviceSP mergedDevice = new KisPaintDevice(*prevLayer->projection());
 
+    KisPainter gc(mergedDevice);
+    gc.setChannelFlags(layer->channelFlags());
     gc.setCompositeOp(layer->compositeOp());
     gc.setOpacity(layer->opacity());
-    gc.bitBlt(rc.topLeft(), layer->projection(), rc);
+    gc.bitBlt(layerExtent.topLeft(), layer->projection(), layerExtent);
+
+    // FIXME: "Merge Down"?
+    undoAdapter()->beginMacro(i18n("Merge with Layer Below"));
+
+    KisPaintLayerSP mergedLayer = new KisPaintLayer(this, prevLayer->name(), OPACITY_OPAQUE_U8, mergedDevice);
+    Q_CHECK_PTR(mergedLayer);
+
 
     // Merge meta data
     QList<const KisMetaData::Store*> srcs;
-    srcs.append(static_cast<KisLayer*>(layer->prevSibling().data())->metaData());
+    srcs.append(prevLayer->metaData());
     srcs.append(layer->metaData());
     QList<double> scores;
-    int layerPrevSiblingArea = layerPrevSiblingExtent.width() * layerPrevSiblingExtent.height();
+    int layerPrevSiblingArea = prevLayerExtent.width() * prevLayerExtent.height();
     int layerArea = layerExtent.width() * layerExtent.height();
     double norm = qMax(layerPrevSiblingArea, layerArea);
     scores.append(layerPrevSiblingArea / norm);
     scores.append(layerArea / norm);
-    strategy->merge(newLayer->metaData(), srcs, scores);
+    strategy->merge(mergedLayer->metaData(), srcs, scores);
 
     KisNodeSP parent = layer->parent(); // parent is set to null when the layer is removed from the node
-    dbgImage << ppVar( parent );
-//     addNode(newLayer, parent, layer);
-//     removeNode(layer->prevSibling());
-//     removeNode(layer);
+    dbgImage << ppVar(parent);
+
+    // XXX: merge the masks!
+    // AAA: do you really think you need it? ;)
+
+    undoAdapter()->addCommand(new KisImageLayerAddCommand(this, mergedLayer, parent, layer));
+    undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, prevLayer));
+    undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, layer));
 
-    undoAdapter()->addCommand(new KisImageLayerAddCommand(this, newLayer, parent, layer ));
-    undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, layer->prevSibling() ));
-    undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, layer ));
-    
     undoAdapter()->endMacro();
-    
+
+    return mergedLayer;
+}
+
+KisLayerSP KisImage::flattenLayer(KisLayerSP layer)
+{
+    if (!layer->firstChild()) return layer;
+
+    undoAdapter()->beginMacro(i18n("Flatten Layer"));
+
+    KisPaintDeviceSP mergedDevice = new KisPaintDevice(*layer->projection());
+    KisPaintLayerSP newLayer = new KisPaintLayer(this, layer->name(), layer->opacity(), mergedDevice);
+    newLayer->setCompositeOp(layer->compositeOp()->id());
+
+    undoAdapter()->addCommand(new KisImageLayerAddCommand(this, newLayer, layer->parent(), layer));
+
+    KisNodeSP node = layer->firstChild();
+    while (node) {
+        undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, node));
+        node = node->nextSibling();
+    }
+    undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, layer));
+
+
+    QList<const KisMetaData::Store*> srcs;
+    srcs.append(layer->metaData());
+
+    const KisMetaData::MergeStrategy* strategy = KisMetaData::MergeStrategyRegistry::instance()->get("Smart");
+    QList<double> scores;
+    scores.append(1.0); //Just give some score, there only is one layer
+    strategy->merge(newLayer->metaData(), srcs, scores);
+
+    undoAdapter()->endMacro();
+
     return newLayer;
 }
 
@@ -811,8 +838,8 @@ void KisImage::renderToPainter(qint32 srcX,
                                QPainter &painter,
                                const KoColorProfile *  monitorProfile)
 {
-    QImage img = convertToQImage(srcX, srcY, width, height, monitorProfile);
-    painter.drawImage(dstX, dstY, img, 0, 0, width, height);
+    QImage image = convertToQImage(srcX, srcY, width, height, monitorProfile);
+    painter.drawImage(dstX, dstY, image, 0, 0, width, height);
 }
 
 QImage KisImage::convertToQImage(qint32 x,
@@ -823,12 +850,14 @@ QImage KisImage::convertToQImage(qint32 x,
 {
     KisPaintDeviceSP dev = m_d->rootLayer->projection();
     if (!dev) return QImage();
-    QImage img = dev->convertToQImage(const_cast<KoColorProfile*>(profile), x, y, w, h);
-
-    if (!img.isNull()) {
+    QImage image = dev->convertToQImage(const_cast<KoColorProfile*>(profile), x, y, w, h);
 
+    if (m_d->backgroundPattern) {
+        m_d->backgroundPattern->paintBackground(image, QRect(x, y, w, h));
+    }
+    if (!image.isNull()) {
 #ifdef WORDS_BIGENDIAN
-        uchar * data = img.bits();
+        uchar * data = image.bits();
         for (int i = 0; i < w * h; ++i) {
             uchar r, g, b, a;
             a = data[0];
@@ -843,55 +872,55 @@ QImage KisImage::convertToQImage(qint32 x,
         }
 #endif
 
-        return img;
+        return image;
     }
 
     return QImage();
 }
 
 
-QImage KisImage::convertToQImage(const QRect& r, const double xScale, const double yScale, const KoColorProfile *profile, KisSelectionSP mask)
+
+QImage KisImage::convertToQImage(const QRect& scaledRect, const QSize& scaledImageSize, const KoColorProfile *profile)
 {
-    Q_UNUSED(mask)
 
-#ifdef __GNUC__
-#warning "KisImage::convertToQImage: Implement direct rendering of current mask onto scaled image pixels"
-#endif
-
-    if (r.isEmpty()) {
+    if (scaledRect.isEmpty() || scaledImageSize.isEmpty()) {
         return QImage();
     }
 
+    qint32 imageWidth = width();
+    qint32 imageHeight = height();
     quint32 pixelSize = colorSpace()->pixelSize();
 
+    double xScale = static_cast<double>(imageWidth) / scaledImageSize.width();
+    double yScale = static_cast<double>(imageHeight) / scaledImageSize.height();
+
     QRect srcRect;
 
-    srcRect.setLeft(static_cast<int>(r.left() * xScale));
-    srcRect.setRight(static_cast<int>(ceil((r.right() + 1) * xScale)) - 1);
-    srcRect.setTop(static_cast<int>(r.top() * yScale));
-    srcRect.setBottom(static_cast<int>(ceil((r.bottom() + 1) * yScale)) - 1);
+    srcRect.setLeft(static_cast<int>(scaledRect.left() * xScale));
+    srcRect.setRight(static_cast<int>(ceil((scaledRect.right() + 1) * xScale)) - 1);
+    srcRect.setTop(static_cast<int>(scaledRect.top() * yScale));
+    srcRect.setBottom(static_cast<int>(ceil((scaledRect.bottom() + 1) * yScale)) - 1);
 
     KisPaintDeviceSP mergedImage = m_d->rootLayer->projection();
-
-    quint8 *scaledImageData = new quint8[r.width() * r.height() * pixelSize];
+    quint8 *scaledImageData = new quint8[scaledRect.width() * scaledRect.height() * pixelSize];
 
     quint8 *imageRow = new quint8[srcRect.width() * pixelSize];
     const qint32 imageRowX = srcRect.x();
 
-    for (qint32 y = 0; y < r.height(); ++y) {
+    for (qint32 y = 0; y < scaledRect.height(); ++y) {
 
-        qint32 dstY = r.y() + y;
-        qint32 dstX = r.x();
-        qint32 srcY = int(dstY * yScale);
+        qint32 dstY = scaledRect.y() + y;
+        qint32 dstX = scaledRect.x();
+        qint32 srcY = (dstY * imageHeight) / scaledImageSize.height();
 
         mergedImage->readBytes(imageRow, imageRowX, srcY, srcRect.width(), 1);
 
-        quint8 *dstPixel = scaledImageData + (y * r.width() * pixelSize);
-        quint32 columnsRemaining = r.width();
+        quint8 *dstPixel = scaledImageData + (y * scaledRect.width() * pixelSize);
+        quint32 columnsRemaining = scaledRect.width();
 
         while (columnsRemaining > 0) {
 
-            qint32 srcX = int(dstX * xScale);
+            qint32 srcX = (dstX * imageWidth) / scaledImageSize.width();
 
             memcpy(dstPixel, imageRow + ((srcX - imageRowX) * pixelSize), pixelSize);
 
@@ -900,13 +929,17 @@ QImage KisImage::convertToQImage(const QRect& r, const double xScale, const doub
             --columnsRemaining;
         }
     }
-
     delete [] imageRow;
 
-    QImage image = colorSpace()->convertToQImage(scaledImageData, r.width(), r.height(), const_cast<KoColorProfile*>(profile), KoColorConversionTransformation::IntentPerceptual);
+    QImage image = colorSpace()->convertToQImage(scaledImageData, scaledRect.width(), scaledRect.height(), const_cast<KoColorProfile*>(profile), KoColorConversionTransformation::IntentPerceptual);
+
+    if (m_d->backgroundPattern) {
+        m_d->backgroundPattern->paintBackground(image, scaledRect, scaledImageSize, QSize(imageWidth, imageHeight));
+    }
+
     delete [] scaledImageData;
 
-#ifdef WORDS_BIGENDIAN
+#ifdef __BIG_ENDIAN__
     uchar * data = image.bits();
     for (int i = 0; i < image.width() * image.height(); ++i) {
         uchar r, g, b, a;
@@ -921,11 +954,14 @@ QImage KisImage::convertToQImage(const QRect& r, const double xScale, const doub
         data += 4;
     }
 #endif
+
     return image;
 }
 
+
 KisPaintDeviceSP KisImage::mergedImage()
 {
+    refreshGraph();
     return m_d->rootLayer->projection();
 }
 
@@ -975,16 +1011,18 @@ const KoColorSpace * KisImage::colorSpace() const
 void KisImage::setColorSpace(const KoColorSpace * colorSpace)
 {
     m_d->colorSpace = colorSpace;
-    m_d->rootLayer->resetProjection();
+    m_d->rootLayer->resetCache();
     emit sigColorSpaceChanged(colorSpace);
 }
 
 void KisImage::setRootLayer(KisGroupLayerSP rootLayer)
 {
+    if(m_d->rootLayer)
+        m_d->rootLayer->disconnect();
+
     m_d->rootLayer = rootLayer;
     m_d->rootLayer->disconnect();
     m_d->rootLayer->setGraphListener(this);
-    m_d->rootLayer->updateStrategy()->setImage(this);
     setRoot(m_d->rootLayer.data());
 }
 
@@ -1032,14 +1070,17 @@ vKisAnnotationSP_it KisImage::beginAnnotations()
     KisAnnotationSP annotation;
 
     if (profile) {
-        // XXX we hardcode icc, this is correct for lcms?
+#ifdef __GNUC__
+#warning "KisImage::beginAnnotations: make it possible to save any profile, not just icc profiles."
+#endif
+#if 0
+        // XXX we hardcode icc, this is correct for icc?
         // XXX productName(), or just "ICC Profile"?
-        if (profile->valid()) {
-            const KoIccColorProfile* iccprofile = dynamic_cast<const KoIccColorProfile*>(profile);
-            if (!iccprofile->rawData().isEmpty()) {
-                annotation = new  KisAnnotation("icc", iccprofile->name(), iccprofile->rawData());
+        if (profile->valid() && profile->type() == "icc" && !profile->rawData().isEmpty()) {
+                annotation = new  KisAnnotation("icc", profile->name(), profile->rawData());
             }
         }
+#endif        
     }
 
     if (annotation)
@@ -1062,10 +1103,27 @@ KisPerspectiveGrid* KisImage::perspectiveGrid()
     return m_d->perspectiveGrid;
 }
 
+void KisImage::refreshGraph(KisNodeSP root)
+{
+    if (!root) root = m_d->rootLayer;
+
+    if (!locked() && m_d->projection) {
+        m_d->projection->fullRefresh(root);
+    }
+}
+
 void KisImage::slotProjectionUpdated(const QRect & rc)
 {
     emit sigImageUpdated(rc);
 }
 
+void KisImage::updateProjection(KisNodeSP node, const QRect& rc)
+{
+    if (!locked() && m_d->projection) {
+        dbgImage << "KisImage: requested and update for" << node->name() << rc;
+        m_d->projection->updateProjection(node, rc);
+    }
+}
+
 #include "kis_image.moc"
 
