@@ -1,7 +1,7 @@
 /*
  *  Copyright (c) 2004 Michael Thaler <michael.thaler@physik.tu-muenchen.de> filters
  *  Copyright (c) 2005-2007 Casper Boemann <cbr@boemann.dk>
- *  Copyright (c) 2005 Boudewijn Rempt <boud@valdyas.org> right angle rotators
+ *  Copyright (c) 2005, 2010 Boudewijn Rempt <boud@valdyas.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 #include <KoProgressUpdater.h>
 #include <KoColorSpace.h>
+#include <KoCompositeOp.h>
 
 #include "kis_paint_device.h"
 #include "kis_debug.h"
@@ -39,7 +40,7 @@ KisTransformWorker::KisTransformWorker(KisPaintDeviceSP dev,
                                        double xshear, double yshear,
                                        double rotation,
                                        qint32 xtranslate, qint32 ytranslate,
-                                       KoUpdater *progress,
+                                       KoUpdaterPtr progress,
                                        KisFilterStrategy *filter,
                                        bool fixBorderAlpha)
 {
@@ -58,7 +59,9 @@ KisTransformWorker::KisTransformWorker(KisPaintDeviceSP dev,
 
 KisTransformWorker::~KisTransformWorker()
 {
-}void KisTransformWorker::rotateNone(KisPaintDeviceSP src, KisPaintDeviceSP dst)
+}
+
+void KisTransformWorker::rotateNone(KisPaintDeviceSP src, KisPaintDeviceSP dst)
 {
     qint32 pixelSize = src->pixelSize();
     QRect r;
@@ -81,11 +84,11 @@ KisTransformWorker::~KisTransformWorker()
 
         //progress info
         m_progressStep += r.width();
-        if(m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
+        if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
             m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
-            m_progressUpdater->setProgress(m_lastProgressReport);
+            if (!m_progressUpdater.isNull()) m_progressUpdater->setProgress(m_lastProgressReport);
         }
-        if (m_progressUpdater->interrupted()) {
+        if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
             break;
         }
     }
@@ -112,11 +115,11 @@ void KisTransformWorker::rotateRight90(KisPaintDeviceSP src, KisPaintDeviceSP ds
 
         //progress info
         m_progressStep += r.width();
-        if(m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
+        if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
             m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
-            m_progressUpdater->setProgress(m_lastProgressReport);
+            if (m_progressUpdater) m_progressUpdater->setProgress(m_lastProgressReport);
         }
-        if (m_progressUpdater->interrupted()) {
+        if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
             break;
         }
     }
@@ -147,11 +150,11 @@ void KisTransformWorker::rotateLeft90(KisPaintDeviceSP src, KisPaintDeviceSP dst
 
         //progress info
         m_progressStep += r.width();
-        if(m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
+        if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
             m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
-            m_progressUpdater->setProgress(m_lastProgressReport);
+            if (m_progressUpdater) m_progressUpdater->setProgress(m_lastProgressReport);
         }
-        if (m_progressUpdater->interrupted()) {
+        if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
             break;
         }
     }
@@ -177,14 +180,14 @@ void KisTransformWorker::rotate180(KisPaintDeviceSP src, KisPaintDeviceSP dst)
             ++dstIt;
         }
         srcIt.nextRow();
-        
+
         //progress info
         m_progressStep += r.width();
-        if(m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
+        if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
             m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
-            m_progressUpdater->setProgress(m_lastProgressReport);
+            if (m_progressUpdater) m_progressUpdater->setProgress(m_lastProgressReport);
         }
-        if (m_progressUpdater->interrupted()) {
+        if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
             break;
         }
     }
@@ -239,9 +242,10 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
                                        double floatscale, double shear, qint32 dx,
                                        KisFilterStrategy *filterStrategy, bool fixBorderAlpha)
 {
+    m_fixBorderAlpha = fixBorderAlpha;
+
     qint32 lineNum, srcStart, firstLine, srcLen, numLines;
     qint32 center, begin, end;    /* filter calculation variables */
-    quint8 *data;
     quint8 pixelSize = src->pixelSize();
     KoColorSpace * cs = src->colorSpace();
     KoMixColorsOp * mixOp = cs->mixColorsOp();
@@ -346,7 +350,7 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
         // Build a temporary line
         T srcIt = createIterator <T>(src, srcStart, lineNum, srcLen);
         quint8 *data;
-        qint32 i=0;
+        qint32 i = 0;
         data = srcIt.rawData(); // take the first pixel as source - in effect duplicating the pixel
         while (i < extraLen) {
             memcpy(&tmpLine[i*pixelSize], data, pixelSize);
@@ -356,8 +360,8 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
         while (i < srcLen + extraLen) {
             data = srcIt.rawData();
             memcpy(&tmpLine[i*pixelSize], data, pixelSize);
-            cs->setAlpha(data, 0, 1);
-            if(i < srcLen + extraLen-1) // duplicate pixels along edge
+            cs->setOpacity(data, quint8(0), 1);
+            if (i < srcLen + extraLen - 1) // duplicate pixels along edge
                 ++srcIt;
             i++;
         }
@@ -400,22 +404,22 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
             data = dstIt.rawData();
             mixOp->mixColors(colors, filterWeights[center&255].weight, filterWeights[center&255].numWeights, data);
 
-/*
-            //possibly fix the alpha of the border if user wants it
-            if (fixBorderAlpha && (i == 0 || i == dstLen - 1))
-                cs->setAlpha(data, cs->alpha(&tmpLine[(center>>8)*pixelSize]), 1);
-*/
+            /*
+                        //possibly fix the alpha of the border if user wants it
+                        if (fixBorderAlpha && (i == 0 || i == dstLen - 1))
+                            cs->setAlpha(data, cs->alpha(&tmpLine[(center>>8)*pixelSize]), 1);
+            */
             ++dstIt;
             i++;
         }
-        
+
         //progress info
         m_progressStep += dstLen;
-        if(m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
+        if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
             m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
-            m_progressUpdater->setProgress(m_lastProgressReport);
+            if (!m_progressUpdater.isNull()) m_progressUpdater->setProgress(m_lastProgressReport);
         }
-        if (m_progressUpdater->interrupted()) {
+        if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
             break;
         }
     }
@@ -432,7 +436,7 @@ bool KisTransformWorker::run()
     m_progressStep = 0;
     QRect r = m_dev->exactBounds();
 
-    KisPaintDeviceSP tmpdev1 = KisPaintDeviceSP(new KisPaintDevice(m_dev->colorSpace(), "transform_tmpdev1"));
+    KisPaintDeviceSP tmpdev1 = KisPaintDeviceSP(new KisPaintDevice(m_dev->colorSpace()));
     KisPaintDeviceSP srcdev = m_dev;
 
     double xscale = m_xscale;
@@ -519,28 +523,27 @@ bool KisTransformWorker::run()
             rotateNone(srcdev, m_dev); //copy it back
         }
         //progress info
-        m_progressUpdater->setProgress(100);
+        if (!m_progressUpdater.isNull()) m_progressUpdater->setProgress(100);
+        return false;
 
-        return false;
-        return m_progressUpdater->interrupted();
     }
-    
-        if ( m_progressUpdater->interrupted()) {
-            m_progressUpdater->setProgress(100);
-            return false;
-        }
-    
+
+    if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
+        if (!m_progressUpdater.isNull()) m_progressUpdater->setProgress(100);
+        return false;
+    }
+
     transformPass <KisHLineIteratorPixel>(srcdev.data(), srcdev.data(), xscale, yscale*xshear, 0, m_filter, m_fixBorderAlpha);
-    
-    if ( m_progressUpdater->interrupted()) {
+
+    if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
         m_progressUpdater->setProgress(100);
         return false;
     }
-    
+
     // Now do the second pass
     transformPass <KisVLineIteratorPixel>(srcdev.data(), srcdev.data(), yscale, yshear, ytranslate, m_filter, m_fixBorderAlpha);
-    
-    if ( m_progressUpdater->interrupted()) {
+
+    if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
         m_progressUpdater->setProgress(100);
         return false;
     }
@@ -557,21 +560,17 @@ bool KisTransformWorker::run()
     //CBRm_dev->setDirty();
 
     //progress info
-    m_progressUpdater->setProgress(100);
+    if (!m_progressUpdater.isNull()) m_progressUpdater->setProgress(100);
 
-    return m_progressUpdater->interrupted();
+    if (!m_progressUpdater.isNull()) return m_progressUpdater->interrupted();
+    return true;
 }
 
 QRect KisTransformWorker::mirrorX(KisPaintDeviceSP dev, const KisSelection* selection)
 {
-    KisPaintDeviceSP dst = dev;
-    bool hasCurrentMemento = dev->dataManager()->hasCurrentMemento();
     int pixelSize = dev->pixelSize();
-    
-    if (!hasCurrentMemento) {
-            dst = new KisPaintDevice(dev->colorSpace());
-    }
-    
+    KisPaintDeviceSP dst = new KisPaintDevice(dev->colorSpace());
+
     QRect r;
     if (selection) {
         r = selection->selectedExactRect();
@@ -588,10 +587,7 @@ QRect KisTransformWorker::mirrorX(KisPaintDeviceSP dev, const KisSelection* sele
 
             while (!srcIt.isDone()) {
                 if (srcIt.isSelected()) {
-                    if (hasCurrentMemento)
-                        memcpy(dstIt.rawData(), srcIt.oldRawData(), pixelSize);
-                    else
-                        memcpy(dstIt.rawData(), srcIt.rawData(), pixelSize);
+                    memcpy(dstIt.rawData(), srcIt.rawData(), pixelSize);
                 }
                 ++srcIt;
                 --dstIt;
@@ -601,23 +597,24 @@ QRect KisTransformWorker::mirrorX(KisPaintDeviceSP dev, const KisSelection* sele
             dstIt.nextRow();
         }
     }
-    if (!hasCurrentMemento) {
-        dev->setDataManager( dst->dataManager(), dst->colorSpace() );
+    KisPainter gc(dev);
+
+    if (selection) {
+        dev->clearSelection(const_cast<KisSelection*>(selection));
     }
+    else {
+        dev->clear(r);
+    }
+    gc.setCompositeOp(COMPOSITE_OVER);
+    gc.bitBlt(r.topLeft(), dst, r);
 
     return r;
 }
 
 QRect KisTransformWorker::mirrorY(KisPaintDeviceSP dev, const KisSelection* selection)
 {
-    KisPaintDeviceSP dst = dev;
-    bool hasCurrentMemento = dev->dataManager()->hasCurrentMemento();
-    
     int pixelSize = dev->pixelSize();
-    
-    if (!hasCurrentMemento) {
-        dst = new KisPaintDevice(dev->colorSpace());
-    }
+    KisPaintDeviceSP dst = new KisPaintDevice(dev->colorSpace());
 
     /* Read a line from bottom to top and and from top to bottom and write their values to each other */
     QRect r;
@@ -630,24 +627,28 @@ QRect KisTransformWorker::mirrorY(KisPaintDeviceSP dev, const KisSelection* sele
         qint32 y1, y2;
         for (y1 = r.top(), y2 = r.bottom(); y1 <= r.bottom(); ++y1, --y2) {
 
-            KisHLineIteratorPixel itTop = dst->createHLineIterator(r.x(), y1, r.width(), selection);
-            KisHLineConstIteratorPixel itBottom = dev->createHLineConstIterator(r.x(), y2, r.width());
+            KisHLineConstIteratorPixel itTop = dev->createHLineConstIterator(r.x(), y1, r.width(), selection);
+            KisHLineIteratorPixel itBottom = dst->createHLineIterator(r.x(), y2, r.width());
 
             while (!itTop.isDone() && !itBottom.isDone()) {
-                if (itBottom.isSelected()) {
-                    if (hasCurrentMemento)
-                        memcpy(itTop.rawData(), itBottom.oldRawData(), pixelSize);
-                    else
-                        memcpy(itTop.rawData(), itBottom.rawData(), pixelSize);
+                if (itTop.isSelected()) {
+                    memcpy(itBottom.rawData(), itTop.rawData(), pixelSize);
                 }
                 ++itBottom;
                 ++itTop;
             }
         }
     }
-    if (!hasCurrentMemento) {
-        dev->setDataManager( dst->dataManager(), dst->colorSpace() );
+    KisPainter gc(dev);
+
+    if (selection) {
+        dev->clearSelection(const_cast<KisSelection*>(selection));
     }
+    else {
+        dev->clear(r);
+    }
+    gc.setCompositeOp(COMPOSITE_OVER);
+    gc.bitBlt(r.topLeft(), dst, r);
 
     return r;
 }
