@@ -21,7 +21,7 @@
 
 #include <kis_debug.h>
 #include <kfiledialog.h>
-#include <kgenericfactory.h>
+#include <kpluginfactory.h>
 #include <kcomponentdata.h>
 #include <klocale.h>
 #include <kmessagebox.h>
@@ -61,15 +61,16 @@
 #undef powf
 #define powf pow
 #endif
+#include <KoColorModelStandardIds.h>
 
 const double epsilonOptimization = 1e-5;
 const double epsilonGaussian = 1e-6;
 
-typedef KGenericFactory<Bracketing2HDRPlugin> Bracketing2HDRPluginFactory;
-K_EXPORT_COMPONENT_FACTORY(kritabracketing2hdr, Bracketing2HDRPluginFactory("krita"))
+K_PLUGIN_FACTORY(Bracketing2HDRPluginFactory, registerPlugin<Bracketing2HDRPlugin>();)
+K_EXPORT_PLUGIN(Bracketing2HDRPluginFactory("krita"))
 
 
-Bracketing2HDRPlugin::Bracketing2HDRPlugin(QObject *parent, const QStringList &)
+Bracketing2HDRPlugin::Bracketing2HDRPlugin(QObject *parent, const QVariantList &)
         : KParts::Plugin(parent), m_wdgBracketing2HDR(0),
         m_responseType(RESPONSE_LINEAR), m_bitDepth(16), m_numberOfInputLevels(2 << (m_bitDepth - 1)), m_cameraResponseIsComputed(false)
 {
@@ -172,13 +173,13 @@ void Bracketing2HDRPlugin::slotNewHDRLayerFromBracketing()
         if (!m_cameraResponseIsComputed) {
             computeCameraResponse();
         }
-        KisImageWSP img = m_view->image();
-        const KoColorSpace* cs = KoColorSpaceRegistry::instance()->colorSpace("RgbAF32", 0);
+        KisImageWSP image = m_view->image();
+        const KoColorSpace* cs = KoColorSpaceRegistry::instance()->colorSpace(RGBAColorModelID.id(), Float32BitsColorDepthID.id(), 0);
         if (!cs) {
             KMessageBox::error(m_view, i18n("HDR colorspace RGBAF32 not found, please check your installation."), i18n("Layer Creation Error"));
             return;
         }
-        KisPaintLayerSP layer = new KisPaintLayer(img, i18n("HDR Layer"), 255, cs);
+        KisPaintLayerSP layer = new KisPaintLayer(image, i18n("HDR Layer"), 255, cs);
         Q_ASSERT(layer);
         KisGroupLayerSP parent;
         KisLayerSP above;
@@ -188,7 +189,7 @@ void Bracketing2HDRPlugin::slotNewHDRLayerFromBracketing()
         if (!parent)
             parent = m_view->image()->rootLayer();
         above = qobject_cast<KisLayer*>(parent->firstChild().data());
-        img->addNode(layer.data(), parent.data(), above.data());
+        image->addNode(layer.data(), parent.data(), above.data());
 
         createHDRPaintDevice(m_imagesFrames, layer->paintDevice());
         m_view->canvas()->update();
@@ -243,7 +244,7 @@ void Bracketing2HDRPlugin::createHDRPaintDevice(QList<BracketingFrame> frames,
             } else {
                 pixelData[KoRgbU16Traits::blue_pos] = 0.0;
             }
-            device->colorSpace()->setAlpha(dstIt.rawData(), 255, 1);
+            device->colorSpace()->setOpacity(dstIt.rawData(), OPACITY_OPAQUE_F, 1);
             ++dstIt;
         }
         dstIt.nextRow();
@@ -358,7 +359,7 @@ void Bracketing2HDRPlugin::computeCameraResponse()
 }
 void Bracketing2HDRPlugin::computeCameraResponse(QList<BracketingFrame> frames)
 {
-    if(frames.size() < 1) return;
+    Q_ASSERT(frames.size() > 0);
 //     return;
     // Normalize the intensity responses
     normalize(m_intensityR);
@@ -367,9 +368,9 @@ void Bracketing2HDRPlugin::computeCameraResponse(QList<BracketingFrame> frames)
 #if 1
     // Now optimize the camera response
     // Create a temporary paint device and fill it with the current value
-    const KoColorSpace* cs = KoColorSpaceRegistry::instance()->colorSpace("RgbAF32", 0);
+    const KoColorSpace* cs = KoColorSpaceRegistry::instance()->colorSpace(RGBAColorModelID.id(), Float32BitsColorDepthID.id(), 0);
     if (!cs) {
-        KMessageBox::error(m_view, i18n("HDR colorspace RGBAF32 not found, please check your installation."), i18n("Layer Creation Error"));
+        KMessageBox::error(m_view, i18n("HDR colorspace RGBAF32 not found, please check your installation."), i18n("Compute Camera Response Error"));
         return;
     }
     KisPaintDeviceSP device = new KisPaintDevice(cs);
