@@ -21,16 +21,13 @@
 #ifndef BLURPLUGIN_H
 #define BLURPLUGIN_H
 
-// TODO: remove that
-#define LCMS_HEADER <lcms.h>
-// TODO: remove it !
+#include <QObject>
+#include <QVariant>
 
-#include <kparts/plugin.h>
-
-class BlurFilterPlugin : public KParts::Plugin
+class BlurFilterPlugin : public QObject
 {
 public:
-    BlurFilterPlugin(QObject *parent, const char *name, const QStringList &);
+    BlurFilterPlugin(QObject *parent, const QVariantList &);
     virtual ~BlurFilterPlugin();
 };
 
