@@ -24,7 +24,7 @@
 #include <QPair>
 #include <QList>
 
-#include "filter/kis_filter.h"
+#include "filter/kis_color_transformation_filter.h"
 #include "filter/kis_filter_configuration.h"
 #include "kis_config_widget.h"
 
@@ -40,29 +40,30 @@ public:
     }
 };
 
-typedef QList<QPointF> KisCurve;
-
 class KisPerChannelFilterConfiguration
-            : public KisFilterConfiguration
+        : public KisFilterConfiguration
 {
 public:
     KisPerChannelFilterConfiguration(int n);
     ~KisPerChannelFilterConfiguration();
 
     using KisFilterConfiguration::fromXML;
+    using KisFilterConfiguration::toXML;
+    using KisFilterConfiguration::fromLegacyXML;
 
-    virtual void fromXML(const QString&);
-    virtual QString toString();
+    virtual void fromLegacyXML(const QDomElement& root);
 
+    virtual void fromXML(const QDomElement& e);
+    virtual void toXML(QDomDocument& doc, QDomElement& root) const;
+
+    void setCurves(QList<KisCubicCurve> &curves);
+    static inline void initDefaultCurves(QList<KisCubicCurve> &curves, int nCh);
     bool isCompatible(const KisPaintDeviceSP) const;
 
 public:
-    QList<KisCurve> curves;
-    quint16 **transfers;
-    quint16 nTransfers;
-    // Caching of adjustment
-    bool dirty;
+    QList<KisCubicCurve> m_curves;
     const KoColorSpace* oldCs;
+
 };
 
 
@@ -70,20 +71,16 @@ public:
  * This class is generic for filters that affect channel separately
  */
 class KisPerChannelFilter
-            : public KisFilter
+        : public KisColorTransformationFilter
 {
 public:
     KisPerChannelFilter();
 public:
-    virtual KisConfigWidget * createConfigurationWidget(QWidget* parent, const KisPaintDeviceSP dev, const KisImageSP image = 0) const;
+    virtual KisConfigWidget * createConfigurationWidget(QWidget* parent, const KisPaintDeviceSP dev, const KisImageWSP image = 0) const;
+    virtual KisFilterConfiguration * factoryConfiguration(const KisPaintDeviceSP) const;
+
+    virtual KoColorTransformation* createTransformation(const KoColorSpace* cs, const KisFilterConfiguration* config) const;
 
-    using KisFilter::process;
-    void process(KisConstProcessingInformation src,
-                 KisProcessingInformation dst,
-                 const QSize& size,
-                 const KisFilterConfiguration* config,
-                 KoUpdater* progressUpdater
-                ) const;
     static inline KoID id() {
         return KoID("perchannel", i18n("Color Adjustment"));
     }
@@ -92,7 +89,6 @@ private:
 
 class KisPerChannelConfigWidget : public KisConfigWidget
 {
-
     Q_OBJECT
 
 public:
@@ -104,13 +100,24 @@ public:
 
 private slots:
     virtual void setActiveChannel(int ch);
+    void setPreview(int state);
 
 private:
+
+    // private routines
+    inline QPixmap getHistogram();
+    inline QPixmap createGradient(Qt::Orientation orient /*, int invert (not used now) */);
+
+    // members
     WdgPerChannel * m_page;
     KisPaintDeviceSP m_dev;
     KisHistogram *m_histogram;
-    mutable QList<KisCurve> m_curves;
+    mutable QList<KisCubicCurve> m_curves;
     int m_activeCh;
+
+    // scales for displaying color numbers
+    double m_scale;
+    double m_shift;
 };
 
 #endif
