@@ -20,18 +20,18 @@
 
 #ifndef _KIS_POLYGON_H_
 #define _KIS_POLYGON_H_
+#include <QPointF>
+#include <QVector>
 
-#include <qvaluevector.h>
-
-typedef QValueVector<KisPoint> KisPointVector;
-class KisPolygon : public KisPointVector
+typedef QVector<QPointF> KoPointVector;
+class KisPolygon : public KoPointVector
 {
-        public:
-                void addPoint(double x, double y);
-                void translate(double tx, double ty);
-                void rotate(double theta);
-                Q_INT32 extents(double &minX, double &minY, double &maxX, double &maxY);
-                Q_INT32 numberOfPoints();
+public:
+    void addPoint(double x, double y);
+    void translate(double tx, double ty);
+    void rotate(double theta);
+    qint32 extents(double &minX, double &minY, double &maxX, double &maxY);
+    qint32 numberOfPoints();
 };
 
 #endif
