@@ -17,28 +17,27 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
- 
+
 #ifndef _KIS_WDG_UNSHARP_H_
 #define _KIS_WDG_UNSHARP_H_
 
-// TODO: remove that
-#define LCMS_HEADER <lcms.h>
-// TODO: remove it !
-
-#include <kis_filter_config_widget.h>
+#include <kis_config_widget.h>
 
 class KisFilter;
-class WdgUnsharp;
+class Ui_WdgUnsharp;
 
-class KisWdgUnsharp : public KisFilterConfigWidget
+class KisWdgUnsharp : public KisConfigWidget
 {
     Q_OBJECT
-    public:
-        KisWdgUnsharp( KisFilter* nfilter, QWidget * parent, const char * name);
-        inline WdgUnsharp* widget() { return m_widget; };
-        virtual void setConfiguration(KisFilterConfiguration*);
-    private:
-        WdgUnsharp* m_widget;
+public:
+    KisWdgUnsharp(QWidget * parent);
+    inline const Ui_WdgUnsharp* widget() const {
+        return m_widget;
+    }
+    virtual void setConfiguration(const KisPropertiesConfiguration*);
+    virtual KisPropertiesConfiguration* configuration() const;
+private:
+    Ui_WdgUnsharp* m_widget;
 };
 
 #endif
