@@ -21,16 +21,13 @@
 #ifndef _UNSHARP_PLUGIN_H_
 #define _UNSHARP_PLUGIN_H_
 
-// TODO: remove that
-#define LCMS_HEADER <lcms.h>
-// TODO: remove it !
+#include <QObject>
+#include <QVariant>
 
-#include <kparts/plugin.h>
-
-class UnsharpPlugin : public KParts::Plugin
+class UnsharpPlugin : public QObject
 {
 public:
-    UnsharpPlugin(QObject *parent, const char *name, const QStringList &);
+    UnsharpPlugin(QObject *parent, const QVariantList &);
     virtual ~UnsharpPlugin();
 };
 
