@@ -17,22 +17,24 @@
  */
 
 #include "complexbrush.h"
-#include "kis_complexop_factory.h"
+#include "kis_simple_paintop_factory.h"
+#include "kis_complexop.h"
+#include "kis_complexop_settings.h"
 
 #include <KGenericFactory>
 #include <kis_paintop_registry.h>
 #include <QStringList>
 
-typedef KGenericFactory<ComplexBrush> ComplexBrushFactory;
-K_EXPORT_COMPONENT_FACTORY(kritacomplexbrush, ComplexBrushFactory("krita"))
+K_PLUGIN_FACTORY(ComplexBrushFactory, registerPlugin<ComplexBrush>();)
+K_EXPORT_PLUGIN(ComplexBrushFactory("krita"))
 
-ComplexBrush::ComplexBrush(QObject *parent, const QStringList &sl)
-        : KParts::Plugin(parent)
+ComplexBrush::ComplexBrush(QObject *parent, const QVariantList &sl)
+        : QObject(parent)
 {
     Q_UNUSED(sl)
 
-    setComponentData(ComplexBrushFactory::componentData());
-    KisPaintOpRegistry::instance()->add(new KisComplexOpFactory);
+    //setComponentData(ComplexBrushFactory::componentData());
+    KisPaintOpRegistry::instance()->add(new KisSimplePaintOpFactory<KisComplexOp, KisComplexOpSettings, KisComplexOpSettingsWidget>("complex", i18n("Mixing Brush"), "krita-mixing.png"));
 }
 
 ComplexBrush::~ComplexBrush()
