@@ -17,7 +17,6 @@
  */
 
 #include "kis_auto_brush_widget.h"
-#include <KoImageResource.h>
 #include <kis_debug.h>
 #include <QSpinBox>
 #include <QToolButton>
@@ -28,6 +27,7 @@
 #include <QResizeEvent>
 
 #include "kis_mask_generator.h"
+#include "kis_slider_spin_box.h"
 
 #define showSlider(input, step) input->setRange(input->minimum(), input->maximum(), step)
 
@@ -43,28 +43,44 @@ KisAutoBrushWidget::KisAutoBrushWidget(QWidget *parent, const char* name, const
 //     linkFadeToggled(m_linkSize);
 //     linkSizeToggled(m_linkFade);
 
-    connect(bnLinkFade, SIGNAL(toggled(bool)), this, SLOT(linkFadeToggled(bool)));
+    connect(aspectButton, SIGNAL(keepAspectRatioChanged(bool)), this, SLOT(linkFadeToggled(bool)));
+    aspectButton->setKeepAspectRatio(m_linkFade);
 
     connect((QObject*)comboBoxShape, SIGNAL(activated(int)), this, SLOT(paramChanged()));
 
-//     showSlider(inputRadius);
-    inputRadius->setRange(inputRadius->minimum(), inputRadius->maximum(), 1.0, false);
-    connect(inputRadius, SIGNAL(valueChanged(double)), this, SLOT(spinBoxRadiusChanged(double)));
+    inputRadius->setRange(0.0, 1000.0, 2);
+    inputRadius->setExponentRatio(3.0);
+    inputRadius->setValue(5.0);
+    connect(inputRadius, SIGNAL(valueChanged(qreal)), this, SLOT(paramChanged()));
 
-    showSlider(inputRatio, 0.1);
-    connect(inputRatio, SIGNAL(valueChanged(double)), this, SLOT(spinBoxRatioChanged(double)));
+    inputRatio->setRange(0.0, 1.0, 2);
+    inputRatio->setSingleStep(0.1);
+    inputRatio->setValue(1.0);
+    connect(inputRatio, SIGNAL(valueChanged(qreal)), this, SLOT(paramChanged()));
 
-    showSlider(inputHFade, 0.1);
-    connect(inputHFade, SIGNAL(valueChanged(double)), this, SLOT(spinBoxHorizontalChanged(double)));
+    inputHFade->setRange(0.0, 1.0, 2);
+    inputHFade->setSingleStep(0.1);
+    inputHFade->setValue(0.5);
+    connect(inputHFade, SIGNAL(valueChanged(qreal)), this, SLOT(spinBoxHorizontalChanged(qreal)));
 
-    showSlider(inputVFade, 0.1);
-    connect(inputVFade, SIGNAL(valueChanged(double)), this, SLOT(spinBoxVerticalChanged(double)));
+    inputVFade->setRange(0.0, 1.0, 2);
+    inputVFade->setSingleStep(0.1);
+    inputVFade->setValue(0.5);
+    connect(inputVFade, SIGNAL(valueChanged(qreal)), this, SLOT(spinBoxVerticalChanged(qreal)));
 
-    inputSpikes->setSliderEnabled(true);
+    inputSpikes->setRange(2, 20);
+    inputSpikes->setValue(2);
     connect(inputSpikes, SIGNAL(valueChanged(int)), this, SLOT(paramChanged()));
 
-    showSlider(inputSpacing, 0.1);
-    connect(inputSpacing, SIGNAL(valueChanged(double)), this, SLOT(paramChanged()));
+    inputAngle->setRange(0, 360);
+    inputAngle->setSuffix(QChar(Qt::Key_degree));
+    inputAngle->setValue(0);
+    connect(inputAngle, SIGNAL(valueChanged(int)), this, SLOT(paramChanged()));
+
+    inputSpacing->setRange(0.0, 10.0, 2);
+    inputSpacing->setSingleStep(0.1);
+    inputSpacing->setValue(0.1);
+    connect(inputSpacing, SIGNAL(valueChanged(qreal)), this, SLOT(paramChanged()));
 
     m_brush = QImage(1, 1, QImage::Format_RGB32);
 
@@ -93,7 +109,6 @@ void KisAutoBrushWidget::paramChanged()
 
     if (comboBoxShape->currentIndex() == 0) { // use index compare instead of comparing a translatable string
         kas = new KisCircleMaskGenerator(inputRadius->value(),  inputRatio->value(), inputHFade->value(), inputVFade->value(), inputSpikes->value());
-//         kas = new KisCircleMaskGenerator(inputRadius->value(), inputRadius->value() * inputRatio->value(), inputHFade->value(), inputVFade->value());
         Q_CHECK_PTR(kas);
 
     } else {
@@ -101,9 +116,9 @@ void KisAutoBrushWidget::paramChanged()
         Q_CHECK_PTR(kas);
 
     }
-    m_autoBrush = new KisAutoBrush(kas);
+    m_autoBrush = new KisAutoBrush(kas, inputAngle->value() / 180.0 * M_PI);
     m_autoBrush->setSpacing(inputSpacing->value());
-    m_brush = m_autoBrush->img();
+    m_brush = m_autoBrush->image();
 
     QImage pi(m_brush);
     double coeff = 1.0;
@@ -125,24 +140,23 @@ void KisAutoBrushWidget::paramChanged()
     emit sigBrushChanged();
 }
 
-void KisAutoBrushWidget::spinBoxRatioChanged(double a)
+void KisAutoBrushWidget::spinBoxHorizontalChanged(qreal a)
 {
-    paramChanged();
-}
-void KisAutoBrushWidget::spinBoxRadiusChanged(double a)
-{
-    paramChanged();
-}
-void KisAutoBrushWidget::spinBoxHorizontalChanged(double a)
-{
-    if (m_linkFade)
-        inputHFade->setValue(a);
-    paramChanged();
-}
-void KisAutoBrushWidget::spinBoxVerticalChanged(double a)
-{
-    if (m_linkFade)
+    if (m_linkFade) {
+        inputVFade->blockSignals(true);
         inputVFade->setValue(a);
+        inputVFade->blockSignals(false);
+    }
+    paramChanged();
+}
+
+void KisAutoBrushWidget::spinBoxVerticalChanged(qreal a)
+{
+    if (m_linkFade) {
+        inputHFade->blockSignals(true);
+        inputHFade->setValue(a);
+        inputHFade->blockSignals(false);
+    }
     paramChanged();
 }
 
@@ -150,12 +164,8 @@ void KisAutoBrushWidget::linkFadeToggled(bool b)
 {
     m_linkFade = b;
 
-    KoImageResource kir;
-    if (b) {
-        bnLinkFade->setIcon(QIcon(kir.chain()));
-    } else {
-        bnLinkFade->setIcon(QIcon(kir.chainBroken()));
-    }
+    if (m_linkFade)
+        inputVFade->setValue(inputHFade->value());
 }
 
 KisBrushSP KisAutoBrushWidget::brush()
@@ -166,9 +176,30 @@ KisBrushSP KisAutoBrushWidget::brush()
 void KisAutoBrushWidget::setBrush(KisBrushSP brush)
 {
     m_autoBrush = brush;
-    m_brush = brush->img();
+    m_brush = brush->image();
     // XXX: lock, set and unlock the widgets.
-
+    KisAutoBrush* aBrush = dynamic_cast<KisAutoBrush*>(brush.data());
+    inputRadius->setValue(aBrush->maskGenerator()->radius());
+    inputRatio->setValue(aBrush->maskGenerator()->ratio());
+    inputHFade->setValue(aBrush->maskGenerator()->horizontalFade());
+    inputVFade->setValue(aBrush->maskGenerator()->verticalFade());
+    inputAngle->setValue(aBrush->angle() * 180 / M_PI);
+    inputSpikes->setValue(aBrush->maskGenerator()->spikes());
+    inputSpacing->setValue(aBrush->spacing());
 }
 
+
+void KisAutoBrushWidget::setAutoBrushDiameter(qreal diameter)
+{
+    inputRadius->setValue(diameter);
+}
+
+
+qreal KisAutoBrushWidget::autoBrushDiameter()
+{
+    return inputRadius->value();
+}
+
+
+
 #include "kis_auto_brush_widget.moc"
