@@ -27,7 +27,7 @@
 
 #include "kis_global.h"
 #include "kis_types.h"
-#include "KoToolFactory.h"
+#include "KoToolFactoryBase.h"
 #include "flake/kis_node_shape.h"
 
 class KisPainter;
@@ -58,6 +58,7 @@ public:
 private:
     void paintLine(QPainter& gc, const QRect& rc);
     QPointF straightLine(QPointF point);
+    void updatePreview();
 
 private:
 
@@ -70,17 +71,17 @@ private:
 };
 
 
-class KisToolLineFactory : public KoToolFactory
+class KisToolLineFactory : public KoToolFactoryBase
 {
 
 public:
 
     KisToolLineFactory(QObject *parent, const QStringList&)
-            : KoToolFactory(parent, "KritaShape/KisToolLine", i18nc("straigh line drawing tool", "Line")) {
+            : KoToolFactoryBase(parent, "KritaShape/KisToolLine") {
         setToolTip(i18n("Draw a straight line with the current brush"));
         // Temporarily
         setToolType(TOOL_TYPE_SHAPE);
-        //setActivationShapeId( KIS_NODE_SHAPE_ID );
+        setActivationShapeId(KRITA_TOOL_ACTIVATION_ID);
         setPriority(1);
         setIcon("krita_tool_line");
         setInputDeviceAgnostic(false);
@@ -88,7 +89,7 @@ public:
 
     virtual ~KisToolLineFactory() {}
 
-    virtual KoTool * createTool(KoCanvasBase *canvas) {
+    virtual KoToolBase * createTool(KoCanvasBase *canvas) {
         return new KisToolLine(canvas);
     }
 
