@@ -31,10 +31,10 @@
 
 #include <kis_debug.h>
 #include <klocale.h>
-#include <knuminput.h>
 #include <kcolorbutton.h>
 
 #include "KoPointerEvent.h"
+#include "KoViewConverter.h"
 
 #include "kis_cursor.h"
 #include "kis_selection_manager.h"
@@ -47,37 +47,26 @@
 #include "kis_fill_painter.h"
 #include "kis_pixel_selection.h"
 #include "kis_selection_tool_helper.h"
+#include "kis_slider_spin_box.h"
 
 KisToolSelectContiguous::KisToolSelectContiguous(KoCanvasBase *canvas)
-        : KisTool(canvas, KisCursor::load("tool_contiguous_selection_cursor.png", 6, 6))
+        : KisToolSelectBase(canvas, KisCursor::load("tool_contiguous_selection_cursor.png", 6, 6))
 {
     setObjectName("tool_select_contiguous");
-    m_optWidget = 0;
     m_fuzziness = 20;
-    m_sampleMerged = false;
-    m_selectAction = SELECTION_REPLACE;
+    m_limitToCurrentLayer = false;
 }
 
 KisToolSelectContiguous::~KisToolSelectContiguous()
 {
 }
 
-void KisToolSelectContiguous::activate(bool tmp)
-{
-    KisTool::activate(tmp);
-
-    if (!m_optWidget)
-        return;
-
-    m_optWidget->slotActivated();
-}
-
 void KisToolSelectContiguous::mousePressEvent(KoPointerEvent * e)
 {
     if (e->button() != Qt::LeftButton)
         return;
 
-    if (m_canvas && currentImage()) {
+    if (canvas() && currentImage()) {
         QApplication::setOverrideCursor(KisCursor::waitCursor());
 
         if (!currentNode())
@@ -93,17 +82,17 @@ void KisToolSelectContiguous::mousePressEvent(KoPointerEvent * e)
         fillpainter.setHeight(rc.height());
         fillpainter.setWidth(rc.width());
         fillpainter.setFillThreshold(m_fuzziness);
-        fillpainter.setSampleMerged(m_sampleMerged);
+        fillpainter.setSampleMerged(!m_limitToCurrentLayer);
         KisSelectionSP selection =
             fillpainter.createFloodSelection(pos.x(), pos.y(), currentImage()->mergedImage());
 
-        KisCanvas2 * kisCanvas = dynamic_cast<KisCanvas2*>(m_canvas);
+        KisCanvas2 * kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
         if (!kisCanvas)
             return;
 
         KisSelectionToolHelper helper(kisCanvas, currentNode(), i18n("Contiguous Area Selection"));
         QUndoCommand* cmd = helper.selectPixelSelection(selection->pixelSelection(), m_selectAction);
-        m_canvas->addCommand(cmd);
+        canvas()->addCommand(cmd);
 
         QApplication::restoreOverrideCursor();
     }
@@ -120,27 +109,9 @@ void KisToolSelectContiguous::slotSetFuzziness(int fuzziness)
     m_fuzziness = fuzziness;
 }
 
-void KisToolSelectContiguous::slotSetAction(int action)
-{
-    if (action >= SELECTION_REPLACE && action <= SELECTION_INTERSECT)
-        m_selectAction = (selectionAction)action;
-// XXX: Fix cursors when then are done.
-//     switch(m_selectAction) {
-//         case SELECTION_ADD:
-//             m_subject->setCanvasCursor(KisCursor::pickerPlusCursor());
-//             break;
-//         case SELECTION_SUBTRACT:
-//             m_subject->setCanvasCursor(KisCursor::pickerMinusCursor());
-//     };
-}
-
 QWidget* KisToolSelectContiguous::createOptionWidget()
 {
-    KisCanvas2* canvas = dynamic_cast<KisCanvas2*>(m_canvas);
-    Q_ASSERT(canvas);
-    m_optWidget = new KisSelectionOptions(canvas);
-    Q_CHECK_PTR(m_optWidget);
-    m_optWidget->setObjectName(toolId() + " option widget");
+    KisToolSelectBase::createOptionWidget();
     m_optWidget->setWindowTitle(i18n("Contiguous Area Selection"));
     m_optWidget->disableAntiAliasSelectionOption();
     m_optWidget->disableSelectionModeOption();
@@ -148,48 +119,37 @@ QWidget* KisToolSelectContiguous::createOptionWidget()
     QVBoxLayout * l = dynamic_cast<QVBoxLayout*>(m_optWidget->layout());
     Q_ASSERT(l);
     if (l) {
-        l->setSpacing(6);
-
-        connect(m_optWidget, SIGNAL(actionChanged(int)), this, SLOT(slotSetAction(int)));
-
         QHBoxLayout * hbox = new QHBoxLayout();
         Q_CHECK_PTR(hbox);
-        l->addLayout(hbox);
+        l->insertLayout(1, hbox);
 
         QLabel * lbl = new QLabel(i18n("Fuzziness: "), m_optWidget);
         hbox->addWidget(lbl);
 
-        KIntNumInput * input = new KIntNumInput(m_optWidget);
+        KisSliderSpinBox *input = new KisSliderSpinBox(m_optWidget);
         Q_CHECK_PTR(input);
         input->setObjectName("fuzziness");
-
-        input->setRange(0, 200, 10, true);
+        input->setRange(0, 200);
+        input->setSingleStep(10);
         input->setValue(20);
         hbox->addWidget(input);
         connect(input, SIGNAL(valueChanged(int)), this, SLOT(slotSetFuzziness(int)));
 
-        QCheckBox* samplemerged = new QCheckBox(i18n("Sample merged"), m_optWidget);
-        l->addWidget(samplemerged);
-        samplemerged->setChecked(m_sampleMerged);
-        connect(samplemerged, SIGNAL(stateChanged(int)),
-                this, SLOT(slotSetSampleMerged(int)));
+        QCheckBox* limitToCurrentLayer = new QCheckBox(i18n("Limit to current layer"), m_optWidget);
+        l->insertWidget(2, limitToCurrentLayer);
+        limitToCurrentLayer->setChecked(m_limitToCurrentLayer);
+        connect(limitToCurrentLayer, SIGNAL(stateChanged(int)),
+                this, SLOT(slotLimitToCurrentLayer(int)));
 
-        l->addItem(new QSpacerItem(1, 1, QSizePolicy::Fixed, QSizePolicy::Expanding));
     }
-    m_optWidget->setFixedHeight(m_optWidget->sizeHint().height());
     return m_optWidget;
 }
 
-QWidget* KisToolSelectContiguous::optionWidget()
-{
-    return m_optWidget;
-}
-
-void KisToolSelectContiguous::slotSetSampleMerged(int state)
+void KisToolSelectContiguous::slotLimitToCurrentLayer(int state)
 {
     if (state == Qt::PartiallyChecked)
         return;
-    m_sampleMerged = (state == Qt::Checked);
+    m_limitToCurrentLayer = (state == Qt::Checked);
 }
 
 #include "kis_tool_select_contiguous.moc"
