@@ -27,17 +27,19 @@
 #include <KoCanvasBase.h>
 #include <krita_export.h>
 #include <kis_types.h>
+#include <KoPointerEvent.h>
 
 class KoToolProxy;
 class KoColorProfile;
 
 class KisCanvasDecoration;
 class KisView2;
+class KisPaintopBox;
+class KoFavoriteResourceManager;
 
 enum KisCanvasType {
     QPAINTER,
-    OPENGL,
-    MITSHM
+    OPENGL
 };
 
 class KoViewConverter;
@@ -140,6 +142,10 @@ signals:
 
     void imageChanged(KisImageWSP image);
 
+    void canvasDestroyed(QWidget *);
+
+    void favoritePaletteCalled(const QPoint&);
+
 public slots:
 
     /// Update the entire canvas area
@@ -177,17 +183,23 @@ private slots:
      */
     void slotSetDisplayProfile(const KoColorProfile * profile);
 
-private:
+    void slotCanvasDestroyed(QWidget* w);
+
+public:
 
     friend class KisView2;
 
-    void resetCanvas();
+    void resetCanvas(bool useOpenGL);
 
     KisCanvas2(const KisCanvas2&);
 
     void createCanvas();
     void createQPainterCanvas();
     void createOpenGLCanvas();
+    void createFavoriteResourceManager(KisPaintopBox*);
+
+    KoFavoriteResourceManager* favoriteResourceManager();
+    bool handlePopupPaletteIsVisible(KoPointerEvent *);
 
     /**
      * Returns a rect in widget pixels that is translated for document
