@@ -20,8 +20,7 @@
 
 #include "kis_dlg_preferences.h"
 
-#include <config-opengl.h>
-#include <config-glew.h>
+#include <opengl/kis_opengl.h>
 
 #include <QBitmap>
 #include <QCheckBox>
@@ -40,20 +39,21 @@
 #include <qgl.h>
 #endif
 
-#include <KoImageResource.h>
-#include <colorprofiles/KoIccColorProfile.h>
+#include <libs/main/KoDocument.h>
+#include <KoColorProfile.h>
 
+#include <kmessagebox.h>
 #include <kcolorbutton.h>
 #include <kcombobox.h>
 #include <kfiledialog.h>
 #include <kiconloader.h>
 #include <klineedit.h>
 #include <klocale.h>
-#include <knuminput.h>
 #include <kurlrequester.h>
 #include <kpagewidgetmodel.h>
 #include <kicon.h>
 #include <kvbox.h>
+
 #include "widgets/squeezedcombobox.h"
 #include "kis_clipboard.h"
 #include "widgets/kis_cmb_idlist.h"
@@ -61,16 +61,13 @@
 #include "KoColorSpaceRegistry.h"
 #include "kis_cursor.h"
 #include "kis_config.h"
+#include "kis_canvas_resource_provider.h"
 
 #include "kis_factory2.h"
 #include "KoID.h"
 
 #include "KoColorProfile.h"
 
-#ifdef HAVE_OPENGL
-#include "opengl/kis_opengl.h"
-#endif
-
 // for the performance update
 #include "tiles/kis_tilemanager.h"
 
@@ -79,8 +76,18 @@ GeneralTab::GeneralTab(QWidget *_parent, const char *_name)
 {
     KisConfig cfg;
 
+#if defined(HAVE_OPENGL)
+    m_cmbCursorShape->addItem("3D Brush Model");
+#endif
+
     m_cmbCursorShape->setCurrentIndex(cfg.cursorStyle());
     chkShowRootLayer->setChecked(cfg.showRootLayer());
+
+    int autosaveInterval = cfg.autoSaveInterval();
+    //convert to minutes
+    m_autosaveSpinBox->setValue(autosaveInterval / 60);
+    m_autosaveCheckBox->setChecked(autosaveInterval > 0);
+    m_undoStackSize->setValue(cfg.undoStackLimit());
 }
 
 void GeneralTab::setDefault()
@@ -89,6 +96,10 @@ void GeneralTab::setDefault()
 
     m_cmbCursorShape->setCurrentIndex(cfg.getDefaultCursorStyle());
     chkShowRootLayer->setChecked(false);
+    m_autosaveCheckBox->setChecked(true);
+    //convert to minutes
+    m_autosaveSpinBox->setValue(KoDocument::defaultAutoSave() / 60);
+    m_undoStackSize->setValue(30);
 }
 
 enumCursorStyle GeneralTab::cursorStyle()
@@ -101,6 +112,17 @@ bool GeneralTab::showRootLayer()
     return chkShowRootLayer->isChecked();
 }
 
+int GeneralTab::autoSaveInterval()
+{
+    //convert to seconds
+    return m_autosaveCheckBox->isChecked() ? m_autosaveSpinBox->value()*60 : 0;
+}
+
+int GeneralTab::undoStackSize()
+{
+    return m_undoStackSize->value();
+}
+
 //---------------------------------------------------------------------------------------------------
 
 ColorSettingsTab::ColorSettingsTab(QWidget *parent, const char *name)
@@ -147,7 +169,7 @@ ColorSettingsTab::ColorSettingsTab(QWidget *parent, const char *name)
 
     m_page->cmbMonitorIntent->setCurrentIndex(cfg.renderIntent());
 
-    if (KoIccColorProfile * profile = KoIccColorProfile::getScreenProfile()) {
+    if (const KoColorProfile * profile = KisCanvasResourceProvider::getScreenProfile()) {
         // We've got an X11 profile, don't allow to override
         m_page->cmbMonitorProfile->hide();
         m_page->lblMonitorProfile->setText(i18n("Monitor profile: ") + profile->name());
@@ -167,6 +189,8 @@ void ColorSettingsTab::setDefault()
     m_page->cmbPrintingColorSpace->setCurrent("CMYK");
     refillPrintProfiles(KoID("CMYK", ""));
 
+    refillMonitorProfiles(KoID("RGBA", ""));
+
     m_page->chkBlackpoint->setChecked(false);
     m_page->cmbMonitorIntent->setCurrentIndex(INTENT_PERCEPTUAL);
 
@@ -181,7 +205,7 @@ void ColorSettingsTab::setDefault()
 
 void ColorSettingsTab::refillMonitorProfiles(const KoID & s)
 {
-    KoColorSpaceFactory * csf = KoColorSpaceRegistry::instance()->value(s.id());
+    const KoColorSpaceFactory * csf = KoColorSpaceRegistry::instance()->colorSpaceFactory(s.id());
 
     m_page->cmbMonitorProfile->clear();
 
@@ -200,7 +224,7 @@ void ColorSettingsTab::refillMonitorProfiles(const KoID & s)
 
 void ColorSettingsTab::refillPrintProfiles(const KoID & s)
 {
-    KoColorSpaceFactory * csf = KoColorSpaceRegistry::instance()->value(s.id());
+    const KoColorSpaceFactory * csf = KoColorSpaceRegistry::instance()->colorSpaceFactory(s.id());
 
     m_page->cmbPrintProfile->clear();
 
@@ -289,6 +313,7 @@ DisplaySettingsTab::DisplaySettingsTab(QWidget *parent, const char *name)
     intCheckSize->setValue(cfg.checkSize());
     chkMoving->setChecked(cfg.scrollCheckers());
     colorChecks->setColor(cfg.checkersColor());
+    chkCurveAntialiasing->setChecked(cfg.antialiasCurves());
 
     connect(cbUseOpenGL, SIGNAL(toggled(bool)), SLOT(slotUseOpenGLToggled(bool)));
 }
@@ -328,15 +353,23 @@ GridSettingsTab::GridSettingsTab(QWidget* parent) : WdgGridSettingsBase(parent)
 
     intHSpacing->setValue(cfg.getGridHSpacing());
     intVSpacing->setValue(cfg.getGridVSpacing());
-    intSubdivision->setValue(cfg.getGridSubdivisions());
-    intOffsetX->setValue(cfg.getGridOffsetX());
-    intOffsetY->setValue(cfg.getGridOffsetY());
+    spacingAspectButton->setKeepAspectRatio(cfg.getGridSpacingAspect());
+    linkSpacingToggled(cfg.getGridSpacingAspect());
 
-    linkSpacingToggled(true);
-    connect(bnLinkSpacing, SIGNAL(toggled(bool)), this, SLOT(linkSpacingToggled(bool)));
+    intSubdivision->setValue(cfg.getGridSubdivisions());
+
+    intXOffset->setValue(cfg.getGridOffsetX());
+    intYOffset->setValue(cfg.getGridOffsetY());
+    offsetAspectButton->setKeepAspectRatio(cfg.getGridOffsetAspect());
+    linkOffsetToggled(cfg.getGridOffsetAspect());
+
+    connect(spacingAspectButton, SIGNAL(keepAspectRatioChanged(bool)), this, SLOT(linkSpacingToggled(bool)));
+    connect(offsetAspectButton, SIGNAL(keepAspectRatioChanged(bool)), this, SLOT(linkOffsetToggled(bool)));
 
     connect(intHSpacing, SIGNAL(valueChanged(int)), this, SLOT(spinBoxHSpacingChanged(int)));
     connect(intVSpacing, SIGNAL(valueChanged(int)), this, SLOT(spinBoxVSpacingChanged(int)));
+    connect(intXOffset, SIGNAL(valueChanged(int)), this, SLOT(spinBoxXOffsetChanged(int)));
+    connect(intYOffset, SIGNAL(valueChanged(int)), this, SLOT(spinBoxYOffsetChanged(int)));
 
 
 }
@@ -352,9 +385,11 @@ void GridSettingsTab::setDefault()
 
     intHSpacing->setValue(10);
     intVSpacing->setValue(10);
+    linkSpacingToggled(false);
     intSubdivision->setValue(1);
-    intOffsetX->setValue(0);
-    intOffsetY->setValue(0);
+    intXOffset->setValue(0);
+    intYOffset->setValue(0);
+    linkOffsetToggled(false);
 }
 
 void GridSettingsTab::spinBoxHSpacingChanged(int v)
@@ -371,23 +406,42 @@ void GridSettingsTab::spinBoxVSpacingChanged(int v)
     }
 }
 
-
 void GridSettingsTab::linkSpacingToggled(bool b)
 {
     m_linkSpacing = b;
 
-    KoImageResource kir;
-    if (b) {
-        bnLinkSpacing->setIcon(QIcon(kir.chain()));
-    } else {
-        bnLinkSpacing->setIcon(QIcon(kir.chainBroken()));
+    if (m_linkSpacing) {
+        intVSpacing->setValue(intHSpacing->value());
+    }
+}
+
+void GridSettingsTab::spinBoxXOffsetChanged(int v)
+{
+    if (m_linkOffset) {
+        intYOffset->setValue(v);
+    }
+}
+
+void GridSettingsTab::spinBoxYOffsetChanged(int v)
+{
+    if (m_linkOffset) {
+        intXOffset->setValue(v);
+    }
+}
+
+void GridSettingsTab::linkOffsetToggled(bool b)
+{
+    m_linkOffset = b;
+
+    if (m_linkOffset) {
+        intYOffset->setValue(intXOffset->value());
     }
 }
 
 
 //---------------------------------------------------------------------------------------------------
 
-PreferencesDialog::PreferencesDialog(QWidget* parent, const char* name)
+KisDlgPreferences::KisDlgPreferences(QWidget* parent, const char* name)
         : KPageDialog(parent)
 {
     Q_UNUSED(name);
@@ -396,6 +450,8 @@ PreferencesDialog::PreferencesDialog(QWidget* parent, const char* name)
     setDefaultButton(Ok);
     showButtonSeparator(true);
     setFaceType(KPageDialog::List);
+
+    // General
     KVBox *vbox = new KVBox();
     KPageWidgetItem *page = new KPageWidgetItem(vbox, i18n("General"));
     page->setHeader(i18n("General"));
@@ -403,14 +459,15 @@ PreferencesDialog::PreferencesDialog(QWidget* parent, const char* name)
     addPage(page);
     m_general = new GeneralTab(vbox);
 
+    // Display
     vbox = new KVBox();
     page = new KPageWidgetItem(vbox, i18n("Display"));
     page->setHeader(i18n("Display"));
     page->setIcon(KIcon("preferences-desktop-display"));
     addPage(page);
-
     m_displaySettings = new DisplaySettingsTab(vbox);
 
+    // Color
     vbox = new KVBox();
     page = new KPageWidgetItem(vbox, i18n("Color Management"));
     page->setHeader(i18n("Color"));
@@ -418,62 +475,77 @@ PreferencesDialog::PreferencesDialog(QWidget* parent, const char* name)
     addPage(page);
     m_colorSettings = new ColorSettingsTab(vbox);
 
+    // Performance
+#if 0
     vbox = new KVBox();
     page = new KPageWidgetItem(vbox, i18n("Performance"));
     page->setHeader(i18n("Performance"));
     page->setIcon(KIcon("preferences-system-performance"));
     addPage(page);
     m_performanceSettings = new PerformanceTab(vbox);
+#endif
 
+    // Grid
     vbox = new KVBox();
     page = new KPageWidgetItem(vbox, i18n("Grid"));
     page->setHeader(i18n("Grid"));
     page->setIcon(KIcon(BarIcon("grid", KIconLoader::SizeMedium)));
     addPage(page);
-
     m_gridSettings = new GridSettingsTab(vbox);
 
+    connect(this, SIGNAL(defaultClicked()), this, SLOT(slotDefault()));
+
 }
 
-PreferencesDialog::~PreferencesDialog()
+KisDlgPreferences::~KisDlgPreferences()
 {
 }
 
-void PreferencesDialog::slotDefault()
+void KisDlgPreferences::slotDefault()
 {
     m_general->setDefault();
     m_colorSettings->setDefault();
+#if 0
     m_performanceSettings->setDefault();
+#endif
 #ifdef HAVE_OPENGL
     m_displaySettings->setDefault();
 #endif
     m_gridSettings->setDefault();
 }
 
-bool PreferencesDialog::editPreferences()
+bool KisDlgPreferences::editPreferences()
 {
-    PreferencesDialog* dialog;
+    KisDlgPreferences* dialog;
 
-    dialog = new PreferencesDialog();
+    dialog = new KisDlgPreferences();
     bool baccept = (dialog->exec() == Accepted);
     if (baccept) {
+        // General settings
         KisConfig cfg;
         cfg.setCursorStyle(dialog->m_general->cursorStyle());
         cfg.setShowRootLayer(dialog->m_general->showRootLayer());
+
+        cfg.setAutoSaveInterval(dialog->m_general->autoSaveInterval());
+        foreach(KoDocument* doc, *KoDocument::documentList()) {
+            doc->setAutoSave(dialog->m_general->autoSaveInterval());
+        }
+        cfg.setUndoStackLimit(dialog->m_general->undoStackSize());
+
         // Color settings
-        cfg.setMonitorProfile(dialog->m_colorSettings->m_page->cmbMonitorProfile->currentText());
-        cfg.setWorkingColorSpace(dialog->m_colorSettings->m_page->cmbWorkingColorSpace->currentText());
-        cfg.setPrinterColorSpace(dialog->m_colorSettings->m_page->cmbPrintingColorSpace->currentText());
-        cfg.setPrinterProfile(dialog->m_colorSettings->m_page->cmbPrintProfile->currentText());
+        cfg.setMonitorProfile(dialog->m_colorSettings->m_page->cmbMonitorProfile->itemHighlighted());
+        cfg.setWorkingColorSpace(dialog->m_colorSettings->m_page->cmbWorkingColorSpace->currentItem().id());
+        cfg.setPrinterColorSpace(dialog->m_colorSettings->m_page->cmbPrintingColorSpace->currentItem().id());
+        cfg.setPrinterProfile(dialog->m_colorSettings->m_page->cmbPrintProfile->itemHighlighted());
 
         cfg.setUseBlackPointCompensation(dialog->m_colorSettings->m_page->chkBlackpoint->isChecked());
         cfg.setPasteBehaviour(dialog->m_colorSettings->m_pasteBehaviourGroup.checkedId());
         cfg.setRenderIntent(dialog->m_colorSettings->m_page->cmbMonitorIntent->currentIndex());
 
+#if 0
         // it's scaled from 0 - 6, but the config is in 0 - 300
         cfg.setSwappiness(dialog->m_performanceSettings->m_swappiness->value() * 50);
         cfg.setMaxTilesInMem(dialog->m_performanceSettings->m_maxTiles->value());
-#if 0
         cfg.setUseProjections(dialog->m_performanceSettings->m_projection->isChecked());
         cfg.setNumProjectThreads(dialog->m_performanceSettings->intNumThreads->value());
         cfg.setProjectionChunkSize(dialog->m_performanceSettings->intChunkSize->value());
@@ -486,12 +558,26 @@ bool PreferencesDialog::editPreferences()
 #endif
 
 #ifdef HAVE_OPENGL
+        if (dialog->m_displaySettings->cbUseOpenGL->isChecked() && cfg.canvasState() == "OPENGL_NOT_TRIED") {
+            cfg.setCanvasState("TRY_OPENGL");
+        }
+        if (dialog->m_displaySettings->cbUseOpenGL->isChecked() && cfg.canvasState() == "OPENGL_FAILED") {
+            if (KMessageBox::warningYesNo(0, i18n("You are trying to enable OpenGL\n\n"
+                                                  "But Krita might have had problems with the OpenGL canvas before,\n"
+                                                  "either because of driver issues, or because of issues with window effects.\n\n"
+                                                  "Are you sure you want to enable OpenGL?\n"), i18n("Krita")) == KMessageBox::Yes) {
+                cfg.setCanvasState("TRY_OPENGL");
+            }
+        }
+
         cfg.setUseOpenGL(dialog->m_displaySettings->cbUseOpenGL->isChecked());
         cfg.setUseOpenGLShaders(dialog->m_displaySettings->cbUseOpenGLShaders->isChecked());
+
 #endif
         cfg.setCheckSize(dialog->m_displaySettings->intCheckSize->value());
         cfg.setScrollingCheckers(dialog->m_displaySettings->chkMoving->isChecked());
         cfg.setCheckersColor(dialog->m_displaySettings->colorChecks->color());
+        cfg.setAntialiasCurves(dialog->m_displaySettings->chkCurveAntialiasing->isChecked());
         // Grid settings
         cfg.setGridMainStyle(dialog->m_gridSettings->selectMainStyle->currentIndex());
         cfg.setGridSubdivisionStyle(dialog->m_gridSettings->selectSubdivisionStyle->currentIndex());
@@ -501,9 +587,11 @@ bool PreferencesDialog::editPreferences()
 
         cfg.setGridHSpacing(dialog->m_gridSettings->intHSpacing->value());
         cfg.setGridVSpacing(dialog->m_gridSettings->intVSpacing->value());
+        cfg.setGridSpacingAspect(dialog->m_gridSettings->spacingAspectButton->keepAspectRatio());
         cfg.setGridSubdivisions(dialog->m_gridSettings->intSubdivision->value());
-        cfg.setGridOffsetX(dialog->m_gridSettings->intOffsetX->value());
-        cfg.setGridOffsetY(dialog->m_gridSettings->intOffsetY->value());
+        cfg.setGridOffsetX(dialog->m_gridSettings->intXOffset->value());
+        cfg.setGridOffsetY(dialog->m_gridSettings->intYOffset->value());
+        cfg.setGridOffsetAspect(dialog->m_gridSettings->offsetAspectButton->keepAspectRatio());
 
     }
     delete dialog;
