@@ -80,6 +80,7 @@ void KisHistogramView::setPaintDevice(KisPaintDeviceSP dev)
 
 void KisHistogramView::setHistogram(KisHistogramSP histogram)
 {
+    if (!histogram) return;
     m_cs = 0;
     m_histogram = histogram;
     m_currentProducer = m_histogram->producer();
@@ -123,11 +124,11 @@ QStringList KisHistogramView::channelStrings()
     return m_channelStrings;
 }
 
-QList<KoID> KisHistogramView::listProducers()
+QList<QString> KisHistogramView::producers()
 {
     if (m_cs)
-        return KoHistogramProducerFactoryRegistry::instance()->listKeysCompatibleWith(m_cs);
-    return QList<KoID>();
+        return KoHistogramProducerFactoryRegistry::instance()->keysCompatibleWith(m_cs);
+    return QList<QString>();
 }
 
 void KisHistogramView::setCurrentChannels(const KoID& producerID, QList<KoChannelInfo *> channels)
@@ -234,16 +235,15 @@ void KisHistogramView::setChannels()
     m_channels.clear();
     m_channelToOffset.clear();
 
-    QList<KoID> list = KoHistogramProducerFactoryRegistry::instance()->listKeysCompatibleWith(m_cs);
+    QList<QString> list = KoHistogramProducerFactoryRegistry::instance()->keysCompatibleWith(m_cs);
 
     if (list.count() == 0) {
         // XXX: No native histogram for this colorspace. Using converted RGB. We should have a warning
         KoGenericRGBHistogramProducerFactory f;
         addProducerChannels(f.generate());
     } else {
-        for (int i = 0; i < list.count(); i++) {
-            KoID id(list.at(i));
-            addProducerChannels(KoHistogramProducerFactoryRegistry::instance()->value(id.id())->generate());
+        foreach (const QString &id, list) {
+            addProducerChannels(KoHistogramProducerFactoryRegistry::instance()->value(id)->generate());
         }
     }
 
@@ -314,7 +314,7 @@ void KisHistogramView::updateHistogram()
             highest = (double)m_histogram->calculations().getHighest();
     }
 
-    QPen pen( Qt::white );
+    QPen pen(Qt::white);
 
     for (int chan = 0; chan < m_channels.count(); chan++) {
         QColor color;
@@ -323,12 +323,10 @@ void KisHistogramView::updateHistogram()
         if (m_color) {
             color = m_channels.at(chan)->color();
             QLinearGradient gradient;
-//             gradient.setStart( 0, 0 );
-//             gradient.setFinalStop( 0, height );
-            gradient.setColorAt( 0, Qt::white );
-            gradient.setColorAt( 1, color );
-            QBrush brush( gradient );
-            pen = QPen( brush, 0 );
+            gradient.setColorAt(0, Qt::white);
+            gradient.setColorAt(1, color);
+            QBrush brush(gradient);
+            pen = QPen(brush, 0);
         } else {
             color = Qt::black;
         }
