@@ -39,10 +39,11 @@ class QHBoxLayout;
 class KoID;
 class KoColorSpace;
 class KoResourceSelector;
+class KoResource;
 
 class KisView2;
 class KisCanvasResourceProvider;
-class KisPresetWidget;
+class KisPopupButton;
 class KisPaintOpPresetsPopup;
 class KisPaintOpSettingsWidget;
 
@@ -63,9 +64,15 @@ class KisPaintopBox : public QWidget
 
 public:
     KisPaintopBox(KisView2 * view,  QWidget * parent, const char * name);
-
+    KisPaintOpPresetSP paintOpPresetSP(KoID * = 0);
+    const KoID & currentPaintop();
+    void setCurrentPaintop(const KoID & paintop);
+    QPixmap paintopPixmap(const KoID & paintop);
     ~KisPaintopBox();
 
+signals:
+    void signalPaintopChanged(KisPaintOpPresetSP paintop);
+
 public slots:
 
     void slotItemSelected(int index);
@@ -77,15 +84,13 @@ public slots:
 
 private:
 
-    QPixmap paintopPixmap(const KoID & paintop);
-    const KoID & currentPaintop();
-    void setCurrentPaintop(const KoID & paintop);
     KoID defaultPaintop(const KoInputDevice & inputDevice);
     KisPaintOpPresetSP activePreset(const KoID & paintop, const KoInputDevice & inputDevice);
 
 private slots:
 
     void updatePaintops();
+    void resourceSelected( KoResource * resource );
 
 private:
 
@@ -96,7 +101,7 @@ private:
 
     QHBoxLayout* m_layout;
     KisPaintOpSettingsWidget* m_optionWidget;
-    KisPresetWidget* m_presetWidget;
+    KisPopupButton* m_presetWidget;
     KisPaintOpPresetsPopup* m_presetsPopup;
     KisView2* m_view;
 
