@@ -28,8 +28,11 @@
 #include "KWDocument.h"
 
 #include <KoShapeRegistry.h>
-#include <KoShapeFactory.h>
+#include <KoShapeFactoryBase.h>
 
+#include <QTextLayout>
+#include <QTextDocument>
+#include <QTextBlock>
 #include <klocale.h>
 #include <kdebug.h>
 #include <limits.h>
@@ -92,7 +95,7 @@ void KWFrameLayout::createNewFramesForPage(int pageNumber)
                 m_parent = parent;
             }
             void create(const KWPage &page, KWTextFrameSet *fs) {
-                KWFrame* frame;
+                KWFrame *frame;
                 if (fs->textFrameSetType() == KWord::MainTextFrameSet)
                     frame = new KWTextFrame(m_parent->createTextShape(page), fs);
                 else
@@ -180,7 +183,9 @@ void KWFrameLayout::createNewFramesForPage(int pageNumber)
                 f = new KWTextFrame(createTextShape(page), tfs);
             } else {
                 Q_ASSERT(frame->newFrameBehavior() == KWord::CopyNewFrame);
-                f = new KWFrame(new KWCopyShape(frame->shape()), frame->frameSet());
+                Q_ASSERT(frame->frameSet());
+                Q_ASSERT(frame->frameSet()->frameCount() > 0);
+                f = new KWFrame(new KWCopyShape(frame->frameSet()->frames().first()->shape(), m_pageManager), frame->frameSet());
             }
             const qreal y = frame->shape()->position().y();
             qreal offsetFromPage = y - pageRect.top();
@@ -207,7 +212,7 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
     Q_ASSERT(page.isValid());
 
     /* +-----------------+
-       |  0              | <- m_pageStyle->pageLayout()->top
+       |  0              | <- m_pageStyle->pageLayout()->topMargin + d:o->topPadding
        |  1  [ header ]  |
        |  2              | <- m_pageStyle->headerDistance()
        |  3  [ maintxt ] |
@@ -217,7 +222,7 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
        |  7 [ footnote ] |
        |  8              | <- m_pageStyle->footerDistance()
        |  9  [ footer ]  |
-       | 10               | <- m_pageStyle->pageLayout()->bottom
+       | 10               | <- m_pageStyle->pageLayout()->bottomMargin + d:o->bottomPadding
        +-----------------+ */
 
     // Create some data structures used for the layouting of the frames later
@@ -228,30 +233,37 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
         minimumHeight[i] = 0;
         resultingPositions[i] = 0;
     }
-    minimumHeight[0] = page.topMargin();
-    minimumHeight[10] = page.bottomMargin();
+    minimumHeight[0] = page.topMargin() + page.topPadding();
+    minimumHeight[10] = page.bottomMargin() + page.bottomPadding();
 
     KoPageLayout layout = page.pageStyle().pageLayout();
-    layout.left = page.leftMargin();
-    layout.right = page.rightMargin();
+    layout.leftMargin = page.leftMargin();
+    layout.rightMargin = page.rightMargin();
+    layout.leftPadding = page.leftPadding();
+    layout.rightPadding = page.rightPadding();
     qreal left = 0, width = page.width();
     if (page.pageSide() == KWPage::PageSpread) {
         width /= 2;
         if (page.pageNumber() != pageNumber) { // doing the 'right' one
             left = width;
-            qreal x = layout.left; // swap margins
-            layout.left = layout.right;
-            layout.right = x;
+            qreal x = layout.leftMargin; // swap margins
+            layout.leftMargin = layout.rightMargin;
+            layout.rightMargin = x;
+            x = layout.leftPadding; // swap paddings
+            layout.leftPadding = layout.rightPadding;
+            layout.rightPadding = x;
         }
     }
-    qreal textWidth = width - layout.left - layout.right;
+    qreal textWidth = width - layout.leftMargin - layout.rightMargin
+                            - layout.leftPadding - layout.rightPadding;
 
     KWPageStyle pageStyle = page.pageStyle();
-    const int columns = pageStyle.hasMainTextFrame() ? pageStyle.columns().columns * (page.pageSide() == KWPage::PageSpread ? 2: 1) : 0;
+    const int columns = pageStyle.hasMainTextFrame() ? pageStyle.columns().columns * (page.pageSide() == KWPage::PageSpread ? 2 : 1) : 0;
     int columnsCount = columns;
     KWTextFrame **main, *footer = 0, *endnote = 0, *header = 0, *footnote = 0;
     main = new KWTextFrame*[columnsCount];
-    main[0] = 0;
+    if (columns > 0)
+        main[0] = 0;
     QRectF pageRect(left, page.offsetInDocument(), width, page.height());
     foreach (KWFrame *frame, framesInPage(pageRect)) {
         KWTextFrameSet *textFrameSet = dynamic_cast<KWTextFrameSet*>(frame->frameSet());
@@ -277,11 +289,17 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
         }
         case KWord::MainTextFrameSet: {
             if (columnsCount < 1) {
-                kWarning() << "Too many columns present on page, ignoring 1" << endl;
+                kWarning(32001) << "Too many columns present on page, ignoring 1";
                 break;
             }
             main[--columnsCount] = static_cast<KWTextFrame *>(frame);
             minimumHeight[3] = 10;
+            // make at least one line fit lest we add endless pages.
+            QTextLayout *layout = textFrameSet->document()->begin().layout();
+            if (layout && layout->lineCount() > 0) {
+                minimumHeight[3] = qMax((qreal) 10, layout->lineAt(0).height());
+            }
+
             requestedHeight[3] = -1; // rest
             break;
         }
@@ -291,7 +309,7 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
     }
     if (minZIndex < INT_MAX) {
         --minZIndex;
-        for (int i=0; i < columns; ++i)
+        for (int i = 0; i < columns; ++i)
             main[i]->shape()->setZIndex(minZIndex);
         if (footer)
             footer->shape()->setZIndex(minZIndex);
@@ -337,11 +355,12 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
     }
 
     // actually move / size the frames.
-    if (main[0]) {
+    if (columns > 0 && main[0]) {
         const qreal columnWidth = textWidth / columns;
         QPointF *points = new QPointF[columns];
         for (int i = columns - 1; i >= 0; i--)
-            points[i] = QPointF(left + layout.left + columnWidth * i, resultingPositions[3]);
+            points[i] = QPointF(left + layout.leftMargin + layout.leftPadding
+                                + columnWidth * i, resultingPositions[3]);
         for (int i = 0; i < columns; i++) {
             for (int f = 0; f < columns; f++) {
                 if (f == i) continue;
@@ -370,22 +389,22 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
 #endif
     if (footnote) {
         footnote->shape()->setPosition(
-            QPointF(left + layout.left, resultingPositions[7]));
+            QPointF(left + layout.leftMargin + layout.leftPadding, resultingPositions[7]));
         footnote->shape()->setSize(QSizeF(textWidth, resultingPositions[8] - resultingPositions[7]));
     }
     if (endnote) {
         endnote->shape()->setPosition(
-            QPointF(left + layout.left, resultingPositions[5]));
+            QPointF(left + layout.leftMargin + layout.leftPadding, resultingPositions[5]));
         endnote->shape()->setSize(QSizeF(textWidth, resultingPositions[6] - resultingPositions[5]));
     }
     if (header) {
         header->shape()->setPosition(
-            QPointF(left + layout.left, resultingPositions[1]));
+            QPointF(left + layout.leftMargin + layout.leftPadding, resultingPositions[1]));
         header->shape()->setSize(QSizeF(textWidth, resultingPositions[2] - resultingPositions[1]));
     }
     if (footer) {
         footer->shape()->setPosition(
-            QPointF(left + layout.left, resultingPositions[9]));
+            QPointF(left + layout.leftMargin + layout.leftPadding, resultingPositions[9]));
         footer->shape()->setSize(QSizeF(textWidth, resultingPositions[10] - resultingPositions[9]));
     }
     delete [] main;
@@ -414,7 +433,7 @@ bool KWFrameLayout::shouldHaveHeaderOrFooter(int pageNumber, bool header, KWord:
     return true;
 }
 
-QList<KWFrame *> KWFrameLayout::framesInPage(QRectF page)
+QList<KWFrame *> KWFrameLayout::framesInPage(const QRectF &page)
 {
     // hopefully replaced with a tree
     QList<KWFrame*> answer;
@@ -495,8 +514,8 @@ void KWFrameLayout::setup()
             break;
         case KWord::MainTextFrameSet:
             m_maintext = tfs;
-            disconnect (tfs, SIGNAL(frameRemoved(KWFrame*)), this, SLOT(mainframeRemoved(KWFrame*)));
-            connect (tfs, SIGNAL(frameRemoved(KWFrame*)), this, SLOT(mainframeRemoved(KWFrame*)));
+            disconnect(tfs, SIGNAL(frameRemoved(KWFrame*)), this, SLOT(mainframeRemoved(KWFrame*)));
+            connect(tfs, SIGNAL(frameRemoved(KWFrame*)), this, SLOT(mainframeRemoved(KWFrame*)));
         default: ;// ignore
         }
         if (tfs->pageStyle().isValid())
@@ -508,12 +527,12 @@ void KWFrameLayout::setup()
 KoShape *KWFrameLayout::createTextShape(const KWPage &page)
 {
     Q_ASSERT(page.isValid());
-    KoShapeFactory *factory = KoShapeRegistry::instance()->value(TextShape_SHAPEID);
+    KoShapeFactoryBase *factory = KoShapeRegistry::instance()->value(TextShape_SHAPEID);
     Q_ASSERT(factory);
-    QMap<QString, KoDataCenter *> dataCenterMap;
+    KoResourceManager *rm = 0;
     if (m_document)
-        dataCenterMap = m_document->dataCenterMap();
-    KoShape *shape = factory->createDefaultShapeAndInit(dataCenterMap);
+        rm = m_document->resourceManager();
+    KoShape *shape = factory->createDefaultShape(rm);
     shape->setPosition(QPointF(0, page.offsetInDocument()));
     return shape;
 }
@@ -655,7 +674,7 @@ void KWFrameLayout::createNewFrameForPage(KWTextFrameSet *fs, int pageNumber)
     }
 }
 
-KWFrame* KWFrameLayout::createCopyFrame(KWFrameSet *fs, const KWPage &page)
+KWFrame *KWFrameLayout::createCopyFrame(KWFrameSet *fs, const KWPage &page)
 {
     Q_ASSERT(page.isValid());
     if (fs->frameCount() == 0) { // special case for the headers. Just return a new textframe.
@@ -667,11 +686,18 @@ KWFrame* KWFrameLayout::createCopyFrame(KWFrameSet *fs, const KWPage &page)
 
         return frame;
     }
-    KWFrame *lastFrame = fs->frames().last();
-    KWCopyShape *shape = new KWCopyShape(lastFrame->shape());
+
+    KoShape *orig = 0;
+    foreach (KWFrame *candidate, fs->frames()) {
+        if (candidate->isCopy())
+            break;
+        orig = candidate->shape();
+    }
+    Q_ASSERT(orig); // can't have a frameset with only copy frames.
+
+    KWCopyShape *shape = new KWCopyShape(orig, m_pageManager);
     shape->setPosition(QPointF(0, page.offsetInDocument()));
     KWFrame *frame = new KWFrame(shape, fs);
-    frame->makeCopyFrame();
     return frame;
 }
 
