@@ -1,6 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
- * Copyright (C) 2008 Pierre Ducroquet <pinaraf@pinaraf.info>
+ * Copyright (C) 2006-2007, 2010 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -37,8 +36,10 @@
 #include <QList>
 #include <QPainterPath>
 #include <QTextBlock>
+#include <qnumeric.h>
 
 // #define DEBUG_TEXT
+// #define DEBUG_ANCHORS
 
 #ifdef DEBUG_TEXT
 #define TDEBUG kDebug(32002)
@@ -46,6 +47,12 @@
 #define TDEBUG if(0) kDebug(32002)
 #endif
 
+#ifdef DEBUG_ANCHORS
+#define ADEBUG kDebug(32002)
+#else
+#define ADEBUG if(0) kDebug(32002)
+#endif
+
 // helper methods
 static qreal xAtY(const QLineF &line, qreal y)
 {
@@ -121,13 +128,20 @@ public:
     }
 
     void init(const QMatrix &matrix, KoShape *shape, qreal distance) {
+        m_shape = shape;
         QPainterPath path =  matrix.map(shape->outline());
         m_bounds = path.boundingRect();
         if (distance >= 0.0) {
             QMatrix grow = matrix;
             grow.translate(m_bounds.width() / 2.0, m_bounds.height() / 2.0);
-            const qreal scaleX = (m_bounds.width() + distance) / m_bounds.width();
-            const qreal scaleY = (m_bounds.height() + distance) / m_bounds.height();
+            qreal scaleX = distance;
+            if (m_bounds.width() > 0)
+                scaleX = (m_bounds.width() + distance) / m_bounds.width();
+            qreal scaleY = distance;
+            if (m_bounds.height() > 0)
+                scaleY = (m_bounds.height() + distance) / m_bounds.height();
+            Q_ASSERT(!qIsNaN(scaleY));
+            Q_ASSERT(!qIsNaN(scaleX));
             grow.scale(scaleX, scaleY);
             grow.translate(-m_bounds.width() / 2.0, -m_bounds.height() / 2.0);
 
@@ -204,11 +218,14 @@ public:
         return answer;
     }
 
+    KoShape *shape() const { return m_shape; }
+
 private:
     enum Side { None, Left, Right, Empty }; // TODO support Auto and Both
     Side m_side;
     QMultiMap<qreal, QLineF> m_edges; //sorted with y-coord
     QRectF m_bounds;
+    KoShape *m_shape;
 };
 
 class KWTextDocumentLayout::DummyShape : public KoShape
@@ -268,19 +285,18 @@ void KWTextDocumentLayout::relayout()
         return;
     const QList<KWFrame*> frames = m_frameSet->frames();
     QList<KWFrame*> dirtyFrames = frames;
-    bool foundADirtyOne = false;
     KWFrame *firstDirtyFrame = 0;
     foreach (KWFrame *frame, frames) {
         KoTextShapeData *data = qobject_cast<KoTextShapeData*>(frame->shape()->userData());
         if (!firstDirtyFrame && data && data->isDirty())
             firstDirtyFrame = frame;
-        if (! firstDirtyFrame)
+        if (!firstDirtyFrame)
             dirtyFrames.removeAll(frame);
     }
 
     // make sure the ordering is proper
     m_frameSet->sortFrames();
-    if (foundADirtyOne) {
+    if (firstDirtyFrame) {
         // if the dirty frame has been resorted to no longer be the first one, then we should
         // mark dirty any frame that were previously later in the flow, but are now before it.
         foreach (KWFrame *frame, frames) {
@@ -312,14 +328,20 @@ void KWTextDocumentLayout::positionInlineObject(QTextInlineObject item, int posi
             if (strategy->anchor() == anchor)
                 return;
         }
-        TDEBUG << "new anchor";
+        ADEBUG << "new anchor";
         m_newAnchors.append(new KWAnchorStrategy(anchor));
     }
 }
 
 void KWTextDocumentLayout::layout()
 {
-    TDEBUG << "starting layout pass";
+    TDEBUG << "starting layout pass" << ((void*)document())
+#ifdef DEBUG_ANCHORS
+     << "m_newAnchors" << m_newAnchors.count() << "m_activeAnchors" << m_activeAnchors.count();
+#else
+    ;
+#endif
+
     QList<Outline*> outlines;
     class End
     {
@@ -342,8 +364,22 @@ void KWTextDocumentLayout::layout()
     qreal bottomOfText = 0.0;
     bool newParagraph = true;
     bool requestFrameResize = false, firstParagraph = true;
+    int startOfBlock = -1; // the first position in a block (aka paragraph)
+    int startOfBlockText = -1; // the first position of text in a block. Will be different only if anchors preceed text.
     KoShape *currentShape = 0;
     while (m_state->shape) {
+        ADEBUG << "> loop.... layout has" << m_state->layout->lineCount() << "lines, we have" << m_activeAnchors.count() << "+" << m_newAnchors.count() <<"anchors";
+#ifndef DEBUG_ANCHORS
+        for (int i = 0; i < m_state->layout->lineCount(); ++i) {
+            QTextLine line = m_state->layout->lineAt(i);
+            ADEBUG << i << "]" << (line.isValid() ? QString("%1 - %2").arg(line.textStart()).arg(line.textLength()) : QString("invalid"));
+        }
+#endif
+        if (m_state->layout->lineCount() == 0 && startOfBlock != m_state->cursorPosition()) {  // new paragraph
+            startOfBlock = m_state->cursorPosition();
+            startOfBlockText = m_state->cursorPosition();
+        }
+
         class Line
         {
         public:
@@ -369,10 +405,13 @@ void KWTextDocumentLayout::layout()
                 }
                 rect = limit(rect);
 
+                qreal movedDown = 0;
+                qreal eligableTop = rect.top();
                 while (true) {
-                    if (m_state->numColumns() > 0)
+                    if (m_state->numColumns() > 0) {
                         line.setNumColumns(m_state->numColumns());
-                    else {
+                        line.setPosition(QPointF(line.x(), rect.y()));
+                    } else {
                         qreal x = rect.x();
                         qreal effectiveLineWidth = rect.width();
                         if (leftIndent < x) { // limit moved the left edge, keep the indent.
@@ -388,15 +427,30 @@ void KWTextDocumentLayout::layout()
                     rect.setHeight(line.height());
 
                     QRectF newLine = limit(rect);
-                    if (newLine.width() <= 0.)
-                        // TODO be more intelligent than just moving down 10 pt
-                        rect = QRectF(leftIndent, rect.top() + 10, width, rect.height());
-                    else if (qAbs(newLine.left() - rect.left()) < 1E-10 && qAbs(newLine.right() - rect.right()) < 1E-10)
+                    if (newLine.width() <= 0 || line.textLength() == 0
+                            || line.cursorToX(line.textStart()+1) > newLine.right()) {
+                        const int Move = 10;
+                        movedDown += Move;
+                        if (movedDown > m_state->shape->size().height() * 1.3) {
+                            // TODO the magic number of shape height needs some history
+                            // investigation here.
+                            rect = QRectF(leftIndent, eligableTop, width, rect.height());
+                            break;
+                        }
+                        if (newLine.width() > 0) {
+                            // at least one char fits
+                            eligableTop = rect.top();
+                        }
+
+                        rect = QRectF(leftIndent, rect.top() + Move, width, rect.height());
+                    } else if (qAbs(newLine.left() - rect.left()) < 1E-10 && qAbs(newLine.right()
+                                - rect.right()) < 1E-10) {
                         break;
-                    else if (newLine.isValid())
+                    } else if (newLine.isValid()) {
                         rect = newLine;
-                    else
+                    } else {
                         break;
+                    }
                 }
             }
             void setOutlines(const QList<Outline*> &outlines) {
@@ -474,34 +528,69 @@ void KWTextDocumentLayout::layout()
         // anchors might require us to do some layout again, give it the chance to 'do as it will'
         bool restartLine = false;
         foreach (KWAnchorStrategy *strategy, m_activeAnchors + m_newAnchors) {
-            TDEBUG << "checking anchor";
-            if (strategy->checkState(m_state)) {
-                TDEBUG << "  restarting line";
+            ADEBUG << "checking anchor";
+            QPointF old;
+            if (strategy->anchoredShape())
+                old = strategy->anchoredShape()->position();
+            if (strategy->checkState(m_state, startOfBlock, startOfBlockText)) {
+                ADEBUG << "  restarting line";
                 restartLine = true;
-                break;
             }
+            if (strategy->anchoredShape() && old != strategy->anchoredShape()->position()) {
+                // refresh outlines in case the shape moved.
+                foreach (Outline *outline, outlines) {
+                    if (outline->shape() == strategy->anchoredShape()) {
+                        ADEBUG << "  refreshing outline";
+                        outlines.removeAll(outline);
+                        QMatrix matrix = strategy->anchoredShape()->absoluteTransformation(0);
+                        matrix = matrix * currentShape->absoluteTransformation(0).inverted();
+                        matrix.translate(0, m_state->documentOffsetInShape());
+                        outlines.append(new Outline(strategy->anchoredShape(), matrix));
+                        break;
+                    }
+                }
+            }
+            if (restartLine)
+                break;
         }
+
         if (restartLine)
             continue;
         foreach (KWAnchorStrategy *strategy, m_activeAnchors + m_newAnchors) {
+            ADEBUG << " + isFinished?"<< strategy->isFinished();
             if (strategy->isFinished() && strategy->anchor()->positionInDocument() < m_state->cursorPosition()) {
-                TDEBUG << "  is finished";
+                ADEBUG << "  is finished";
                 m_activeAnchors.removeAll(strategy);
                 m_newAnchors.removeAll(strategy);
                 delete strategy;
             }
         }
-
         foreach (KWAnchorStrategy *strategy, m_newAnchors) {
-            if (strategy->anchoredShape() != 0) {
+            ADEBUG << "  migrating strategy!";
+            if (strategy->anchoredShape()) {
                 QMatrix matrix = strategy->anchoredShape()->absoluteTransformation(0);
                 matrix = matrix * currentShape->absoluteTransformation(0).inverted();
                 matrix.translate(0, m_state->documentOffsetInShape());
                 outlines.append(new Outline(strategy->anchoredShape(), matrix));
+
+                // if the anchor occupies the first character of our block, create one line for it.
+                const int cursorPosition = m_state->cursorPosition();
+                if (cursorPosition == startOfBlockText && strategy->anchor()->positionInDocument() == cursorPosition) {
+                    ADEBUG << "   creating line for anchor";
+                    QTextLine line = m_state->layout->createLine();
+                    line.setNumColumns(1);
+                    line.setPosition(QPointF(m_state->x(), m_state->y()));
+                    m_state->addLine(line);
+                    ++startOfBlockText;
+                }
+                restartLine = true;
+                ADEBUG << "adding child outline";
             }
             m_activeAnchors.append(strategy);
         }
         m_newAnchors.clear();
+        if (restartLine)
+            continue;
 
         Line line(m_state);
         if (!line.isValid()) { // end of parag
@@ -543,6 +632,8 @@ void KWTextDocumentLayout::layout()
                 }
 
                 m_frameSet->layoutDone();
+                cleanupAnchors();
+                emit finishedLayout();
                 return; // done!
             } else if (m_state->shape == 0) {
                 TDEBUG << "encountered an 'end of page' break, we need an extra page to honor that!";
@@ -555,7 +646,7 @@ void KWTextDocumentLayout::layout()
             newParagraph = true;
             continue;
         }
-        if (m_state->interrupted() || (newParagraph && m_state->y() > endPos)) {
+        if (m_state->isInterrupted() || (newParagraph && m_state->y() > endPos)) {
             // enough for now. Try again later.
             TDEBUG << "schedule a next layout due to having done a layout of quite some space";
             scheduleLayoutWithoutInterrupt();
@@ -563,27 +654,37 @@ void KWTextDocumentLayout::layout()
         }
         newParagraph = false;
         line.setOutlines(outlines);
+        const int anchorCount = m_newAnchors.count();
         line.tryFit();
+        if (m_state->layout->lineCount() == 1 && anchorCount != m_newAnchors.count()) {
+            // start parag over so we can correctly take the just found anchors into account.
+            m_state->layout->endLayout();
+            m_state->layout->beginLayout();
+            continue;
+        }
 #ifdef DEBUG_TEXT
         if (line.line.isValid()) {
             QTextBlock b = document()->findBlock(m_state->cursorPosition());
             if (b.isValid()) {
                 TDEBUG << "fitted line" << b.text().mid(line.line.textStart(), line.line.textLength());
                 TDEBUG << "       1 @ " << line.line.position() << " from parag at pos " << b.position();
+                TDEBUG << "         y " << line.line.y() << " h" <<  line.line.height();
             }
         }
 #endif
 
         bottomOfText = line.line.y() + line.line.height();
         if (bottomOfText > m_state->shape->size().height() && document()->blockCount() == 1) {
+            TDEBUG << "requestMoreFrames" << (bottomOfText - m_state->shape->size().height());
             m_frameSet->requestMoreFrames(bottomOfText - m_state->shape->size().height());
+            cleanupAnchors();
             return;
         }
 
         while (m_state->addLine(line.line)) {
             if (m_state->shape == 0) { // no more shapes to put the text in!
                 TDEBUG << "no more shape for our text; bottom is" << m_state->y();
-                line.line.setPosition(QPointF(0, m_state->y() + 20));
+                line.line.setPosition(QPointF(0, m_state->y() + 10000)); // move it away from any place that we're likely going to paint
 
                 if (requestFrameResize) { // plenty more text, but first lets resize the shape.
                     TDEBUG << "  we need more space; we require at least:" << m_dummyShape->size().height();
@@ -602,6 +703,7 @@ void KWTextDocumentLayout::layout()
                         || dynamic_cast<KWCopyShape*>(lastFrame)) {
                     m_state->clearTillEnd();
                     m_frameSet->layoutDone();
+                    emit finishedLayout();
                     return; // done!
                 }
 
@@ -626,7 +728,7 @@ void KWTextDocumentLayout::layout()
                 if (list.count() > 0)
                     down2 = QLineF(down2.p1(), list.last());
                 const qreal maxFrameLength = qMin(down.length(), down2.length());
-                if (maxFrameLength <= currentShape->size().height()) {
+                if (qAbs(maxFrameLength - currentShape->size().height()) < 1) {
                     m_state->clearTillEnd();
                     TDEBUG << "  we need another page";
                     m_frameSet->requestMoreFrames(0); // new page, please.
@@ -640,6 +742,7 @@ void KWTextDocumentLayout::layout()
                 if (! m_state->setFollowupShape(m_dummyShape)) { // if I can't render into a dummy shape
                     m_state->clearTillEnd();
                     m_frameSet->layoutDone();
+                    emit finishedLayout();
                     return; // done!
                 }
                 requestFrameResize = true;
@@ -655,15 +758,17 @@ void KWTextDocumentLayout::layout()
             }
 #endif
         }
-
-        QRectF repaintRect = line.line.rect();
-        repaintRect.moveTop(repaintRect.y() - m_state->docOffsetInShape());
-        repaintRect.setX(0.0); // just take full width since we can't force a repaint of
-        repaintRect.setWidth(m_state->shape->size().width()); // where lines were before layout.
-        m_state->shape->update(repaintRect);
     }
     if (requestFrameResize) {
         TDEBUG << "  requestFrameResize" << m_dummyShape->size().height();
         m_frameSet->requestMoreFrames(m_dummyShape->size().height());
     }
 }
+
+void KWTextDocumentLayout::cleanupAnchors()
+{
+    qDeleteAll(m_activeAnchors);
+    m_activeAnchors.clear();
+    qDeleteAll(m_newAnchors);
+    m_newAnchors.clear();
+}
