@@ -26,18 +26,14 @@
 /**
 * The KWScriptingPart class implements a KPart component
 * to integrate scripting into KWord.
+* \internal
 */
 class KWScriptingPart : public KoScriptingPart
 {
     Q_OBJECT
 public:
-    KWScriptingPart(QObject* parent, const QStringList&);
+    KWScriptingPart(QObject *parent, const QStringList&);
     virtual ~KWScriptingPart();
-private:
-    /// \internal d-pointer class.
-    class Private;
-    /// \internal d-pointer instance.
-    Private* const d;
 };
 
 #endif
