@@ -29,10 +29,9 @@
 #include "KoShape.h"
 #include "KoViewConverter.h"
 #include "KoCanvasBase.h"
-#include "KoCanvasObserver.h"
-#include "KoCanvasObserverProvider.h"
-#include "tools/KoGuidesTool.h"
-#include "KoToolManager.h"
+#include "KoCanvasObserverBase.h"
+#include "KoCanvasSupervisor.h"
+#include "KoToolManager_p.h"
 
 #include <ksharedconfig.h>
 #include <KDebug>
@@ -50,33 +49,130 @@
 #include <QtOpenGL/QGLWidget>
 #endif
 
-class KoCanvasController::Private
+KoCanvasController::Private::Private(KoCanvasController *qq)
+    : q(qq),
+    canvas(0),
+    canvasMode(Centered),
+    margin(0),
+    preferredCenterFractionX(0.5),
+    preferredCenterFractionY(0.5),
+    ignoreScrollSignals(false)
 {
-public:
-    Private()
-        : canvas(0),
-        canvasMode(Centered),
-        margin(0),
-        preferredCenterFractionX(0.5),
-        preferredCenterFractionY(0.5),
-        ignoreScrollSignals(false)
-    {
+}
+
+void KoCanvasController::Private::setDocumentOffset()
+{
+    // The margins scroll the canvas widget inside the viewport, not
+    // the document. The documentOffset is meant the be the value that
+    // the canvas must add to the update rect in its paint event, to
+    // compensate.
+
+    QPoint pt(q->horizontalScrollBar()->value(), q->verticalScrollBar()->value());
+    if (pt.x() < margin) pt.setX(0);
+    if (pt.y() < margin) pt.setY(0);
+    if (pt.x() > documentSize.width()) pt.setX(documentSize.width());
+    if (pt.y() > documentSize.height()) pt.setY(documentSize.height());
+    emit q->moveDocumentOffset(pt);
+
+    QWidget *canvasWidget = canvas->canvasWidget();
+
+    if (canvasWidget) {
+        if (!q->isCanvasOpenGL()) {
+            QPoint diff = documentOffset - pt;
+            canvasWidget->scroll(diff.x(), diff.y());
+        }
     }
 
-    KoCanvasBase * canvas;
-    CanvasMode canvasMode;
-    int margin; // The viewport margin around the document // TODO can we remove this one? The viewport has a copy...
-    QSize documentSize;
-    QPoint documentOffset;
-    Viewport * viewportWidget;
-    qreal preferredCenterFractionX;
-    qreal preferredCenterFractionY;
-    bool ignoreScrollSignals;
-};
+    documentOffset = pt;
+}
 
+void KoCanvasController::Private::resetScrollBars()
+{
+    // The scrollbar value always points at the top-left corner of the
+    // bit of image we paint.
+
+    int docH = documentSize.height() + margin;
+    int docW = documentSize.width() + margin;
+    int drawH = viewportWidget->height();
+    int drawW = viewportWidget->width();
+
+    QScrollBar *hScroll = q->horizontalScrollBar();
+    QScrollBar *vScroll = q->verticalScrollBar();
+
+    if (docH <= drawH && docW <= drawW) {
+        // we need no scrollbars
+        vScroll->setRange(0, 0);
+        hScroll->setRange(0, 0);
+    } else if (docH <= drawH) {
+        // we need a horizontal scrollbar only
+        vScroll->setRange(0, 0);
+        hScroll->setRange(0, docW - drawW);
+    } else if (docW <= drawW) {
+        // we need a vertical scrollbar only
+        hScroll->setRange(0, 0);
+        vScroll->setRange(0, docH - drawH);
+    } else {
+        // we need both scrollbars
+        vScroll->setRange(0, docH - drawH);
+        hScroll->setRange(0, docW - drawW);
+    }
+
+    int fontheight = QFontMetrics(q->font()).height();
+
+    vScroll->setPageStep(drawH);
+    vScroll->setSingleStep(fontheight);
+    hScroll->setPageStep(drawW);
+    hScroll->setSingleStep(fontheight);
+
+}
+
+void KoCanvasController::Private::emitPointerPositionChangedSignals(QEvent *event)
+{
+    if (!canvas) return;
+    if (!canvas->viewConverter()) return;
+
+    QPoint pointerPos;
+    QMouseEvent *mouseEvent = dynamic_cast<QMouseEvent*>(event);
+    if (mouseEvent) {
+        pointerPos = mouseEvent->pos();
+    } else {
+        QTabletEvent *tabletEvent = dynamic_cast<QTabletEvent*>(event);
+        if (tabletEvent) {
+            pointerPos = tabletEvent->pos();
+        }
+    }
+
+    QPoint pixelPos = (pointerPos - canvas->documentOrigin()) + documentOffset;
+    QPointF documentPos = canvas->viewConverter()->viewToDocument(pixelPos);
+
+    emit q->documentMousePositionChanged(documentPos);
+    emit q->canvasMousePositionChanged(pointerPos);
+}
+
+
+void KoCanvasController::Private::activate()
+{
+    QWidget *parent = q;
+    while (parent->parentWidget())
+        parent = parent->parentWidget();
+
+    KoCanvasSupervisor *observerProvider = dynamic_cast<KoCanvasSupervisor*>(parent);
+    if (!observerProvider)
+        return;
+
+    foreach(KoCanvasObserverBase *docker, observerProvider->canvasObservers()) {
+        KoCanvasObserverBase *observer = dynamic_cast<KoCanvasObserverBase*>(docker);
+        if (observer) {
+            observer->setCanvas(q->canvas());
+        }
+    }
+}
+
+
+////////////
 KoCanvasController::KoCanvasController(QWidget *parent)
         : QAbstractScrollArea(parent),
-        d(new Private())
+        d(new Private(this))
 {
     setFrameShape(NoFrame);
     d->viewportWidget = new Viewport(this);
@@ -107,7 +203,7 @@ void KoCanvasController::scrollContentsBy(int dx, int dy)
 {
     Q_UNUSED(dx);
     Q_UNUSED(dy);
-    setDocumentOffset();
+    d->setDocumentOffset();
 }
 
 void KoCanvasController::setDrawShadow(bool drawShadow)
@@ -121,8 +217,8 @@ void KoCanvasController::resizeEvent(QResizeEvent *resizeEvent)
 
     // XXX: When resizing, keep the area we're looking at now in the
     // center of the resized view.
-    resetScrollBars();
-    setDocumentOffset();
+    d->resetScrollBars();
+    d->setDocumentOffset();
 }
 
 void KoCanvasController::setCanvas(KoCanvasBase *canvas)
@@ -133,11 +229,11 @@ void KoCanvasController::setCanvas(KoCanvasBase *canvas)
         canvas->setCanvasController(0);
         d->canvas->canvasWidget()->removeEventFilter(this);
     }
+    canvas->setCanvasController(this);
     d->viewportWidget->setCanvas(canvas->canvasWidget());
     d->canvas = canvas;
     d->canvas->canvasWidget()->installEventFilter(this);
     d->canvas->canvasWidget()->setMouseTracking(true);
-    canvas->setCanvasController(this);
     setFocusProxy(d->canvas->canvasWidget());
 
     emit canvasSet(this);
@@ -275,35 +371,12 @@ bool KoCanvasController::eventFilter(QObject *watched, QEvent *event)
             updateCanvasOffsetX();
             updateCanvasOffsetY();
         } else if (event->type() == QEvent::MouseMove || event->type() == QEvent::TabletMove) {
-            emitPointerPositionChangedSignals(event);
+            d->emitPointerPositionChangedSignals(event);
         }
     }
     return false;
 }
 
-void KoCanvasController::emitPointerPositionChangedSignals(QEvent *event)
-{
-    if (!d->canvas) return;
-    if (!d->canvas->viewConverter()) return;
-
-    QPoint pointerPos;
-    QMouseEvent *mouseEvent = dynamic_cast<QMouseEvent*>(event);
-    if (mouseEvent) {
-        pointerPos = mouseEvent->pos();
-    } else {
-        QTabletEvent *tabletEvent = dynamic_cast<QTabletEvent*>(event);
-        if (tabletEvent) {
-            pointerPos = tabletEvent->pos();
-        }
-    }
-
-    QPoint pixelPos = (pointerPos - d->canvas->documentOrigin()) + d->documentOffset;
-    QPointF documentPos = d->canvas->viewConverter()->viewToDocument(pixelPos);
-
-    emit documentMousePositionChanged(documentPos);
-    emit canvasMousePositionChanged(pointerPos);
-}
-
 void KoCanvasController::ensureVisible(KoShape *shape)
 {
     Q_ASSERT(shape);
@@ -447,7 +520,7 @@ void KoCanvasController::setDocumentSize(const QSize &sz, bool recalculateCenter
     d->ignoreScrollSignals = true;
     d->documentSize = sz;
     d->viewportWidget->setDocumentSize(sz);
-    resetScrollBars();
+    d->resetScrollBars();
     d->ignoreScrollSignals = oldIgnoreScrollSignals;
 
     // in case the document got so small a slider dissapeared; emit the new offset.
@@ -457,32 +530,6 @@ void KoCanvasController::setDocumentSize(const QSize &sz, bool recalculateCenter
         updateCanvasOffsetY();
 }
 
-void KoCanvasController::setDocumentOffset()
-{
-    // The margins scroll the canvas widget inside the viewport, not
-    // the document. The documentOffset is meant the be the value that
-    // the canvas must add to the update rect in its paint event, to
-    // compensate.
-
-    QPoint pt(horizontalScrollBar()->value(), verticalScrollBar()->value());
-    if (pt.x() < d->margin) pt.setX(0);
-    if (pt.y() < d->margin) pt.setY(0);
-    if (pt.x() > d->documentSize.width()) pt.setX(d->documentSize.width());
-    if (pt.y() > d->documentSize.height()) pt.setY(d->documentSize.height());
-    emit(moveDocumentOffset(pt));
-
-    QWidget *canvasWidget = d->canvas->canvasWidget();
-
-    if (canvasWidget) {
-        if (!isCanvasOpenGL()) {
-            QPoint diff = d->documentOffset - pt;
-            canvasWidget->scroll(diff.x(), diff.y());
-        }
-    }
-
-    d->documentOffset = pt;
-}
-
 bool KoCanvasController::isCanvasOpenGL() const
 {
     QWidget *canvasWidget = d->canvas->canvasWidget();
@@ -498,46 +545,6 @@ bool KoCanvasController::isCanvasOpenGL() const
     return false;
 }
 
-void KoCanvasController::resetScrollBars()
-{
-    // The scrollbar value always points at the top-left corner of the
-    // bit of image we paint.
-
-    int docH = d->documentSize.height() + d->margin;
-    int docW = d->documentSize.width() + d->margin;
-    int drawH = d->viewportWidget->height();
-    int drawW = d->viewportWidget->width();
-
-    QScrollBar * hScroll = horizontalScrollBar();
-    QScrollBar * vScroll = verticalScrollBar();
-
-    if (docH <= drawH && docW <= drawW) {
-        // we need no scrollbars
-        vScroll->setRange(0, 0);
-        hScroll->setRange(0, 0);
-    } else if (docH <= drawH) {
-        // we need a horizontal scrollbar only
-        vScroll->setRange(0, 0);
-        hScroll->setRange(0, docW - drawW);
-    } else if (docW <= drawW) {
-        // we need a vertical scrollbar only
-        hScroll->setRange(0, 0);
-        vScroll->setRange(0, docH - drawH);
-    } else {
-        // we need both scrollbars
-        vScroll->setRange(0, docH - drawH);
-        hScroll->setRange(0, docW - drawW);
-    }
-
-    int fontheight = QFontMetrics(font()).height();
-
-    vScroll->setPageStep(drawH);
-    vScroll->setSingleStep(fontheight);
-    hScroll->setPageStep(drawW);
-    hScroll->setSingleStep(fontheight);
-
-}
-
 void KoCanvasController::pan(const QPoint &distance)
 {
     QScrollBar *hBar = horizontalScrollBar();
@@ -591,7 +598,7 @@ void KoCanvasController::dragLeaveEvent(QDragLeaveEvent *event)
 
 void KoCanvasController::keyPressEvent(QKeyEvent *event)
 {
-    KoToolManager::instance()->switchToolByShortcut(event);
+    KoToolManager::instance()->priv()->switchToolByShortcut(event);
 }
 
 void KoCanvasController::wheelEvent(QWheelEvent *event)
@@ -668,39 +675,9 @@ int KoCanvasController::margin() const
     return d->margin;
 }
 
-void KoCanvasController::activate()
+KoCanvasController::Private *KoCanvasController::priv()
 {
-    QWidget *parent = this;
-    while (parent->parentWidget())
-        parent = parent->parentWidget();
-
-    KoCanvasObserverProvider *observerProvider = dynamic_cast<KoCanvasObserverProvider*>(parent);
-    if (!observerProvider)
-        return;
-
-    foreach(KoCanvasObserver *docker, observerProvider->canvasObservers()) {
-        KoCanvasObserver *observer = dynamic_cast<KoCanvasObserver*>(docker);
-        if (observer) {
-            observer->setCanvas(canvas());
-        }
-    }
+    return d;
 }
 
-void KoCanvasController::addGuideLine(Qt::Orientation orientation, int viewPosition)
-{
-    KoGuidesTool *guidesTool = KoToolManager::instance()->guidesTool(d->canvas);
-    if (!guidesTool)
-        return;
-    // check if the canvas does provide access to guides data
-    if (!d->canvas->guidesData())
-        return;
-
-    if (orientation == Qt::Horizontal)
-        guidesTool->addGuideLine(orientation, d->canvas->viewConverter()->viewToDocumentY(viewPosition));
-    else
-        guidesTool->addGuideLine(orientation, d->canvas->viewConverter()->viewToDocumentX(viewPosition));
-
-    KoToolManager::instance()->switchToolTemporaryRequested(guidesTool->toolId());
-}
-
-#include "KoCanvasController.moc"
+#include <KoCanvasController.moc>
