@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006,2008 Jan Hambrecht <jaham@gmx.net>
+ * Copyright (C) 2006,2008-2009 Jan Hambrecht <jaham@gmx.net>
  * Copyright (C) 2006,2007 Thorsten Zachmann <zachmann@kde.org>
- *               2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2007, 2010 Thomas Zander <zander@kde.org>
  * Copyright (C) 2007 Boudewijn Rempt <boud@valdyas.org>
  *
  * This library is free software; you can redistribute it and/or
@@ -21,10 +21,14 @@
  */
 
 #include "KoPathTool.h"
+#include "KoToolBase_p.h"
+#include "KoPathShape_p.h"
 #include "KoPathToolHandle.h"
 #include "KoCanvasBase.h"
 #include "KoShapeManager.h"
-#include "KoCanvasResourceProvider.h"
+#include "KoResourceManager.h"
+#include "KoViewConverter.h"
+#include "KoSelection.h"
 #include "KoPointerEvent.h"
 #include "commands/KoPathPointTypeCommand.h"
 #include "commands/KoPathPointInsertCommand.h"
@@ -34,18 +38,19 @@
 #include "commands/KoPathSegmentBreakCommand.h"
 #include "commands/KoParameterToPathCommand.h"
 #include "commands/KoSubpathJoinCommand.h"
+#include "commands/KoPathPointMergeCommand.h"
 #include "KoParameterShape.h"
-#include <KoPathPoint.h>
+#include "KoPathPoint.h"
 #include "KoPathPointRubberSelectStrategy.h"
+#include "KoPathSegmentChangeStrategy.h"
 #include "PathToolOptionWidget.h"
 #include "KoConnectionShape.h"
 #include "KoSnapGuide.h"
-#include "SnapGuideConfigWidget.h"
 
 #include <KAction>
 #include <KIcon>
-#include <kdebug.h>
-#include <klocale.h>
+#include <KDebug>
+#include <KLocale>
 #include <QtGui/QPainter>
 #include <QtGui/QBitmap>
 #include <QtGui/QTabWidget>
@@ -62,9 +67,16 @@ static unsigned char needle_move_bits[] = {
     0x54, 0x7c, 0x38, 0x1c, 0x10, 0x18, 0x00, 0x00
 };
 
+// helper function to calculate the squared distance between two points
+qreal squaredDistance( const QPointF p1, const QPointF &p2 )
+{
+    qreal dx = p1.x()-p2.x();
+    qreal dy = p1.y()-p2.y();
+    return dx*dx + dy*dy;
+}
 
 KoPathTool::KoPathTool(KoCanvasBase *canvas)
-        : KoTool(canvas)
+        : KoToolBase(canvas)
         , m_activeHandle(0)
         , m_handleRadius(3)
         , m_pointSelection(this)
@@ -128,10 +140,9 @@ KoPathTool::KoPathTool(KoCanvasBase *canvas)
     addAction("pathpoint-join", m_actionJoinSegment);
     connect(m_actionJoinSegment, SIGNAL(triggered()), this, SLOT(joinPoints()));
 
-    // TODO: implement me
     m_actionMergePoints = new KAction(KIcon("pathpoint-merge"), i18n("Merge points"), this);
-    m_actionMergePoints->setEnabled(false); // not implemented yet.
     addAction("pathpoint-merge", m_actionMergePoints);
+    connect(m_actionMergePoints, SIGNAL(triggered()), this, SLOT(mergePoints()));
 
     m_actionConvertToPath = new KAction(KIcon("convert-to-path"), i18n("To Path"), this);
     m_actionConvertToPath->setShortcut(Qt::Key_P);
@@ -158,6 +169,7 @@ KoPathTool::~KoPathTool()
 
 QMap<QString, QWidget *>  KoPathTool::createOptionWidgets()
 {
+    Q_D(KoToolBase);
     QMap<QString, QWidget *> map;
 
     PathToolOptionWidget * toolOptions = new PathToolOptionWidget(this);
@@ -165,16 +177,15 @@ QMap<QString, QWidget *>  KoPathTool::createOptionWidgets()
     //connect(this, SIGNAL(pathChanged(KoPathShape*)), widget, SLOT(setSelectedPath(KoPathShape*)));
     updateOptionsWidget();
 
-    SnapGuideConfigWidget * snapOptions = new SnapGuideConfigWidget(m_canvas->snapGuide());
-
     map.insert(i18n("Line/Curve"), toolOptions);
-    map.insert(i18n("Snapping"), snapOptions);
+    map.insert(i18n("Snapping"), d->canvas->createSnapGuideConfigWidget());
 
     return map;
 }
 
 void KoPathTool::pointTypeChanged(QAction *type)
 {
+    Q_D(KoToolBase);
     if (m_pointSelection.hasSelection()) {
         QList<KoPathPointData> selectedPoints = m_pointSelection.selectedPointsData();
         QList<KoPathPointData> pointToChange;
@@ -192,7 +203,7 @@ void KoPathTool::pointTypeChanged(QAction *type)
         if (!pointToChange.isEmpty()) {
             KoPathPointTypeCommand *cmd = new KoPathPointTypeCommand(pointToChange,
                     static_cast<KoPathPointTypeCommand::PointType>(type->data().toInt()));
-            m_canvas->addCommand(cmd);
+            d->canvas->addCommand(cmd);
             updateActions();
         }
     }
@@ -200,12 +211,13 @@ void KoPathTool::pointTypeChanged(QAction *type)
 
 void KoPathTool::insertPoints()
 {
+    Q_D(KoToolBase);
     if (m_pointSelection.size() > 1) {
         QList<KoPathPointData> segments(m_pointSelection.selectedSegmentsData());
         if (!segments.isEmpty()) {
             KoPathPointInsertCommand *cmd = new KoPathPointInsertCommand(segments, 0.5);
-            m_canvas->addCommand(cmd);
-            
+            d->canvas->addCommand(cmd);
+
             foreach( KoPathPoint * p, cmd->insertedPoints() ) {
                 m_pointSelection.add( p, false );
             }
@@ -216,21 +228,23 @@ void KoPathTool::insertPoints()
 
 void KoPathTool::removePoints()
 {
+    Q_D(KoToolBase);
     // TODO finish current action or should this not possible during actions???
     if (m_pointSelection.size() > 0) {
-        QUndoCommand *cmd = KoPathPointRemoveCommand::createCommand(m_pointSelection.selectedPointsData(), m_canvas->shapeController());
+        QUndoCommand *cmd = KoPathPointRemoveCommand::createCommand(m_pointSelection.selectedPointsData(), d->canvas->shapeController());
         PointHandle *pointHandle = dynamic_cast<PointHandle*>(m_activeHandle);
         if (pointHandle && m_pointSelection.contains(pointHandle->activePoint())) {
             delete m_activeHandle;
             m_activeHandle = 0;
         }
         m_pointSelection.clear();
-        m_canvas->addCommand(cmd);
+        d->canvas->addCommand(cmd);
     }
 }
 
 void KoPathTool::pointToLine()
 {
+    Q_D(KoToolBase);
     if (m_pointSelection.hasSelection()) {
         QList<KoPathPointData> selectedPoints = m_pointSelection.selectedPointsData();
         QList<KoPathPointData> pointToChange;
@@ -243,7 +257,7 @@ void KoPathTool::pointToLine()
         }
 
         if (! pointToChange.isEmpty()) {
-            m_canvas->addCommand(new KoPathPointTypeCommand(pointToChange, KoPathPointTypeCommand::Line));
+            d->canvas->addCommand(new KoPathPointTypeCommand(pointToChange, KoPathPointTypeCommand::Line));
             updateActions();
         }
     }
@@ -251,6 +265,7 @@ void KoPathTool::pointToLine()
 
 void KoPathTool::pointToCurve()
 {
+    Q_D(KoToolBase);
     if (m_pointSelection.hasSelection()) {
         QList<KoPathPointData> selectedPoints = m_pointSelection.selectedPointsData();
         QList<KoPathPointData> pointToChange;
@@ -263,7 +278,7 @@ void KoPathTool::pointToCurve()
         }
 
         if (! pointToChange.isEmpty()) {
-            m_canvas->addCommand(new KoPathPointTypeCommand(pointToChange, KoPathPointTypeCommand::Curve));
+            d->canvas->addCommand(new KoPathPointTypeCommand(pointToChange, KoPathPointTypeCommand::Curve));
             updateActions();
         }
     }
@@ -271,10 +286,11 @@ void KoPathTool::pointToCurve()
 
 void KoPathTool::segmentToLine()
 {
+    Q_D(KoToolBase);
     if (m_pointSelection.size() > 1) {
         QList<KoPathPointData> segments(m_pointSelection.selectedSegmentsData());
         if (segments.size() > 0) {
-            m_canvas->addCommand(new KoPathSegmentTypeCommand(segments, KoPathSegmentTypeCommand::Line));
+            d->canvas->addCommand(new KoPathSegmentTypeCommand(segments, KoPathSegmentTypeCommand::Line));
             updateActions();
         }
     }
@@ -282,10 +298,11 @@ void KoPathTool::segmentToLine()
 
 void KoPathTool::segmentToCurve()
 {
+    Q_D(KoToolBase);
     if (m_pointSelection.size() > 1) {
         QList<KoPathPointData> segments(m_pointSelection.selectedSegmentsData());
         if (segments.size() > 0) {
-            m_canvas->addCommand(new KoPathSegmentTypeCommand(segments, KoPathSegmentTypeCommand::Curve));
+            d->canvas->addCommand(new KoPathSegmentTypeCommand(segments, KoPathSegmentTypeCommand::Curve));
             updateActions();
         }
     }
@@ -293,6 +310,7 @@ void KoPathTool::segmentToCurve()
 
 void KoPathTool::convertToPath()
 {
+    Q_D(KoToolBase);
     QList<KoParameterShape*> shapesToConvert;
     foreach(KoShape *shape, m_pointSelection.selectedShapes()) {
         KoParameterShape * parameterShape = dynamic_cast<KoParameterShape*>(shape);
@@ -300,12 +318,13 @@ void KoPathTool::convertToPath()
             shapesToConvert.append(parameterShape);
     }
     if (shapesToConvert.count())
-        m_canvas->addCommand(new KoParameterToPathCommand(shapesToConvert));
+        d->canvas->addCommand(new KoParameterToPathCommand(shapesToConvert));
     updateOptionsWidget();
 }
 
 void KoPathTool::joinPoints()
 {
+    Q_D(KoToolBase);
     if (m_pointSelection.objectCount() == 1 && m_pointSelection.size() == 2) {
         QList<KoPathPointData> pd(m_pointSelection.selectedPointsData());
         const KoPathPointData & pd1 = pd.at(0);
@@ -313,32 +332,64 @@ void KoPathTool::joinPoints()
         KoPathShape * pathShape = pd1.pathShape;
         if (!pathShape->isClosedSubpath(pd1.pointIndex.first) &&
                 (pd1.pointIndex.second == 0 ||
-                 pd1.pointIndex.second == pathShape->pointCountSubpath(pd1.pointIndex.first) - 1) &&
+                 pd1.pointIndex.second == pathShape->subpathPointCount(pd1.pointIndex.first) - 1) &&
                 !pathShape->isClosedSubpath(pd2.pointIndex.first) &&
                 (pd2.pointIndex.second == 0 ||
-                 pd2.pointIndex.second == pathShape->pointCountSubpath(pd2.pointIndex.first) - 1)) {
+                 pd2.pointIndex.second == pathShape->subpathPointCount(pd2.pointIndex.first) - 1)) {
             KoSubpathJoinCommand *cmd = new KoSubpathJoinCommand(pd1, pd2);
-            m_canvas->addCommand(cmd);
+            d->canvas->addCommand(cmd);
         }
         updateActions();
     }
 }
 
+void KoPathTool::mergePoints()
+{
+    Q_D(KoToolBase);
+    if (m_pointSelection.objectCount() != 1 || m_pointSelection.size() != 2)
+        return;
+
+    QList<KoPathPointData> pointData = m_pointSelection.selectedPointsData();
+    const KoPathPointData & pd1 = pointData.at(0);
+    const KoPathPointData & pd2 = pointData.at(1);
+    const KoPathPointIndex & index1 = pd1.pointIndex;
+    const KoPathPointIndex & index2 = pd2.pointIndex;
+
+    KoPathShape * path = pd1.pathShape;
+
+    // check if subpaths are already closed
+    if (path->isClosedSubpath(index1.first) || path->isClosedSubpath(index2.first))
+        return;
+    // check if first point is an endpoint
+    if (index1.second != 0 && index1.second != path->subpathPointCount(index1.first)-1)
+        return;
+    // check if second point is an endpoint
+    if (index2.second != 0 && index2.second != path->subpathPointCount(index2.first)-1)
+        return;
+
+    // now we can start merging the endpoints
+    KoPathPointMergeCommand *cmd = new KoPathPointMergeCommand(pd1, pd2);
+    d->canvas->addCommand(cmd);
+    updateActions();
+}
+
 void KoPathTool::breakAtPoint()
 {
+    Q_D(KoToolBase);
     if (m_pointSelection.hasSelection()) {
-        m_canvas->addCommand(new KoPathBreakAtPointCommand(m_pointSelection.selectedPointsData()));
+        d->canvas->addCommand(new KoPathBreakAtPointCommand(m_pointSelection.selectedPointsData()));
         updateActions();
     }
 }
 
 void KoPathTool::breakAtSegment()
 {
+    Q_D(KoToolBase);
     // only try to break a segment when 2 points of the same object are selected
     if (m_pointSelection.objectCount() == 1 && m_pointSelection.size() == 2) {
         QList<KoPathPointData> segments(m_pointSelection.selectedSegmentsData());
         if (segments.size() == 1) {
-            m_canvas->addCommand(new KoPathSegmentBreakCommand(segments.at(0)));
+            d->canvas->addCommand(new KoPathSegmentBreakCommand(segments.at(0)));
             updateActions();
         }
     }
@@ -346,6 +397,7 @@ void KoPathTool::breakAtSegment()
 
 void KoPathTool::paint(QPainter &painter, const KoViewConverter &converter)
 {
+    Q_D(KoToolBase);
     painter.setRenderHint(QPainter::Antialiasing, true);
     // use different colors so that it is also visible on a background of the same color
     painter.setBrush(Qt::white);   //TODO make configurable
@@ -391,7 +443,7 @@ void KoPathTool::paint(QPainter &painter, const KoViewConverter &converter)
     if (m_currentStrategy) {
         painter.save();
         KoShape::applyConversion(painter, converter);
-        m_canvas->snapGuide()->paint(painter, converter);
+        d->canvas->snapGuide()->paint(painter, converter);
         painter.restore();
     }
 }
@@ -415,12 +467,24 @@ void KoPathTool::mousePressEvent(KoPointerEvent *event)
         event->accept();
     } else {
         if (event->button() & Qt::LeftButton) {
-            if ((event->modifiers() & Qt::ControlModifier) == 0) {
-                m_pointSelection.clear();
+
+            // check if we hit a path segment
+            KoPathShape * clickedShape = 0;
+            KoPathPoint * clickedPoint = 0;
+            qreal clickedPointParam = 0.0;
+            if (segmentAtPoint(event->point, clickedShape, clickedPoint, clickedPointParam)) {
+                KoPathPointIndex index = clickedShape->pathPointIndex(clickedPoint);
+                KoPathPointData data(clickedShape, index);
+                m_currentStrategy = new KoPathSegmentChangeStrategy(this, event->point, data, clickedPointParam);
+                event->accept();
+            } else {
+                if ((event->modifiers() & Qt::ControlModifier) == 0) {
+                    m_pointSelection.clear();
+                }
+                // start rubberband selection
+                Q_ASSERT(m_currentStrategy == 0);
+                m_currentStrategy = new KoPathPointRubberSelectStrategy(this, event->point);
             }
-            // start rubberband selection
-            Q_ASSERT(m_currentStrategy == 0);
-            m_currentStrategy = new KoPathPointRubberSelectStrategy(this, m_canvas, event->point);
         }
     }
 }
@@ -442,7 +506,7 @@ void KoPathTool::mouseMoveEvent(KoPointerEvent *event)
     }
 
     foreach(KoPathShape *shape, m_pointSelection.selectedShapes()) {
-        QRectF roi = handleRect(shape->documentToShape(event->point));
+        QRectF roi = handleGrabRect(shape->documentToShape(event->point));
         KoParameterShape * parameterShape = dynamic_cast<KoParameterShape*>(shape);
         if (parameterShape && parameterShape->isParametricShape()) {
             int handleId = parameterShape->handleIdAt(roi);
@@ -469,34 +533,61 @@ void KoPathTool::mouseMoveEvent(KoPointerEvent *event)
         } else {
             QList<KoPathPoint*> points = shape->pointsAt(roi);
             if (! points.empty()) {
-                KoPathPoint *p = points.first();
-                KoPathPoint::KoPointType type = KoPathPoint::Node;
+                // find the nearest control point from all points within the roi
+                KoPathPoint * bestPoint = 0;
+                KoPathPoint::PointType bestPointType = KoPathPoint::Node;
+                qreal minDistance = HUGE_VAL;
+                foreach(KoPathPoint *p, points) {
+                    // the node point must be hit if the point is not selected yet
+                    if (! m_pointSelection.contains(p) && ! roi.contains(p->point()))
+                        continue;
 
-                // the node point must be hit if the point is not selected yet
-                if (! m_pointSelection.contains(p) && ! roi.contains(p->point()))
+                    // check for the control points first as otherwise it is no longer
+                    // possible to change the control points when they are the same as the point
+                    if (p->activeControlPoint1() && roi.contains(p->controlPoint1())) {
+                        qreal dist = squaredDistance(roi.center(), p->controlPoint1());
+                        if (dist < minDistance) {
+                            bestPoint = p;
+                            bestPointType = KoPathPoint::ControlPoint1;
+                            minDistance = dist;
+                        }
+                    }
+
+                    if (p->activeControlPoint2() && roi.contains(p->controlPoint2())) {
+                        qreal dist = squaredDistance(roi.center(), p->controlPoint2());
+                        if (dist < minDistance) {
+                            bestPoint = p;
+                            bestPointType = KoPathPoint::ControlPoint2;
+                            minDistance = dist;
+                        }
+                    }
+
+                    // check the node point at last
+                    qreal dist = squaredDistance(roi.center(), p->point());
+                    if (dist < minDistance) {
+                        bestPoint = p;
+                        bestPointType = KoPathPoint::Node;
+                        minDistance = dist;
+                    }
+                }
+
+                if (! bestPoint)
                     return;
 
-                // check for the control points as otherwise it is no longer
-                // possible to change the control points when they are the same as the point
-                if (p->activeControlPoint1() && roi.contains(p->controlPoint1()))
-                    type = KoPathPoint::ControlPoint1;
-                else if (p->activeControlPoint2() && roi.contains(p->controlPoint2()))
-                    type = KoPathPoint::ControlPoint2;
-
                 useCursor(m_moveCursor);
-                if (type == KoPathPoint::Node)
+                if (bestPointType == KoPathPoint::Node)
                     emit statusTextChanged(i18n("Drag to move point. Shift click to change point type."));
                 else
                     emit statusTextChanged(i18n("Drag to move control point."));
 
                 PointHandle *prev = dynamic_cast<PointHandle*>(m_activeHandle);
-                if (prev && prev->activePoint() == p && prev->activePointType() == type)
+                if (prev && prev->activePoint() == bestPoint && prev->activePointType() == bestPointType)
                     return; // no change;
 
                 if (m_activeHandle)
                     m_activeHandle->repaint();
                 delete m_activeHandle;
-                m_activeHandle = new PointHandle(this, p, type);
+                m_activeHandle = new PointHandle(this, bestPoint, bestPointType);
                 m_activeHandle->repaint();
                 return;
             }
@@ -519,12 +610,13 @@ void KoPathTool::mouseMoveEvent(KoPointerEvent *event)
 
 void KoPathTool::mouseReleaseEvent(KoPointerEvent *event)
 {
+    Q_D(KoToolBase);
     if (m_currentStrategy) {
         const bool hadNoSelection = !m_pointSelection.hasSelection();
         m_currentStrategy->finishInteraction(event->modifiers());
         QUndoCommand *command = m_currentStrategy->createCommand();
         if (command)
-            m_canvas->addCommand(command);
+            d->canvas->addCommand(command);
         if (hadNoSelection && dynamic_cast<KoPathPointRubberSelectStrategy*>(m_currentStrategy)
                 && !m_pointSelection.hasSelection()) {
             // the click didn't do anything at all. Allow it to be used by others.
@@ -542,13 +634,16 @@ void KoPathTool::mouseReleaseEvent(KoPointerEvent *event)
 
 void KoPathTool::keyPressEvent(QKeyEvent *event)
 {
+    Q_D(KoToolBase);
     if (m_currentStrategy) {
         switch (event->key()) {
         case Qt::Key_Control:
         case Qt::Key_Alt:
         case Qt::Key_Shift:
         case Qt::Key_Meta:
-            m_currentStrategy->handleMouseMove(m_lastPoint, event->modifiers());
+            if (! event->isAutoRepeat()) {
+                m_currentStrategy->handleMouseMove(m_lastPoint, event->modifiers());
+            }
             break;
         case Qt::Key_Escape:
             m_currentStrategy->cancelInteraction();
@@ -563,19 +658,20 @@ void KoPathTool::keyPressEvent(QKeyEvent *event)
         switch (event->key()) {
 // TODO move these to the actions in the constructor.
         case Qt::Key_I: {
-            int handleRadius = m_canvas->resourceProvider()->handleRadius();
+            int handleRadius = d->canvas->resourceManager()->handleRadius();
             if (event->modifiers() & Qt::ControlModifier)
                 handleRadius--;
             else
                 handleRadius++;
-            m_canvas->resourceProvider()->setHandleRadius(handleRadius);
+            d->canvas->resourceManager()->setHandleRadius(handleRadius);
             break;
         }
 #ifndef NDEBUG
         case Qt::Key_D:
             if (m_pointSelection.objectCount() == 1) {
                 QList<KoPathPointData> selectedPoints = m_pointSelection.selectedPointsData();
-                selectedPoints[0].pathShape->debugPath();
+                KoPathShapePrivate *p = static_cast<KoPathShapePrivate*>(selectedPoints[0].pathShape->priv());
+                p->debugPath();
             }
             break;
 #endif
@@ -601,7 +697,9 @@ void KoPathTool::keyReleaseEvent(QKeyEvent *event)
         case Qt::Key_Alt:
         case Qt::Key_Shift:
         case Qt::Key_Meta:
-            m_currentStrategy->handleMouseMove(m_lastPoint, Qt::NoModifier);
+            if (! event->isAutoRepeat()) {
+                m_currentStrategy->handleMouseMove(m_lastPoint, Qt::NoModifier);
+            }
             break;
         default:
             break;
@@ -610,19 +708,144 @@ void KoPathTool::keyReleaseEvent(QKeyEvent *event)
     event->accept();
 }
 
-void KoPathTool::activate(bool temporary)
+void KoPathTool::mouseDoubleClickEvent(KoPointerEvent *event)
 {
-    Q_UNUSED(temporary);
+    Q_D(KoToolBase);
+    event->ignore();
+
+    // check if we are doing something else at the moment
+    if (m_currentStrategy)
+        return;
+
+    /*
+    // TODO: use global click proximity once added to the canvas resource provider
+    const int clickProximity = 5;
+
+    // convert click proximity to point using the current zoom level
+    QPointF clickOffset = d->canvas->viewConverter()->viewToDocument( QPointF(clickProximity, clickProximity) );
+    // the max allowed distance from a segment
+    const qreal maxSquaredDistance = clickOffset.x()*clickOffset.x();
+
+    KoPathShape * clickedShape = 0;
+    KoPathPoint * clickedSegmentStart = 0;
+    qreal clickedPointParam = 0.0;
+
+    foreach(KoPathShape *shape, m_pointSelection.selectedShapes()) {
+        if (dynamic_cast<KoParameterShape*>( shape ))
+            continue;
+
+        // convert document point to shape coordinates
+        QPointF point = shape->documentToShape( event->point );
+        // our region of interest, i.e. a region around our mouse position
+        QRectF roi( point - clickOffset, point + clickOffset );
+
+        qreal minSqaredDistance = HUGE_VAL;
+        // check all segments of this shape which intersect the region of interest
+        QList<KoPathSegment> segments = shape->segmentsAt( roi );
+        foreach( const KoPathSegment &s, segments ) {
+            qreal nearestPointParam = s.nearestPoint( point );
+            QPointF nearestPoint = s.pointAt( nearestPointParam );
+            QPointF diff = point - nearestPoint;
+            qreal squaredDistance = diff.x()*diff.x() + diff.y()*diff.y();
+            // are we within the allowed distance ?
+            if (squaredDistance > maxSquaredDistance)
+                continue;
+            // are we closer to the last closest point ?
+            if (squaredDistance < minSqaredDistance) {
+                clickedShape = shape;
+                clickedSegmentStart = s.first();
+                clickedPointParam = nearestPointParam;
+            }
+        }
+    }
+    */
+
+    KoPathShape * clickedShape = 0;
+    KoPathPoint * clickedSegmentStart = 0;
+    qreal clickedPointParam = 0.0;
+    if (! segmentAtPoint(event->point, clickedShape, clickedSegmentStart, clickedPointParam))
+        return;
+
+    if (clickedShape && clickedSegmentStart) {
+        QList<KoPathPointData> segments;
+        segments.append(KoPathPointData(clickedShape, clickedShape->pathPointIndex(clickedSegmentStart)));
+        KoPathPointInsertCommand *cmd = new KoPathPointInsertCommand(segments, clickedPointParam);
+        d->canvas->addCommand(cmd);
+
+        foreach( KoPathPoint * p, cmd->insertedPoints() ) {
+            m_pointSelection.add( p, false );
+        }
+        updateActions();
+        event->accept();
+    }
+}
+
+bool KoPathTool::segmentAtPoint( const QPointF &point, KoPathShape* &shape, KoPathPoint* &segmentStart, qreal &pointParam )
+{
+    Q_D(KoToolBase);
+    // TODO: use global click proximity once added to the canvas resource provider
+    const int clickProximity = 5;
+
+    // convert click proximity to point using the current zoom level
+    QPointF clickOffset = d->canvas->viewConverter()->viewToDocument( QPointF(clickProximity, clickProximity) );
+    // the max allowed distance from a segment
+    const qreal maxSquaredDistance = clickOffset.x()*clickOffset.x();
+
+    KoPathShape * clickedShape = 0;
+    KoPathPoint * clickedSegmentStart = 0;
+    qreal clickedPointParam = 0.0;
+
+    foreach(KoPathShape *shape, m_pointSelection.selectedShapes()) {
+        KoParameterShape * parameterShape = dynamic_cast<KoParameterShape*>( shape );
+        if (parameterShape && parameterShape->isParametricShape())
+            continue;
+
+        // convert document point to shape coordinates
+        QPointF p = shape->documentToShape( point );
+        // our region of interest, i.e. a region around our mouse position
+        QRectF roi( p - clickOffset, p + clickOffset );
+
+        qreal minSqaredDistance = HUGE_VAL;
+        // check all segments of this shape which intersect the region of interest
+        QList<KoPathSegment> segments = shape->segmentsAt( roi );
+        foreach( const KoPathSegment &s, segments ) {
+            qreal nearestPointParam = s.nearestPoint( p );
+            QPointF nearestPoint = s.pointAt( nearestPointParam );
+            QPointF diff = p - nearestPoint;
+            qreal squaredDistance = diff.x()*diff.x() + diff.y()*diff.y();
+            // are we within the allowed distance ?
+            if (squaredDistance > maxSquaredDistance)
+                continue;
+            // are we closer to the last closest point ?
+            if (squaredDistance < minSqaredDistance) {
+                clickedShape = shape;
+                clickedSegmentStart = s.first();
+                clickedPointParam = nearestPointParam;
+            }
+        }
+    }
+
+    shape = clickedShape;
+    segmentStart = clickedSegmentStart;
+    pointParam = clickedPointParam;
+
+    return (shape && segmentStart);
+}
+
+void KoPathTool::activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes)
+{
+    Q_D(KoToolBase);
+    Q_UNUSED(toolActivation);
     // retrieve the actual global handle radius
-    m_handleRadius = m_canvas->resourceProvider()->handleRadius();
-    m_canvas->snapGuide()->reset();
+    m_handleRadius = d->canvas->resourceManager()->handleRadius();
+    d->canvas->snapGuide()->reset();
 
     repaintDecorations();
     QList<KoPathShape*> selectedShapes;
-    foreach(KoShape *shape, m_canvas->shapeManager()->selection()->selectedShapes()) {
+    foreach(KoShape *shape, shapes) {
         KoPathShape *pathShape = dynamic_cast<KoPathShape*>(shape);
 
-        if (!shape->isLocked() && pathShape) {
+        if (shape->isEditable() && pathShape) {
             // as the tool is just in activation repaintDecorations does not yet get called
             // so we need to use repaint of the tool and it is only needed to repaint the
             // current canvas
@@ -635,12 +858,27 @@ void KoPathTool::activate(bool temporary)
         return;
     }
     m_pointSelection.setSelectedShapes(selectedShapes);
-    useCursor(m_selectCursor, true);
-    connect(m_canvas->shapeManager()->selection(), SIGNAL(selectionChanged()), this, SLOT(activate()));
+    useCursor(m_selectCursor);
+    connect(d->canvas->shapeManager()->selection(), SIGNAL(selectionChanged()), this, SLOT(activate()));
     updateOptionsWidget();
     updateActions();
 }
 
+void KoPathTool::activate()
+{
+    Q_D(KoToolBase);
+    QSet<KoShape*> shapes;
+    foreach(KoShape *shape, d->canvas->shapeManager()->selection()->selectedShapes()) {
+        QSet<KoShape*> delegates = shape->toolDelegates();
+        if (delegates.isEmpty()) {
+            shapes << shape;
+        } else {
+            shapes += delegates;
+        }
+    }
+    activate(DefaultActivation, shapes);
+}
+
 void KoPathTool::updateOptionsWidget()
 {
     PathToolOptionWidget::Types type;
@@ -674,59 +912,58 @@ void KoPathTool::updateActions()
     m_actionAddPoint->setEnabled(hasSegmentsSelected);
     m_actionLineSegment->setEnabled(hasSegmentsSelected);
     m_actionCurveSegment->setEnabled(hasSegmentsSelected);
-    m_actionBreakSegment->setEnabled(hasPointsSelected && m_pointSelection.objectCount() == 1 && m_pointSelection.size() == 2);
-    m_actionJoinSegment->setEnabled(hasPointsSelected && m_pointSelection.objectCount() == 1 && m_pointSelection.size() == 2);
-    m_actionMergePoints->setEnabled(false);
+
+    const uint objectCount = m_pointSelection.objectCount();
+    const uint pointCount = m_pointSelection.size();
+    m_actionBreakSegment->setEnabled(objectCount == 1 && pointCount == 2);
+    m_actionJoinSegment->setEnabled(objectCount == 1 && pointCount == 2);
+    m_actionMergePoints->setEnabled(objectCount == 1 && pointCount == 2);
 }
 
 void KoPathTool::deactivate()
 {
-    disconnect(m_canvas->shapeManager()->selection(), SIGNAL(selectionChanged()), this, SLOT(activate()));
+    Q_D(KoToolBase);
+    disconnect(d->canvas->shapeManager()->selection(), SIGNAL(selectionChanged()), this, SLOT(activate()));
     m_pointSelection.clear();
     m_pointSelection.setSelectedShapes(QList<KoPathShape*>());
     delete m_activeHandle;
     m_activeHandle = 0;
     delete m_currentStrategy;
     m_currentStrategy = 0;
-    m_canvas->snapGuide()->reset();
+    d->canvas->snapGuide()->reset();
 }
 
 void KoPathTool::resourceChanged(int key, const QVariant & res)
 {
-    if (key != KoCanvasResource::HandleRadius)
-        return;
+    if (key == KoCanvasResource::HandleRadius) {
+        int oldHandleRadius = m_handleRadius;
 
-    int oldHandleRadius = m_handleRadius;
+        m_handleRadius = res.toUInt();
 
-    m_handleRadius = res.toUInt();
-
-    // repaint with the bigger of old and new handle radius
-    int maxRadius = qMax(m_handleRadius, oldHandleRadius);
-    foreach(KoPathShape *shape, m_pointSelection.selectedShapes()) {
-        QRectF controlPointRect = shape->absoluteTransformation(0).map(shape->outline()).controlPointRect();
-        repaint(controlPointRect.adjusted(-maxRadius, -maxRadius, maxRadius, maxRadius));
+        // repaint with the bigger of old and new handle radius
+        int maxRadius = qMax(m_handleRadius, oldHandleRadius);
+        foreach(KoPathShape *shape, m_pointSelection.selectedShapes()) {
+            QRectF controlPointRect = shape->absoluteTransformation(0).map(shape->outline()).controlPointRect();
+            repaint(controlPointRect.adjusted(-maxRadius, -maxRadius, maxRadius, maxRadius));
+        }
     }
 }
 
 void KoPathTool::pointSelectionChanged()
 {
+    Q_D(KoToolBase);
     updateActions();
-    m_canvas->snapGuide()->setIgnoredPathPoints(m_pointSelection.selectedPoints().toList());
+    d->canvas->snapGuide()->setIgnoredPathPoints(m_pointSelection.selectedPoints().toList());
     emit selectionChanged(m_pointSelection.hasSelection());
 }
 
 void KoPathTool::repaint(const QRectF &repaintRect)
 {
+    Q_D(KoToolBase);
     //kDebug(30006) <<"KoPathTool::repaint(" << repaintRect <<")" << m_handleRadius;
     // widen border to take antialiasing into account
     qreal radius = m_handleRadius + 1;
-    m_canvas->updateCanvas(repaintRect.adjusted(-radius, -radius, radius, radius));
-}
-
-QRectF KoPathTool::handleRect(const QPointF &p)
-{
-    QSizeF hsize = m_canvas->viewConverter()->viewToDocument(QSizeF(m_handleRadius, m_handleRadius));
-    return QRectF(p.x() - hsize.width(), p.y() - hsize.height(), 2*hsize.width(), 2*hsize.height());
+    d->canvas->updateCanvas(repaintRect.adjusted(-radius, -radius, radius, radius));
 }
 
 void KoPathTool::deleteSelection()
@@ -739,4 +976,4 @@ KoToolSelection * KoPathTool::selection()
     return &m_pointSelection;
 }
 
-#include "KoPathTool.moc"
+#include <KoPathTool.moc>
