@@ -21,12 +21,15 @@
 #include "KoPADocument.h"
 
 #include <KoStore.h>
+#include <KoResourceManager.h>
 #include <KoXmlWriter.h>
 #include <KoXmlReader.h>
 #include <KoOdfStylesReader.h>
 #include <KoOdfReadStore.h>
 #include <KoOdfWriteStore.h>
 #include <KoOdfLoadingContext.h>
+#include <KoOasisSettings.h>
+#include <KoStoreDevice.h>
 #include <KoShapeManager.h>
 #include <KoShapeLayer.h>
 #include <KoShapeRegistry.h>
@@ -38,7 +41,6 @@
 #include <KoPathShape.h>
 #include <KoLineBorder.h>
 #include <KoXmlNS.h>
-#include <KoDataCenter.h>
 
 #include "KoPACanvas.h"
 #include "KoPAView.h"
@@ -47,6 +49,7 @@
 #include "KoPASavingContext.h"
 #include "KoPALoadingContext.h"
 #include "KoPAViewMode.h"
+#include "KoPAPageProvider.h"
 #include "commands/KoPAPageDeleteCommand.h"
 
 #include <kdebug.h>
@@ -62,7 +65,7 @@ public:
     QList<KoPAPageBase*> masterPages;
     KoInlineTextObjectManager *inlineTextObjectManager;
     bool rulersVisible;
-    QMap<QString, KoDataCenter *>  dataCenterMap;
+    KoPAPageProvider *pageProvider;
 };
 
 KoPADocument::KoPADocument( QWidget* parentWidget, QObject* parent, bool singleViewMode )
@@ -70,14 +73,14 @@ KoPADocument::KoPADocument( QWidget* parentWidget, QObject* parent, bool singleV
     d(new Private())
 {
     d->inlineTextObjectManager = new KoInlineTextObjectManager(this);
+    d->rulersVisible = false;
 
-    // Ask every shapefactory to populate the dataCenterMap
-    foreach(const QString & id, KoShapeRegistry::instance()->keys())
-    {
-        KoShapeFactory *shapeFactory = KoShapeRegistry::instance()->value(id);
-        shapeFactory->populateDataCenterMap(d->dataCenterMap);
-    }
-
+    resourceManager()->setUndoStack(undoStack());
+    resourceManager()->setOdfDocument(this);
+    QVariant variant;
+    d->pageProvider = new KoPAPageProvider();
+    variant.setValue<void*>(d->pageProvider);
+    resourceManager()->setResource(KoText::PageProvider, variant);
     loadConfig();
 }
 
@@ -86,7 +89,7 @@ KoPADocument::~KoPADocument()
     saveConfig();
     qDeleteAll( d->pages );
     qDeleteAll( d->masterPages );
-    qDeleteAll( d->dataCenterMap );
+    delete d->pageProvider;
     delete d;
 }
 
@@ -94,7 +97,7 @@ void KoPADocument::paintContent( QPainter &painter, const QRect &rect)
 {
     KoPAPageBase * page = pageByIndex( 0, false );
     Q_ASSERT( page );
-    QPixmap thumbnail = page->thumbnail( rect.size() );
+    QPixmap thumbnail( pageThumbnail( page, rect.size() ) );
     painter.drawPixmap( rect, thumbnail );
 }
 
@@ -111,32 +114,44 @@ bool KoPADocument::loadOdf( KoOdfReadStore & odfStore )
 {
     emit sigProgress( 0 );
     KoOdfLoadingContext loadingContext( odfStore.styles(), odfStore.store(), componentData() );
-    KoPALoadingContext paContext( loadingContext, dataCenterMap() );
+    KoPALoadingContext paContext(loadingContext, resourceManager());
 
     KoXmlElement content = odfStore.contentDoc().documentElement();
     KoXmlElement realBody ( KoXml::namedItemNS( content, KoXmlNS::office, "body" ) );
 
     if ( realBody.isNull() ) {
-        kError() << "No body tag found!" << endl;
+        kError(30010) << "No body tag found!" << endl;
         return false;
     }
 
     KoXmlElement body = KoXml::namedItemNS(realBody, KoXmlNS::office, odfTagName( false ));
 
     if ( body.isNull() ) {
-        kError() << "No office:" << odfTagName( false ) << " tag found!" << endl;
+        kError(30010) << "No office:" << odfTagName( false ) << " tag found!" << endl;
         return false;
     }
 
     // Load text styles before the corresponding text shapes try to use them!
     KoTextSharedLoadingData * sharedData = new KoTextSharedLoadingData();
-    KoStyleManager * styleManager = dynamic_cast<KoStyleManager *>( dataCenterMap()["StyleManager"] );
-    sharedData->loadOdfStyles(loadingContext, styleManager);
+    KoStyleManager *styleManager = resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
+
+    sharedData->loadOdfStyles(paContext, styleManager);
     paContext.addSharedData( KOTEXT_SHARED_LOADING_ID, sharedData );
 
     d->masterPages = loadOdfMasterPages( odfStore.styles().masterPages(), paContext );
+    if ( !loadOdfProlog( body, paContext ) ) {
+        return false;
+    }
     d->pages = loadOdfPages( body, paContext );
 
+    // create pages if there are none
+    if (d->masterPages.empty()) {
+        d->masterPages.append(newMasterPage());
+    }
+    if (d->pages.empty()) {
+        d->pages.append(newPage(static_cast<KoPAMasterPage*>(d->masterPages.first())));
+    }
+
     if ( !loadOdfEpilogue( body, paContext ) ) {
         return false;
     }
@@ -147,20 +162,12 @@ bool KoPADocument::loadOdf( KoOdfReadStore & odfStore )
         setActionEnabled( KoPAView::ActionDeletePage, false );
     }
 
+    updatePageCount();
+
     emit sigProgress( -1 );
     return true;
 }
 
-bool KoPADocument::completeLoading( KoStore* store )
-{
-    bool ok=true;
-    foreach(KoDataCenter *dataCenter, d->dataCenterMap)
-    {
-        ok = ok && dataCenter->completeLoading(store);
-    }
-    return ok;
-}
-
 bool KoPADocument::saveOdf( SavingContext & documentContext )
 {
     KoXmlWriter* contentWriter = documentContext.odfStore.contentWriter();
@@ -170,13 +177,17 @@ bool KoPADocument::saveOdf( SavingContext & documentContext )
     KoGenStyles mainStyles;
     KoXmlWriter * bodyWriter = documentContext.odfStore.bodyWriter();
 
-    KoPASavingContext paContext( *bodyWriter, mainStyles, documentContext.embeddedSaver, 1, KoShapeSavingContext::Store );
+    KoPASavingContext paContext(*bodyWriter, mainStyles, documentContext.embeddedSaver, 1);
 
     saveOdfDocumentStyles( paContext );
 
     bodyWriter->startElement( "office:body" );
     bodyWriter->startElement( odfTagName( true ) );
 
+    if ( !saveOdfProlog( paContext ) ) {
+        return false;
+    }
+
     if ( !saveOdfPages( paContext, d->pages, d->masterPages ) ) {
         return false;
     }
@@ -188,7 +199,7 @@ bool KoPADocument::saveOdf( SavingContext & documentContext )
     bodyWriter->endElement(); // office:odfTagName()
     bodyWriter->endElement(); // office:body
 
-    mainStyles.saveOdfAutomaticStyles( contentWriter, false );
+    mainStyles.saveOdfStyles( KoGenStyles::DocumentAutomaticStyles, contentWriter );
 
     documentContext.odfStore.closeContentWriter();
 
@@ -199,13 +210,22 @@ bool KoPADocument::saveOdf( SavingContext & documentContext )
         return false;
     }
 
-    return paContext.saveDataCenter( documentContext.odfStore.store(), documentContext.odfStore.manifestWriter() );
-}
+    KoStore * store = documentContext.odfStore.store();
+    if ( ! store->open( "settings.xml" ) ) {
+        return false;
+    }
 
-bool KoPADocument::completeSaving( KoStore* store)
-{
-    Q_UNUSED(store)
-    return true;
+    saveOdfSettings( store );
+
+    if ( ! store->close() ) {
+        return false;
+    }
+
+    documentContext.odfStore.manifestWriter()->addManifestEntry( "settings.xml", "text/xml" );
+
+    //setModified( false );
+
+    return paContext.saveDataCenter( documentContext.odfStore.store(), documentContext.odfStore.manifestWriter() );
 }
 
 QList<KoPAPageBase *> KoPADocument::loadOdfMasterPages( const QHash<QString, KoXmlElement*> masterStyles, KoPALoadingContext & context )
@@ -228,12 +248,16 @@ QList<KoPAPageBase *> KoPADocument::loadOdfMasterPages( const QHash<QString, KoX
 
 QList<KoPAPageBase *> KoPADocument::loadOdfPages( const KoXmlElement & body, KoPALoadingContext & context )
 {
+    if (d->masterPages.isEmpty()) { // we require at least one master page. Auto create one if the doc was faulty.
+        d->masterPages << newMasterPage();
+    }
+
     QList<KoPAPageBase *> pages;
     KoXmlElement element;
     forEachElement( element, body )
     {
         if ( element.tagName() == "page" && element.namespaceURI() == KoXmlNS::draw ) {
-            KoPAPage* page = newPage();
+            KoPAPage *page = newPage(static_cast<KoPAMasterPage*>(d->masterPages.first()));
             page->loadOdf( element, context );
             pages.append( page );
         }
@@ -248,6 +272,13 @@ bool KoPADocument::loadOdfEpilogue( const KoXmlElement & body, KoPALoadingContex
     return true;
 }
 
+bool KoPADocument::loadOdfProlog( const KoXmlElement & body, KoPALoadingContext & context )
+{
+    Q_UNUSED( body );
+    Q_UNUSED( context );
+    return true;
+}
+
 bool KoPADocument::saveOdfPages( KoPASavingContext &paContext, QList<KoPAPageBase *> &pages, QList<KoPAPageBase *> &masterPages )
 {
     paContext.addOption( KoPASavingContext::DrawId );
@@ -272,15 +303,77 @@ bool KoPADocument::saveOdfPages( KoPASavingContext &paContext, QList<KoPAPageBas
     return true;
 }
 
+bool KoPADocument::saveOdfProlog( KoPASavingContext & paContext )
+{
+    Q_UNUSED( paContext );
+    return true;
+}
+
 bool KoPADocument::saveOdfEpilogue( KoPASavingContext & paContext )
 {
     Q_UNUSED( paContext );
     return true;
 }
 
+bool KoPADocument::saveOdfSettings( KoStore * store )
+{
+    KoStoreDevice settingsDev( store );
+    KoXmlWriter * settingsWriter = KoOdfWriteStore::createOasisXmlWriter( &settingsDev, "office:document-settings" );
+
+    // add this so that OOo reads guides lines and grid data from ooo:view-settings
+    settingsWriter->addAttribute( "xmlns:ooo", "http://openoffice.org/2004/office" );
+
+    settingsWriter->startElement("office:settings");
+    settingsWriter->startElement("config:config-item-set");
+    settingsWriter->addAttribute("config:name", "view-settings");
+
+    saveUnitOdf(settingsWriter);
+
+    settingsWriter->endElement(); // config:config-item-set
+
+    settingsWriter->startElement("config:config-item-set");
+    settingsWriter->addAttribute("config:name", "ooo:view-settings");
+    settingsWriter->startElement("config:config-item-map-indexed" );
+    settingsWriter->addAttribute("config:name", "Views" );
+    settingsWriter->startElement("config:config-item-map-entry" );
+
+    guidesData().saveOdfSettings( *settingsWriter );
+    gridData().saveOdfSettings( *settingsWriter );
+
+    settingsWriter->endElement(); // config:config-item-map-entry
+    settingsWriter->endElement(); // config:config-item-map-indexed
+    settingsWriter->endElement(); // config:config-item-set
+
+    settingsWriter->endElement(); // office:settings
+    settingsWriter->endElement(); // office:document-settings
+
+    settingsWriter->endDocument();
+
+    delete settingsWriter;
+
+    return true;
+}
+
+void KoPADocument::loadOdfSettings(  const KoXmlDocument & settingsDoc )
+{
+    if ( settingsDoc.isNull() ) {
+        return ; // not an error if some file doesn't have settings.xml
+    }
+
+    KoOasisSettings settings( settingsDoc );
+    KoOasisSettings::Items viewSettings = settings.itemSet( "view-settings" );
+    if ( !viewSettings.isNull() ) {
+        setUnit( KoUnit::unit( viewSettings.parseConfigItemString( "unit" ) ) );
+        // FIXME: add other config here.
+    }
+
+    guidesData().loadOdfSettings( settingsDoc );
+    gridData().loadOdfSettings( settingsDoc );
+}
+
 void KoPADocument::saveOdfDocumentStyles( KoPASavingContext & context )
 {
-    KoStyleManager *styleManager = dynamic_cast<KoStyleManager *>( dataCenterMap()["StyleManager"] );
+    KoStyleManager *styleManager = resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
     Q_ASSERT( styleManager );
     styleManager->saveOdf( context.mainStyles() );
 }
@@ -416,11 +509,6 @@ void KoPADocument::pageRemoved( KoPAPageBase * page, QUndoCommand * parent )
     Q_UNUSED( parent );
 }
 
-QMap<QString, KoDataCenter *> KoPADocument::dataCenterMap() const
-{
-    return d->dataCenterMap;
-}
-
 KoPAPageBase * KoPADocument::pageByShape( KoShape * shape ) const
 {
     KoShape * parent = shape;
@@ -438,8 +526,16 @@ void KoPADocument::updateViews(KoPAPageBase *page)
 
     foreach (KoView *view, views()) {
         KoPAView *paView = static_cast<KoPAView *>(view);
-        if (paView->activePage() == page)
-            paView->updateActivePage(page);
+        if ( paView->activePage() == page ) {
+            paView->updateActivePage( page );
+        }
+        else if ( dynamic_cast<KoPAMasterPage *>( page ) ) {
+            // if the page changed is a master page, we need to check whether it is the current page's master page
+            KoPAPage *activePage = dynamic_cast<KoPAPage *>( paView->activePage() );
+            if ( activePage && activePage->masterPage() == page ) {
+                paView->updateActivePage( activePage );
+            }
+        }
     }
 }
 
@@ -448,6 +544,24 @@ KoPageApp::PageType KoPADocument::pageType() const
     return KoPageApp::Page;
 }
 
+QPixmap KoPADocument::pageThumbnail(KoPAPageBase* page, const QSize& size)
+{
+    int pageNumber = pageIndex(page) + 1;
+    d->pageProvider->setPageData(pageNumber, page);
+    return page->thumbnail(size);
+}
+
+void KoPADocument::initEmpty()
+{
+    d->masterPages.clear();
+    d->pages.clear();
+    KoPAMasterPage * masterPage = newMasterPage();
+    d->masterPages.append( masterPage );
+    KoPAPage * page = newPage( masterPage );
+    d->pages.append( page );
+    KoDocument::initEmpty();
+}
+
 void KoPADocument::setActionEnabled( int actions, bool enable )
 {
     foreach( KoView *view, views() )
@@ -470,6 +584,7 @@ void KoPADocument::insertPage( KoPAPageBase* page, int index )
     }
 
     pages.insert( index, page );
+    updatePageCount();
 
     setActionEnabled( KoPAView::ActionDeletePage, pages.size() > 1 );
 
@@ -495,6 +610,7 @@ void KoPADocument::insertPage( KoPAPageBase* page, KoPAPageBase* after )
     }
 
     pages.insert( index, page );
+    updatePageCount();
 
     setActionEnabled( KoPAView::ActionDeletePage, pages.size() > 1 );
 
@@ -525,6 +641,7 @@ int KoPADocument::takePage( KoPAPageBase *page )
                 kopaView->updateActivePage( newActivePage );
             }
         }
+        updatePageCount();
     }
 
     if ( pages.size() == 1 ) {
@@ -638,15 +755,18 @@ bool KoPADocument::rulersVisible() const
     return d->rulersVisible;
 }
 
-
 int KoPADocument::pageCount() const
 {
     return d->pages.count();
 }
 
-void KoPADocument::insertIntoDataCenterMap(QString key, KoDataCenter *dc)
+void KoPADocument::updatePageCount()
 {
-    d->dataCenterMap[key] = dc;
+    if (resourceManager()->hasResource(KoText::InlineTextObjectManager)) {
+        QVariant var = resourceManager()->resource(KoText::InlineTextObjectManager);
+        KoInlineTextObjectManager *om = var.value<KoInlineTextObjectManager*>();
+        om->setProperty( KoInlineObject::PageCount, pageCount() );
+    }
 }
 
-#include "KoPADocument.moc"
+#include <KoPADocument.moc>
