@@ -1,6 +1,7 @@
 /* This file is part of the KDE project
  * Copyright (C) 2006-2007 Jan Hambrecht <jaham@gmx.net>
  * Copyright (C) 2008-2009 Fredy Yanardi <fyanardi@gmail.com>
+ * Copyright (C) 2009 Jean-Nicolas Artaud <jeannicolasartaud@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,6 +28,7 @@
 #include "KoPAMasterPage.h"
 #include "KoPAPage.h"
 #include "commands/KoPAPageInsertCommand.h"
+#include "commands/KoPAPageDeleteCommand.h"
 
 #include <KoShapeManager.h>
 #include <KoShapeBorderModel.h>
@@ -36,10 +38,14 @@
 #include <KoCanvasController.h>
 #include <KoShapeControllerBase.h>
 #include <KoSelection.h>
+#include <KoShapeOdfSaveHelper.h>
+#include <KoPAOdfPageSaveHelper.h>
+#include <KoDrag.h>
 #include <KoShapeCreateCommand.h>
 #include <KoShapeDeleteCommand.h>
 #include <KoShapeReorderCommand.h>
 #include <KoShapeLayer.h>
+#include <KoShapePaste.h>
 
 #include <KMenu>
 #include <klocale.h>
@@ -48,11 +54,14 @@
 #include <kinputdialog.h>
 #include <kmessagebox.h>
 #include <KConfigGroup>
+#include <KDebug>
 
 #include <QtGui/QGridLayout>
 #include <QtGui/QToolButton>
 #include <QtGui/QButtonGroup>
 #include <QItemSelection>
+#include <QApplication>
+#include <QClipboard>
 
 enum ButtonIds
 {
@@ -79,7 +88,7 @@ QDockWidget* KoPADocumentStructureDockerFactory::createDockWidget()
 
 KoPADocumentStructureDocker::KoPADocumentStructureDocker( KoDocumentSectionView::DisplayMode mode, KoPageApp::PageType pageType, QWidget* parent )
 : QDockWidget( parent )
-, KoCanvasObserver()
+, KoCanvasObserverBase()
 , m_doc( 0 )
 , m_model( 0 )
 {
@@ -195,6 +204,13 @@ void KoPADocumentStructureDocker::setPart( KParts::Part * part )
 {
     m_doc = dynamic_cast<KoPADocument *>( part );
     m_model->setDocument( m_doc ); // this either contains the doc or is 0
+
+    m_buttonGroup->button(Button_Delete)->setEnabled(false);
+    if (m_doc) {
+        if (m_sectionView->selectionModel()->selectedIndexes().count() < m_doc->pages().count()) {
+            m_buttonGroup->button(Button_Delete)->setEnabled(true);
+        }
+    }
 }
 
 void KoPADocumentStructureDocker::slotButtonClicked( int buttonId )
@@ -317,6 +333,11 @@ void KoPADocumentStructureDocker::addLayer()
         if ( canvas ) {
             layer->setParent( canvas->koPAView()->activePage() );
             layer->setName( name );
+            QList<KoShape*> layers( canvas->koPAView()->activePage()->childShapes() );
+            if ( !layers.isEmpty() ) {
+                qSort( layers.begin(), layers.end(), KoShape::compareShapeZIndex );
+                layer->setZIndex( layers.last()->zIndex() + 1 );
+            }
             QUndoCommand *cmd = new KoShapeCreateCommand( m_doc, layer, 0 );
             cmd->setText( i18n( "Create Layer") );
             m_doc->addCommand( cmd );
@@ -327,7 +348,7 @@ void KoPADocumentStructureDocker::addLayer()
 
 void KoPADocumentStructureDocker::deleteItem()
 {
-    QList<KoPAPageBase*> selectedPages; // XXX: Fix this!
+    QList<KoPAPageBase*> selectedPages;
     QList<KoShapeLayer*> selectedLayers;
     QList<KoShape*> selectedShapes;
 
@@ -343,7 +364,7 @@ void KoPADocumentStructureDocker::deleteItem()
             QList<KoShape*> deleteShapes;
             foreach( KoPAPageBase* page, selectedPages )
             {
-                deleteShapes += page->iterator();
+                deleteShapes += page->childShapes();
                 deleteShapes.append( page );
             }
             cmd = new KoShapeDeleteCommand( m_doc, deleteShapes );
@@ -351,13 +372,16 @@ void KoPADocumentStructureDocker::deleteItem()
         }
         else
         {
-            KMessageBox::error( 0L, i18n( "Could not delete all layers. At least one layer is required."), i18n( "Error deleting layers") );
+            KMessageBox::error(0, i18n("Could not delete all layers. At least one layer is required."), i18n("Error deleting layers"));
         }
     }
     else if( selectedShapes.count() )
     {
         cmd = new KoShapeDeleteCommand( m_doc, selectedShapes );
     }
+    else if (!selectedPages.isEmpty() && selectedPages.count() < m_doc->pages().count()) {
+        cmd = new KoPAPageDeleteCommand(m_doc, selectedPages);
+    }
 
     if( cmd )
     {
@@ -582,14 +606,19 @@ void KoPADocumentStructureDocker::itemSelected( const QItemSelection& selected,
     if( selected.indexes().isEmpty() ) {
         m_buttonGroup->button( Button_Raise )->setEnabled( false );
         m_buttonGroup->button( Button_Lower )->setEnabled( false );
-        m_buttonGroup->button( Button_Delete )->setEnabled( false );
         m_addLayerAction->setEnabled( false );
     } else {
-        m_buttonGroup->button( Button_Delete )->setEnabled( true );
         m_buttonGroup->button( Button_Raise )->setEnabled( true );
         m_buttonGroup->button( Button_Lower )->setEnabled( true );
         m_addLayerAction->setEnabled( true );
     }
+
+    if (!m_sectionView->selectionModel()->selectedIndexes().empty() &&
+            m_sectionView->selectionModel()->selectedIndexes().count() < m_doc->pages().count()) {
+        m_buttonGroup->button(Button_Delete)->setEnabled(true);
+    } else {
+        m_buttonGroup->button(Button_Delete)->setEnabled(false);
+    }
 }
 
 void KoPADocumentStructureDocker::addPage()
@@ -600,6 +629,76 @@ void KoPADocumentStructureDocker::addPage()
     }
 }
 
-#include "KoPADocumentStructureDocker.moc"
+void KoPADocumentStructureDocker::contextMenuEvent(QContextMenuEvent* event)
+{
+    QMenu menu( this );
+
+    // Not connected yet
+    menu.addAction( SmallIcon( "document-new" ), i18n("Add a new slide"), this, SLOT( addPage() ) );
+    menu.addAction( i18n("Delete selected objects"), this, SLOT( deleteItem() ));
+
+    menu.addAction( i18n( "Cut" ) ,this,  SLOT( editCut() ) );
+    menu.addAction( i18n( "Copy" ), this,  SLOT( editCopy() ));
+    menu.addAction( i18n( "Paste" ), this, SLOT( editPaste() ) );
+
+    menu.exec(event->globalPos());
+}
+
+void KoPADocumentStructureDocker::editCut()
+{
+    editCopy();
+    deleteItem();
+}
+
+void KoPADocumentStructureDocker::editCopy()
+{
+    QList<KoPAPageBase*> pages;
+    QList<KoShapeLayer*> layers;
+    QList<KoShape*> shapes;
+
+    // separate selected layers and selected shapes
+    extractSelectedLayersAndShapes( pages, layers, shapes );
+
+    foreach ( KoShape* shape, layers ) {
+        // Add layers to shapes
+        shapes.append(shape);
+    }
+
+    if ( !shapes.empty() ) {
+        // Copy Shapes or Layers
+        KoShapeOdfSaveHelper saveHelper(shapes);
+        KoDrag drag;
+        drag.setOdf(KoOdf::mimeType(KoOdf::Text), saveHelper);
+        drag.addToClipboard();
+        return;
+    }
+
+    if ( !pages.empty() ) {
+        // Copy Pages
+        KoPAOdfPageSaveHelper saveHelper( m_doc, pages );
+        KoDrag drag;
+        drag.setOdf( KoOdf::mimeType( m_doc->documentType() ), saveHelper );
+        drag.addToClipboard();
+    }
+}
+
+void KoPADocumentStructureDocker::editPaste()
+{
+    const QMimeData * data = QApplication::clipboard()->mimeData();
+
+    if (data->hasFormat(KoOdf::mimeType(KoOdf::Text))) {
+        // Paste Shapes or Layers
+        KoCanvasBase* canvas = KoToolManager::instance()->activeCanvasController()->canvas();
+        KoShapeManager * shapeManager = canvas->shapeManager();
+        KoShapePaste paste(canvas, shapeManager->selection()->activeLayer());
+        paste.paste(KoOdf::Text, data);
+
+    } else {
+        // Paste Pages
+        KoPACanvas * canvas = dynamic_cast<KoPACanvas *>( KoToolManager::instance()->activeCanvasController()->canvas() );
+        canvas->koPAView()->pagePaste();
+    }
+}
+#include <KoPADocumentStructureDocker.moc>
 
 // kate: replace-tabs on; space-indent on; indent-width 4; mixedindent off; indent-mode cstyle;
