@@ -197,7 +197,7 @@ bool TestLoading::compareBlockFormats(const QTextBlockFormat &actualFormat, cons
         switch (id) {
         case KoParagraphStyle::UnnumberedListItem:
         case KoParagraphStyle::IsListHeader:
-            if (actualProperty[id].toInt() != expectedProperty[id].toBool())
+            if (actualProperty[id].toInt() != static_cast<int>(expectedProperty[id].toBool()))
                 match = false;
             break;
         case KoParagraphStyle::AutoTextIndent:
@@ -328,6 +328,17 @@ bool TestLoading::compareTableCellFormats(QTextTableCellFormat &actualFormat, QT
             || actualFormat.rightPadding() != expectedFormat.rightPadding()
             || actualFormat.topPadding() != expectedFormat.topPadding()
             || actualFormat.bottomPadding() != expectedFormat.bottomPadding()) {
+        qDebug() << "Background or Padding mismatch";
+        qDebug() << "Actual Background: " << actualFormat.background();
+        qDebug() << "Expected background: " << expectedFormat.background();
+        qDebug() << "Actual Left Padding: " << actualFormat.leftPadding();
+        qDebug() << "Expected Left Padding: " << expectedFormat.leftPadding();
+        qDebug() << "Actual Right Padding: " << actualFormat.rightPadding();
+        qDebug() << "Expected Right Padding: " << expectedFormat.rightPadding();
+        qDebug() << "Actual Top Padding: " << actualFormat.topPadding();
+        qDebug() << "Expected Top Padding: " << expectedFormat.topPadding();
+        qDebug() << "Actual Bottom Padding: " << actualFormat.bottomPadding();
+        qDebug() << "Expected Bottom Padding: " << expectedFormat.bottomPadding();
         return false;
     }
 
@@ -345,8 +356,12 @@ bool TestLoading::compareTableCellFormats(QTextTableCellFormat &actualFormat, QT
         case KoTableCellStyle::RightBorderSpacing:
         case KoTableCellStyle::TopBorderSpacing:
         case KoTableCellStyle::BottomBorderSpacing:
-            if (abs(actualProperty[id].toDouble() - expectedProperty[id].toDouble()) > 1e-10)
+            if (abs(actualProperty[id].toDouble() - expectedProperty[id].toDouble()) > 1e-10) {
+                qDebug() << "Cell Border Spacing Mismatch";
+                qDebug() << "Expected Spacing: " << expectedProperty[id].toDouble();
+                qDebug() << "Actual Spacing: " << actualProperty[id].toDouble();
                 match = false;
+            }
             break;
         // string properties
         case KoTableCellStyle::MasterPageName:
@@ -364,8 +379,12 @@ bool TestLoading::compareTableCellFormats(QTextTableCellFormat &actualFormat, QT
         case KoTableCellStyle::BottomBorderInnerPen: {
             QPen actualPen = qvariant_cast<QPen>(actualProperty[id]);
             QPen expectedPen = qvariant_cast<QPen>(expectedProperty[id]);
-            if (actualPen != expectedPen)
+            if (actualPen != expectedPen) {
+                qDebug() << "Cell Border Pen MisMatch";
+                qDebug() << "Actual Pen:" << actualPen;
+                qDebug() << "Expected Pen:" << expectedPen;
                 match = false;
+            }
             break;
         }
         // brush properties
@@ -389,6 +408,21 @@ bool TestLoading::compareTableCellFormats(QTextTableCellFormat &actualFormat, QT
 
 bool TestLoading::compareTableCells(QTextTableCell &actualCell, QTextTableCell &expectedCell)
 {
+    //compare row and column spans
+    if (actualCell.rowSpan() != expectedCell.rowSpan()) {
+        qDebug() << "compareTableCells: table cell row span mismatch";
+        qDebug() << "Actual Row Span : " << actualCell.rowSpan();
+        qDebug() << "Expected Row Span : " << expectedCell.rowSpan();
+        return false;
+    }
+
+    if (actualCell.columnSpan() != expectedCell.columnSpan()) {
+        qDebug() << "comparetableCells: table cell column span mismatch";
+        qDebug() << "Actual Column Span : " << actualCell.columnSpan();
+        qDebug() << "Expected Column Span : " << expectedCell.columnSpan();
+        return false;
+    }
+
     // compare cell formats
     QTextTableCellFormat actualFormat = actualCell.format().toTableCellFormat();
     QTextTableCellFormat expectedFormat = expectedCell.format().toTableCellFormat();
@@ -447,9 +481,24 @@ bool TestLoading::compareTableCells(QTextTableCell &actualCell, QTextTableCell &
 
 bool TestLoading::compareTableFormats(QTextTableFormat &actualFormat, QTextTableFormat &expectedFormat)
 {
-    if (actualFormat.background() != expectedFormat.background()
-            || actualFormat.alignment() != expectedFormat.alignment()
-            || actualFormat.width() != expectedFormat.width()) {
+    if (actualFormat.background() != expectedFormat.background()) {
+        qDebug() << "compareTableFormats: Background mismatch";
+        qDebug() << "Expected Background: " << expectedFormat.background();
+        qDebug() << "Actual Background: " << actualFormat.background();
+        return false;
+    }
+
+    if (actualFormat.alignment() != expectedFormat.alignment()) {
+        qDebug() << "compareTableFormats: Alignment mismatch";
+        qDebug() << "Expected Alignment: " << expectedFormat.alignment();
+        qDebug() << "Actual Alignment: " << actualFormat.alignment();
+        return false;
+    }
+
+    if (actualFormat.width() != expectedFormat.width()) {
+        qDebug() << "compareTableFormats: Width mismatch";
+        qDebug() << "Expected Width: " << expectedFormat.width();
+        qDebug() << "Actual Width: " << actualFormat.width();
         return false;
     }
 
@@ -505,6 +554,20 @@ bool TestLoading::compareTables(QTextTable *actualTable, QTextTable *expectedTab
         return false;
     }
 
+    if (actualTable->rows() != expectedTable->rows()) {
+        qDebug() << "compareTables: Rows mismatch";
+        qDebug() << "Actual Rows : " << actualTable->rows();
+        qDebug() << "Expected Rows : " << expectedTable->rows();
+        return false;
+    }
+
+    if (actualTable->columns() != expectedTable->columns()) {
+        qDebug() << "compareTables: Columns mismatch";
+        qDebug() << "Actual Columns : " << actualTable->columns();
+        qDebug() << "Expected Columns : " << expectedTable->columns();
+        return false;
+    }
+
     for (int row = 0; row < expectedTable->rows(); ++row) {
         for (int col = 0; col < expectedTable->columns(); ++col) {
             QTextTableCell actualCell = actualTable->cellAt(row, col);
@@ -748,7 +811,7 @@ void TestLoading::initTestCase()
     QScriptValue rc = engine->importExtension("qt.core");
     if (rc.isError()) {
         kWarning() << "Failed to find Qt bindings, aborting";
-        abort();
+        QSKIP("Failed to find Qt bindings (qtscript generator), aborting", SkipAll);
     }
     rc = engine->importExtension("qt.gui");
     if (rc.isError()) {
@@ -812,9 +875,23 @@ void TestLoading::addData(LoadSave loadSave)
     QTest::newRow("fontColors") << "TextContents/TextFormatting/fontColors";
 
     if (loadSave == TestLoadingData) {
-        QTest::newRow("tableWidth") << "FormattingProperties/TableFormattingProperties/tableWidth";
-        QTest::newRow("tableTopAndBottomMargin") << "FormattingProperties/TableFormattingProperties/tableTopAndBottomMargin";
-        QTest::newRow("tableLeftAndRightMargin") << "FormattingProperties/TableFormattingProperties/tableLeftAndRightMargin";
+        QTest::newRow("simpleTable") << "Tables/simpleTable";
+        QTest::newRow("spanningCells") << "Tables/spanningCells";
+        QTest::newRow("tableWidth") << "Tables/tableWidth";
+        QTest::newRow("tableAlignment") << "Tables/tableAlignment";
+        QTest::newRow("tableTopAndBottomMargin") << "Tables/tableTopAndBottomMargin";
+        QTest::newRow("tableLeftAndRightMargin") << "Tables/tableLeftAndRightMargin";
+        QTest::newRow("tableMargins") << "Tables/tableMargins";
+        QTest::newRow("breakBeforeAndBreakAfter") << "Tables/breakBeforeAndBreakAfter";
+        QTest::newRow("mayBreakBetweenRows") << "Tables/mayBreakBetweenRows";
+        QTest::newRow("tableBackground") << "Tables/tableBackground";
+        QTest::newRow("borderModelProperty") << "Tables/borderModelProperty";
+        QTest::newRow("keepWithNext") << "Tables/keepWithNext";
+        QTest::newRow("cellBackground") << "Tables/cellBackground";
+        QTest::newRow("cellPadding") << "Tables/cellPadding";
+        QTest::newRow("cellBorder") << "Tables/cellBorder";
+        QTest::newRow("cellBorderStyle") << "Tables/cellBorderStyle";
+        QTest::newRow("cellBorderSpacing") << "Tables/cellBorderSpacing";
     }
 
     // TODO: Write tests for these.
@@ -913,10 +990,9 @@ QTextDocument *TestLoading::documentFromOdt(const QString &odt)
     KoChangeTracker *changeTracker = new KoChangeTracker;
 
     KoOdfLoadingContext odfLoadingContext(odfReadStore.styles(), odfReadStore.store(), *componentData);
-    QMap<QString, KoDataCenter *> dataCenterMap;
-    KoShapeLoadingContext shapeLoadingContext(odfLoadingContext, dataCenterMap);
+    KoShapeLoadingContext shapeLoadingContext(odfLoadingContext, 0);
     KoTextSharedLoadingData *textSharedLoadingData = new KoTextSharedLoadingData;
-    textSharedLoadingData->loadOdfStyles(odfLoadingContext, styleManager);
+    textSharedLoadingData->loadOdfStyles(shapeLoadingContext, styleManager);
     shapeLoadingContext.addSharedData(KOTEXT_SHARED_LOADING_ID, textSharedLoadingData);
 
     KoTextShapeData *textShapeData = new KoTextShapeData;
@@ -964,13 +1040,14 @@ QString TestLoading::documentToOdt(QTextDocument *document)
 
     KoGenStyles mainStyles;
     KoStyleManager *styleMan = KoTextDocument(document).styleManager();
+    Q_UNUSED(styleMan);
     KoEmbeddedDocumentSaver embeddedSaver;
 
     KoGenChanges changes;
     KoShapeSavingContext context(xmlWriter, mainStyles, embeddedSaver);
 
     KoSharedSavingData *sharedData = context.sharedData(KOTEXT_SHARED_SAVING_ID);
-    KoTextSharedSavingData *textSharedData;
+    KoTextSharedSavingData *textSharedData = 0;
     if (sharedData) {
         textSharedData = dynamic_cast<KoTextSharedSavingData *>(sharedData);
     }
@@ -1005,7 +1082,7 @@ QString TestLoading::documentToOdt(QTextDocument *document)
 
     contentTmpFile.close();
 
-    mainStyles.saveOdfAutomaticStyles(contentWriter, false);
+    mainStyles.saveOdfStyles(KoGenStyles::DocumentAutomaticStyles, contentWriter);
 
     contentWriter->startElement("office:body");
     contentWriter->startElement("office:text");
@@ -1056,6 +1133,7 @@ void TestLoading::testLoading()
 
     bool documentsEqual = compareDocuments(actualDocument, expectedDocument);
 
+    Q_UNUSED(showDocument);
 //    showDocument(actualDocument);
 //    showDocument(expectedDocument);
     if (!documentsEqual) {
@@ -1106,4 +1184,4 @@ int main(int argc, char *argv[])
     return QTest::qExec(new TestLoading, argc, argv);
 }
 
-#include "TestLoading.moc"
+#include <TestLoading.moc>
