@@ -22,30 +22,43 @@
 
 #include "ui_koDocumentInfoAboutWidget.h"
 #include "ui_koDocumentInfoAuthorWidget.h"
+
 #include "KoDocumentInfo.h"
 #include "KoDocument.h"
 #include "KoMainWindow.h"
+#include "rdf/KoDocumentRdfEditWidgetBase.h"
+#ifdef SHOULD_BUILD_RDF
+#include "rdf/KoDocumentRdfEditWidget.h"
+#endif
+
 #include <kmimetype.h>
 #include <klocale.h>
 #include <kglobal.h>
-#include <kabc/addressee.h>
-#include <kabc/stdaddressbook.h>
-#include <KoGlobal.h>
 #include <kiconloader.h>
 #include <kmessagebox.h>
-#include <kstdguiitem.h>
+#include <kconfiggroup.h>
+
+#ifdef KDEPIMLIBS_FOUND
+#include <kabc/addressee.h>
+#include <kabc/stdaddressbook.h>
+#endif
+
+#include <KoGlobal.h>
+#include <KoEncryptionChecker.h>
 
 #include <QLabel>
 #include <QLineEdit>
 #include <QTextEdit>
 #include <QPixmap>
 #include <QDateTime>
-#include <kconfiggroup.h>
+
 
 class KoDocumentInfoDlg::KoDocumentInfoDlgPrivate
 {
 public:
     KoDocumentInfoDlgPrivate() :
+            m_rdf(0),
+            m_rdfEditWidget(0),
             m_toggleEncryption(false),
             m_applyToggleEncryption(false),
             m_documentSaved(false) {}
@@ -54,18 +67,24 @@ public:
     KoDocumentInfo* m_info;
     Ui::KoDocumentInfoAboutWidget* m_aboutUi;
     Ui::KoDocumentInfoAuthorWidget* m_authorUi;
-
+    KoDocumentRdf* m_rdf;
+#ifdef SHOULD_BUILD_RDF
+    KoDocumentRdfEditWidget* m_rdfEditWidget;
+#else
+    KoDocumentRdfEditWidgetBase* m_rdfEditWidget;
+#endif
     bool m_toggleEncryption;
     bool m_applyToggleEncryption;
     bool m_documentSaved;
 };
 
 
-KoDocumentInfoDlg::KoDocumentInfoDlg(QWidget* parent, KoDocumentInfo* docInfo)
+KoDocumentInfoDlg::KoDocumentInfoDlg(QWidget* parent, KoDocumentInfo* docInfo, KoDocumentRdf* docRdf)
         : KPageDialog(parent)
         , d(new KoDocumentInfoDlgPrivate)
 {
     d->m_info = docInfo;
+    d->m_rdf = docRdf;
 
     setCaption(i18n("Document Information"));
     setInitialSize(QSize(500, 500));
@@ -76,6 +95,12 @@ KoDocumentInfoDlg::KoDocumentInfoDlg(QWidget* parent, KoDocumentInfo* docInfo)
     d->m_aboutUi = new Ui::KoDocumentInfoAboutWidget();
     QWidget *infodlg = new QWidget();
     d->m_aboutUi->setupUi(infodlg);
+    if (!KoEncryptionChecker::isEncryptionSupported()) {
+        d->m_aboutUi->lblEncryptedDesc->setVisible(false);
+        d->m_aboutUi->lblEncrypted->setVisible(false);
+        d->m_aboutUi->pbEncrypt->setVisible(false);
+        d->m_aboutUi->lblEncryptedPic->setVisible(false);
+    }
     KPageWidgetItem *page = new KPageWidgetItem(infodlg, i18n("General"));
     page->setHeader(i18n("General"));
     KoDocument* doc = dynamic_cast< KoDocument* >(d->m_info->parent());
@@ -97,9 +122,20 @@ KoDocumentInfoDlg::KoDocumentInfoDlg(QWidget* parent, KoDocumentInfo* docInfo)
 
     initAuthorTab();
 
-    connect(this, SIGNAL(okClicked()), this, SLOT(slotApply()));
     // Saving encryption implies saving the document, this is done after closing the dialog
     connect(this, SIGNAL(hidden()), this, SLOT(slotSaveEncryption()));
+
+    if (d->m_rdf) {
+        d->m_rdfEditWidget = 0;
+
+#ifdef SHOULD_BUILD_RDF
+        d->m_rdfEditWidget = new KoDocumentRdfEditWidget(this, d->m_rdf);
+        page = new KPageWidgetItem(d->m_rdfEditWidget->widget(), i18n("Rdf"));
+        page->setHeader(i18n("Rdf"));
+        page->setIcon(KIcon("text-rdf"));
+        addPage(page);
+#endif
+    }
 }
 
 KoDocumentInfoDlg::~KoDocumentInfoDlg()
@@ -109,6 +145,23 @@ KoDocumentInfoDlg::~KoDocumentInfoDlg()
     delete d;
 }
 
+void KoDocumentInfoDlg::slotButtonClicked(int button)
+{
+    emit buttonClicked(static_cast<KDialog::ButtonCode>(button));
+    switch (button) {
+    case Ok:
+        if (d->m_rdfEditWidget) {
+            if (d->m_rdfEditWidget->shouldDialogCloseBeVetoed()) {
+                return;
+            }
+        }
+        slotApply();
+        accept();
+        return;
+    }
+    KPageDialog::slotButtonClicked(button);
+}
+
 bool KoDocumentInfoDlg::isDocumentSaved()
 {
     return d->m_documentSaved;
@@ -151,32 +204,36 @@ void KoDocumentInfoDlg::initAboutTab()
 
     d->m_aboutUi->lblRevision->setText(d->m_info->aboutInfo("editing-cycles"));
 
-    if (doc->specialOutputFlag() == KoDocument::SaveEncrypted) {
-        if (d->m_toggleEncryption) {
-            QPixmap p = KIconLoader::global()->loadIcon("object-unlocked", KIconLoader::Small);
-            d->m_aboutUi->lblEncrypted->setText(i18n("This document will be decrypted"));
-            d->m_aboutUi->lblEncryptedPic->setPixmap(p);
-            d->m_aboutUi->pbEncrypt->setText(i18n("Do not decrypt"));
+    if ( doc->supportedSpecialFormats() & KoDocument::SaveEncrypted ) {
+        if (doc->specialOutputFlag() == KoDocument::SaveEncrypted) {
+            if (d->m_toggleEncryption) {
+                QPixmap p = KIconLoader::global()->loadIcon("object-unlocked", KIconLoader::Small);
+                d->m_aboutUi->lblEncrypted->setText(i18n("This document will be decrypted"));
+                d->m_aboutUi->lblEncryptedPic->setPixmap(p);
+                d->m_aboutUi->pbEncrypt->setText(i18n("Do not decrypt"));
+            } else {
+                QPixmap p = KIconLoader::global()->loadIcon("object-locked", KIconLoader::Small);
+                d->m_aboutUi->lblEncrypted->setText(i18n("This document is encrypted"));
+                d->m_aboutUi->lblEncryptedPic->setPixmap(p);
+                d->m_aboutUi->pbEncrypt->setText(i18n("D&ecrypt"));
+            }
         } else {
-            QPixmap p = KIconLoader::global()->loadIcon("object-locked", KIconLoader::Small);
-            d->m_aboutUi->lblEncrypted->setText(i18n("This document is encrypted"));
-            d->m_aboutUi->lblEncryptedPic->setPixmap(p);
-            d->m_aboutUi->pbEncrypt->setText(i18n("D&ecrypt"));
+            if (d->m_toggleEncryption) {
+                QPixmap p = KIconLoader::global()->loadIcon("object-locked", KIconLoader::Small);
+                d->m_aboutUi->lblEncrypted->setText(i18n("This document will be encrypted."));
+                d->m_aboutUi->lblEncryptedPic->setPixmap(p);
+                d->m_aboutUi->pbEncrypt->setText(i18n("Do not encrypt"));
+            } else {
+                QPixmap p = KIconLoader::global()->loadIcon("object-unlocked", KIconLoader::Small);
+                d->m_aboutUi->lblEncrypted->setText(i18n("This document is not encrypted"));
+                d->m_aboutUi->lblEncryptedPic->setPixmap(p);
+                d->m_aboutUi->pbEncrypt->setText(i18n("&Encrypt"));
+            }
         }
     } else {
-        if (d->m_toggleEncryption) {
-            QPixmap p = KIconLoader::global()->loadIcon("object-locked", KIconLoader::Small);
-            d->m_aboutUi->lblEncrypted->setText(i18n("This document will be encrypted."));
-            d->m_aboutUi->lblEncryptedPic->setPixmap(p);
-            d->m_aboutUi->pbEncrypt->setText(i18n("Do not encrypt"));
-        } else {
-            QPixmap p = KIconLoader::global()->loadIcon("object-unlocked", KIconLoader::Small);
-            d->m_aboutUi->lblEncrypted->setText(i18n("This document is not encrypted"));
-            d->m_aboutUi->lblEncryptedPic->setPixmap(p);
-            d->m_aboutUi->pbEncrypt->setText(i18n("&Encrypt"));
-        }
+        d->m_aboutUi->lblEncrypted->setText(i18n("This document does not support encryption"));
+        d->m_aboutUi->pbEncrypt->setEnabled( false );
     }
-
     connect(d->m_aboutUi->pbReset, SIGNAL(clicked()),
             this, SLOT(slotResetMetaData()));
     connect(d->m_aboutUi->pbEncrypt, SIGNAL(clicked()),
@@ -204,8 +261,13 @@ void KoDocumentInfoDlg::initAuthorTab()
     d->m_authorUi->leStreet->setText(d->m_info->authorInfo("street"));
     d->m_authorUi->lePosition->setText(d->m_info->authorInfo("position"));
 
+#ifdef KDEPIMLIBS_FOUND
     connect(d->m_authorUi->pbLoadKABC, SIGNAL(clicked()),
             this, SLOT(slotLoadFromKABC()));
+#else
+    d->m_authorUi->pbLoadKABC->hide();
+#endif
+
     connect(d->m_authorUi->pbDelete, SIGNAL(clicked()),
             this, SLOT(slotDeleteAuthorInfo()));
 }
@@ -214,6 +276,9 @@ void KoDocumentInfoDlg::slotApply()
 {
     saveAboutData();
     saveAuthorData();
+    if (d->m_rdfEditWidget) {
+        d->m_rdfEditWidget->apply();
+    }
 }
 
 void KoDocumentInfoDlg::saveAboutData()
@@ -327,6 +392,7 @@ void KoDocumentInfoDlg::slotDeleteAuthorInfo()
 
 void KoDocumentInfoDlg::slotLoadFromKABC()
 {
+#ifdef KDEPIMLIBS_FOUND
     KABC::StdAddressBook *ab = static_cast<KABC::StdAddressBook*>
                                (KABC::StdAddressBook::self());
     if (!ab)
@@ -334,7 +400,7 @@ void KoDocumentInfoDlg::slotLoadFromKABC()
 
     KABC::Addressee addr = ab->whoAmI();
     if (addr.isEmpty()) {
-        KMessageBox::sorry(0L, i18n("No personal contact data set, please use the option \
+        KMessageBox::sorry(0, i18n("No personal contact data set, please use the option \
                                     \"Set as Personal Contact Data\" from the \"Edit\"     menu in KAddressbook to set one."));
         return;
     }
@@ -359,6 +425,7 @@ void KoDocumentInfoDlg::slotLoadFromKABC()
     d->m_authorUi->lePostal->setText(a.postalCode());
     d->m_authorUi->leCity->setText(a.locality());
     d->m_authorUi->leStreet->setText(a.street());
+#endif
 }
 
 void KoDocumentInfoDlg::slotSaveEncryption()
@@ -456,4 +523,4 @@ void KoDocumentInfoDlg::slotSaveEncryption()
     d->m_documentSaved = !doc->url().isEmpty();
 }
 
-#include "KoDocumentInfoDlg.moc"
+#include <KoDocumentInfoDlg.moc>
