@@ -1,6 +1,7 @@
 /* This file is part of the KDE project
  *
  * Copyright (C) 2009 Inge Wallin <inge@lysator.liu.se>
+ * Copyright (C) 2009 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,7 +19,6 @@
  * Boston, MA 02110-1301, USA.
  */
 
-
 #include "KoBorder.h"
 
 #include <KDebug>
@@ -36,35 +36,38 @@ public:
     KoBorderPrivate();
     ~KoBorderPrivate();
 
-    KoBorder::BorderData  leftBorder;
-    KoBorder::BorderData  topBorder;
-    KoBorder::BorderData  rightBorder;
-    KoBorder::BorderData  bottomBorder;
+    KoBorder::BorderData leftBorder;
+    KoBorder::BorderData topBorder;
+    KoBorder::BorderData rightBorder;
+    KoBorder::BorderData bottomBorder;
 };
 
 KoBorderPrivate::KoBorderPrivate()
 {
-    leftBorder.style  = KoBorder::BorderNone;
-    topBorder.style   = KoBorder::BorderNone;
-    rightBorder.style = KoBorder::BorderNone;
-    topBorder.style   = KoBorder::BorderNone;
 }
 
 KoBorderPrivate::~KoBorderPrivate()
 {
 }
 
+KoBorder::BorderData::BorderData()
+    : style(KoBorder::BorderNone),
+    width(0),
+    innerWidth(0),
+    spacing(0)
+{
+}
+
 
 // ----------------------------------------------------------------
 
-
 KoBorder::KoBorder()
-    : d( new KoBorderPrivate )
+    : d(new KoBorderPrivate)
 {
 }
 
 KoBorder::KoBorder(const KoBorder &kb)
-    : d( kb.d )
+    : d(kb.d)
 {
 }
 
@@ -76,7 +79,6 @@ KoBorder::~KoBorder()
 // ----------------------------------------------------------------
 //                             operators
 
-
 KoBorder &KoBorder::operator=(const KoBorder &other)
 {
     d = other.d;
@@ -84,10 +86,9 @@ KoBorder &KoBorder::operator=(const KoBorder &other)
     return *this;
 }
 
-
 bool KoBorder::operator==(const KoBorder &other) const
 {
-    if (d == other.d)
+    if (d.data() == other.d.data())
         return true;
 
     // Left Borders
@@ -213,7 +214,6 @@ bool KoBorder::operator==(const KoBorder &other) const
 // ----------------------------------------------------------------
 //                 public, non-class functions
 
-
 KoBorder::BorderStyle KoBorder::odfBorderStyle(const QString &borderstyle)
 {
     // Note: the styles marked "Not odf compatible" below are legacies
@@ -484,10 +484,30 @@ qreal KoBorder::bottomBorderSpacing() const
 }
 
 
+KoBorder::BorderData KoBorder::leftBorderData() const
+{
+    return d->leftBorder;
+}
+
+KoBorder::BorderData KoBorder::topBorderData() const
+{
+    return d->topBorder;
+}
+
+KoBorder::BorderData KoBorder::rightBorderData() const
+{
+    return d->rightBorder;
+}
+
+KoBorder::BorderData KoBorder::bottomBorderData() const
+{
+    return d->bottomBorder;
+}
+
+
 // ----------------------------------------------------------------
 //                         load and save
 
-
 void KoBorder::loadOdf(const KoXmlElement &style)
 {
     if (style.hasAttributeNS(KoXmlNS::fo, "border")) {
@@ -623,7 +643,6 @@ void KoBorder::loadOdf(const KoXmlElement &style)
     }
 }
 
-
 void KoBorder::saveOdf(KoGenStyle &style) const
 {
     // Get the strings that describe respective borders.
