@@ -21,15 +21,15 @@
 #ifndef DEFAULTTOOLFACTORY_H
 #define DEFAULTTOOLFACTORY_H
 
-#include <KoToolFactory.h>
+#include <KoToolFactoryBase.h>
 
 /// Factory for the KoInteractionTool
-class DefaultToolFactory : public KoToolFactory {
+class DefaultToolFactory : public KoToolFactoryBase {
 public:
     /// constructor
     DefaultToolFactory(QObject *parent);
     ~DefaultToolFactory();
 
-    KoTool* createTool(KoCanvasBase *canvas);
+    KoToolBase* createTool(KoCanvasBase *canvas);
 };
 #endif
