@@ -20,13 +20,12 @@
 #ifndef SHADOWDOCKER_H
 #define SHADOWDOCKER_H
 
-#include <KoCanvasObserver.h>
+#include <KoCanvasObserverBase.h>
 #include <QtGui/QDockWidget>
 
-class KoUnit;
 
 /// A docker for setting properties of a shape shadow
-class ShadowDocker : public QDockWidget, public KoCanvasObserver
+class ShadowDocker : public QDockWidget, public KoCanvasObserverBase
 {
     Q_OBJECT
 
@@ -43,6 +42,8 @@ private slots:
     virtual void setCanvas( KoCanvasBase *canvas );
 
     void shadowChanged();
+    void locationChanged(Qt::DockWidgetArea area);
+
 private:
     class Private;
     Private * const d;
